import{CredentialUnavailableError}from"../errors";import{credentialLogger,formatError,formatSuccess}from"../util/logging";import{tracingClient}from"../util/tracing";import{ensureValidScope,getScopeResource}from"../util/scopeUtils";import{processUtils}from"../util/processUtils";import{processMultiTenantRequest}from"../util/validateMultiTenant";import{checkTenantId}from"../util/checkTenantId";const logger=credentialLogger("AzurePowerShellCredential"),isWindows="win32"===process.platform;export function formatCommand(commandName){return isWindows?`${commandName}.exe`:commandName}async function runCommands(commands){const results=[];for(const command of commands){const[file,...parameters]=command,result=await processUtils.execFile(file,parameters,{encoding:"utf8"});results.push(result)}return results}export const powerShellErrors={login:"Run Connect-AzAccount to login",installed:"The specified module 'Az.Accounts' with version '2.2.0' was not loaded because no valid module file was found in any module directory"};export const powerShellPublicErrorMessages={login:"Please run 'Connect-AzAccount' from PowerShell to authenticate before using this credential.",installed:"The 'Az.Account' module >= 2.2.0 is not installed. Install the Azure Az PowerShell module with: \"Install-Module -Name Az -Scope CurrentUser -Repository PSGallery -Force\".",troubleshoot:"To troubleshoot, visit https://aka.ms/azsdk/js/identity/powershellcredential/troubleshoot."};const isLoginError=err=>err.message.match(`(.*)${powerShellErrors.login}(.*)`),isNotInstalledError=err=>err.message.match(powerShellErrors.installed);export const commandStack=[formatCommand("pwsh")];isWindows&&commandStack.push(formatCommand("powershell"));export class AzurePowerShellCredential{constructor(options){this.tenantId=null==options?void 0:options.tenantId}async getAzurePowerShellAccessToken(resource,tenantId){for(const powerShellCommand of[...commandStack]){try{await runCommands([[powerShellCommand,"/?"]])}catch(e){commandStack.shift();continue}let tenantSection="";tenantId&&(tenantSection=`-TenantId "${tenantId}"`);const result=(await runCommands([[powerShellCommand,"-Command","Import-Module Az.Accounts -MinimumVersion 2.2.0 -PassThru"],[powerShellCommand,"-Command",`Get-AzAccessToken ${tenantSection} -ResourceUrl "${resource}" | ConvertTo-Json`]]))[1];try{return JSON.parse(result)}catch(e){throw new Error(`Unable to parse the output of PowerShell. Received output: ${result}`)}}throw new Error("Unable to execute PowerShell. Ensure that it is installed in your system")}async getToken(scopes,options={}){return tracingClient.withSpan(`${this.constructor.name}.getToken`,options,(async()=>{const tenantId=processMultiTenantRequest(this.tenantId,options);tenantId&&checkTenantId(logger,tenantId);const scope="string"==typeof scopes?scopes:scopes[0];ensureValidScope(scope,logger),logger.getToken.info(`Using the scope ${scope}`);const resource=getScopeResource(scope);try{const response=await this.getAzurePowerShellAccessToken(resource,tenantId);return logger.getToken.info(formatSuccess(scopes)),{token:response.Token,expiresOnTimestamp:new Date(response.ExpiresOn).getTime()}}catch(err){if(isNotInstalledError(err)){const error=new CredentialUnavailableError(powerShellPublicErrorMessages.installed);throw logger.getToken.info(formatError(scope,error)),error}if(isLoginError(err)){const error=new CredentialUnavailableError(powerShellPublicErrorMessages.login);throw logger.getToken.info(formatError(scope,error)),error}const error=new CredentialUnavailableError(`${err}. ${powerShellPublicErrorMessages.troubleshoot}`);throw logger.getToken.info(formatError(scope,error)),error}}))}}