import fs from"fs";import os from"os";import path from"path";import{AzureAuthorityHosts}from"../constants";import{checkTenantId}from"../util/checkTenantId";import{CredentialUnavailableError}from"../errors";import{IdentityClient}from"../client/identityClient";import{processMultiTenantRequest}from"../util/validateMultiTenant";import{credentialLogger,formatError,formatSuccess}from"../util/logging";const CommonTenantId="common",AzureAccountClientId="aebc6443-996d-45c2-90f0-388ff96faa56",logger=credentialLogger("VisualStudioCodeCredential");let findCredentials;export const vsCodeCredentialControl={setVsCodeCredentialFinder(finder){findCredentials=finder}};const unsupportedTenantIds={adfs:"The VisualStudioCodeCredential does not support authentication with ADFS tenants."};function checkUnsupportedTenant(tenantId){const unsupportedTenantError=unsupportedTenantIds[tenantId];if(unsupportedTenantError)throw new CredentialUnavailableError(unsupportedTenantError)}const mapVSCodeAuthorityHosts={AzureCloud:AzureAuthorityHosts.AzurePublicCloud,AzureChina:AzureAuthorityHosts.AzureChina,AzureGermanCloud:AzureAuthorityHosts.AzureGermany,AzureUSGovernment:AzureAuthorityHosts.AzureGovernment};export function getPropertyFromVSCode(property){const settingsPath=["User","settings.json"],vsCodeFolder="Code",homedir=os.homedir();function loadProperty(...pathSegments){const fullPath=path.join(...pathSegments,vsCodeFolder,...settingsPath);return JSON.parse(fs.readFileSync(fullPath,{encoding:"utf8"}))[property]}try{let appData;switch(process.platform){case"win32":return appData=process.env.APPDATA,appData?loadProperty(appData):void 0;case"darwin":return loadProperty(homedir,"Library","Application Support");case"linux":return loadProperty(homedir,".config");default:return}}catch(e){return void logger.info(`Failed to load the Visual Studio Code configuration file. Error: ${e.message}`)}}export class VisualStudioCodeCredential{constructor(options){this.cloudName=getPropertyFromVSCode("azure.cloud")||"AzureCloud";const authorityHost=mapVSCodeAuthorityHosts[this.cloudName];this.identityClient=new IdentityClient(Object.assign({authorityHost:authorityHost},options)),options&&options.tenantId?(checkTenantId(logger,options.tenantId),this.tenantId=options.tenantId):this.tenantId=CommonTenantId,checkUnsupportedTenant(this.tenantId)}async prepare(){const settingsTenant=getPropertyFromVSCode("azure.tenant");settingsTenant&&(this.tenantId=settingsTenant),checkUnsupportedTenant(this.tenantId)}prepareOnce(){return this.preparePromise||(this.preparePromise=this.prepare()),this.preparePromise}async getToken(scopes,options){var _a,_b;await this.prepareOnce();const tenantId=processMultiTenantRequest(this.tenantId,options)||this.tenantId;if(void 0===findCredentials)throw new CredentialUnavailableError(["No implementation of `VisualStudioCodeCredential` is available.","You must install the identity-vscode plugin package (`npm install --save-dev @azure/identity-vscode`)","and enable it by importing `useIdentityPlugin` from `@azure/identity` and calling","`useIdentityPlugin(vsCodePlugin)` before creating a `VisualStudioCodeCredential`."].join(" "));let scopeString="string"==typeof scopes?scopes:scopes.join(" ");if(!scopeString.match(/^[0-9a-zA-Z-.:/]+$/)){const error=new Error("Invalid scope was specified by the user or calling client");throw logger.getToken.info(formatError(scopes,error)),error}scopeString.indexOf("offline_access")<0&&(scopeString+=" offline_access");const credentials=await findCredentials(),{password:refreshToken}=null!==(_b=null!==(_a=credentials.find((({account:account})=>account===this.cloudName)))&&void 0!==_a?_a:credentials[0])&&void 0!==_b?_b:{};if(refreshToken){const tokenResponse=await this.identityClient.refreshAccessToken(tenantId,AzureAccountClientId,scopeString,refreshToken,void 0);if(tokenResponse)return logger.getToken.info(formatSuccess(scopes)),tokenResponse.accessToken;{const error=new CredentialUnavailableError("Could not retrieve the token associated with Visual Studio Code. Have you connected using the 'Azure Account' extension recently? To troubleshoot, visit https://aka.ms/azsdk/js/identity/vscodecredential/troubleshoot.");throw logger.getToken.info(formatError(scopes,error)),error}}{const error=new CredentialUnavailableError("Could not retrieve the token associated with Visual Studio Code. Did you connect using the 'Azure Account' extension? To troubleshoot, visit https://aka.ms/azsdk/js/identity/vscodecredential/troubleshoot.");throw logger.getToken.info(formatError(scopes,error)),error}}}