/*! @azure/msal-browser v2.37.1 2023-06-07 */
"use strict";import{__awaiter,__generator,__assign}from"../_virtual/_tslib.js";import{CryptoOps}from"../crypto/CryptoOps.js";import{PerformanceEvents,Constants,InteractionRequiredAuthError,AuthError,ClientAuthError,PromptValue,Logger,StubPerformanceClient,DEFAULT_CRYPTO_IMPLEMENTATION}from"@azure/msal-common";import{BrowserCacheManager,DEFAULT_BROWSER_CACHE_MANAGER}from"../cache/BrowserCacheManager.js";import{buildConfiguration}from"../config/Configuration.js";import{InteractionType,ApiId,TemporaryCacheKeys,CacheLookupPolicy,BrowserCacheLocation}from"../utils/BrowserConstants.js";import{BrowserUtils}from"../utils/BrowserUtils.js";import{name,version}from"../packageMetadata.js";import{EventType}from"../event/EventType.js";import{BrowserConfigurationAuthError}from"../error/BrowserConfigurationAuthError.js";import{EventHandler}from"../event/EventHandler.js";import{PopupClient}from"../interaction_client/PopupClient.js";import{RedirectClient}from"../interaction_client/RedirectClient.js";import{SilentIframeClient}from"../interaction_client/SilentIframeClient.js";import{SilentRefreshClient}from"../interaction_client/SilentRefreshClient.js";import{TokenCache}from"../cache/TokenCache.js";import{NativeInteractionClient}from"../interaction_client/NativeInteractionClient.js";import{NativeMessageHandler}from"../broker/nativeBroker/NativeMessageHandler.js";import{NativeAuthError}from"../error/NativeAuthError.js";import{SilentCacheClient}from"../interaction_client/SilentCacheClient.js";import{SilentAuthCodeClient}from"../interaction_client/SilentAuthCodeClient.js";import{BrowserAuthError}from"../error/BrowserAuthError.js";import{BrowserPerformanceClient}from"../telemetry/BrowserPerformanceClient.js";var ClientApplication=function(){function ClientApplication(configuration){this.isBrowserEnvironment="undefined"!=typeof window,this.config=buildConfiguration(configuration,this.isBrowserEnvironment),this.initialized=!1,this.logger=new Logger(this.config.system.loggerOptions,name,version),this.networkClient=this.config.system.networkClient,this.navigationClient=this.config.system.navigationClient,this.redirectResponse=new Map,this.hybridAuthCodeResponses=new Map,this.performanceClient=this.isBrowserEnvironment?new BrowserPerformanceClient(this.config.auth.clientId,this.config.auth.authority,this.logger,name,version,this.config.telemetry.application,this.config.system.cryptoOptions):new StubPerformanceClient(this.config.auth.clientId,this.config.auth.authority,this.logger,name,version,this.config.telemetry.application),this.browserCrypto=this.isBrowserEnvironment?new CryptoOps(this.logger,this.performanceClient,this.config.system.cryptoOptions):DEFAULT_CRYPTO_IMPLEMENTATION,this.eventHandler=new EventHandler(this.logger,this.browserCrypto),this.browserStorage=this.isBrowserEnvironment?new BrowserCacheManager(this.config.auth.clientId,this.config.cache,this.browserCrypto,this.logger):DEFAULT_BROWSER_CACHE_MANAGER(this.config.auth.clientId,this.logger);var nativeCacheOptions={cacheLocation:BrowserCacheLocation.MemoryStorage,temporaryCacheLocation:BrowserCacheLocation.MemoryStorage,storeAuthStateInCookie:!1,secureCookies:!1,cacheMigrationEnabled:!1};this.nativeInternalStorage=new BrowserCacheManager(this.config.auth.clientId,nativeCacheOptions,this.browserCrypto,this.logger),this.tokenCache=new TokenCache(this.config,this.browserStorage,this.logger,this.browserCrypto),this.trackPageVisibilityWithMeasurement=this.trackPageVisibilityWithMeasurement.bind(this)}return ClientApplication.prototype.initialize=function(){return __awaiter(this,void 0,void 0,(function(){var allowNativeBroker,initMeasurement,_a,e_1;return __generator(this,(function(_b){switch(_b.label){case 0:if(this.logger.trace("initialize called"),this.initialized)return this.logger.info("initialize has already been called, exiting early."),[2];if(allowNativeBroker=this.config.system.allowNativeBroker,initMeasurement=this.performanceClient.startMeasurement(PerformanceEvents.InitializeClientApplication),this.eventHandler.emitEvent(EventType.INITIALIZE_START),!allowNativeBroker)return[3,4];_b.label=1;case 1:return _b.trys.push([1,3,,4]),_a=this,[4,NativeMessageHandler.createProvider(this.logger,this.config.system.nativeBrokerHandshakeTimeout,this.performanceClient)];case 2:return _a.nativeExtensionProvider=_b.sent(),[3,4];case 3:return e_1=_b.sent(),this.logger.verbose(e_1),[3,4];case 4:return this.initialized=!0,this.eventHandler.emitEvent(EventType.INITIALIZE_END),initMeasurement.endMeasurement({allowNativeBroker:allowNativeBroker,success:!0}),[2]}}))}))},ClientApplication.prototype.handleRedirectPromise=function(hash){return __awaiter(this,void 0,void 0,(function(){var loggedInAccounts,redirectResponseKey,response,request,redirectResponse,nativeClient,correlationId,redirectClient,_this=this;return __generator(this,(function(_a){return this.logger.verbose("handleRedirectPromise called"),BrowserUtils.blockNativeBrokerCalledBeforeInitialized(this.config.system.allowNativeBroker,this.initialized),loggedInAccounts=this.getAllAccounts(),this.isBrowserEnvironment?(redirectResponseKey=hash||Constants.EMPTY_STRING,void 0===(response=this.redirectResponse.get(redirectResponseKey))?(this.eventHandler.emitEvent(EventType.HANDLE_REDIRECT_START,InteractionType.Redirect),this.logger.verbose("handleRedirectPromise has been called for the first time, storing the promise"),request=this.browserStorage.getCachedNativeRequest(),redirectResponse=void 0,request&&NativeMessageHandler.isNativeAvailable(this.config,this.logger,this.nativeExtensionProvider)&&this.nativeExtensionProvider&&!hash?(this.logger.trace("handleRedirectPromise - acquiring token from native platform"),nativeClient=new NativeInteractionClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,ApiId.handleRedirectPromise,this.performanceClient,this.nativeExtensionProvider,request.accountId,this.nativeInternalStorage,request.correlationId),redirectResponse=nativeClient.handleRedirectPromise()):(this.logger.trace("handleRedirectPromise - acquiring token from web flow"),correlationId=this.browserStorage.getTemporaryCache(TemporaryCacheKeys.CORRELATION_ID,!0)||Constants.EMPTY_STRING,redirectClient=this.createRedirectClient(correlationId),redirectResponse=redirectClient.handleRedirectPromise(hash)),response=redirectResponse.then((function(result){result&&(loggedInAccounts.length<_this.getAllAccounts().length?(_this.eventHandler.emitEvent(EventType.LOGIN_SUCCESS,InteractionType.Redirect,result),_this.logger.verbose("handleRedirectResponse returned result, login success")):(_this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_SUCCESS,InteractionType.Redirect,result),_this.logger.verbose("handleRedirectResponse returned result, acquire token success")));return _this.eventHandler.emitEvent(EventType.HANDLE_REDIRECT_END,InteractionType.Redirect),result})).catch((function(e){throw loggedInAccounts.length>0?_this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_FAILURE,InteractionType.Redirect,null,e):_this.eventHandler.emitEvent(EventType.LOGIN_FAILURE,InteractionType.Redirect,null,e),_this.eventHandler.emitEvent(EventType.HANDLE_REDIRECT_END,InteractionType.Redirect),e})),this.redirectResponse.set(redirectResponseKey,response)):this.logger.verbose("handleRedirectPromise has been called previously, returning the result from the first call"),[2,response]):(this.logger.verbose("handleRedirectPromise returns null, not browser environment"),[2,null])}))}))},ClientApplication.prototype.acquireTokenRedirect=function(request){return __awaiter(this,void 0,void 0,(function(){var correlationId,isLoggedIn,result,nativeClient,redirectClient,_this=this;return __generator(this,(function(_a){return correlationId=this.getRequestCorrelationId(request),this.logger.verbose("acquireTokenRedirect called",correlationId),this.preflightBrowserEnvironmentCheck(InteractionType.Redirect),(isLoggedIn=this.getAllAccounts().length>0)?this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_START,InteractionType.Redirect,request):this.eventHandler.emitEvent(EventType.LOGIN_START,InteractionType.Redirect,request),this.nativeExtensionProvider&&this.canUseNative(request)?(nativeClient=new NativeInteractionClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,ApiId.acquireTokenRedirect,this.performanceClient,this.nativeExtensionProvider,this.getNativeAccountId(request),this.nativeInternalStorage,request.correlationId),result=nativeClient.acquireTokenRedirect(request).catch((function(e){if(e instanceof NativeAuthError&&e.isFatal())return _this.nativeExtensionProvider=void 0,_this.createRedirectClient(request.correlationId).acquireToken(request);if(e instanceof InteractionRequiredAuthError)return _this.logger.verbose("acquireTokenRedirect - Resolving interaction required error thrown by native broker by falling back to web flow"),_this.createRedirectClient(request.correlationId).acquireToken(request);throw _this.browserStorage.setInteractionInProgress(!1),e}))):(redirectClient=this.createRedirectClient(request.correlationId),result=redirectClient.acquireToken(request)),[2,result.catch((function(e){throw isLoggedIn?_this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_FAILURE,InteractionType.Redirect,null,e):_this.eventHandler.emitEvent(EventType.LOGIN_FAILURE,InteractionType.Redirect,null,e),e}))]}))}))},ClientApplication.prototype.acquireTokenPopup=function(request){var _this=this,correlationId=this.getRequestCorrelationId(request),atPopupMeasurement=this.performanceClient.startMeasurement(PerformanceEvents.AcquireTokenPopup,correlationId);try{this.logger.verbose("acquireTokenPopup called",correlationId),this.preflightBrowserEnvironmentCheck(InteractionType.Popup)}catch(e){return Promise.reject(e)}var result,loggedInAccounts=this.getAllAccounts();(loggedInAccounts.length>0?this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_START,InteractionType.Popup,request):this.eventHandler.emitEvent(EventType.LOGIN_START,InteractionType.Popup,request),this.canUseNative(request))?result=this.acquireTokenNative(request,ApiId.acquireTokenPopup).then((function(response){return _this.browserStorage.setInteractionInProgress(!1),atPopupMeasurement.endMeasurement({success:!0,isNativeBroker:!0,requestId:response.requestId}),response})).catch((function(e){if(e instanceof NativeAuthError&&e.isFatal())return _this.nativeExtensionProvider=void 0,_this.createPopupClient(request.correlationId).acquireToken(request);if(e instanceof InteractionRequiredAuthError)return _this.logger.verbose("acquireTokenPopup - Resolving interaction required error thrown by native broker by falling back to web flow"),_this.createPopupClient(request.correlationId).acquireToken(request);throw _this.browserStorage.setInteractionInProgress(!1),e})):result=this.createPopupClient(request.correlationId).acquireToken(request);return result.then((function(result){return loggedInAccounts.length<_this.getAllAccounts().length?_this.eventHandler.emitEvent(EventType.LOGIN_SUCCESS,InteractionType.Popup,result):_this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_SUCCESS,InteractionType.Popup,result),atPopupMeasurement.addStaticFields({accessTokenSize:result.accessToken.length,idTokenSize:result.idToken.length}),atPopupMeasurement.endMeasurement({success:!0,requestId:result.requestId}),result})).catch((function(e){return loggedInAccounts.length>0?_this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_FAILURE,InteractionType.Popup,null,e):_this.eventHandler.emitEvent(EventType.LOGIN_FAILURE,InteractionType.Popup,null,e),atPopupMeasurement.endMeasurement({errorCode:e.errorCode,subErrorCode:e.subError,success:!1}),Promise.reject(e)}))},ClientApplication.prototype.trackPageVisibilityWithMeasurement=function(){var measurement=this.ssoSilentMeasurement||this.acquireTokenByCodeAsyncMeasurement;measurement&&(this.logger.info("Perf: Visibility change detected in ",measurement.event.name),measurement.increment({visibilityChangeCount:1}))},ClientApplication.prototype.ssoSilent=function(request){var _a;return __awaiter(this,void 0,void 0,(function(){var correlationId,validRequest,result,silentIframeClient,_this=this;return __generator(this,(function(_b){return correlationId=this.getRequestCorrelationId(request),validRequest=__assign(__assign({},request),{prompt:request.prompt,correlationId:correlationId}),this.preflightBrowserEnvironmentCheck(InteractionType.Silent),this.ssoSilentMeasurement=this.performanceClient.startMeasurement(PerformanceEvents.SsoSilent,correlationId),null===(_a=this.ssoSilentMeasurement)||void 0===_a||_a.increment({visibilityChangeCount:0}),document.addEventListener("visibilitychange",this.trackPageVisibilityWithMeasurement),this.logger.verbose("ssoSilent called",correlationId),this.eventHandler.emitEvent(EventType.SSO_SILENT_START,InteractionType.Silent,validRequest),this.canUseNative(validRequest)?result=this.acquireTokenNative(validRequest,ApiId.ssoSilent).catch((function(e){if(e instanceof NativeAuthError&&e.isFatal())return _this.nativeExtensionProvider=void 0,_this.createSilentIframeClient(validRequest.correlationId).acquireToken(validRequest);throw e})):(silentIframeClient=this.createSilentIframeClient(validRequest.correlationId),result=silentIframeClient.acquireToken(validRequest)),[2,result.then((function(response){var _a,_b;return _this.eventHandler.emitEvent(EventType.SSO_SILENT_SUCCESS,InteractionType.Silent,response),null===(_a=_this.ssoSilentMeasurement)||void 0===_a||_a.addStaticFields({accessTokenSize:response.accessToken.length,idTokenSize:response.idToken.length}),null===(_b=_this.ssoSilentMeasurement)||void 0===_b||_b.endMeasurement({success:!0,isNativeBroker:response.fromNativeBroker,requestId:response.requestId}),response})).catch((function(e){var _a;throw _this.eventHandler.emitEvent(EventType.SSO_SILENT_FAILURE,InteractionType.Silent,null,e),null===(_a=_this.ssoSilentMeasurement)||void 0===_a||_a.endMeasurement({errorCode:e.errorCode,subErrorCode:e.subError,success:!1}),e})).finally((function(){document.removeEventListener("visibilitychange",_this.trackPageVisibilityWithMeasurement)}))]}))}))},ClientApplication.prototype.acquireTokenByCode=function(request){return __awaiter(this,void 0,void 0,(function(){var correlationId,atbcMeasurement,hybridAuthCode_1,response,_this=this;return __generator(this,(function(_a){correlationId=this.getRequestCorrelationId(request),this.preflightBrowserEnvironmentCheck(InteractionType.Silent),this.logger.trace("acquireTokenByCode called",correlationId),this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_BY_CODE_START,InteractionType.Silent,request),atbcMeasurement=this.performanceClient.startMeasurement(PerformanceEvents.AcquireTokenByCode,request.correlationId);try{if(request.code&&request.nativeAccountId)throw BrowserAuthError.createSpaCodeAndNativeAccountIdPresentError();if(request.code)return hybridAuthCode_1=request.code,(response=this.hybridAuthCodeResponses.get(hybridAuthCode_1))?(this.logger.verbose("Existing acquireTokenByCode request found",request.correlationId),atbcMeasurement.discardMeasurement()):(this.logger.verbose("Initiating new acquireTokenByCode request",correlationId),response=this.acquireTokenByCodeAsync(__assign(__assign({},request),{correlationId:correlationId})).then((function(result){return _this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_BY_CODE_SUCCESS,InteractionType.Silent,result),_this.hybridAuthCodeResponses.delete(hybridAuthCode_1),atbcMeasurement.addStaticFields({accessTokenSize:result.accessToken.length,idTokenSize:result.idToken.length}),atbcMeasurement.endMeasurement({success:!0,isNativeBroker:result.fromNativeBroker,requestId:result.requestId}),result})).catch((function(error){throw _this.hybridAuthCodeResponses.delete(hybridAuthCode_1),_this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_BY_CODE_FAILURE,InteractionType.Silent,null,error),atbcMeasurement.endMeasurement({errorCode:error.errorCode,subErrorCode:error.subError,success:!1}),error})),this.hybridAuthCodeResponses.set(hybridAuthCode_1,response)),[2,response];if(request.nativeAccountId){if(this.canUseNative(request,request.nativeAccountId))return[2,this.acquireTokenNative(request,ApiId.acquireTokenByCode,request.nativeAccountId).catch((function(e){throw e instanceof NativeAuthError&&e.isFatal()&&(_this.nativeExtensionProvider=void 0),e}))];throw BrowserAuthError.createUnableToAcquireTokenFromNativePlatformError()}throw BrowserAuthError.createAuthCodeOrNativeAccountIdRequiredError()}catch(e){throw this.eventHandler.emitEvent(EventType.ACQUIRE_TOKEN_BY_CODE_FAILURE,InteractionType.Silent,null,e),atbcMeasurement.endMeasurement({errorCode:e instanceof AuthError&&e.errorCode||void 0,subErrorCode:e instanceof AuthError&&e.subError||void 0,success:!1}),e}return[2]}))}))},ClientApplication.prototype.acquireTokenByCodeAsync=function(request){var _a;return __awaiter(this,void 0,void 0,(function(){var _this=this;return __generator(this,(function(_b){switch(_b.label){case 0:return this.logger.trace("acquireTokenByCodeAsync called",request.correlationId),this.acquireTokenByCodeAsyncMeasurement=this.performanceClient.startMeasurement(PerformanceEvents.AcquireTokenByCodeAsync,request.correlationId),null===(_a=this.acquireTokenByCodeAsyncMeasurement)||void 0===_a||_a.increment({visibilityChangeCount:0}),document.addEventListener("visibilitychange",this.trackPageVisibilityWithMeasurement),[4,this.createSilentAuthCodeClient(request.correlationId).acquireToken(request).then((function(response){var _a;return null===(_a=_this.acquireTokenByCodeAsyncMeasurement)||void 0===_a||_a.endMeasurement({success:!0,fromCache:response.fromCache,isNativeBroker:response.fromNativeBroker,requestId:response.requestId}),response})).catch((function(tokenRenewalError){var _a;throw null===(_a=_this.acquireTokenByCodeAsyncMeasurement)||void 0===_a||_a.endMeasurement({errorCode:tokenRenewalError.errorCode,subErrorCode:tokenRenewalError.subError,success:!1}),tokenRenewalError})).finally((function(){document.removeEventListener("visibilitychange",_this.trackPageVisibilityWithMeasurement)}))];case 1:return[2,_b.sent()]}}))}))},ClientApplication.prototype.acquireTokenFromCache=function(silentCacheClient,commonRequest,silentRequest){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){switch(this.performanceClient.addQueueMeasurement(PerformanceEvents.AcquireTokenFromCache,commonRequest.correlationId),silentRequest.cacheLookupPolicy){case CacheLookupPolicy.Default:case CacheLookupPolicy.AccessToken:case CacheLookupPolicy.AccessTokenAndRefreshToken:return[2,silentCacheClient.acquireToken(commonRequest)];default:throw ClientAuthError.createRefreshRequiredError()}return[2]}))}))},ClientApplication.prototype.acquireTokenByRefreshToken=function(commonRequest,silentRequest){return __awaiter(this,void 0,void 0,(function(){var silentRefreshClient;return __generator(this,(function(_a){switch(this.performanceClient.addQueueMeasurement(PerformanceEvents.AcquireTokenByRefreshToken,commonRequest.correlationId),silentRequest.cacheLookupPolicy){case CacheLookupPolicy.Default:case CacheLookupPolicy.AccessTokenAndRefreshToken:case CacheLookupPolicy.RefreshToken:case CacheLookupPolicy.RefreshTokenAndNetwork:return silentRefreshClient=this.createSilentRefreshClient(commonRequest.correlationId),this.performanceClient.setPreQueueTime(PerformanceEvents.SilentRefreshClientAcquireToken,commonRequest.correlationId),[2,silentRefreshClient.acquireToken(commonRequest)];default:throw ClientAuthError.createRefreshRequiredError()}return[2]}))}))},ClientApplication.prototype.acquireTokenBySilentIframe=function(request){return __awaiter(this,void 0,void 0,(function(){var silentIframeClient;return __generator(this,(function(_a){return this.performanceClient.addQueueMeasurement(PerformanceEvents.AcquireTokenBySilentIframe,request.correlationId),silentIframeClient=this.createSilentIframeClient(request.correlationId),this.performanceClient.setPreQueueTime(PerformanceEvents.SilentIframeClientAcquireToken,request.correlationId),[2,silentIframeClient.acquireToken(request)]}))}))},ClientApplication.prototype.logout=function(logoutRequest){return __awaiter(this,void 0,void 0,(function(){var correlationId;return __generator(this,(function(_a){return correlationId=this.getRequestCorrelationId(logoutRequest),this.logger.warning("logout API is deprecated and will be removed in msal-browser v3.0.0. Use logoutRedirect instead.",correlationId),[2,this.logoutRedirect(__assign({correlationId:correlationId},logoutRequest))]}))}))},ClientApplication.prototype.logoutRedirect=function(logoutRequest){return __awaiter(this,void 0,void 0,(function(){var correlationId;return __generator(this,(function(_a){return correlationId=this.getRequestCorrelationId(logoutRequest),this.preflightBrowserEnvironmentCheck(InteractionType.Redirect),[2,this.createRedirectClient(correlationId).logout(logoutRequest)]}))}))},ClientApplication.prototype.logoutPopup=function(logoutRequest){try{var correlationId=this.getRequestCorrelationId(logoutRequest);return this.preflightBrowserEnvironmentCheck(InteractionType.Popup),this.createPopupClient(correlationId).logout(logoutRequest)}catch(e){return Promise.reject(e)}},ClientApplication.prototype.getAllAccounts=function(){return this.logger.verbose("getAllAccounts called"),this.isBrowserEnvironment?this.browserStorage.getAllAccounts():[]},ClientApplication.prototype.getAccountByUsername=function(username){if(this.logger.trace("getAccountByUsername called"),!username)return this.logger.warning("getAccountByUsername: No username provided"),null;var account=this.browserStorage.getAccountInfoFilteredBy({username:username});return account?(this.logger.verbose("getAccountByUsername: Account matching username found, returning"),this.logger.verbosePii("getAccountByUsername: Returning signed-in accounts matching username: "+username),account):(this.logger.verbose("getAccountByUsername: No matching account found, returning null"),null)},ClientApplication.prototype.getAccountByHomeId=function(homeAccountId){if(this.logger.trace("getAccountByHomeId called"),!homeAccountId)return this.logger.warning("getAccountByHomeId: No homeAccountId provided"),null;var account=this.browserStorage.getAccountInfoFilteredBy({homeAccountId:homeAccountId});return account?(this.logger.verbose("getAccountByHomeId: Account matching homeAccountId found, returning"),this.logger.verbosePii("getAccountByHomeId: Returning signed-in accounts matching homeAccountId: "+homeAccountId),account):(this.logger.verbose("getAccountByHomeId: No matching account found, returning null"),null)},ClientApplication.prototype.getAccountByLocalId=function(localAccountId){if(this.logger.trace("getAccountByLocalId called"),!localAccountId)return this.logger.warning("getAccountByLocalId: No localAccountId provided"),null;var account=this.browserStorage.getAccountInfoFilteredBy({localAccountId:localAccountId});return account?(this.logger.verbose("getAccountByLocalId: Account matching localAccountId found, returning"),this.logger.verbosePii("getAccountByLocalId: Returning signed-in accounts matching localAccountId: "+localAccountId),account):(this.logger.verbose("getAccountByLocalId: No matching account found, returning null"),null)},ClientApplication.prototype.setActiveAccount=function(account){this.browserStorage.setActiveAccount(account)},ClientApplication.prototype.getActiveAccount=function(){return this.browserStorage.getActiveAccount()},ClientApplication.prototype.preflightBrowserEnvironmentCheck=function(interactionType,setInteractionInProgress){if(void 0===setInteractionInProgress&&(setInteractionInProgress=!0),this.logger.verbose("preflightBrowserEnvironmentCheck started"),BrowserUtils.blockNonBrowserEnvironment(this.isBrowserEnvironment),BrowserUtils.blockRedirectInIframe(interactionType,this.config.system.allowRedirectInIframe),BrowserUtils.blockReloadInHiddenIframes(),BrowserUtils.blockAcquireTokenInPopups(),BrowserUtils.blockNativeBrokerCalledBeforeInitialized(this.config.system.allowNativeBroker,this.initialized),interactionType===InteractionType.Redirect&&this.config.cache.cacheLocation===BrowserCacheLocation.MemoryStorage&&!this.config.cache.storeAuthStateInCookie)throw BrowserConfigurationAuthError.createInMemoryRedirectUnavailableError();interactionType!==InteractionType.Redirect&&interactionType!==InteractionType.Popup||this.preflightInteractiveRequest(setInteractionInProgress)},ClientApplication.prototype.preflightInteractiveRequest=function(setInteractionInProgress){this.logger.verbose("preflightInteractiveRequest called, validating app environment"),BrowserUtils.blockReloadInHiddenIframes(),setInteractionInProgress&&this.browserStorage.setInteractionInProgress(!0)},ClientApplication.prototype.acquireTokenNative=function(request,apiId,accountId){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){if(this.logger.trace("acquireTokenNative called"),!this.nativeExtensionProvider)throw BrowserAuthError.createNativeConnectionNotEstablishedError();return[2,new NativeInteractionClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,apiId,this.performanceClient,this.nativeExtensionProvider,accountId||this.getNativeAccountId(request),this.nativeInternalStorage,request.correlationId).acquireToken(request)]}))}))},ClientApplication.prototype.canUseNative=function(request,accountId){if(this.logger.trace("canUseNative called"),!NativeMessageHandler.isNativeAvailable(this.config,this.logger,this.nativeExtensionProvider,request.authenticationScheme))return this.logger.trace("canUseNative: isNativeAvailable returned false, returning false"),!1;if(request.prompt)switch(request.prompt){case PromptValue.NONE:case PromptValue.CONSENT:case PromptValue.LOGIN:this.logger.trace("canUseNative: prompt is compatible with native flow");break;default:return this.logger.trace("canUseNative: prompt = "+request.prompt+" is not compatible with native flow, returning false"),!1}return!(!accountId&&!this.getNativeAccountId(request))||(this.logger.trace("canUseNative: nativeAccountId is not available, returning false"),!1)},ClientApplication.prototype.getNativeAccountId=function(request){var account=request.account||this.browserStorage.getAccountInfoByHints(request.loginHint,request.sid)||this.getActiveAccount();return account&&account.nativeAccountId||""},ClientApplication.prototype.createPopupClient=function(correlationId){return new PopupClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,this.performanceClient,this.nativeInternalStorage,this.nativeExtensionProvider,correlationId)},ClientApplication.prototype.createRedirectClient=function(correlationId){return new RedirectClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,this.performanceClient,this.nativeInternalStorage,this.nativeExtensionProvider,correlationId)},ClientApplication.prototype.createSilentIframeClient=function(correlationId){return new SilentIframeClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,ApiId.ssoSilent,this.performanceClient,this.nativeInternalStorage,this.nativeExtensionProvider,correlationId)},ClientApplication.prototype.createSilentCacheClient=function(correlationId){return new SilentCacheClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,this.performanceClient,this.nativeExtensionProvider,correlationId)},ClientApplication.prototype.createSilentRefreshClient=function(correlationId){return new SilentRefreshClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,this.performanceClient,this.nativeExtensionProvider,correlationId)},ClientApplication.prototype.createSilentAuthCodeClient=function(correlationId){return new SilentAuthCodeClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,ApiId.acquireTokenByCode,this.performanceClient,this.nativeExtensionProvider,correlationId)},ClientApplication.prototype.addEventCallback=function(callback){return this.eventHandler.addEventCallback(callback)},ClientApplication.prototype.removeEventCallback=function(callbackId){this.eventHandler.removeEventCallback(callbackId)},ClientApplication.prototype.addPerformanceCallback=function(callback){return this.performanceClient.addPerformanceCallback(callback)},ClientApplication.prototype.removePerformanceCallback=function(callbackId){return this.performanceClient.removePerformanceCallback(callbackId)},ClientApplication.prototype.enableAccountStorageEvents=function(){this.eventHandler.enableAccountStorageEvents()},ClientApplication.prototype.disableAccountStorageEvents=function(){this.eventHandler.disableAccountStorageEvents()},ClientApplication.prototype.getTokenCache=function(){return this.tokenCache},ClientApplication.prototype.getLogger=function(){return this.logger},ClientApplication.prototype.setLogger=function(logger){this.logger=logger},ClientApplication.prototype.initializeWrapperLibrary=function(sku,version){this.browserStorage.setWrapperMetadata(sku,version)},ClientApplication.prototype.setNavigationClient=function(navigationClient){this.navigationClient=navigationClient},ClientApplication.prototype.getConfiguration=function(){return this.config},ClientApplication.prototype.getRequestCorrelationId=function(request){return(null==request?void 0:request.correlationId)?request.correlationId:this.isBrowserEnvironment?this.browserCrypto.createNewGuid():Constants.EMPTY_STRING},ClientApplication}();export{ClientApplication};