/*! @azure/msal-browser v2.37.1 2023-06-07 */
"use strict";import{__awaiter,__generator}from"../_virtual/_tslib.js";import{PerformanceEvents,JoseHeader}from"@azure/msal-common";import{GuidGenerator}from"./GuidGenerator.js";import{Base64Encode}from"../encode/Base64Encode.js";import{Base64Decode}from"../encode/Base64Decode.js";import{PkceGenerator}from"./PkceGenerator.js";import{BrowserCrypto}from"./BrowserCrypto.js";import{BrowserStringUtils}from"../utils/BrowserStringUtils.js";import{BrowserAuthError}from"../error/BrowserAuthError.js";import{CryptoKeyStore}from"../cache/CryptoKeyStore.js";var CryptoOps=function(){function CryptoOps(logger,performanceClient,cryptoConfig){this.logger=logger,this.browserCrypto=new BrowserCrypto(this.logger,cryptoConfig),this.b64Encode=new Base64Encode,this.b64Decode=new Base64Decode,this.guidGenerator=new GuidGenerator(this.browserCrypto),this.pkceGenerator=new PkceGenerator(this.browserCrypto),this.cache=new CryptoKeyStore(this.logger),this.performanceClient=performanceClient}return CryptoOps.prototype.createNewGuid=function(){return this.guidGenerator.generateGuid()},CryptoOps.prototype.base64Encode=function(input){return this.b64Encode.encode(input)},CryptoOps.prototype.base64Decode=function(input){return this.b64Decode.decode(input)},CryptoOps.prototype.generatePkceCodes=function(){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){return[2,this.pkceGenerator.generateCodes()]}))}))},CryptoOps.prototype.getPublicKeyThumbprint=function(request){var _a;return __awaiter(this,void 0,void 0,(function(){var publicKeyThumbMeasurement,keyPair,publicKeyJwk,pubKeyThumprintObj,publicJwkString,publicJwkHash,privateKeyJwk,unextractablePrivateKey;return __generator(this,(function(_b){switch(_b.label){case 0:return publicKeyThumbMeasurement=null===(_a=this.performanceClient)||void 0===_a?void 0:_a.startMeasurement(PerformanceEvents.CryptoOptsGetPublicKeyThumbprint,request.correlationId),[4,this.browserCrypto.generateKeyPair(CryptoOps.EXTRACTABLE,CryptoOps.POP_KEY_USAGES)];case 1:return keyPair=_b.sent(),[4,this.browserCrypto.exportJwk(keyPair.publicKey)];case 2:return publicKeyJwk=_b.sent(),pubKeyThumprintObj={e:publicKeyJwk.e,kty:publicKeyJwk.kty,n:publicKeyJwk.n},publicJwkString=BrowserStringUtils.getSortedObjectString(pubKeyThumprintObj),[4,this.hashString(publicJwkString)];case 3:return publicJwkHash=_b.sent(),[4,this.browserCrypto.exportJwk(keyPair.privateKey)];case 4:return privateKeyJwk=_b.sent(),[4,this.browserCrypto.importJwk(privateKeyJwk,!1,["sign"])];case 5:return unextractablePrivateKey=_b.sent(),[4,this.cache.asymmetricKeys.setItem(publicJwkHash,{privateKey:unextractablePrivateKey,publicKey:keyPair.publicKey,requestMethod:request.resourceRequestMethod,requestUri:request.resourceRequestUri})];case 6:return _b.sent(),publicKeyThumbMeasurement&&publicKeyThumbMeasurement.endMeasurement({success:!0}),[2,publicJwkHash]}}))}))},CryptoOps.prototype.removeTokenBindingKey=function(kid){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){switch(_a.label){case 0:return[4,this.cache.asymmetricKeys.removeItem(kid)];case 1:return _a.sent(),[4,this.cache.asymmetricKeys.containsKey(kid)];case 2:return[2,!_a.sent()]}}))}))},CryptoOps.prototype.clearKeystore=function(){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){switch(_a.label){case 0:return[4,this.cache.clear()];case 1:return[2,_a.sent()]}}))}))},CryptoOps.prototype.signJwt=function(payload,kid,correlationId){var _a;return __awaiter(this,void 0,void 0,(function(){var signJwtMeasurement,cachedKeyPair,publicKeyJwk,publicKeyJwkString,encodedKeyIdThumbprint,shrHeader,encodedShrHeader,encodedPayload,tokenString,tokenBuffer,signatureBuffer,encodedSignature,signedJwt;return __generator(this,(function(_b){switch(_b.label){case 0:return signJwtMeasurement=null===(_a=this.performanceClient)||void 0===_a?void 0:_a.startMeasurement(PerformanceEvents.CryptoOptsSignJwt,correlationId),[4,this.cache.asymmetricKeys.getItem(kid)];case 1:if(!(cachedKeyPair=_b.sent()))throw BrowserAuthError.createSigningKeyNotFoundInStorageError(kid);return[4,this.browserCrypto.exportJwk(cachedKeyPair.publicKey)];case 2:return publicKeyJwk=_b.sent(),publicKeyJwkString=BrowserStringUtils.getSortedObjectString(publicKeyJwk),encodedKeyIdThumbprint=this.b64Encode.urlEncode(JSON.stringify({kid:kid})),shrHeader=JoseHeader.getShrHeaderString({kid:encodedKeyIdThumbprint,alg:publicKeyJwk.alg}),encodedShrHeader=this.b64Encode.urlEncode(shrHeader),payload.cnf={jwk:JSON.parse(publicKeyJwkString)},encodedPayload=this.b64Encode.urlEncode(JSON.stringify(payload)),tokenString=encodedShrHeader+"."+encodedPayload,tokenBuffer=BrowserStringUtils.stringToArrayBuffer(tokenString),[4,this.browserCrypto.sign(cachedKeyPair.privateKey,tokenBuffer)];case 3:return signatureBuffer=_b.sent(),encodedSignature=this.b64Encode.urlEncodeArr(new Uint8Array(signatureBuffer)),signedJwt=tokenString+"."+encodedSignature,signJwtMeasurement&&signJwtMeasurement.endMeasurement({success:!0}),[2,signedJwt]}}))}))},CryptoOps.prototype.hashString=function(plainText){return __awaiter(this,void 0,void 0,(function(){var hashBuffer,hashBytes;return __generator(this,(function(_a){switch(_a.label){case 0:return[4,this.browserCrypto.sha256Digest(plainText)];case 1:return hashBuffer=_a.sent(),hashBytes=new Uint8Array(hashBuffer),[2,this.b64Encode.urlEncodeArr(hashBytes)]}}))}))},CryptoOps.POP_KEY_USAGES=["sign","verify"],CryptoOps.EXTRACTABLE=!0,CryptoOps}();export{CryptoOps};