/*! @azure/msal-common v13.1.0 2023-06-07 */
"use strict";import{__awaiter,__generator,__assign,__extends}from"../_virtual/_tslib.js";import{Separators,CredentialType,AuthenticationScheme,THE_FAMILY_ID,APP_METADATA,AUTHORITY_METADATA_CONSTANTS}from"../utils/Constants.js";import{ScopeSet}from"../request/ScopeSet.js";import{AccountEntity}from"./entities/AccountEntity.js";import{AuthError}from"../error/AuthError.js";import{ClientAuthError}from"../error/ClientAuthError.js";import{AuthToken}from"../account/AuthToken.js";import{name,version}from"../packageMetadata.js";var CacheManager=function(){function CacheManager(clientId,cryptoImpl,logger){this.clientId=clientId,this.cryptoImpl=cryptoImpl,this.commonLogger=logger.clone(name,version)}return CacheManager.prototype.getAllAccounts=function(){var _this=this,allAccountKeys=this.getAccountKeys();if(allAccountKeys.length<1)return[];var accountEntities=allAccountKeys.reduce((function(accounts,key){var entity=_this.getAccount(key);return entity?(accounts.push(entity),accounts):accounts}),[]);return accountEntities.length<1?[]:accountEntities.map((function(accountEntity){return _this.getAccountInfoFromEntity(accountEntity)}))},CacheManager.prototype.getAccountInfoFilteredBy=function(accountFilter){var allAccounts=this.getAccountsFilteredBy(accountFilter);return allAccounts.length>0?this.getAccountInfoFromEntity(allAccounts[0]):null},CacheManager.prototype.getAccountInfoFromEntity=function(accountEntity){var accountInfo=accountEntity.getAccountInfo(),idToken=this.getIdToken(accountInfo);return idToken&&(accountInfo.idToken=idToken.secret,accountInfo.idTokenClaims=new AuthToken(idToken.secret,this.cryptoImpl).claims),accountInfo},CacheManager.prototype.saveCacheRecord=function(cacheRecord){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){switch(_a.label){case 0:if(!cacheRecord)throw ClientAuthError.createNullOrUndefinedCacheRecord();return cacheRecord.account&&this.setAccount(cacheRecord.account),cacheRecord.idToken&&this.setIdTokenCredential(cacheRecord.idToken),cacheRecord.accessToken?[4,this.saveAccessToken(cacheRecord.accessToken)]:[3,2];case 1:_a.sent(),_a.label=2;case 2:return cacheRecord.refreshToken&&this.setRefreshTokenCredential(cacheRecord.refreshToken),cacheRecord.appMetadata&&this.setAppMetadata(cacheRecord.appMetadata),[2]}}))}))},CacheManager.prototype.saveAccessToken=function(credential){return __awaiter(this,void 0,void 0,(function(){var accessTokenFilter,tokenKeys,currentScopes,removedAccessTokens,_this=this;return __generator(this,(function(_a){switch(_a.label){case 0:return accessTokenFilter={clientId:credential.clientId,credentialType:credential.credentialType,environment:credential.environment,homeAccountId:credential.homeAccountId,realm:credential.realm,tokenType:credential.tokenType,requestedClaimsHash:credential.requestedClaimsHash},tokenKeys=this.getTokenKeys(),currentScopes=ScopeSet.fromString(credential.target),removedAccessTokens=[],tokenKeys.accessToken.forEach((function(key){if(_this.accessTokenKeyMatchesFilter(key,accessTokenFilter,!1)){var tokenEntity=_this.getAccessTokenCredential(key);if(tokenEntity&&_this.credentialMatchesFilter(tokenEntity,accessTokenFilter))ScopeSet.fromString(tokenEntity.target).intersectingScopeSets(currentScopes)&&removedAccessTokens.push(_this.removeAccessToken(key))}})),[4,Promise.all(removedAccessTokens)];case 1:return _a.sent(),this.setAccessTokenCredential(credential),[2]}}))}))},CacheManager.prototype.getAccountsFilteredBy=function(accountFilter){var _this=this,allAccountKeys=this.getAccountKeys(),matchingAccounts=[];return allAccountKeys.forEach((function(cacheKey){if(_this.isAccountKey(cacheKey,accountFilter.homeAccountId,accountFilter.realm)){var entity=_this.getAccount(cacheKey);entity&&(accountFilter.homeAccountId&&!_this.matchHomeAccountId(entity,accountFilter.homeAccountId)||accountFilter.localAccountId&&!_this.matchLocalAccountId(entity,accountFilter.localAccountId)||accountFilter.username&&!_this.matchUsername(entity,accountFilter.username)||accountFilter.environment&&!_this.matchEnvironment(entity,accountFilter.environment)||accountFilter.realm&&!_this.matchRealm(entity,accountFilter.realm)||accountFilter.nativeAccountId&&!_this.matchNativeAccountId(entity,accountFilter.nativeAccountId)||matchingAccounts.push(entity))}})),matchingAccounts},CacheManager.prototype.isAccountKey=function(key,homeAccountId,tenantId){return!(key.split(Separators.CACHE_KEY_SEPARATOR).length<3)&&(!(homeAccountId&&!key.toLowerCase().includes(homeAccountId.toLowerCase()))&&!(tenantId&&!key.toLowerCase().includes(tenantId.toLowerCase())))},CacheManager.prototype.isCredentialKey=function(key){if(key.split(Separators.CACHE_KEY_SEPARATOR).length<6)return!1;var lowerCaseKey=key.toLowerCase();if(-1===lowerCaseKey.indexOf(CredentialType.ID_TOKEN.toLowerCase())&&-1===lowerCaseKey.indexOf(CredentialType.ACCESS_TOKEN.toLowerCase())&&-1===lowerCaseKey.indexOf(CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME.toLowerCase())&&-1===lowerCaseKey.indexOf(CredentialType.REFRESH_TOKEN.toLowerCase()))return!1;if(lowerCaseKey.indexOf(CredentialType.REFRESH_TOKEN.toLowerCase())>-1){var clientIdValidation=""+CredentialType.REFRESH_TOKEN+Separators.CACHE_KEY_SEPARATOR+this.clientId+Separators.CACHE_KEY_SEPARATOR,familyIdValidation=""+CredentialType.REFRESH_TOKEN+Separators.CACHE_KEY_SEPARATOR+THE_FAMILY_ID+Separators.CACHE_KEY_SEPARATOR;if(-1===lowerCaseKey.indexOf(clientIdValidation.toLowerCase())&&-1===lowerCaseKey.indexOf(familyIdValidation.toLowerCase()))return!1}else if(-1===lowerCaseKey.indexOf(this.clientId.toLowerCase()))return!1;return!0},CacheManager.prototype.credentialMatchesFilter=function(entity,filter){if(filter.clientId&&!this.matchClientId(entity,filter.clientId))return!1;if(filter.userAssertionHash&&!this.matchUserAssertionHash(entity,filter.userAssertionHash))return!1;if("string"==typeof filter.homeAccountId&&!this.matchHomeAccountId(entity,filter.homeAccountId))return!1;if(filter.environment&&!this.matchEnvironment(entity,filter.environment))return!1;if(filter.realm&&!this.matchRealm(entity,filter.realm))return!1;if(filter.credentialType&&!this.matchCredentialType(entity,filter.credentialType))return!1;if(filter.familyId&&!this.matchFamilyId(entity,filter.familyId))return!1;if(filter.target&&!this.matchTarget(entity,filter.target))return!1;if((filter.requestedClaimsHash||entity.requestedClaimsHash)&&entity.requestedClaimsHash!==filter.requestedClaimsHash)return!1;if(entity.credentialType===CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME){if(filter.tokenType&&!this.matchTokenType(entity,filter.tokenType))return!1;if(filter.tokenType===AuthenticationScheme.SSH&&filter.keyId&&!this.matchKeyId(entity,filter.keyId))return!1}return!0},CacheManager.prototype.getAppMetadataFilteredBy=function(filter){return this.getAppMetadataFilteredByInternal(filter.environment,filter.clientId)},CacheManager.prototype.getAppMetadataFilteredByInternal=function(environment,clientId){var _this=this,allCacheKeys=this.getKeys(),matchingAppMetadata={};return allCacheKeys.forEach((function(cacheKey){if(_this.isAppMetadata(cacheKey)){var entity=_this.getAppMetadata(cacheKey);entity&&(environment&&!_this.matchEnvironment(entity,environment)||clientId&&!_this.matchClientId(entity,clientId)||(matchingAppMetadata[cacheKey]=entity))}})),matchingAppMetadata},CacheManager.prototype.getAuthorityMetadataByAlias=function(host){var _this=this,allCacheKeys=this.getAuthorityMetadataKeys(),matchedEntity=null;return allCacheKeys.forEach((function(cacheKey){if(_this.isAuthorityMetadata(cacheKey)&&-1!==cacheKey.indexOf(_this.clientId)){var entity=_this.getAuthorityMetadata(cacheKey);entity&&-1!==entity.aliases.indexOf(host)&&(matchedEntity=entity)}})),matchedEntity},CacheManager.prototype.removeAllAccounts=function(){return __awaiter(this,void 0,void 0,(function(){var allAccountKeys,removedAccounts,_this=this;return __generator(this,(function(_a){switch(_a.label){case 0:return allAccountKeys=this.getAccountKeys(),removedAccounts=[],allAccountKeys.forEach((function(cacheKey){removedAccounts.push(_this.removeAccount(cacheKey))})),[4,Promise.all(removedAccounts)];case 1:return _a.sent(),[2]}}))}))},CacheManager.prototype.removeAccount=function(accountKey){return __awaiter(this,void 0,void 0,(function(){var account;return __generator(this,(function(_a){switch(_a.label){case 0:if(!(account=this.getAccount(accountKey)))throw ClientAuthError.createNoAccountFoundError();return[4,this.removeAccountContext(account)];case 1:return _a.sent(),this.removeItem(accountKey),[2]}}))}))},CacheManager.prototype.removeAccountContext=function(account){return __awaiter(this,void 0,void 0,(function(){var allTokenKeys,accountId,removedCredentials,_this=this;return __generator(this,(function(_a){switch(_a.label){case 0:return allTokenKeys=this.getTokenKeys(),accountId=account.generateAccountId(),removedCredentials=[],allTokenKeys.idToken.forEach((function(key){0===key.indexOf(accountId)&&_this.removeIdToken(key)})),allTokenKeys.accessToken.forEach((function(key){0===key.indexOf(accountId)&&removedCredentials.push(_this.removeAccessToken(key))})),allTokenKeys.refreshToken.forEach((function(key){0===key.indexOf(accountId)&&_this.removeRefreshToken(key)})),[4,Promise.all(removedCredentials)];case 1:return _a.sent(),[2]}}))}))},CacheManager.prototype.removeAccessToken=function(key){return __awaiter(this,void 0,void 0,(function(){var credential,kid;return __generator(this,(function(_a){switch(_a.label){case 0:if(!(credential=this.getAccessTokenCredential(key)))return[2];if(credential.credentialType.toLowerCase()!==CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME.toLowerCase())return[3,4];if(credential.tokenType!==AuthenticationScheme.POP)return[3,4];if(!(kid=credential.keyId))return[3,4];_a.label=1;case 1:return _a.trys.push([1,3,,4]),[4,this.cryptoImpl.removeTokenBindingKey(kid)];case 2:return _a.sent(),[3,4];case 3:throw _a.sent(),ClientAuthError.createBindingKeyNotRemovedError();case 4:return[2,this.removeItem(key)]}}))}))},CacheManager.prototype.removeAppMetadata=function(){var _this=this;return this.getKeys().forEach((function(cacheKey){_this.isAppMetadata(cacheKey)&&_this.removeItem(cacheKey)})),!0},CacheManager.prototype.readCacheRecord=function(account,request,environment){var tokenKeys=this.getTokenKeys(),cachedAccount=this.readAccountFromCache(account),cachedIdToken=this.getIdToken(account,tokenKeys),cachedAccessToken=this.getAccessToken(account,request,tokenKeys),cachedRefreshToken=this.getRefreshToken(account,!1,tokenKeys),cachedAppMetadata=this.readAppMetadataFromCache(environment);return cachedAccount&&cachedIdToken&&(cachedAccount.idTokenClaims=new AuthToken(cachedIdToken.secret,this.cryptoImpl).claims),{account:cachedAccount,idToken:cachedIdToken,accessToken:cachedAccessToken,refreshToken:cachedRefreshToken,appMetadata:cachedAppMetadata}},CacheManager.prototype.readAccountFromCache=function(account){var accountKey=AccountEntity.generateAccountCacheKey(account);return this.getAccount(accountKey)},CacheManager.prototype.getIdToken=function(account,tokenKeys){this.commonLogger.trace("CacheManager - getIdToken called");var idTokenFilter={homeAccountId:account.homeAccountId,environment:account.environment,credentialType:CredentialType.ID_TOKEN,clientId:this.clientId,realm:account.tenantId},idTokens=this.getIdTokensByFilter(idTokenFilter,tokenKeys),numIdTokens=idTokens.length;if(numIdTokens<1)return this.commonLogger.info("CacheManager:getIdToken - No token found"),null;if(numIdTokens>1)throw ClientAuthError.createMultipleMatchingTokensInCacheError();return this.commonLogger.info("CacheManager:getIdToken - Returning id token"),idTokens[0]},CacheManager.prototype.getIdTokensByFilter=function(filter,tokenKeys){var _this=this,idTokenKeys=tokenKeys&&tokenKeys.idToken||this.getTokenKeys().idToken,idTokens=[];return idTokenKeys.forEach((function(key){if(_this.idTokenKeyMatchesFilter(key,__assign({clientId:_this.clientId},filter))){var idToken=_this.getIdTokenCredential(key);idToken&&_this.credentialMatchesFilter(idToken,filter)&&idTokens.push(idToken)}})),idTokens},CacheManager.prototype.idTokenKeyMatchesFilter=function(inputKey,filter){var key=inputKey.toLowerCase();return(!filter.clientId||-1!==key.indexOf(filter.clientId.toLowerCase()))&&(!filter.homeAccountId||-1!==key.indexOf(filter.homeAccountId.toLowerCase()))},CacheManager.prototype.removeIdToken=function(key){this.removeItem(key)},CacheManager.prototype.removeRefreshToken=function(key){this.removeItem(key)},CacheManager.prototype.getAccessToken=function(account,request,tokenKeys){var _this=this;this.commonLogger.trace("CacheManager - getAccessToken called");var scopes=ScopeSet.createSearchScopes(request.scopes),authScheme=request.authenticationScheme||AuthenticationScheme.BEARER,credentialType=authScheme&&authScheme.toLowerCase()!==AuthenticationScheme.BEARER.toLowerCase()?CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME:CredentialType.ACCESS_TOKEN,accessTokenFilter={homeAccountId:account.homeAccountId,environment:account.environment,credentialType:credentialType,clientId:this.clientId,realm:account.tenantId,target:scopes,tokenType:authScheme,keyId:request.sshKid,requestedClaimsHash:request.requestedClaimsHash},accessTokenKeys=tokenKeys&&tokenKeys.accessToken||this.getTokenKeys().accessToken,accessTokens=[];accessTokenKeys.forEach((function(key){if(_this.accessTokenKeyMatchesFilter(key,accessTokenFilter,!0)){var accessToken=_this.getAccessTokenCredential(key);accessToken&&_this.credentialMatchesFilter(accessToken,accessTokenFilter)&&accessTokens.push(accessToken)}}));var numAccessTokens=accessTokens.length;if(numAccessTokens<1)return this.commonLogger.info("CacheManager:getAccessToken - No token found"),null;if(numAccessTokens>1)throw ClientAuthError.createMultipleMatchingTokensInCacheError();return this.commonLogger.info("CacheManager:getAccessToken - Returning access token"),accessTokens[0]},CacheManager.prototype.accessTokenKeyMatchesFilter=function(inputKey,filter,keyMustContainAllScopes){var key=inputKey.toLowerCase();if(filter.clientId&&-1===key.indexOf(filter.clientId.toLowerCase()))return!1;if(filter.homeAccountId&&-1===key.indexOf(filter.homeAccountId.toLowerCase()))return!1;if(filter.realm&&-1===key.indexOf(filter.realm.toLowerCase()))return!1;if(filter.requestedClaimsHash&&-1===key.indexOf(filter.requestedClaimsHash.toLowerCase()))return!1;if(filter.target)for(var scopes=filter.target.asArray(),i=0;i<scopes.length;i++){if(keyMustContainAllScopes&&!key.includes(scopes[i].toLowerCase()))return!1;if(!keyMustContainAllScopes&&key.includes(scopes[i].toLowerCase()))return!0}return!0},CacheManager.prototype.getAccessTokensByFilter=function(filter){var _this=this,tokenKeys=this.getTokenKeys(),accessTokens=[];return tokenKeys.accessToken.forEach((function(key){if(_this.accessTokenKeyMatchesFilter(key,filter,!0)){var accessToken=_this.getAccessTokenCredential(key);accessToken&&_this.credentialMatchesFilter(accessToken,filter)&&accessTokens.push(accessToken)}})),accessTokens},CacheManager.prototype.getRefreshToken=function(account,familyRT,tokenKeys){var _this=this;this.commonLogger.trace("CacheManager - getRefreshToken called");var id=familyRT?THE_FAMILY_ID:void 0,refreshTokenFilter={homeAccountId:account.homeAccountId,environment:account.environment,credentialType:CredentialType.REFRESH_TOKEN,clientId:this.clientId,familyId:id},refreshTokenKeys=tokenKeys&&tokenKeys.refreshToken||this.getTokenKeys().refreshToken,refreshTokens=[];return refreshTokenKeys.forEach((function(key){if(_this.refreshTokenKeyMatchesFilter(key,refreshTokenFilter)){var refreshToken=_this.getRefreshTokenCredential(key);refreshToken&&_this.credentialMatchesFilter(refreshToken,refreshTokenFilter)&&refreshTokens.push(refreshToken)}})),refreshTokens.length<1?(this.commonLogger.info("CacheManager:getRefreshToken - No refresh token found."),null):(this.commonLogger.info("CacheManager:getRefreshToken - returning refresh token"),refreshTokens[0])},CacheManager.prototype.refreshTokenKeyMatchesFilter=function(inputKey,filter){var key=inputKey.toLowerCase();return(!filter.familyId||-1!==key.indexOf(filter.familyId.toLowerCase()))&&(!(!filter.familyId&&filter.clientId&&-1===key.indexOf(filter.clientId.toLowerCase()))&&(!filter.homeAccountId||-1!==key.indexOf(filter.homeAccountId.toLowerCase())))},CacheManager.prototype.readAppMetadataFromCache=function(environment){var appMetadataFilter={environment:environment,clientId:this.clientId},appMetadata=this.getAppMetadataFilteredBy(appMetadataFilter),appMetadataEntries=Object.keys(appMetadata).map((function(key){return appMetadata[key]})),numAppMetadata=appMetadataEntries.length;if(numAppMetadata<1)return null;if(numAppMetadata>1)throw ClientAuthError.createMultipleMatchingAppMetadataInCacheError();return appMetadataEntries[0]},CacheManager.prototype.isAppMetadataFOCI=function(environment){var appMetadata=this.readAppMetadataFromCache(environment);return!(!appMetadata||appMetadata.familyId!==THE_FAMILY_ID)},CacheManager.prototype.matchHomeAccountId=function(entity,homeAccountId){return!("string"!=typeof entity.homeAccountId||homeAccountId!==entity.homeAccountId)},CacheManager.prototype.matchLocalAccountId=function(entity,localAccountId){return!("string"!=typeof entity.localAccountId||localAccountId!==entity.localAccountId)},CacheManager.prototype.matchUsername=function(entity,username){return!("string"!=typeof entity.username||username.toLowerCase()!==entity.username.toLowerCase())},CacheManager.prototype.matchUserAssertionHash=function(entity,userAssertionHash){return!(!entity.userAssertionHash||userAssertionHash!==entity.userAssertionHash)},CacheManager.prototype.matchEnvironment=function(entity,environment){var cloudMetadata=this.getAuthorityMetadataByAlias(environment);return!!(cloudMetadata&&cloudMetadata.aliases.indexOf(entity.environment)>-1)},CacheManager.prototype.matchCredentialType=function(entity,credentialType){return entity.credentialType&&credentialType.toLowerCase()===entity.credentialType.toLowerCase()},CacheManager.prototype.matchClientId=function(entity,clientId){return!(!entity.clientId||clientId!==entity.clientId)},CacheManager.prototype.matchFamilyId=function(entity,familyId){return!(!entity.familyId||familyId!==entity.familyId)},CacheManager.prototype.matchRealm=function(entity,realm){return!(!entity.realm||realm!==entity.realm)},CacheManager.prototype.matchNativeAccountId=function(entity,nativeAccountId){return!(!entity.nativeAccountId||nativeAccountId!==entity.nativeAccountId)},CacheManager.prototype.matchTarget=function(entity,target){return!(entity.credentialType!==CredentialType.ACCESS_TOKEN&&entity.credentialType!==CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME||!entity.target)&&ScopeSet.fromString(entity.target).containsScopeSet(target)},CacheManager.prototype.matchTokenType=function(entity,tokenType){return!(!entity.tokenType||entity.tokenType!==tokenType)},CacheManager.prototype.matchKeyId=function(entity,keyId){return!(!entity.keyId||entity.keyId!==keyId)},CacheManager.prototype.isAppMetadata=function(key){return-1!==key.indexOf(APP_METADATA)},CacheManager.prototype.isAuthorityMetadata=function(key){return-1!==key.indexOf(AUTHORITY_METADATA_CONSTANTS.CACHE_KEY)},CacheManager.prototype.generateAuthorityMetadataCacheKey=function(authority){return AUTHORITY_METADATA_CONSTANTS.CACHE_KEY+"-"+this.clientId+"-"+authority},CacheManager.toObject=function(obj,json){for(var propertyName in json)obj[propertyName]=json[propertyName];return obj},CacheManager}(),DefaultStorageClass=function(_super){function DefaultStorageClass(){return null!==_super&&_super.apply(this,arguments)||this}return __extends(DefaultStorageClass,_super),DefaultStorageClass.prototype.setAccount=function(){var notImplErr="Storage interface - setAccount() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAccount=function(){var notImplErr="Storage interface - getAccount() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setIdTokenCredential=function(){var notImplErr="Storage interface - setIdTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getIdTokenCredential=function(){var notImplErr="Storage interface - getIdTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setAccessTokenCredential=function(){var notImplErr="Storage interface - setAccessTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAccessTokenCredential=function(){var notImplErr="Storage interface - getAccessTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setRefreshTokenCredential=function(){var notImplErr="Storage interface - setRefreshTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getRefreshTokenCredential=function(){var notImplErr="Storage interface - getRefreshTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setAppMetadata=function(){var notImplErr="Storage interface - setAppMetadata() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAppMetadata=function(){var notImplErr="Storage interface - getAppMetadata() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setServerTelemetry=function(){var notImplErr="Storage interface - setServerTelemetry() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getServerTelemetry=function(){var notImplErr="Storage interface - getServerTelemetry() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setAuthorityMetadata=function(){var notImplErr="Storage interface - setAuthorityMetadata() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAuthorityMetadata=function(){var notImplErr="Storage interface - getAuthorityMetadata() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAuthorityMetadataKeys=function(){var notImplErr="Storage interface - getAuthorityMetadataKeys() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setThrottlingCache=function(){var notImplErr="Storage interface - setThrottlingCache() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getThrottlingCache=function(){var notImplErr="Storage interface - getThrottlingCache() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.removeItem=function(){var notImplErr="Storage interface - removeItem() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.containsKey=function(){var notImplErr="Storage interface - containsKey() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getKeys=function(){var notImplErr="Storage interface - getKeys() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAccountKeys=function(){var notImplErr="Storage interface - getAccountKeys() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getTokenKeys=function(){var notImplErr="Storage interface - getTokenKeys() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.clear=function(){return __awaiter(this,void 0,void 0,(function(){var notImplErr;return __generator(this,(function(_a){throw notImplErr="Storage interface - clear() has not been implemented for the cacheStorage interface.",AuthError.createUnexpectedError(notImplErr)}))}))},DefaultStorageClass.prototype.updateCredentialCacheKey=function(){var notImplErr="Storage interface - updateCredentialCacheKey() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass}(CacheManager);export{CacheManager,DefaultStorageClass};