/*! @azure/msal-common v7.6.0 2022-10-10 */
"use strict";import{__awaiter,__generator,__extends}from"../_virtual/_tslib.js";import{Constants,CredentialType,AuthenticationScheme,CacheSchemaType,THE_FAMILY_ID,APP_METADATA,AUTHORITY_METADATA_CONSTANTS}from"../utils/Constants.js";import{CredentialEntity}from"./entities/CredentialEntity.js";import{ScopeSet}from"../request/ScopeSet.js";import{AccountEntity}from"./entities/AccountEntity.js";import{AuthError}from"../error/AuthError.js";import{ClientAuthError}from"../error/ClientAuthError.js";import{AuthToken}from"../account/AuthToken.js";var CacheManager=function(){function CacheManager(clientId,cryptoImpl){this.clientId=clientId,this.cryptoImpl=cryptoImpl}return CacheManager.prototype.getAllAccounts=function(){var _this=this,currentAccounts=this.getAccountsFilteredBy(),accountValues=Object.keys(currentAccounts).map((function(accountKey){return currentAccounts[accountKey]}));return accountValues.length<1?[]:accountValues.map((function(value){var accountInfo=CacheManager.toObject(new AccountEntity,value).getAccountInfo(),idToken=_this.readIdTokenFromCache(_this.clientId,accountInfo);return idToken&&!accountInfo.idTokenClaims&&(accountInfo.idToken=idToken.secret,accountInfo.idTokenClaims=new AuthToken(idToken.secret,_this.cryptoImpl).claims),accountInfo}))},CacheManager.prototype.saveCacheRecord=function(cacheRecord){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){switch(_a.label){case 0:if(!cacheRecord)throw ClientAuthError.createNullOrUndefinedCacheRecord();return cacheRecord.account&&this.setAccount(cacheRecord.account),cacheRecord.idToken&&this.setIdTokenCredential(cacheRecord.idToken),cacheRecord.accessToken?[4,this.saveAccessToken(cacheRecord.accessToken)]:[3,2];case 1:_a.sent(),_a.label=2;case 2:return cacheRecord.refreshToken&&this.setRefreshTokenCredential(cacheRecord.refreshToken),cacheRecord.appMetadata&&this.setAppMetadata(cacheRecord.appMetadata),[2]}}))}))},CacheManager.prototype.saveAccessToken=function(credential){return __awaiter(this,void 0,void 0,(function(){var currentTokenCache,currentScopes,currentAccessTokens,removedAccessTokens_1,_this=this;return __generator(this,(function(_a){switch(_a.label){case 0:return currentTokenCache=this.getCredentialsFilteredBy({clientId:credential.clientId,credentialType:credential.credentialType,environment:credential.environment,homeAccountId:credential.homeAccountId,realm:credential.realm,tokenType:credential.tokenType,requestedClaimsHash:credential.requestedClaimsHash}),currentScopes=ScopeSet.fromString(credential.target),(currentAccessTokens=Object.keys(currentTokenCache.accessTokens).map((function(key){return currentTokenCache.accessTokens[key]})))?(removedAccessTokens_1=[],currentAccessTokens.forEach((function(tokenEntity){ScopeSet.fromString(tokenEntity.target).intersectingScopeSets(currentScopes)&&removedAccessTokens_1.push(_this.removeCredential(tokenEntity))})),[4,Promise.all(removedAccessTokens_1)]):[3,2];case 1:_a.sent(),_a.label=2;case 2:return this.setAccessTokenCredential(credential),[2]}}))}))},CacheManager.prototype.getAccountsFilteredBy=function(accountFilter){return this.getAccountsFilteredByInternal(accountFilter?accountFilter.homeAccountId:Constants.EMPTY_STRING,accountFilter?accountFilter.environment:Constants.EMPTY_STRING,accountFilter?accountFilter.realm:Constants.EMPTY_STRING,accountFilter?accountFilter.nativeAccountId:Constants.EMPTY_STRING)},CacheManager.prototype.getAccountsFilteredByInternal=function(homeAccountId,environment,realm,nativeAccountId){var _this=this,allCacheKeys=this.getKeys(),matchingAccounts={};return allCacheKeys.forEach((function(cacheKey){var entity=_this.getAccount(cacheKey);entity&&(homeAccountId&&!_this.matchHomeAccountId(entity,homeAccountId)||environment&&!_this.matchEnvironment(entity,environment)||realm&&!_this.matchRealm(entity,realm)||nativeAccountId&&!_this.matchNativeAccountId(entity,nativeAccountId)||(matchingAccounts[cacheKey]=entity))})),matchingAccounts},CacheManager.prototype.getCredentialsFilteredBy=function(filter){return this.getCredentialsFilteredByInternal(filter.homeAccountId,filter.environment,filter.credentialType,filter.clientId,filter.familyId,filter.realm,filter.target,filter.userAssertionHash,filter.tokenType,filter.keyId,filter.requestedClaimsHash)},CacheManager.prototype.getCredentialsFilteredByInternal=function(homeAccountId,environment,credentialType,clientId,familyId,realm,target,userAssertionHash,tokenType,keyId,requestedClaimsHash){var _this=this,allCacheKeys=this.getKeys(),matchingCredentials={idTokens:{},accessTokens:{},refreshTokens:{}};return allCacheKeys.forEach((function(cacheKey){var credType=CredentialEntity.getCredentialType(cacheKey);if(credType!==Constants.NOT_DEFINED){var entity=_this.getSpecificCredential(cacheKey,credType);if(entity&&(!userAssertionHash||_this.matchUserAssertionHash(entity,userAssertionHash))&&("string"!=typeof homeAccountId||_this.matchHomeAccountId(entity,homeAccountId))&&(!environment||_this.matchEnvironment(entity,environment))&&(!realm||_this.matchRealm(entity,realm))&&(!credentialType||_this.matchCredentialType(entity,credentialType))&&(!clientId||_this.matchClientId(entity,clientId))&&(!familyId||_this.matchFamilyId(entity,familyId))&&(!target||_this.matchTarget(entity,target))&&(!requestedClaimsHash&&!entity.requestedClaimsHash||entity.requestedClaimsHash===requestedClaimsHash)){if(credentialType===CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME){if(tokenType&&!_this.matchTokenType(entity,tokenType))return;if(tokenType===AuthenticationScheme.SSH&&keyId&&!_this.matchKeyId(entity,keyId))return}var updatedCacheKey=_this.updateCredentialCacheKey(cacheKey,entity);switch(credType){case CredentialType.ID_TOKEN:matchingCredentials.idTokens[updatedCacheKey]=entity;break;case CredentialType.ACCESS_TOKEN:case CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME:matchingCredentials.accessTokens[updatedCacheKey]=entity;break;case CredentialType.REFRESH_TOKEN:matchingCredentials.refreshTokens[updatedCacheKey]=entity}}}})),matchingCredentials},CacheManager.prototype.getAppMetadataFilteredBy=function(filter){return this.getAppMetadataFilteredByInternal(filter.environment,filter.clientId)},CacheManager.prototype.getAppMetadataFilteredByInternal=function(environment,clientId){var _this=this,allCacheKeys=this.getKeys(),matchingAppMetadata={};return allCacheKeys.forEach((function(cacheKey){if(_this.isAppMetadata(cacheKey)){var entity=_this.getAppMetadata(cacheKey);entity&&(environment&&!_this.matchEnvironment(entity,environment)||clientId&&!_this.matchClientId(entity,clientId)||(matchingAppMetadata[cacheKey]=entity))}})),matchingAppMetadata},CacheManager.prototype.getAuthorityMetadataByAlias=function(host){var _this=this,allCacheKeys=this.getAuthorityMetadataKeys(),matchedEntity=null;return allCacheKeys.forEach((function(cacheKey){if(_this.isAuthorityMetadata(cacheKey)&&-1!==cacheKey.indexOf(_this.clientId)){var entity=_this.getAuthorityMetadata(cacheKey);entity&&-1!==entity.aliases.indexOf(host)&&(matchedEntity=entity)}})),matchedEntity},CacheManager.prototype.removeAllAccounts=function(){return __awaiter(this,void 0,void 0,(function(){var allCacheKeys,removedAccounts,_this=this;return __generator(this,(function(_a){switch(_a.label){case 0:return allCacheKeys=this.getKeys(),removedAccounts=[],allCacheKeys.forEach((function(cacheKey){_this.getAccount(cacheKey)&&removedAccounts.push(_this.removeAccount(cacheKey))})),[4,Promise.all(removedAccounts)];case 1:return _a.sent(),[2,!0]}}))}))},CacheManager.prototype.removeAccount=function(accountKey){return __awaiter(this,void 0,void 0,(function(){var account;return __generator(this,(function(_a){switch(_a.label){case 0:if(!(account=this.getAccount(accountKey)))throw ClientAuthError.createNoAccountFoundError();return[4,this.removeAccountContext(account)];case 1:return[2,_a.sent()&&this.removeItem(accountKey,CacheSchemaType.ACCOUNT)]}}))}))},CacheManager.prototype.removeAccountContext=function(account){return __awaiter(this,void 0,void 0,(function(){var allCacheKeys,accountId,removedCredentials,_this=this;return __generator(this,(function(_a){switch(_a.label){case 0:return allCacheKeys=this.getKeys(),accountId=account.generateAccountId(),removedCredentials=[],allCacheKeys.forEach((function(cacheKey){var credType=CredentialEntity.getCredentialType(cacheKey);if(credType!==Constants.NOT_DEFINED){var cacheEntity=_this.getSpecificCredential(cacheKey,credType);cacheEntity&&accountId===cacheEntity.generateAccountId()&&removedCredentials.push(_this.removeCredential(cacheEntity))}})),[4,Promise.all(removedCredentials)];case 1:return _a.sent(),[2,!0]}}))}))},CacheManager.prototype.removeCredential=function(credential){return __awaiter(this,void 0,void 0,(function(){var key,kid;return __generator(this,(function(_a){switch(_a.label){case 0:if(key=credential.generateCredentialKey(),credential.credentialType.toLowerCase()!==CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME.toLowerCase())return[3,4];if(credential.tokenType!==AuthenticationScheme.POP)return[3,4];if(!(kid=credential.keyId))return[3,4];_a.label=1;case 1:return _a.trys.push([1,3,,4]),[4,this.cryptoImpl.removeTokenBindingKey(kid)];case 2:return _a.sent(),[3,4];case 3:throw _a.sent(),ClientAuthError.createBindingKeyNotRemovedError();case 4:return[2,this.removeItem(key,CacheSchemaType.CREDENTIAL)]}}))}))},CacheManager.prototype.removeAppMetadata=function(){var _this=this;return this.getKeys().forEach((function(cacheKey){_this.isAppMetadata(cacheKey)&&_this.removeItem(cacheKey,CacheSchemaType.APP_METADATA)})),!0},CacheManager.prototype.readCacheRecord=function(account,clientId,request,environment){var cachedAccount=this.readAccountFromCache(account),cachedIdToken=this.readIdTokenFromCache(clientId,account),cachedAccessToken=this.readAccessTokenFromCache(clientId,account,request),cachedRefreshToken=this.readRefreshTokenFromCache(clientId,account,!1),cachedAppMetadata=this.readAppMetadataFromCache(environment,clientId);return cachedAccount&&cachedIdToken&&(cachedAccount.idTokenClaims=new AuthToken(cachedIdToken.secret,this.cryptoImpl).claims),{account:cachedAccount,idToken:cachedIdToken,accessToken:cachedAccessToken,refreshToken:cachedRefreshToken,appMetadata:cachedAppMetadata}},CacheManager.prototype.readAccountFromCache=function(account){var accountKey=AccountEntity.generateAccountCacheKey(account);return this.getAccount(accountKey)},CacheManager.prototype.readAccountFromCacheWithNativeAccountId=function(nativeAccountId){var accountFilter={nativeAccountId:nativeAccountId},accountCache=this.getAccountsFilteredBy(accountFilter),accounts=Object.keys(accountCache).map((function(key){return accountCache[key]}));if(accounts.length<1)return null;if(accounts.length>1)throw ClientAuthError.createMultipleMatchingAccountsInCacheError();return accountCache[0]},CacheManager.prototype.readIdTokenFromCache=function(clientId,account){var idTokenFilter={homeAccountId:account.homeAccountId,environment:account.environment,credentialType:CredentialType.ID_TOKEN,clientId:clientId,realm:account.tenantId},credentialCache=this.getCredentialsFilteredBy(idTokenFilter),idTokens=Object.keys(credentialCache.idTokens).map((function(key){return credentialCache.idTokens[key]})),numIdTokens=idTokens.length;if(numIdTokens<1)return null;if(numIdTokens>1)throw ClientAuthError.createMultipleMatchingTokensInCacheError();return idTokens[0]},CacheManager.prototype.readAccessTokenFromCache=function(clientId,account,request){var scopes=new ScopeSet(request.scopes||[]),authScheme=request.authenticationScheme||AuthenticationScheme.BEARER,credentialType=authScheme&&authScheme.toLowerCase()!==AuthenticationScheme.BEARER.toLowerCase()?CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME:CredentialType.ACCESS_TOKEN,accessTokenFilter={homeAccountId:account.homeAccountId,environment:account.environment,credentialType:credentialType,clientId:clientId,realm:account.tenantId,target:scopes.printScopesLowerCase(),tokenType:authScheme,keyId:request.sshKid,requestedClaimsHash:request.requestedClaimsHash},credentialCache=this.getCredentialsFilteredBy(accessTokenFilter),accessTokens=Object.keys(credentialCache.accessTokens).map((function(key){return credentialCache.accessTokens[key]})),numAccessTokens=accessTokens.length;if(numAccessTokens<1)return null;if(numAccessTokens>1)throw ClientAuthError.createMultipleMatchingTokensInCacheError();return accessTokens[0]},CacheManager.prototype.readRefreshTokenFromCache=function(clientId,account,familyRT){var id=familyRT?THE_FAMILY_ID:void 0,refreshTokenFilter={homeAccountId:account.homeAccountId,environment:account.environment,credentialType:CredentialType.REFRESH_TOKEN,clientId:clientId,familyId:id},credentialCache=this.getCredentialsFilteredBy(refreshTokenFilter),refreshTokens=Object.keys(credentialCache.refreshTokens).map((function(key){return credentialCache.refreshTokens[key]}));return refreshTokens.length<1?null:refreshTokens[0]},CacheManager.prototype.readAppMetadataFromCache=function(environment,clientId){var appMetadataFilter={environment:environment,clientId:clientId},appMetadata=this.getAppMetadataFilteredBy(appMetadataFilter),appMetadataEntries=Object.keys(appMetadata).map((function(key){return appMetadata[key]})),numAppMetadata=appMetadataEntries.length;if(numAppMetadata<1)return null;if(numAppMetadata>1)throw ClientAuthError.createMultipleMatchingAppMetadataInCacheError();return appMetadataEntries[0]},CacheManager.prototype.isAppMetadataFOCI=function(environment,clientId){var appMetadata=this.readAppMetadataFromCache(environment,clientId);return!(!appMetadata||appMetadata.familyId!==THE_FAMILY_ID)},CacheManager.prototype.matchHomeAccountId=function(entity,homeAccountId){return!("string"!=typeof entity.homeAccountId||homeAccountId!==entity.homeAccountId)},CacheManager.prototype.matchUserAssertionHash=function(entity,userAssertionHash){return!(!entity.userAssertionHash||userAssertionHash!==entity.userAssertionHash)},CacheManager.prototype.matchEnvironment=function(entity,environment){var cloudMetadata=this.getAuthorityMetadataByAlias(environment);return!!(cloudMetadata&&cloudMetadata.aliases.indexOf(entity.environment)>-1)},CacheManager.prototype.matchCredentialType=function(entity,credentialType){return entity.credentialType&&credentialType.toLowerCase()===entity.credentialType.toLowerCase()},CacheManager.prototype.matchClientId=function(entity,clientId){return!(!entity.clientId||clientId!==entity.clientId)},CacheManager.prototype.matchFamilyId=function(entity,familyId){return!(!entity.familyId||familyId!==entity.familyId)},CacheManager.prototype.matchRealm=function(entity,realm){return!(!entity.realm||realm!==entity.realm)},CacheManager.prototype.matchNativeAccountId=function(entity,nativeAccountId){return!(!entity.nativeAccountId||nativeAccountId!==entity.nativeAccountId)},CacheManager.prototype.matchTarget=function(entity,target){if(entity.credentialType!==CredentialType.ACCESS_TOKEN&&entity.credentialType!==CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME||!entity.target)return!1;var entityScopeSet=ScopeSet.fromString(entity.target),requestTargetScopeSet=ScopeSet.fromString(target);return requestTargetScopeSet.containsOnlyOIDCScopes()?requestTargetScopeSet.removeScope(Constants.OFFLINE_ACCESS_SCOPE):requestTargetScopeSet.removeOIDCScopes(),entityScopeSet.containsScopeSet(requestTargetScopeSet)},CacheManager.prototype.matchTokenType=function(entity,tokenType){return!(!entity.tokenType||entity.tokenType!==tokenType)},CacheManager.prototype.matchKeyId=function(entity,keyId){return!(!entity.keyId||entity.keyId!==keyId)},CacheManager.prototype.isAppMetadata=function(key){return-1!==key.indexOf(APP_METADATA)},CacheManager.prototype.isAuthorityMetadata=function(key){return-1!==key.indexOf(AUTHORITY_METADATA_CONSTANTS.CACHE_KEY)},CacheManager.prototype.generateAuthorityMetadataCacheKey=function(authority){return AUTHORITY_METADATA_CONSTANTS.CACHE_KEY+"-"+this.clientId+"-"+authority},CacheManager.prototype.getSpecificCredential=function(key,credType){switch(credType){case CredentialType.ID_TOKEN:return this.getIdTokenCredential(key);case CredentialType.ACCESS_TOKEN:case CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME:return this.getAccessTokenCredential(key);case CredentialType.REFRESH_TOKEN:return this.getRefreshTokenCredential(key);default:return null}},CacheManager.toObject=function(obj,json){for(var propertyName in json)obj[propertyName]=json[propertyName];return obj},CacheManager}(),DefaultStorageClass=function(_super){function DefaultStorageClass(){return null!==_super&&_super.apply(this,arguments)||this}return __extends(DefaultStorageClass,_super),DefaultStorageClass.prototype.setAccount=function(){var notImplErr="Storage interface - setAccount() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAccount=function(){var notImplErr="Storage interface - getAccount() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setIdTokenCredential=function(){var notImplErr="Storage interface - setIdTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getIdTokenCredential=function(){var notImplErr="Storage interface - getIdTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setAccessTokenCredential=function(){var notImplErr="Storage interface - setAccessTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAccessTokenCredential=function(){var notImplErr="Storage interface - getAccessTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setRefreshTokenCredential=function(){var notImplErr="Storage interface - setRefreshTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getRefreshTokenCredential=function(){var notImplErr="Storage interface - getRefreshTokenCredential() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setAppMetadata=function(){var notImplErr="Storage interface - setAppMetadata() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAppMetadata=function(){var notImplErr="Storage interface - getAppMetadata() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setServerTelemetry=function(){var notImplErr="Storage interface - setServerTelemetry() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getServerTelemetry=function(){var notImplErr="Storage interface - getServerTelemetry() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setAuthorityMetadata=function(){var notImplErr="Storage interface - setAuthorityMetadata() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAuthorityMetadata=function(){var notImplErr="Storage interface - getAuthorityMetadata() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getAuthorityMetadataKeys=function(){var notImplErr="Storage interface - getAuthorityMetadataKeys() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.setThrottlingCache=function(){var notImplErr="Storage interface - setThrottlingCache() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getThrottlingCache=function(){var notImplErr="Storage interface - getThrottlingCache() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.removeItem=function(){var notImplErr="Storage interface - removeItem() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.containsKey=function(){var notImplErr="Storage interface - containsKey() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.getKeys=function(){var notImplErr="Storage interface - getKeys() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass.prototype.clear=function(){return __awaiter(this,void 0,void 0,(function(){var notImplErr;return __generator(this,(function(_a){throw notImplErr="Storage interface - clear() has not been implemented for the cacheStorage interface.",AuthError.createUnexpectedError(notImplErr)}))}))},DefaultStorageClass.prototype.updateCredentialCacheKey=function(){var notImplErr="Storage interface - updateCredentialCacheKey() has not been implemented for the cacheStorage interface.";throw AuthError.createUnexpectedError(notImplErr)},DefaultStorageClass}(CacheManager);export{CacheManager,DefaultStorageClass};