/*! @azure/msal-common v7.6.0 2022-10-10 */
"use strict";import{__assign}from"../../_virtual/_tslib.js";import{PerformanceEventStatus}from"./PerformanceEvent.js";var PerformanceClient=function(){function PerformanceClient(clientId,authority,logger,libraryName,libraryVersion,applicationTelemetry){this.authority=authority,this.libraryName=libraryName,this.libraryVersion=libraryVersion,this.applicationTelemetry=applicationTelemetry,this.clientId=clientId,this.logger=logger,this.callbacks=new Map,this.eventsByCorrelationId=new Map,this.measurementsById=new Map}return PerformanceClient.prototype.startMeasurement=function(measureName,correlationId){var _a,_b,_this=this,eventCorrelationId=correlationId||this.generateId();correlationId||this.logger.info("PerformanceClient: No correlation id provided for "+measureName+", generating",eventCorrelationId),this.logger.trace("PerformanceClient: Performance measurement started for "+measureName,eventCorrelationId);var performanceMeasurement=this.startPerformanceMeasuremeant(measureName,eventCorrelationId);performanceMeasurement.startMeasurement();var inProgressEvent={eventId:this.generateId(),status:PerformanceEventStatus.InProgress,authority:this.authority,libraryName:this.libraryName,libraryVersion:this.libraryVersion,appName:null===(_a=this.applicationTelemetry)||void 0===_a?void 0:_a.appName,appVersion:null===(_b=this.applicationTelemetry)||void 0===_b?void 0:_b.appVersion,clientId:this.clientId,name:measureName,startTimeMs:Date.now(),correlationId:eventCorrelationId};return this.cacheEventByCorrelationId(inProgressEvent),this.cacheMeasurement(inProgressEvent,performanceMeasurement),{endMeasurement:function(event){var completedEvent=_this.endMeasurement(__assign(__assign({},inProgressEvent),event));return completedEvent&&_this.cacheEventByCorrelationId(completedEvent),completedEvent},flushMeasurement:function(){return _this.flushMeasurements(inProgressEvent.name,inProgressEvent.correlationId)},discardMeasurement:function(){return _this.discardMeasurements(inProgressEvent.correlationId)},measurement:performanceMeasurement,event:inProgressEvent}},PerformanceClient.prototype.endMeasurement=function(event){var performanceMeasurement=this.measurementsById.get(event.eventId);if(performanceMeasurement){this.measurementsById.delete(event.eventId),performanceMeasurement.endMeasurement();var durationMs=performanceMeasurement.flushMeasurement();if(null!==durationMs)return this.logger.trace("PerformanceClient: Performance measurement ended for "+event.name+": "+durationMs+" ms",event.correlationId),__assign(__assign({durationMs:Math.round(durationMs)},event),{status:PerformanceEventStatus.Completed});this.logger.trace("PerformanceClient: Performance measurement not taken",event.correlationId)}else this.logger.trace("PerformanceClient: Measurement not found for "+event.eventId,event.correlationId);return null},PerformanceClient.prototype.cacheEventByCorrelationId=function(event){var existingEvents=this.eventsByCorrelationId.get(event.correlationId);existingEvents?(this.logger.trace("PerformanceClient: Performance measurement for "+event.name+" added/updated",event.correlationId),existingEvents.set(event.eventId,event)):(this.logger.trace("PerformanceClient: Performance measurement for "+event.name+" started",event.correlationId),this.eventsByCorrelationId.set(event.correlationId,(new Map).set(event.eventId,event)))},PerformanceClient.prototype.cacheMeasurement=function(event,measurement){this.measurementsById.set(event.eventId,measurement)},PerformanceClient.prototype.flushMeasurements=function(measureName,correlationId){var _this=this;this.logger.trace("PerformanceClient: Performance measurements flushed for "+measureName,correlationId);var eventsForCorrelationId=this.eventsByCorrelationId.get(correlationId);if(eventsForCorrelationId){this.discardMeasurements(correlationId);var completedEvents_1=[];eventsForCorrelationId.forEach((function(event){if(event.name!==measureName&&event.status!==PerformanceEventStatus.Completed){_this.logger.trace("PerformanceClient: Incomplete submeasurement "+event.name+" found for "+measureName,correlationId);var completedEvent=_this.endMeasurement(event);completedEvent&&completedEvents_1.push(completedEvent)}completedEvents_1.push(event)}));var sortedCompletedEvents=completedEvents_1.sort((function(eventA,eventB){return eventA.startTimeMs-eventB.startTimeMs})),topLevelEvents=sortedCompletedEvents.filter((function(event){return event.name===measureName&&event.status===PerformanceEventStatus.Completed}));if(topLevelEvents.length>0){topLevelEvents.length>1&&this.logger.verbose("PerformanceClient: Multiple distinct top-level performance events found, using the first",correlationId);var topLevelEvent=topLevelEvents[0];this.logger.verbose("PerformanceClient: Measurement found for "+measureName,correlationId);var eventToEmit=sortedCompletedEvents.reduce((function(previous,current){if(current.name!==measureName){_this.logger.trace("PerformanceClient: Complete submeasurement found for "+current.name,correlationId);var subMeasurementName=current.name+"DurationMs";previous[subMeasurementName]?_this.logger.verbose("PerformanceClient: Submeasurement for "+measureName+" already exists for "+current.name+", ignoring",correlationId):previous[subMeasurementName]=current.durationMs,current.accessTokenSize&&(previous.accessTokenSize=current.accessTokenSize),current.idTokenSize&&(previous.idTokenSize=current.idTokenSize),current.refreshTokenSize&&(previous.refreshTokenSize=current.refreshTokenSize)}return previous}),topLevelEvent);this.emitEvents([eventToEmit],eventToEmit.correlationId)}else this.logger.verbose("PerformanceClient: No completed top-level measurements found for "+measureName,correlationId)}else this.logger.verbose("PerformanceClient: No measurements found",correlationId)},PerformanceClient.prototype.discardMeasurements=function(correlationId){this.logger.trace("PerformanceClient: Performance measurements discarded",correlationId),this.eventsByCorrelationId.delete(correlationId)},PerformanceClient.prototype.addPerformanceCallback=function(callback){var callbackId=this.generateId();return this.callbacks.set(callbackId,callback),this.logger.verbose("PerformanceClient: Performance callback registered with id: "+callbackId),callbackId},PerformanceClient.prototype.removePerformanceCallback=function(callbackId){var result=this.callbacks.delete(callbackId);return result?this.logger.verbose("PerformanceClient: Performance callback "+callbackId+" removed."):this.logger.verbose("PerformanceClient: Performance callback "+callbackId+" not removed."),result},PerformanceClient.prototype.emitEvents=function(events,correlationId){var _this=this;this.logger.verbose("PerformanceClient: Emitting performance events",correlationId),this.callbacks.forEach((function(callback,callbackId){_this.logger.trace("PerformanceClient: Emitting event to callback "+callbackId,correlationId),callback.apply(null,[events])}))},PerformanceClient}();export{PerformanceClient};