/*! @azure/msal-common v13.1.0 2023-06-07 */
"use strict";import{__awaiter,__generator,__assign}from"../_virtual/_tslib.js";import{AuthorityType}from"./AuthorityType.js";import{isOpenIdConfigResponse}from"./OpenIdConfigResponse.js";import{UrlString}from"../url/UrlString.js";import{ClientAuthError}from"../error/ClientAuthError.js";import{Constants,AADAuthorityConstants,AuthorityMetadataSource,RegionDiscoveryOutcomes}from"../utils/Constants.js";import{EndpointMetadata,InstanceDiscoveryMetadata}from"./AuthorityMetadata.js";import{ClientConfigurationError}from"../error/ClientConfigurationError.js";import{ProtocolMode}from"./ProtocolMode.js";import{AuthorityMetadataEntity}from"../cache/entities/AuthorityMetadataEntity.js";import{AzureCloudInstance}from"./AuthorityOptions.js";import{isCloudInstanceDiscoveryResponse}from"./CloudInstanceDiscoveryResponse.js";import{isCloudInstanceDiscoveryErrorResponse}from"./CloudInstanceDiscoveryErrorResponse.js";import{RegionDiscovery}from"./RegionDiscovery.js";import{AuthError}from"../error/AuthError.js";import{PerformanceEvents}from"../telemetry/performance/PerformanceEvent.js";var Authority=function(){function Authority(authority,networkInterface,cacheManager,authorityOptions,logger,performanceClient,correlationId){this.canonicalAuthority=authority,this._canonicalAuthority.validateAsUri(),this.networkInterface=networkInterface,this.cacheManager=cacheManager,this.authorityOptions=authorityOptions,this.regionDiscoveryMetadata={region_used:void 0,region_source:void 0,region_outcome:void 0},this.logger=logger,this.performanceClient=performanceClient,this.correlationId=correlationId,this.regionDiscovery=new RegionDiscovery(networkInterface,this.performanceClient,this.correlationId)}return Authority.prototype.getAuthorityType=function(authorityUri){if(authorityUri.HostNameAndPort.endsWith(Constants.CIAM_AUTH_URL))return AuthorityType.Ciam;var pathSegments=authorityUri.PathSegments;if(pathSegments.length)switch(pathSegments[0].toLowerCase()){case Constants.ADFS:return AuthorityType.Adfs;case Constants.DSTS:return AuthorityType.Dsts}return AuthorityType.Default},Object.defineProperty(Authority.prototype,"authorityType",{get:function(){return this.getAuthorityType(this.canonicalAuthorityUrlComponents)},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"protocolMode",{get:function(){return this.authorityOptions.protocolMode},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"options",{get:function(){return this.authorityOptions},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"canonicalAuthority",{get:function(){return this._canonicalAuthority.urlString},set:function(url){this._canonicalAuthority=new UrlString(url),this._canonicalAuthority.validateAsUri(),this._canonicalAuthorityUrlComponents=null},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"canonicalAuthorityUrlComponents",{get:function(){return this._canonicalAuthorityUrlComponents||(this._canonicalAuthorityUrlComponents=this._canonicalAuthority.getUrlComponents()),this._canonicalAuthorityUrlComponents},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"hostnameAndPort",{get:function(){return this.canonicalAuthorityUrlComponents.HostNameAndPort.toLowerCase()},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"tenant",{get:function(){return this.canonicalAuthorityUrlComponents.PathSegments[0]},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"authorizationEndpoint",{get:function(){if(this.discoveryComplete())return this.replacePath(this.metadata.authorization_endpoint);throw ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"tokenEndpoint",{get:function(){if(this.discoveryComplete())return this.replacePath(this.metadata.token_endpoint);throw ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"deviceCodeEndpoint",{get:function(){if(this.discoveryComplete())return this.replacePath(this.metadata.token_endpoint.replace("/token","/devicecode"));throw ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"endSessionEndpoint",{get:function(){if(this.discoveryComplete()){if(!this.metadata.end_session_endpoint)throw ClientAuthError.createLogoutNotSupportedError();return this.replacePath(this.metadata.end_session_endpoint)}throw ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"selfSignedJwtAudience",{get:function(){if(this.discoveryComplete())return this.replacePath(this.metadata.issuer);throw ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),Object.defineProperty(Authority.prototype,"jwksUri",{get:function(){if(this.discoveryComplete())return this.replacePath(this.metadata.jwks_uri);throw ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),Authority.prototype.canReplaceTenant=function(authorityUri){return 1===authorityUri.PathSegments.length&&!Authority.reservedTenantDomains.has(authorityUri.PathSegments[0])&&this.getAuthorityType(authorityUri)===AuthorityType.Default&&this.protocolMode===ProtocolMode.AAD},Authority.prototype.replaceTenant=function(urlString){return urlString.replace(/{tenant}|{tenantid}/g,this.tenant)},Authority.prototype.replacePath=function(urlString){var _this=this,endpoint=urlString,cachedAuthorityUrlComponents=new UrlString(this.metadata.canonical_authority).getUrlComponents(),cachedAuthorityParts=cachedAuthorityUrlComponents.PathSegments;return this.canonicalAuthorityUrlComponents.PathSegments.forEach((function(currentPart,index){var cachedPart=cachedAuthorityParts[index];if(0===index&&_this.canReplaceTenant(cachedAuthorityUrlComponents)){var tenantId=new UrlString(_this.metadata.authorization_endpoint).getUrlComponents().PathSegments[0];cachedPart!==tenantId&&(_this.logger.verbose("Replacing tenant domain name "+cachedPart+" with id "+tenantId),cachedPart=tenantId)}currentPart!==cachedPart&&(endpoint=endpoint.replace("/"+cachedPart+"/","/"+currentPart+"/"))})),this.replaceTenant(endpoint)},Object.defineProperty(Authority.prototype,"defaultOpenIdConfigurationEndpoint",{get:function(){return this.authorityType===AuthorityType.Adfs||this.authorityType===AuthorityType.Dsts||this.protocolMode===ProtocolMode.OIDC?this.canonicalAuthority+".well-known/openid-configuration":this.canonicalAuthority+"v2.0/.well-known/openid-configuration"},enumerable:!1,configurable:!0}),Authority.prototype.discoveryComplete=function(){return!!this.metadata},Authority.prototype.resolveEndpointsAsync=function(){var _a,_b,_c;return __awaiter(this,void 0,void 0,(function(){var metadataEntity,cloudDiscoverySource,endpointSource,cacheKey;return __generator(this,(function(_d){switch(_d.label){case 0:return null===(_a=this.performanceClient)||void 0===_a||_a.addQueueMeasurement(PerformanceEvents.AuthorityResolveEndpointsAsync,this.correlationId),(metadataEntity=this.cacheManager.getAuthorityMetadataByAlias(this.hostnameAndPort))||(metadataEntity=new AuthorityMetadataEntity).updateCanonicalAuthority(this.canonicalAuthority),null===(_b=this.performanceClient)||void 0===_b||_b.setPreQueueTime(PerformanceEvents.AuthorityUpdateCloudDiscoveryMetadata,this.correlationId),[4,this.updateCloudDiscoveryMetadata(metadataEntity)];case 1:return cloudDiscoverySource=_d.sent(),this.canonicalAuthority=this.canonicalAuthority.replace(this.hostnameAndPort,metadataEntity.preferred_network),null===(_c=this.performanceClient)||void 0===_c||_c.setPreQueueTime(PerformanceEvents.AuthorityUpdateEndpointMetadata,this.correlationId),[4,this.updateEndpointMetadata(metadataEntity)];case 2:return endpointSource=_d.sent(),cloudDiscoverySource!==AuthorityMetadataSource.CACHE&&endpointSource!==AuthorityMetadataSource.CACHE&&(metadataEntity.resetExpiresAt(),metadataEntity.updateCanonicalAuthority(this.canonicalAuthority)),cacheKey=this.cacheManager.generateAuthorityMetadataCacheKey(metadataEntity.preferred_cache),this.cacheManager.setAuthorityMetadata(cacheKey,metadataEntity),this.metadata=metadataEntity,[2]}}))}))},Authority.prototype.updateEndpointMetadata=function(metadataEntity){var _a,_b,_c,_d,_e,_f;return __awaiter(this,void 0,void 0,(function(){var metadata,harcodedMetadata;return __generator(this,(function(_g){switch(_g.label){case 0:return null===(_a=this.performanceClient)||void 0===_a||_a.addQueueMeasurement(PerformanceEvents.AuthorityUpdateEndpointMetadata,this.correlationId),(metadata=this.getEndpointMetadataFromConfig())?(metadataEntity.updateEndpointMetadata(metadata,!1),[2,AuthorityMetadataSource.CONFIG]):this.isAuthoritySameType(metadataEntity)&&metadataEntity.endpointsFromNetwork&&!metadataEntity.isExpired()?[2,AuthorityMetadataSource.CACHE]:(null===(_b=this.performanceClient)||void 0===_b||_b.setPreQueueTime(PerformanceEvents.AuthorityGetEndpointMetadataFromNetwork,this.correlationId),[4,this.getEndpointMetadataFromNetwork()]);case 1:return(metadata=_g.sent())?(null===(_c=this.authorityOptions.azureRegionConfiguration)||void 0===_c?void 0:_c.azureRegion)?(null===(_d=this.performanceClient)||void 0===_d||_d.setPreQueueTime(PerformanceEvents.AuthorityUpdateMetadataWithRegionalInformation,this.correlationId),[4,this.updateMetadataWithRegionalInformation(metadata)]):[3,3]:[3,4];case 2:metadata=_g.sent(),_g.label=3;case 3:return metadataEntity.updateEndpointMetadata(metadata,!0),[2,AuthorityMetadataSource.NETWORK];case 4:return!(harcodedMetadata=this.getEndpointMetadataFromHardcodedValues())||this.authorityOptions.skipAuthorityMetadataCache?[3,7]:(null===(_e=this.authorityOptions.azureRegionConfiguration)||void 0===_e?void 0:_e.azureRegion)?(null===(_f=this.performanceClient)||void 0===_f||_f.setPreQueueTime(PerformanceEvents.AuthorityUpdateMetadataWithRegionalInformation,this.correlationId),[4,this.updateMetadataWithRegionalInformation(harcodedMetadata)]):[3,6];case 5:harcodedMetadata=_g.sent(),_g.label=6;case 6:return metadataEntity.updateEndpointMetadata(harcodedMetadata,!1),[2,AuthorityMetadataSource.HARDCODED_VALUES];case 7:throw ClientAuthError.createUnableToGetOpenidConfigError(this.defaultOpenIdConfigurationEndpoint)}}))}))},Authority.prototype.isAuthoritySameType=function(metadataEntity){return new UrlString(metadataEntity.canonical_authority).getUrlComponents().PathSegments.length===this.canonicalAuthorityUrlComponents.PathSegments.length},Authority.prototype.getEndpointMetadataFromConfig=function(){if(this.authorityOptions.authorityMetadata)try{return JSON.parse(this.authorityOptions.authorityMetadata)}catch(e){throw ClientConfigurationError.createInvalidAuthorityMetadataError()}return null},Authority.prototype.getEndpointMetadataFromNetwork=function(){var _a;return __awaiter(this,void 0,void 0,(function(){var options,response;return __generator(this,(function(_b){switch(_b.label){case 0:null===(_a=this.performanceClient)||void 0===_a||_a.addQueueMeasurement(PerformanceEvents.AuthorityGetEndpointMetadataFromNetwork,this.correlationId),options={},_b.label=1;case 1:return _b.trys.push([1,3,,4]),[4,this.networkInterface.sendGetRequestAsync(this.defaultOpenIdConfigurationEndpoint,options)];case 2:return response=_b.sent(),[2,isOpenIdConfigResponse(response.body)?response.body:null];case 3:return _b.sent(),[2,null];case 4:return[2]}}))}))},Authority.prototype.getEndpointMetadataFromHardcodedValues=function(){return this.canonicalAuthority in EndpointMetadata?EndpointMetadata[this.canonicalAuthority]:null},Authority.prototype.updateMetadataWithRegionalInformation=function(metadata){var _a,_b,_c,_d,_e,_f,_g;return __awaiter(this,void 0,void 0,(function(){var autodetectedRegionName,azureRegion;return __generator(this,(function(_h){switch(_h.label){case 0:return null===(_a=this.performanceClient)||void 0===_a||_a.addQueueMeasurement(PerformanceEvents.AuthorityUpdateMetadataWithRegionalInformation,this.correlationId),null===(_b=this.performanceClient)||void 0===_b||_b.setPreQueueTime(PerformanceEvents.RegionDiscoveryDetectRegion,this.correlationId),[4,this.regionDiscovery.detectRegion(null===(_c=this.authorityOptions.azureRegionConfiguration)||void 0===_c?void 0:_c.environmentRegion,this.regionDiscoveryMetadata)];case 1:return autodetectedRegionName=_h.sent(),azureRegion=(null===(_d=this.authorityOptions.azureRegionConfiguration)||void 0===_d?void 0:_d.azureRegion)===Constants.AZURE_REGION_AUTO_DISCOVER_FLAG?autodetectedRegionName:null===(_e=this.authorityOptions.azureRegionConfiguration)||void 0===_e?void 0:_e.azureRegion,(null===(_f=this.authorityOptions.azureRegionConfiguration)||void 0===_f?void 0:_f.azureRegion)===Constants.AZURE_REGION_AUTO_DISCOVER_FLAG?this.regionDiscoveryMetadata.region_outcome=autodetectedRegionName?RegionDiscoveryOutcomes.AUTO_DETECTION_REQUESTED_SUCCESSFUL:RegionDiscoveryOutcomes.AUTO_DETECTION_REQUESTED_FAILED:this.regionDiscoveryMetadata.region_outcome=autodetectedRegionName?(null===(_g=this.authorityOptions.azureRegionConfiguration)||void 0===_g?void 0:_g.azureRegion)===autodetectedRegionName?RegionDiscoveryOutcomes.CONFIGURED_MATCHES_DETECTED:RegionDiscoveryOutcomes.CONFIGURED_NOT_DETECTED:RegionDiscoveryOutcomes.CONFIGURED_NO_AUTO_DETECTION,azureRegion?(this.regionDiscoveryMetadata.region_used=azureRegion,[2,Authority.replaceWithRegionalInformation(metadata,azureRegion)]):[2,metadata]}}))}))},Authority.prototype.updateCloudDiscoveryMetadata=function(metadataEntity){var _a,_b;return __awaiter(this,void 0,void 0,(function(){var metadata,metadataEntityExpired,harcodedMetadata;return __generator(this,(function(_c){switch(_c.label){case 0:return null===(_a=this.performanceClient)||void 0===_a||_a.addQueueMeasurement(PerformanceEvents.AuthorityUpdateCloudDiscoveryMetadata,this.correlationId),this.logger.verbose("Attempting to get cloud discovery metadata in the config"),this.logger.verbosePii("Known Authorities: "+(this.authorityOptions.knownAuthorities||Constants.NOT_APPLICABLE)),this.logger.verbosePii("Authority Metadata: "+(this.authorityOptions.authorityMetadata||Constants.NOT_APPLICABLE)),this.logger.verbosePii("Canonical Authority: "+(metadataEntity.canonical_authority||Constants.NOT_APPLICABLE)),(metadata=this.getCloudDiscoveryMetadataFromConfig())?(this.logger.verbose("Found cloud discovery metadata in the config."),metadataEntity.updateCloudDiscoveryMetadata(metadata,!1),[2,AuthorityMetadataSource.CONFIG]):(this.logger.verbose("Did not find cloud discovery metadata in the config... Attempting to get cloud discovery metadata from the cache."),metadataEntityExpired=metadataEntity.isExpired(),this.isAuthoritySameType(metadataEntity)&&metadataEntity.aliasesFromNetwork&&!metadataEntityExpired?(this.logger.verbose("Found metadata in the cache."),[2,AuthorityMetadataSource.CACHE]):(metadataEntityExpired&&this.logger.verbose("The metadata entity is expired."),this.logger.verbose("Did not find cloud discovery metadata in the cache... Attempting to get cloud discovery metadata from the network."),null===(_b=this.performanceClient)||void 0===_b||_b.setPreQueueTime(PerformanceEvents.AuthorityGetCloudDiscoveryMetadataFromNetwork,this.correlationId),[4,this.getCloudDiscoveryMetadataFromNetwork()]));case 1:if(metadata=_c.sent())return this.logger.verbose("cloud discovery metadata was successfully returned from getCloudDiscoveryMetadataFromNetwork()"),metadataEntity.updateCloudDiscoveryMetadata(metadata,!0),[2,AuthorityMetadataSource.NETWORK];if(this.logger.verbose("Did not find cloud discovery metadata from the network... Attempting to get cloud discovery metadata from hardcoded values."),(harcodedMetadata=this.getCloudDiscoveryMetadataFromHarcodedValues())&&!this.options.skipAuthorityMetadataCache)return this.logger.verbose("Found cloud discovery metadata from hardcoded values."),metadataEntity.updateCloudDiscoveryMetadata(harcodedMetadata,!1),[2,AuthorityMetadataSource.HARDCODED_VALUES];throw this.logger.error("Did not find cloud discovery metadata from hardcoded values... Metadata could not be obtained from config, cache, network or hardcoded values. Throwing Untrusted Authority Error."),ClientConfigurationError.createUntrustedAuthorityError()}}))}))},Authority.prototype.getCloudDiscoveryMetadataFromConfig=function(){if(this.authorityType===AuthorityType.Ciam)return this.logger.verbose("CIAM authorities do not support cloud discovery metadata, generate the aliases from authority host."),Authority.createCloudDiscoveryMetadataFromHost(this.hostnameAndPort);if(this.authorityOptions.cloudDiscoveryMetadata){this.logger.verbose("The cloud discovery metadata has been provided as a network response, in the config.");try{this.logger.verbose("Attempting to parse the cloud discovery metadata.");var parsedResponse=JSON.parse(this.authorityOptions.cloudDiscoveryMetadata),metadata=Authority.getCloudDiscoveryMetadataFromNetworkResponse(parsedResponse.metadata,this.hostnameAndPort);if(this.logger.verbose("Parsed the cloud discovery metadata."),metadata)return this.logger.verbose("There is returnable metadata attached to the parsed cloud discovery metadata."),metadata;this.logger.verbose("There is no metadata attached to the parsed cloud discovery metadata.")}catch(e){throw this.logger.verbose("Unable to parse the cloud discovery metadata. Throwing Invalid Cloud Discovery Metadata Error."),ClientConfigurationError.createInvalidCloudDiscoveryMetadataError()}}return this.isInKnownAuthorities()?(this.logger.verbose("The host is included in knownAuthorities. Creating new cloud discovery metadata from the host."),Authority.createCloudDiscoveryMetadataFromHost(this.hostnameAndPort)):null},Authority.prototype.getCloudDiscoveryMetadataFromNetwork=function(){var _a;return __awaiter(this,void 0,void 0,(function(){var instanceDiscoveryEndpoint,options,match,response,typedResponseBody,metadata,error_1,typedError;return __generator(this,(function(_b){switch(_b.label){case 0:null===(_a=this.performanceClient)||void 0===_a||_a.addQueueMeasurement(PerformanceEvents.AuthorityGetCloudDiscoveryMetadataFromNetwork,this.correlationId),instanceDiscoveryEndpoint=""+Constants.AAD_INSTANCE_DISCOVERY_ENDPT+this.canonicalAuthority+"oauth2/v2.0/authorize",options={},match=null,_b.label=1;case 1:return _b.trys.push([1,3,,4]),[4,this.networkInterface.sendGetRequestAsync(instanceDiscoveryEndpoint,options)];case 2:if(response=_b.sent(),typedResponseBody=void 0,metadata=void 0,isCloudInstanceDiscoveryResponse(response.body))typedResponseBody=response.body,metadata=typedResponseBody.metadata,this.logger.verbosePii("tenant_discovery_endpoint is: "+typedResponseBody.tenant_discovery_endpoint);else{if(!isCloudInstanceDiscoveryErrorResponse(response.body))return this.logger.error("AAD did not return a CloudInstanceDiscoveryResponse or CloudInstanceDiscoveryErrorResponse"),[2,null];if(this.logger.warning("A CloudInstanceDiscoveryErrorResponse was returned. The cloud instance discovery network request's status code is: "+response.status),(typedResponseBody=response.body).error===Constants.INVALID_INSTANCE)return this.logger.error("The CloudInstanceDiscoveryErrorResponse error is invalid_instance."),[2,null];this.logger.warning("The CloudInstanceDiscoveryErrorResponse error is "+typedResponseBody.error),this.logger.warning("The CloudInstanceDiscoveryErrorResponse error description is "+typedResponseBody.error_description),this.logger.warning("Setting the value of the CloudInstanceDiscoveryMetadata (returned from the network) to []"),metadata=[]}return this.logger.verbose("Attempting to find a match between the developer's authority and the CloudInstanceDiscoveryMetadata returned from the network request."),match=Authority.getCloudDiscoveryMetadataFromNetworkResponse(metadata,this.hostnameAndPort),[3,4];case 3:return(error_1=_b.sent())instanceof AuthError?this.logger.error("There was a network error while attempting to get the cloud discovery instance metadata.\nError: "+error_1.errorCode+"\nError Description: "+error_1.errorMessage):(typedError=error_1,this.logger.error("A non-MSALJS error was thrown while attempting to get the cloud instance discovery metadata.\nError: "+typedError.name+"\nError Description: "+typedError.message)),[2,null];case 4:return match||(this.logger.warning("The developer's authority was not found within the CloudInstanceDiscoveryMetadata returned from the network request."),this.logger.verbose("Creating custom Authority for custom domain scenario."),match=Authority.createCloudDiscoveryMetadataFromHost(this.hostnameAndPort)),[2,match]}}))}))},Authority.prototype.getCloudDiscoveryMetadataFromHarcodedValues=function(){return this.canonicalAuthority in InstanceDiscoveryMetadata?InstanceDiscoveryMetadata[this.canonicalAuthority]:null},Authority.prototype.isInKnownAuthorities=function(){var _this=this;return this.authorityOptions.knownAuthorities.filter((function(authority){return UrlString.getDomainFromUrl(authority).toLowerCase()===_this.hostnameAndPort})).length>0},Authority.generateAuthority=function(authorityString,azureCloudOptions){var authorityAzureCloudInstance;if(azureCloudOptions&&azureCloudOptions.azureCloudInstance!==AzureCloudInstance.None){var tenant=azureCloudOptions.tenant?azureCloudOptions.tenant:Constants.DEFAULT_COMMON_TENANT;authorityAzureCloudInstance=azureCloudOptions.azureCloudInstance+"/"+tenant+"/"}return authorityAzureCloudInstance||authorityString},Authority.createCloudDiscoveryMetadataFromHost=function(host){return{preferred_network:host,preferred_cache:host,aliases:[host]}},Authority.getCloudDiscoveryMetadataFromNetworkResponse=function(response,authority){for(var i=0;i<response.length;i++){var metadata=response[i];if(metadata.aliases.indexOf(authority)>-1)return metadata}return null},Authority.prototype.getPreferredCache=function(){if(this.discoveryComplete())return this.metadata.preferred_cache;throw ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},Authority.prototype.isAlias=function(host){return this.metadata.aliases.indexOf(host)>-1},Authority.isPublicCloudAuthority=function(host){return Constants.KNOWN_PUBLIC_CLOUDS.indexOf(host)>=0},Authority.buildRegionalAuthorityString=function(host,region,queryString){var authorityUrlInstance=new UrlString(host);authorityUrlInstance.validateAsUri();var authorityUrlParts=authorityUrlInstance.getUrlComponents(),hostNameAndPort=region+"."+authorityUrlParts.HostNameAndPort;this.isPublicCloudAuthority(authorityUrlParts.HostNameAndPort)&&(hostNameAndPort=region+"."+Constants.REGIONAL_AUTH_PUBLIC_CLOUD_SUFFIX);var url=UrlString.constructAuthorityUriFromObject(__assign(__assign({},authorityUrlInstance.getUrlComponents()),{HostNameAndPort:hostNameAndPort})).urlString;return queryString?url+"?"+queryString:url},Authority.replaceWithRegionalInformation=function(metadata,azureRegion){return metadata.authorization_endpoint=Authority.buildRegionalAuthorityString(metadata.authorization_endpoint,azureRegion),metadata.token_endpoint=Authority.buildRegionalAuthorityString(metadata.token_endpoint,azureRegion,Constants.REGIONAL_AUTH_NON_MSI_QUERY_STRING),metadata.end_session_endpoint&&(metadata.end_session_endpoint=Authority.buildRegionalAuthorityString(metadata.end_session_endpoint,azureRegion)),metadata},Authority.transformCIAMAuthority=function(authority){var ciamAuthority=authority.endsWith(Constants.FORWARD_SLASH)?authority:""+authority+Constants.FORWARD_SLASH,authorityUrlComponents=new UrlString(authority).getUrlComponents();0===authorityUrlComponents.PathSegments.length&&authorityUrlComponents.HostNameAndPort.endsWith(Constants.CIAM_AUTH_URL)&&(ciamAuthority=""+ciamAuthority+authorityUrlComponents.HostNameAndPort.split(".")[0]+Constants.AAD_TENANT_DOMAIN_SUFFIX);return ciamAuthority},Authority.reservedTenantDomains=new Set(["{tenant}","{tenantid}",AADAuthorityConstants.COMMON,AADAuthorityConstants.CONSUMERS,AADAuthorityConstants.ORGANIZATIONS]),Authority}();export{Authority};