/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{abstractMethodFail,requireNonNull}from"./assert";import{Instant}from"./Instant";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";export class Clock{static systemUTC(){return new SystemClock(ZoneOffset.UTC)}static systemDefaultZone(){return new SystemClock(ZoneId.systemDefault())}static system(zone){return new SystemClock(zone)}static fixed(fixedInstant,zoneId){return new FixedClock(fixedInstant,zoneId)}static offset(baseClock,duration){return new OffsetClock(baseClock,duration)}millis(){abstractMethodFail("Clock.millis")}instant(){abstractMethodFail("Clock.instant")}zone(){abstractMethodFail("Clock.zone")}withZone(){abstractMethodFail("Clock.withZone")}}class SystemClock extends Clock{constructor(zone){requireNonNull(zone,"zone"),super(),this._zone=zone}zone(){return this._zone}millis(){return(new Date).getTime()}instant(){return Instant.ofEpochMilli(this.millis())}equals(obj){return obj instanceof SystemClock&&this._zone.equals(obj._zone)}withZone(zone){return zone.equals(this._zone)?this:new SystemClock(zone)}toString(){return`SystemClock[${this._zone.toString()}]`}}class FixedClock extends Clock{constructor(fixedInstant,zoneId){super(),this._instant=fixedInstant,this._zoneId=zoneId}instant(){return this._instant}millis(){return this._instant.toEpochMilli()}zone(){return this._zoneId}toString(){return"FixedClock[]"}equals(obj){return obj instanceof FixedClock&&(this._instant.equals(obj._instant)&&this._zoneId.equals(obj._zoneId))}withZone(zone){return zone.equals(this._zoneId)?this:new FixedClock(this._instant,zone)}}class OffsetClock extends Clock{constructor(baseClock,offset){super(),this._baseClock=baseClock,this._offset=offset}zone(){return this._baseClock.zone()}withZone(zone){return zone.equals(this._baseClock.zone())?this:new OffsetClock(this._baseClock.withZone(zone),this._offset)}millis(){return this._baseClock.millis()+this._offset.toMillis()}instant(){return this._baseClock.instant().plus(this._offset)}equals(obj){return obj instanceof OffsetClock&&(this._baseClock.equals(obj._baseClock)&&this._offset.equals(obj._offset))}toString(){return`OffsetClock[${this._baseClock},${this._offset}]`}}