/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull,requireInstance}from"./assert";import{ArithmeticException,DateTimeParseException,UnsupportedTemporalTypeException}from"./errors";import{MathUtil,MAX_SAFE_INTEGER,MIN_SAFE_INTEGER}from"./MathUtil";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{TemporalAmount}from"./temporal/TemporalAmount";import{TemporalUnit}from"./temporal/TemporalUnit";import{LocalTime}from"./LocalTime";export class Duration extends TemporalAmount{constructor(seconds,nanos){super(),this._seconds=MathUtil.safeToInt(seconds),this._nanos=MathUtil.safeToInt(nanos)}static ofDays(days){return Duration._create(MathUtil.safeMultiply(days,LocalTime.SECONDS_PER_DAY),0)}static ofHours(hours){return Duration._create(MathUtil.safeMultiply(hours,LocalTime.SECONDS_PER_HOUR),0)}static ofMinutes(minutes){return Duration._create(MathUtil.safeMultiply(minutes,LocalTime.SECONDS_PER_MINUTE),0)}static ofSeconds(seconds,nanoAdjustment=0){const secs=MathUtil.safeAdd(seconds,MathUtil.floorDiv(nanoAdjustment,LocalTime.NANOS_PER_SECOND)),nos=MathUtil.floorMod(nanoAdjustment,LocalTime.NANOS_PER_SECOND);return Duration._create(secs,nos)}static ofMillis(millis){let secs=MathUtil.intDiv(millis,1e3),mos=MathUtil.intMod(millis,1e3);return mos<0&&(mos+=1e3,secs--),Duration._create(secs,1e6*mos)}static ofNanos(nanos){let secs=MathUtil.intDiv(nanos,LocalTime.NANOS_PER_SECOND),nos=MathUtil.intMod(nanos,LocalTime.NANOS_PER_SECOND);return nos<0&&(nos+=LocalTime.NANOS_PER_SECOND,secs--),this._create(secs,nos)}static of(amount,unit){return Duration.ZERO.plus(amount,unit)}static from(amount){requireNonNull(amount,"amount"),requireInstance(amount,TemporalAmount);let duration=Duration.ZERO;return amount.units().forEach((unit=>{duration=duration.plus(amount.get(unit),unit)})),duration}static between(startInclusive,endExclusive){requireNonNull(startInclusive,"startInclusive"),requireNonNull(endExclusive,"endExclusive");let secs=startInclusive.until(endExclusive,ChronoUnit.SECONDS),nanos=0;if(startInclusive.isSupported(ChronoField.NANO_OF_SECOND)&&endExclusive.isSupported(ChronoField.NANO_OF_SECOND))try{const startNos=startInclusive.getLong(ChronoField.NANO_OF_SECOND);if(nanos=endExclusive.getLong(ChronoField.NANO_OF_SECOND)-startNos,secs>0&&nanos<0)nanos+=LocalTime.NANOS_PER_SECOND;else if(secs<0&&nanos>0)nanos-=LocalTime.NANOS_PER_SECOND;else if(0===secs&&0!==nanos){const adjustedEnd=endExclusive.with(ChronoField.NANO_OF_SECOND,startNos);secs=startInclusive.until(adjustedEnd,ChronoUnit.SECONDS)}}catch(e){}return this.ofSeconds(secs,nanos)}static parse(text){requireNonNull(text,"text");const matches=new RegExp("([-+]?)P(?:([-+]?[0-9]+)D)?(T(?:([-+]?[0-9]+)H)?(?:([-+]?[0-9]+)M)?(?:([-+]?[0-9]+)(?:[.,]([0-9]{0,9}))?S)?)?","i").exec(text);if(null!==matches&&"T"===matches[3]==!1){const negate="-"===matches[1],dayMatch=matches[2],hourMatch=matches[4],minuteMatch=matches[5],secondMatch=matches[6],fractionMatch=matches[7];if(null!=dayMatch||null!=hourMatch||null!=minuteMatch||null!=secondMatch){const daysAsSecs=Duration._parseNumber(text,dayMatch,LocalTime.SECONDS_PER_DAY,"days"),hoursAsSecs=Duration._parseNumber(text,hourMatch,LocalTime.SECONDS_PER_HOUR,"hours"),minsAsSecs=Duration._parseNumber(text,minuteMatch,LocalTime.SECONDS_PER_MINUTE,"minutes"),seconds=Duration._parseNumber(text,secondMatch,1,"seconds"),negativeSecs=null!=secondMatch&&"-"===secondMatch.charAt(0),nanos=Duration._parseFraction(text,fractionMatch,negativeSecs?-1:1);try{return Duration._create(negate,daysAsSecs,hoursAsSecs,minsAsSecs,seconds,nanos)}catch(ex){throw new DateTimeParseException("Text cannot be parsed to a Duration: overflow",text,0,ex)}}}throw new DateTimeParseException("Text cannot be parsed to a Duration",text,0)}static _parseNumber(text,parsed,multiplier,errorText){if(null==parsed)return 0;try{return"+"===parsed[0]&&(parsed=parsed.substring(1)),MathUtil.safeMultiply(parseFloat(parsed),multiplier)}catch(ex){throw new DateTimeParseException(`Text cannot be parsed to a Duration: ${errorText}`,text,0,ex)}}static _parseFraction(text,parsed,negate){return null==parsed||0===parsed.length?0:(parsed=`${parsed}000000000`.substring(0,9),parseFloat(parsed)*negate)}static _create(){return arguments.length<=2?Duration._createSecondsNanos(arguments[0],arguments[1]):Duration._createNegateDaysHoursMinutesSecondsNanos(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],arguments[5])}static _createNegateDaysHoursMinutesSecondsNanos(negate,daysAsSecs,hoursAsSecs,minsAsSecs,secs,nanos){const seconds=MathUtil.safeAdd(daysAsSecs,MathUtil.safeAdd(hoursAsSecs,MathUtil.safeAdd(minsAsSecs,secs)));return negate?Duration.ofSeconds(seconds,nanos).negated():Duration.ofSeconds(seconds,nanos)}static _createSecondsNanos(seconds=0,nanoAdjustment=0){return 0===seconds&&0===nanoAdjustment?Duration.ZERO:new Duration(seconds,nanoAdjustment)}get(unit){if(unit===ChronoUnit.SECONDS)return this._seconds;if(unit===ChronoUnit.NANOS)return this._nanos;throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}units(){return[ChronoUnit.SECONDS,ChronoUnit.NANOS]}isZero(){return 0===this._seconds&&0===this._nanos}isNegative(){return this._seconds<0}seconds(){return this._seconds}nano(){return this._nanos}withSeconds(seconds){return Duration._create(seconds,this._nanos)}withNanos(nanoOfSecond){return ChronoField.NANO_OF_SECOND.checkValidIntValue(nanoOfSecond),Duration._create(this._seconds,nanoOfSecond)}plusDuration(duration){return requireNonNull(duration,"duration"),this.plus(duration.seconds(),duration.nano())}plus(durationOrNumber,unitOrNumber){return 1===arguments.length?this.plusDuration(durationOrNumber):2===arguments.length&&unitOrNumber instanceof TemporalUnit?this.plusAmountUnit(durationOrNumber,unitOrNumber):this.plusSecondsNanos(durationOrNumber,unitOrNumber)}plusAmountUnit(amountToAdd,unit){if(requireNonNull(amountToAdd,"amountToAdd"),requireNonNull(unit,"unit"),unit===ChronoUnit.DAYS)return this.plusSecondsNanos(MathUtil.safeMultiply(amountToAdd,LocalTime.SECONDS_PER_DAY),0);if(unit.isDurationEstimated())throw new UnsupportedTemporalTypeException("Unit must not have an estimated duration");if(0===amountToAdd)return this;if(unit instanceof ChronoUnit){switch(unit){case ChronoUnit.NANOS:return this.plusNanos(amountToAdd);case ChronoUnit.MICROS:return this.plusSecondsNanos(1e3*MathUtil.intDiv(amountToAdd,1e9),1e3*MathUtil.intMod(amountToAdd,1e9));case ChronoUnit.MILLIS:return this.plusMillis(amountToAdd);case ChronoUnit.SECONDS:return this.plusSeconds(amountToAdd)}return this.plusSecondsNanos(MathUtil.safeMultiply(unit.duration().seconds(),amountToAdd),0)}const duration=unit.duration().multipliedBy(amountToAdd);return this.plusSecondsNanos(duration.seconds(),duration.nano())}plusDays(daysToAdd){return this.plusSecondsNanos(MathUtil.safeMultiply(daysToAdd,LocalTime.SECONDS_PER_DAY),0)}plusHours(hoursToAdd){return this.plusSecondsNanos(MathUtil.safeMultiply(hoursToAdd,LocalTime.SECONDS_PER_HOUR),0)}plusMinutes(minutesToAdd){return this.plusSecondsNanos(MathUtil.safeMultiply(minutesToAdd,LocalTime.SECONDS_PER_MINUTE),0)}plusSeconds(secondsToAdd){return this.plusSecondsNanos(secondsToAdd,0)}plusMillis(millisToAdd){return this.plusSecondsNanos(MathUtil.intDiv(millisToAdd,1e3),1e6*MathUtil.intMod(millisToAdd,1e3))}plusNanos(nanosToAdd){return this.plusSecondsNanos(0,nanosToAdd)}plusSecondsNanos(secondsToAdd,nanosToAdd){if(requireNonNull(secondsToAdd,"secondsToAdd"),requireNonNull(nanosToAdd,"nanosToAdd"),0===secondsToAdd&&0===nanosToAdd)return this;let epochSec=MathUtil.safeAdd(this._seconds,secondsToAdd);epochSec=MathUtil.safeAdd(epochSec,MathUtil.intDiv(nanosToAdd,LocalTime.NANOS_PER_SECOND)),nanosToAdd=MathUtil.intMod(nanosToAdd,LocalTime.NANOS_PER_SECOND);const nanoAdjustment=MathUtil.safeAdd(this._nanos,nanosToAdd);return Duration.ofSeconds(epochSec,nanoAdjustment)}minus(durationOrNumber,unit){return 1===arguments.length?this.minusDuration(durationOrNumber):this.minusAmountUnit(durationOrNumber,unit)}minusDuration(duration){requireNonNull(duration,"duration");const secsToSubtract=duration.seconds(),nanosToSubtract=duration.nano();return secsToSubtract===MIN_SAFE_INTEGER?this.plus(MAX_SAFE_INTEGER,-nanosToSubtract):this.plus(-secsToSubtract,-nanosToSubtract)}minusAmountUnit(amountToSubtract,unit){return requireNonNull(amountToSubtract,"amountToSubtract"),requireNonNull(unit,"unit"),amountToSubtract===MIN_SAFE_INTEGER?this.plusAmountUnit(MAX_SAFE_INTEGER,unit):this.plusAmountUnit(-amountToSubtract,unit)}minusDays(daysToSubtract){return daysToSubtract===MIN_SAFE_INTEGER?this.plusDays(MAX_SAFE_INTEGER):this.plusDays(-daysToSubtract)}minusHours(hoursToSubtract){return hoursToSubtract===MIN_SAFE_INTEGER?this.plusHours(MAX_SAFE_INTEGER):this.plusHours(-hoursToSubtract)}minusMinutes(minutesToSubtract){return minutesToSubtract===MIN_SAFE_INTEGER?this.plusMinutes(MAX_SAFE_INTEGER):this.plusMinutes(-minutesToSubtract)}minusSeconds(secondsToSubtract){return secondsToSubtract===MIN_SAFE_INTEGER?this.plusSeconds(MAX_SAFE_INTEGER):this.plusSeconds(-secondsToSubtract)}minusMillis(millisToSubtract){return millisToSubtract===MIN_SAFE_INTEGER?this.plusMillis(MAX_SAFE_INTEGER):this.plusMillis(-millisToSubtract)}minusNanos(nanosToSubtract){return nanosToSubtract===MIN_SAFE_INTEGER?this.plusNanos(MAX_SAFE_INTEGER):this.plusNanos(-nanosToSubtract)}multipliedBy(multiplicand){if(0===multiplicand)return Duration.ZERO;if(1===multiplicand)return this;let secs=MathUtil.safeMultiply(this._seconds,multiplicand),nos=MathUtil.safeMultiply(this._nanos,multiplicand);return secs+=MathUtil.intDiv(nos,LocalTime.NANOS_PER_SECOND),nos=MathUtil.intMod(nos,LocalTime.NANOS_PER_SECOND),Duration.ofSeconds(secs,nos)}dividedBy(divisor){if(0===divisor)throw new ArithmeticException("Cannot divide by zero");if(1===divisor)return this;const secs=MathUtil.intDiv(this._seconds,divisor),secsMod=MathUtil.roundDown((this._seconds/divisor-secs)*LocalTime.NANOS_PER_SECOND);let nos=MathUtil.intDiv(this._nanos,divisor);return nos=secsMod+nos,Duration.ofSeconds(secs,nos)}negated(){return this.multipliedBy(-1)}abs(){return this.isNegative()?this.negated():this}addTo(temporal){return requireNonNull(temporal,"temporal"),0!==this._seconds&&(temporal=temporal.plus(this._seconds,ChronoUnit.SECONDS)),0!==this._nanos&&(temporal=temporal.plus(this._nanos,ChronoUnit.NANOS)),temporal}subtractFrom(temporal){return requireNonNull(temporal,"temporal"),0!==this._seconds&&(temporal=temporal.minus(this._seconds,ChronoUnit.SECONDS)),0!==this._nanos&&(temporal=temporal.minus(this._nanos,ChronoUnit.NANOS)),temporal}toDays(){return MathUtil.intDiv(this._seconds,LocalTime.SECONDS_PER_DAY)}toHours(){return MathUtil.intDiv(this._seconds,LocalTime.SECONDS_PER_HOUR)}toMinutes(){return MathUtil.intDiv(this._seconds,LocalTime.SECONDS_PER_MINUTE)}toMillis(){let millis=Math.round(MathUtil.safeMultiply(this._seconds,1e3));return millis=MathUtil.safeAdd(millis,MathUtil.intDiv(this._nanos,1e6)),millis}toNanos(){let totalNanos=MathUtil.safeMultiply(this._seconds,LocalTime.NANOS_PER_SECOND);return totalNanos=MathUtil.safeAdd(totalNanos,this._nanos),totalNanos}compareTo(otherDuration){requireNonNull(otherDuration,"otherDuration"),requireInstance(otherDuration,Duration,"otherDuration");const cmp=MathUtil.compareNumbers(this._seconds,otherDuration.seconds());return 0!==cmp?cmp:this._nanos-otherDuration.nano()}equals(otherDuration){return this===otherDuration||otherDuration instanceof Duration&&(this.seconds()===otherDuration.seconds()&&this.nano()===otherDuration.nano())}toString(){if(this===Duration.ZERO)return"PT0S";const hours=MathUtil.intDiv(this._seconds,LocalTime.SECONDS_PER_HOUR),minutes=MathUtil.intDiv(MathUtil.intMod(this._seconds,LocalTime.SECONDS_PER_HOUR),LocalTime.SECONDS_PER_MINUTE),secs=MathUtil.intMod(this._seconds,LocalTime.SECONDS_PER_MINUTE);let rval="PT";if(0!==hours&&(rval+=`${hours}H`),0!==minutes&&(rval+=`${minutes}M`),0===secs&&0===this._nanos&&rval.length>2)return rval;if(secs<0&&this._nanos>0?rval+=-1===secs?"-0":secs+1:rval+=secs,this._nanos>0){let nanoString;for(rval+=".",nanoString=secs<0?""+(2*LocalTime.NANOS_PER_SECOND-this._nanos):`${LocalTime.NANOS_PER_SECOND+this._nanos}`,nanoString=nanoString.slice(1,nanoString.length),rval+=nanoString;"0"===rval.charAt(rval.length-1);)rval=rval.slice(0,rval.length-1)}return rval+="S",rval}toJSON(){return this.toString()}}export function _init(){Duration.ZERO=new Duration(0,0)}