/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{Clock}from"./Clock";import{LocalTime}from"./LocalTime";import{ZonedDateTime}from"./ZonedDateTime";import{MathUtil}from"./MathUtil";import{OffsetDateTime}from"./OffsetDateTime";import{Temporal}from"./temporal/Temporal";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{TemporalQueries}from"./temporal/TemporalQueries";import{TemporalUnit}from"./temporal/TemporalUnit";import{createTemporalQuery}from"./temporal/TemporalQuery";import{DateTimeFormatter}from"./format/DateTimeFormatter";const NANOS_PER_MILLI=1e6;export class Instant extends Temporal{static now(clock=Clock.systemUTC()){return clock.instant()}static ofEpochSecond(epochSecond,nanoAdjustment=0){const secs=epochSecond+MathUtil.floorDiv(nanoAdjustment,LocalTime.NANOS_PER_SECOND),nos=MathUtil.floorMod(nanoAdjustment,LocalTime.NANOS_PER_SECOND);return Instant._create(secs,nos)}static ofEpochMilli(epochMilli){const secs=MathUtil.floorDiv(epochMilli,1e3),mos=MathUtil.floorMod(epochMilli,1e3);return Instant._create(secs,1e6*mos)}static ofEpochMicro(epochMicro){const secs=MathUtil.floorDiv(epochMicro,1e6),mos=MathUtil.floorMod(epochMicro,1e6);return Instant._create(secs,1e3*mos)}static from(temporal){try{const instantSecs=temporal.getLong(ChronoField.INSTANT_SECONDS),nanoOfSecond=temporal.get(ChronoField.NANO_OF_SECOND);return Instant.ofEpochSecond(instantSecs,nanoOfSecond)}catch(ex){throw new DateTimeException(`Unable to obtain Instant from TemporalAccessor: ${temporal}, type ${typeof temporal}`,ex)}}static parse(text){return DateTimeFormatter.ISO_INSTANT.parse(text,Instant.FROM)}static _create(seconds,nanoOfSecond){return 0===seconds&&0===nanoOfSecond?Instant.EPOCH:new Instant(seconds,nanoOfSecond)}static _validate(seconds,nanoOfSecond){if(seconds<Instant.MIN_SECONDS||seconds>Instant.MAX_SECONDS)throw new DateTimeException("Instant exceeds minimum or maximum instant");if(nanoOfSecond<0||nanoOfSecond>LocalTime.NANOS_PER_SECOND)throw new DateTimeException("Instant exceeds minimum or maximum instant")}constructor(seconds,nanoOfSecond){super(),Instant._validate(seconds,nanoOfSecond),this._seconds=MathUtil.safeToInt(seconds),this._nanos=MathUtil.safeToInt(nanoOfSecond)}isSupported(fieldOrUnit){return fieldOrUnit instanceof ChronoField?fieldOrUnit===ChronoField.INSTANT_SECONDS||fieldOrUnit===ChronoField.NANO_OF_SECOND||fieldOrUnit===ChronoField.MICRO_OF_SECOND||fieldOrUnit===ChronoField.MILLI_OF_SECOND:fieldOrUnit instanceof ChronoUnit?fieldOrUnit.isTimeBased()||fieldOrUnit===ChronoUnit.DAYS:null!=fieldOrUnit&&fieldOrUnit.isSupportedBy(this)}range(field){return super.range(field)}get(field){return this.getLong(field)}getLong(field){if(field instanceof ChronoField){switch(field){case ChronoField.NANO_OF_SECOND:return this._nanos;case ChronoField.MICRO_OF_SECOND:return MathUtil.intDiv(this._nanos,1e3);case ChronoField.MILLI_OF_SECOND:return MathUtil.intDiv(this._nanos,NANOS_PER_MILLI);case ChronoField.INSTANT_SECONDS:return this._seconds}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}return field.getFrom(this)}epochSecond(){return this._seconds}nano(){return this._nanos}_withField(field,newValue){if(requireNonNull(field,"field"),field instanceof ChronoField){switch(field.checkValidValue(newValue),field){case ChronoField.MILLI_OF_SECOND:{const nval=newValue*NANOS_PER_MILLI;return nval!==this._nanos?Instant._create(this._seconds,nval):this}case ChronoField.MICRO_OF_SECOND:{const nval=1e3*newValue;return nval!==this._nanos?Instant._create(this._seconds,nval):this}case ChronoField.NANO_OF_SECOND:return newValue!==this._nanos?Instant._create(this._seconds,newValue):this;case ChronoField.INSTANT_SECONDS:return newValue!==this._seconds?Instant._create(newValue,this._nanos):this}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}return field.adjustInto(this,newValue)}truncatedTo(unit){if(requireNonNull(unit,"unit"),unit===ChronoUnit.NANOS)return this;const unitDur=unit.duration();if(unitDur.seconds()>LocalTime.SECONDS_PER_DAY)throw new DateTimeException("Unit is too large to be used for truncation");const dur=unitDur.toNanos();if(0!==MathUtil.intMod(LocalTime.NANOS_PER_DAY,dur))throw new DateTimeException("Unit must divide into a standard day without remainder");const nod=MathUtil.intMod(this._seconds,LocalTime.SECONDS_PER_DAY)*LocalTime.NANOS_PER_SECOND+this._nanos,result=MathUtil.intDiv(nod,dur)*dur;return this.plusNanos(result-nod)}_plusUnit(amountToAdd,unit){if(requireNonNull(amountToAdd,"amountToAdd"),requireNonNull(unit,"unit"),requireInstance(unit,TemporalUnit),unit instanceof ChronoUnit){switch(unit){case ChronoUnit.NANOS:return this.plusNanos(amountToAdd);case ChronoUnit.MICROS:return this.plusMicros(amountToAdd);case ChronoUnit.MILLIS:return this.plusMillis(amountToAdd);case ChronoUnit.SECONDS:return this.plusSeconds(amountToAdd);case ChronoUnit.MINUTES:return this.plusSeconds(MathUtil.safeMultiply(amountToAdd,LocalTime.SECONDS_PER_MINUTE));case ChronoUnit.HOURS:return this.plusSeconds(MathUtil.safeMultiply(amountToAdd,LocalTime.SECONDS_PER_HOUR));case ChronoUnit.HALF_DAYS:return this.plusSeconds(MathUtil.safeMultiply(amountToAdd,LocalTime.SECONDS_PER_DAY/2));case ChronoUnit.DAYS:return this.plusSeconds(MathUtil.safeMultiply(amountToAdd,LocalTime.SECONDS_PER_DAY))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.addTo(this,amountToAdd)}plusSeconds(secondsToAdd){return this._plus(secondsToAdd,0)}plusMillis(millisToAdd){return this._plus(MathUtil.intDiv(millisToAdd,1e3),MathUtil.intMod(millisToAdd,1e3)*NANOS_PER_MILLI)}plusNanos(nanosToAdd){return this._plus(0,nanosToAdd)}plusMicros(microsToAdd){return this._plus(MathUtil.intDiv(microsToAdd,1e6),1e3*MathUtil.intMod(microsToAdd,1e6))}_plus(secondsToAdd,nanosToAdd){if(0===secondsToAdd&&0===nanosToAdd)return this;let epochSec=this._seconds+secondsToAdd;epochSec+=MathUtil.intDiv(nanosToAdd,LocalTime.NANOS_PER_SECOND);const nanoAdjustment=this._nanos+nanosToAdd%LocalTime.NANOS_PER_SECOND;return Instant.ofEpochSecond(epochSec,nanoAdjustment)}_minusUnit(amountToSubtract,unit){return this._plusUnit(-1*amountToSubtract,unit)}minusSeconds(secondsToSubtract){return this.plusSeconds(-1*secondsToSubtract)}minusMillis(millisToSubtract){return this.plusMillis(-1*millisToSubtract)}minusNanos(nanosToSubtract){return this.plusNanos(-1*nanosToSubtract)}minusMicros(microsToSubtract){return this.plusMicros(-1*microsToSubtract)}query(query){return requireNonNull(query,"query"),query===TemporalQueries.precision()?ChronoUnit.NANOS:query===TemporalQueries.localDate()||query===TemporalQueries.localTime()||query===TemporalQueries.chronology()||query===TemporalQueries.zoneId()||query===TemporalQueries.zone()||query===TemporalQueries.offset()?null:query.queryFrom(this)}adjustInto(temporal){return requireNonNull(temporal,"temporal"),temporal.with(ChronoField.INSTANT_SECONDS,this._seconds).with(ChronoField.NANO_OF_SECOND,this._nanos)}until(endExclusive,unit){requireNonNull(endExclusive,"endExclusive"),requireNonNull(unit,"unit");const end=Instant.from(endExclusive);if(unit instanceof ChronoUnit){switch(unit){case ChronoUnit.NANOS:return this._nanosUntil(end);case ChronoUnit.MICROS:return this._microsUntil(end);case ChronoUnit.MILLIS:return MathUtil.safeSubtract(end.toEpochMilli(),this.toEpochMilli());case ChronoUnit.SECONDS:return this._secondsUntil(end);case ChronoUnit.MINUTES:return MathUtil.intDiv(this._secondsUntil(end),LocalTime.SECONDS_PER_MINUTE);case ChronoUnit.HOURS:return MathUtil.intDiv(this._secondsUntil(end),LocalTime.SECONDS_PER_HOUR);case ChronoUnit.HALF_DAYS:return MathUtil.intDiv(this._secondsUntil(end),12*LocalTime.SECONDS_PER_HOUR);case ChronoUnit.DAYS:return MathUtil.intDiv(this._secondsUntil(end),LocalTime.SECONDS_PER_DAY)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.between(this,end)}_microsUntil(end){const secsDiff=MathUtil.safeSubtract(end.epochSecond(),this.epochSecond()),totalMicros=MathUtil.safeMultiply(secsDiff,1e6);return MathUtil.safeAdd(totalMicros,MathUtil.intDiv(end.nano()-this.nano(),1e3))}_nanosUntil(end){const secsDiff=MathUtil.safeSubtract(end.epochSecond(),this.epochSecond()),totalNanos=MathUtil.safeMultiply(secsDiff,LocalTime.NANOS_PER_SECOND);return MathUtil.safeAdd(totalNanos,end.nano()-this.nano())}_secondsUntil(end){let secsDiff=MathUtil.safeSubtract(end.epochSecond(),this.epochSecond());const nanosDiff=end.nano()-this.nano();return secsDiff>0&&nanosDiff<0?secsDiff--:secsDiff<0&&nanosDiff>0&&secsDiff++,secsDiff}atOffset(offset){return OffsetDateTime.ofInstant(this,offset)}atZone(zone){return ZonedDateTime.ofInstant(this,zone)}toEpochMilli(){return MathUtil.safeMultiply(this._seconds,1e3)+MathUtil.intDiv(this._nanos,NANOS_PER_MILLI)}compareTo(otherInstant){requireNonNull(otherInstant,"otherInstant"),requireInstance(otherInstant,Instant,"otherInstant");const cmp=MathUtil.compareNumbers(this._seconds,otherInstant._seconds);return 0!==cmp?cmp:this._nanos-otherInstant._nanos}isAfter(otherInstant){return this.compareTo(otherInstant)>0}isBefore(otherInstant){return this.compareTo(otherInstant)<0}equals(other){return this===other||other instanceof Instant&&(this.epochSecond()===other.epochSecond()&&this.nano()===other.nano())}hashCode(){return MathUtil.hashCode(this._seconds,this._nanos)}toString(){return DateTimeFormatter.ISO_INSTANT.format(this)}toJSON(){return this.toString()}}export function _init(){Instant.MIN_SECONDS=-31619119219200,Instant.MAX_SECONDS=31494816403199,Instant.EPOCH=new Instant(0,0),Instant.MIN=Instant.ofEpochSecond(Instant.MIN_SECONDS,0),Instant.MAX=Instant.ofEpochSecond(Instant.MAX_SECONDS,999999999),Instant.FROM=createTemporalQuery("Instant.FROM",(temporal=>Instant.from(temporal)))}