/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{MathUtil}from"./MathUtil";import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{Clock}from"./Clock";import{Instant}from"./Instant";import{LocalDate}from"./LocalDate";import{LocalTime}from"./LocalTime";import{OffsetDateTime}from"./OffsetDateTime";import{ZonedDateTime}from"./ZonedDateTime";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{TemporalQueries}from"./temporal/TemporalQueries";import{createTemporalQuery}from"./temporal/TemporalQuery";import{ChronoLocalDateTime}from"./chrono/ChronoLocalDateTime";export class LocalDateTime extends ChronoLocalDateTime{static now(clockOrZone){return null==clockOrZone?LocalDateTime._now(Clock.systemDefaultZone()):clockOrZone instanceof Clock?LocalDateTime._now(clockOrZone):LocalDateTime._now(Clock.system(clockOrZone))}static _now(clock){return requireNonNull(clock,"clock"),LocalDateTime.ofInstant(clock.instant(),clock.zone())}static _ofEpochMillis(epochMilli,offset){const localSecond=MathUtil.floorDiv(epochMilli,1e3)+offset.totalSeconds(),localEpochDay=MathUtil.floorDiv(localSecond,LocalTime.SECONDS_PER_DAY),secsOfDay=MathUtil.floorMod(localSecond,LocalTime.SECONDS_PER_DAY),nanoOfSecond=1e6*MathUtil.floorMod(epochMilli,1e3),date=LocalDate.ofEpochDay(localEpochDay),time=LocalTime.ofSecondOfDay(secsOfDay,nanoOfSecond);return new LocalDateTime(date,time)}static of(){return arguments.length<=2?LocalDateTime.ofDateAndTime.apply(this,arguments):LocalDateTime.ofNumbers.apply(this,arguments)}static ofNumbers(year,month,dayOfMonth,hour=0,minute=0,second=0,nanoOfSecond=0){const date=LocalDate.of(year,month,dayOfMonth),time=LocalTime.of(hour,minute,second,nanoOfSecond);return new LocalDateTime(date,time)}static ofDateAndTime(date,time){return requireNonNull(date,"date"),requireNonNull(time,"time"),new LocalDateTime(date,time)}static ofInstant(instant,zone=ZoneId.systemDefault()){requireNonNull(instant,"instant"),requireInstance(instant,Instant,"instant"),requireNonNull(zone,"zone");const offset=zone.rules().offset(instant);return LocalDateTime.ofEpochSecond(instant.epochSecond(),instant.nano(),offset)}static ofEpochSecond(epochSecond=0,nanoOfSecond=0,offset){2===arguments.length&&nanoOfSecond instanceof ZoneOffset&&(offset=nanoOfSecond,nanoOfSecond=0),requireNonNull(offset,"offset");const localSecond=epochSecond+offset.totalSeconds(),localEpochDay=MathUtil.floorDiv(localSecond,LocalTime.SECONDS_PER_DAY),secsOfDay=MathUtil.floorMod(localSecond,LocalTime.SECONDS_PER_DAY),date=LocalDate.ofEpochDay(localEpochDay),time=LocalTime.ofSecondOfDay(secsOfDay,nanoOfSecond);return new LocalDateTime(date,time)}static from(temporal){if(requireNonNull(temporal,"temporal"),temporal instanceof LocalDateTime)return temporal;if(temporal instanceof ZonedDateTime)return temporal.toLocalDateTime();try{const date=LocalDate.from(temporal),time=LocalTime.from(temporal);return new LocalDateTime(date,time)}catch(ex){throw new DateTimeException(`Unable to obtain LocalDateTime TemporalAccessor: ${temporal}, type ${null!=temporal.constructor?temporal.constructor.name:""}`)}}static parse(text,formatter=DateTimeFormatter.ISO_LOCAL_DATE_TIME){return requireNonNull(formatter,"formatter"),formatter.parse(text,LocalDateTime.FROM)}constructor(date,time){super(),requireInstance(date,LocalDate,"date"),requireInstance(time,LocalTime,"time"),this._date=date,this._time=time}_withDateTime(newDate,newTime){return this._date.equals(newDate)&&this._time.equals(newTime)?this:new LocalDateTime(newDate,newTime)}isSupported(fieldOrUnit){return fieldOrUnit instanceof ChronoField||fieldOrUnit instanceof ChronoUnit?fieldOrUnit.isDateBased()||fieldOrUnit.isTimeBased():null!=fieldOrUnit&&fieldOrUnit.isSupportedBy(this)}range(field){return field instanceof ChronoField?field.isTimeBased()?this._time.range(field):this._date.range(field):field.rangeRefinedBy(this)}get(field){return field instanceof ChronoField?field.isTimeBased()?this._time.get(field):this._date.get(field):super.get(field)}getLong(field){return requireNonNull(field,"field"),field instanceof ChronoField?field.isTimeBased()?this._time.getLong(field):this._date.getLong(field):field.getFrom(this)}year(){return this._date.year()}monthValue(){return this._date.monthValue()}month(){return this._date.month()}dayOfMonth(){return this._date.dayOfMonth()}dayOfYear(){return this._date.dayOfYear()}dayOfWeek(){return this._date.dayOfWeek()}hour(){return this._time.hour()}minute(){return this._time.minute()}second(){return this._time.second()}nano(){return this._time.nano()}_withAdjuster(adjuster){return requireNonNull(adjuster,"adjuster"),adjuster instanceof LocalDate?this._withDateTime(adjuster,this._time):adjuster instanceof LocalTime?this._withDateTime(this._date,adjuster):adjuster instanceof LocalDateTime?adjuster:super._withAdjuster(adjuster)}_withField(field,newValue){return requireNonNull(field,"field"),field instanceof ChronoField?field.isTimeBased()?this._withDateTime(this._date,this._time.with(field,newValue)):this._withDateTime(this._date.with(field,newValue),this._time):field.adjustInto(this,newValue)}withYear(year){return this._withDateTime(this._date.withYear(year),this._time)}withMonth(month){return this._withDateTime(this._date.withMonth(month),this._time)}withDayOfMonth(dayOfMonth){return this._withDateTime(this._date.withDayOfMonth(dayOfMonth),this._time)}withDayOfYear(dayOfYear){return this._withDateTime(this._date.withDayOfYear(dayOfYear),this._time)}withHour(hour){const newTime=this._time.withHour(hour);return this._withDateTime(this._date,newTime)}withMinute(minute){const newTime=this._time.withMinute(minute);return this._withDateTime(this._date,newTime)}withSecond(second){const newTime=this._time.withSecond(second);return this._withDateTime(this._date,newTime)}withNano(nanoOfSecond){const newTime=this._time.withNano(nanoOfSecond);return this._withDateTime(this._date,newTime)}truncatedTo(unit){return this._withDateTime(this._date,this._time.truncatedTo(unit))}_plusUnit(amountToAdd,unit){if(requireNonNull(unit,"unit"),unit instanceof ChronoUnit){switch(unit){case ChronoUnit.NANOS:return this.plusNanos(amountToAdd);case ChronoUnit.MICROS:return this.plusDays(MathUtil.intDiv(amountToAdd,LocalTime.MICROS_PER_DAY)).plusNanos(1e3*MathUtil.intMod(amountToAdd,LocalTime.MICROS_PER_DAY));case ChronoUnit.MILLIS:return this.plusDays(MathUtil.intDiv(amountToAdd,LocalTime.MILLIS_PER_DAY)).plusNanos(1e6*MathUtil.intMod(amountToAdd,LocalTime.MILLIS_PER_DAY));case ChronoUnit.SECONDS:return this.plusSeconds(amountToAdd);case ChronoUnit.MINUTES:return this.plusMinutes(amountToAdd);case ChronoUnit.HOURS:return this.plusHours(amountToAdd);case ChronoUnit.HALF_DAYS:return this.plusDays(MathUtil.intDiv(amountToAdd,256)).plusHours(12*MathUtil.intMod(amountToAdd,256))}return this._withDateTime(this._date.plus(amountToAdd,unit),this._time)}return unit.addTo(this,amountToAdd)}plusYears(years){const newDate=this._date.plusYears(years);return this._withDateTime(newDate,this._time)}plusMonths(months){const newDate=this._date.plusMonths(months);return this._withDateTime(newDate,this._time)}plusWeeks(weeks){const newDate=this._date.plusWeeks(weeks);return this._withDateTime(newDate,this._time)}plusDays(days){const newDate=this._date.plusDays(days);return this._withDateTime(newDate,this._time)}plusHours(hours){return this._plusWithOverflow(this._date,hours,0,0,0,1)}plusMinutes(minutes){return this._plusWithOverflow(this._date,0,minutes,0,0,1)}plusSeconds(seconds){return this._plusWithOverflow(this._date,0,0,seconds,0,1)}plusNanos(nanos){return this._plusWithOverflow(this._date,0,0,0,nanos,1)}_minusUnit(amountToSubtract,unit){return requireNonNull(unit,"unit"),this._plusUnit(-1*amountToSubtract,unit)}minusYears(years){return this.plusYears(-1*years)}minusMonths(months){return this.plusMonths(-1*months)}minusWeeks(weeks){return this.plusWeeks(-1*weeks)}minusDays(days){return this.plusDays(-1*days)}minusHours(hours){return this._plusWithOverflow(this._date,hours,0,0,0,-1)}minusMinutes(minutes){return this._plusWithOverflow(this._date,0,minutes,0,0,-1)}minusSeconds(seconds){return this._plusWithOverflow(this._date,0,0,seconds,0,-1)}minusNanos(nanos){return this._plusWithOverflow(this._date,0,0,0,nanos,-1)}_plusWithOverflow(newDate,hours,minutes,seconds,nanos,sign){if(0===hours&&0===minutes&&0===seconds&&0===nanos)return this._withDateTime(newDate,this._time);let totDays=MathUtil.intDiv(nanos,LocalTime.NANOS_PER_DAY)+MathUtil.intDiv(seconds,LocalTime.SECONDS_PER_DAY)+MathUtil.intDiv(minutes,LocalTime.MINUTES_PER_DAY)+MathUtil.intDiv(hours,LocalTime.HOURS_PER_DAY);totDays*=sign;let totNanos=MathUtil.intMod(nanos,LocalTime.NANOS_PER_DAY)+MathUtil.intMod(seconds,LocalTime.SECONDS_PER_DAY)*LocalTime.NANOS_PER_SECOND+MathUtil.intMod(minutes,LocalTime.MINUTES_PER_DAY)*LocalTime.NANOS_PER_MINUTE+MathUtil.intMod(hours,LocalTime.HOURS_PER_DAY)*LocalTime.NANOS_PER_HOUR;const curNoD=this._time.toNanoOfDay();totNanos=totNanos*sign+curNoD,totDays+=MathUtil.floorDiv(totNanos,LocalTime.NANOS_PER_DAY);const newNoD=MathUtil.floorMod(totNanos,LocalTime.NANOS_PER_DAY),newTime=newNoD===curNoD?this._time:LocalTime.ofNanoOfDay(newNoD);return this._withDateTime(newDate.plusDays(totDays),newTime)}query(query){return requireNonNull(query,"query"),query===TemporalQueries.localDate()?this.toLocalDate():super.query(query)}adjustInto(temporal){return super.adjustInto(temporal)}until(endExclusive,unit){requireNonNull(endExclusive,"endExclusive"),requireNonNull(unit,"unit");const end=LocalDateTime.from(endExclusive);if(unit instanceof ChronoUnit){if(unit.isTimeBased()){let daysUntil=this._date.daysUntil(end._date),timeUntil=end._time.toNanoOfDay()-this._time.toNanoOfDay();daysUntil>0&&timeUntil<0?(daysUntil--,timeUntil+=LocalTime.NANOS_PER_DAY):daysUntil<0&&timeUntil>0&&(daysUntil++,timeUntil-=LocalTime.NANOS_PER_DAY);let amount=daysUntil;switch(unit){case ChronoUnit.NANOS:return amount=MathUtil.safeMultiply(amount,LocalTime.NANOS_PER_DAY),MathUtil.safeAdd(amount,timeUntil);case ChronoUnit.MICROS:return amount=MathUtil.safeMultiply(amount,LocalTime.MICROS_PER_DAY),MathUtil.safeAdd(amount,MathUtil.intDiv(timeUntil,1e3));case ChronoUnit.MILLIS:return amount=MathUtil.safeMultiply(amount,LocalTime.MILLIS_PER_DAY),MathUtil.safeAdd(amount,MathUtil.intDiv(timeUntil,1e6));case ChronoUnit.SECONDS:return amount=MathUtil.safeMultiply(amount,LocalTime.SECONDS_PER_DAY),MathUtil.safeAdd(amount,MathUtil.intDiv(timeUntil,LocalTime.NANOS_PER_SECOND));case ChronoUnit.MINUTES:return amount=MathUtil.safeMultiply(amount,LocalTime.MINUTES_PER_DAY),MathUtil.safeAdd(amount,MathUtil.intDiv(timeUntil,LocalTime.NANOS_PER_MINUTE));case ChronoUnit.HOURS:return amount=MathUtil.safeMultiply(amount,LocalTime.HOURS_PER_DAY),MathUtil.safeAdd(amount,MathUtil.intDiv(timeUntil,LocalTime.NANOS_PER_HOUR));case ChronoUnit.HALF_DAYS:return amount=MathUtil.safeMultiply(amount,2),MathUtil.safeAdd(amount,MathUtil.intDiv(timeUntil,12*LocalTime.NANOS_PER_HOUR))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}let endDate=end._date;const endTime=end._time;return endDate.isAfter(this._date)&&endTime.isBefore(this._time)?endDate=endDate.minusDays(1):endDate.isBefore(this._date)&&endTime.isAfter(this._time)&&(endDate=endDate.plusDays(1)),this._date.until(endDate,unit)}return unit.between(this,end)}atOffset(offset){return OffsetDateTime.of(this,offset)}atZone(zone){return ZonedDateTime.of(this,zone)}toLocalDate(){return this._date}toLocalTime(){return this._time}compareTo(other){return requireNonNull(other,"other"),requireInstance(other,LocalDateTime,"other"),this._compareTo0(other)}_compareTo0(other){let cmp=this._date.compareTo(other.toLocalDate());return 0===cmp&&(cmp=this._time.compareTo(other.toLocalTime())),cmp}isAfter(other){return this.compareTo(other)>0}isBefore(other){return this.compareTo(other)<0}isEqual(other){return 0===this.compareTo(other)}equals(other){return this===other||other instanceof LocalDateTime&&(this._date.equals(other._date)&&this._time.equals(other._time))}hashCode(){return this._date.hashCode()^this._time.hashCode()}toString(){return`${this._date.toString()}T${this._time.toString()}`}toJSON(){return this.toString()}format(formatter){return requireNonNull(formatter,"formatter"),formatter.format(this)}}export function _init(){LocalDateTime.MIN=LocalDateTime.of(LocalDate.MIN,LocalTime.MIN),LocalDateTime.MAX=LocalDateTime.of(LocalDate.MAX,LocalTime.MAX),LocalDateTime.FROM=createTemporalQuery("LocalDateTime.FROM",(temporal=>LocalDateTime.from(temporal)))}