/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{MathUtil}from"./MathUtil";import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{Clock}from"./Clock";import{LocalDateTime}from"./LocalDateTime";import{ZoneId}from"./ZoneId";import{OffsetTime}from"./OffsetTime";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{Temporal}from"./temporal/Temporal";import{TemporalField}from"./temporal/TemporalField";import{TemporalQueries}from"./temporal/TemporalQueries";import{createTemporalQuery}from"./temporal/TemporalQuery";export class LocalTime extends Temporal{static now(clockOrZone){return null==clockOrZone?LocalTime._now(Clock.systemDefaultZone()):clockOrZone instanceof Clock?LocalTime._now(clockOrZone):LocalTime._now(Clock.system(clockOrZone))}static _now(clock=Clock.systemDefaultZone()){return requireNonNull(clock,"clock"),LocalTime.ofInstant(clock.instant(),clock.zone())}static ofInstant(instant,zone=ZoneId.systemDefault()){const offset=zone.rules().offset(instant);let secsOfDay=MathUtil.intMod(instant.epochSecond(),LocalTime.SECONDS_PER_DAY);return secsOfDay=MathUtil.intMod(secsOfDay+offset.totalSeconds(),LocalTime.SECONDS_PER_DAY),secsOfDay<0&&(secsOfDay+=LocalTime.SECONDS_PER_DAY),LocalTime.ofSecondOfDay(secsOfDay,instant.nano())}static of(hour,minute,second,nanoOfSecond){return new LocalTime(hour,minute,second,nanoOfSecond)}static ofSecondOfDay(secondOfDay=0,nanoOfSecond=0){ChronoField.SECOND_OF_DAY.checkValidValue(secondOfDay),ChronoField.NANO_OF_SECOND.checkValidValue(nanoOfSecond);const hours=MathUtil.intDiv(secondOfDay,LocalTime.SECONDS_PER_HOUR);secondOfDay-=hours*LocalTime.SECONDS_PER_HOUR;const minutes=MathUtil.intDiv(secondOfDay,LocalTime.SECONDS_PER_MINUTE);return secondOfDay-=minutes*LocalTime.SECONDS_PER_MINUTE,new LocalTime(hours,minutes,secondOfDay,nanoOfSecond)}static ofNanoOfDay(nanoOfDay=0){ChronoField.NANO_OF_DAY.checkValidValue(nanoOfDay);const hours=MathUtil.intDiv(nanoOfDay,LocalTime.NANOS_PER_HOUR);nanoOfDay-=hours*LocalTime.NANOS_PER_HOUR;const minutes=MathUtil.intDiv(nanoOfDay,LocalTime.NANOS_PER_MINUTE);nanoOfDay-=minutes*LocalTime.NANOS_PER_MINUTE;const seconds=MathUtil.intDiv(nanoOfDay,LocalTime.NANOS_PER_SECOND);return nanoOfDay-=seconds*LocalTime.NANOS_PER_SECOND,new LocalTime(hours,minutes,seconds,nanoOfDay)}static from(temporal){requireNonNull(temporal,"temporal");const time=temporal.query(TemporalQueries.localTime());if(null==time)throw new DateTimeException(`Unable to obtain LocalTime TemporalAccessor: ${temporal}, type ${null!=temporal.constructor?temporal.constructor.name:""}`);return time}static parse(text,formatter=DateTimeFormatter.ISO_LOCAL_TIME){return requireNonNull(formatter,"formatter"),formatter.parse(text,LocalTime.FROM)}constructor(hour=0,minute=0,second=0,nanoOfSecond=0){super();const _hour=MathUtil.safeToInt(hour),_minute=MathUtil.safeToInt(minute),_second=MathUtil.safeToInt(second),_nanoOfSecond=MathUtil.safeToInt(nanoOfSecond);if(LocalTime._validate(_hour,_minute,_second,_nanoOfSecond),0===_minute&&0===_second&&0===_nanoOfSecond)return LocalTime.HOURS[_hour]||(this._hour=_hour,this._minute=_minute,this._second=_second,this._nano=_nanoOfSecond,LocalTime.HOURS[_hour]=this),LocalTime.HOURS[_hour];this._hour=_hour,this._minute=_minute,this._second=_second,this._nano=_nanoOfSecond}static _validate(hour,minute,second,nanoOfSecond){ChronoField.HOUR_OF_DAY.checkValidValue(hour),ChronoField.MINUTE_OF_HOUR.checkValidValue(minute),ChronoField.SECOND_OF_MINUTE.checkValidValue(second),ChronoField.NANO_OF_SECOND.checkValidValue(nanoOfSecond)}isSupported(fieldOrUnit){return fieldOrUnit instanceof ChronoField||fieldOrUnit instanceof ChronoUnit?fieldOrUnit.isTimeBased():null!=fieldOrUnit&&fieldOrUnit.isSupportedBy(this)}range(field){return requireNonNull(field),super.range(field)}get(field){return this.getLong(field)}getLong(field){return requireNonNull(field,"field"),field instanceof ChronoField?this._get0(field):field.getFrom(this)}_get0(field){switch(field){case ChronoField.NANO_OF_SECOND:return this._nano;case ChronoField.NANO_OF_DAY:return this.toNanoOfDay();case ChronoField.MICRO_OF_SECOND:return MathUtil.intDiv(this._nano,1e3);case ChronoField.MICRO_OF_DAY:return MathUtil.intDiv(this.toNanoOfDay(),1e3);case ChronoField.MILLI_OF_SECOND:return MathUtil.intDiv(this._nano,1e6);case ChronoField.MILLI_OF_DAY:return MathUtil.intDiv(this.toNanoOfDay(),1e6);case ChronoField.SECOND_OF_MINUTE:return this._second;case ChronoField.SECOND_OF_DAY:return this.toSecondOfDay();case ChronoField.MINUTE_OF_HOUR:return this._minute;case ChronoField.MINUTE_OF_DAY:return 60*this._hour+this._minute;case ChronoField.HOUR_OF_AMPM:return MathUtil.intMod(this._hour,12);case ChronoField.CLOCK_HOUR_OF_AMPM:{const ham=MathUtil.intMod(this._hour,12);return ham%12==0?12:ham}case ChronoField.HOUR_OF_DAY:return this._hour;case ChronoField.CLOCK_HOUR_OF_DAY:return 0===this._hour?24:this._hour;case ChronoField.AMPM_OF_DAY:return MathUtil.intDiv(this._hour,12)}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}hour(){return this._hour}minute(){return this._minute}second(){return this._second}nano(){return this._nano}_withAdjuster(adjuster){return requireNonNull(adjuster,"adjuster"),adjuster instanceof LocalTime?adjuster:super._withAdjuster(adjuster)}_withField(field,newValue){if(requireNonNull(field,"field"),requireInstance(field,TemporalField,"field"),field instanceof ChronoField){switch(field.checkValidValue(newValue),field){case ChronoField.NANO_OF_SECOND:return this.withNano(newValue);case ChronoField.NANO_OF_DAY:return LocalTime.ofNanoOfDay(newValue);case ChronoField.MICRO_OF_SECOND:return this.withNano(1e3*newValue);case ChronoField.MICRO_OF_DAY:return LocalTime.ofNanoOfDay(1e3*newValue);case ChronoField.MILLI_OF_SECOND:return this.withNano(1e6*newValue);case ChronoField.MILLI_OF_DAY:return LocalTime.ofNanoOfDay(1e6*newValue);case ChronoField.SECOND_OF_MINUTE:return this.withSecond(newValue);case ChronoField.SECOND_OF_DAY:return this.plusSeconds(newValue-this.toSecondOfDay());case ChronoField.MINUTE_OF_HOUR:return this.withMinute(newValue);case ChronoField.MINUTE_OF_DAY:return this.plusMinutes(newValue-(60*this._hour+this._minute));case ChronoField.HOUR_OF_AMPM:return this.plusHours(newValue-MathUtil.intMod(this._hour,12));case ChronoField.CLOCK_HOUR_OF_AMPM:return this.plusHours((12===newValue?0:newValue)-MathUtil.intMod(this._hour,12));case ChronoField.HOUR_OF_DAY:return this.withHour(newValue);case ChronoField.CLOCK_HOUR_OF_DAY:return this.withHour(24===newValue?0:newValue);case ChronoField.AMPM_OF_DAY:return this.plusHours(12*(newValue-MathUtil.intDiv(this._hour,12)))}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}return field.adjustInto(this,newValue)}withHour(hour=0){return this._hour===hour?this:new LocalTime(hour,this._minute,this._second,this._nano)}withMinute(minute=0){return this._minute===minute?this:new LocalTime(this._hour,minute,this._second,this._nano)}withSecond(second=0){return this._second===second?this:new LocalTime(this._hour,this._minute,second,this._nano)}withNano(nanoOfSecond=0){return this._nano===nanoOfSecond?this:new LocalTime(this._hour,this._minute,this._second,nanoOfSecond)}truncatedTo(unit){if(requireNonNull(unit,"unit"),unit===ChronoUnit.NANOS)return this;const unitDur=unit.duration();if(unitDur.seconds()>LocalTime.SECONDS_PER_DAY)throw new DateTimeException("Unit is too large to be used for truncation");const dur=unitDur.toNanos();if(0!==MathUtil.intMod(LocalTime.NANOS_PER_DAY,dur))throw new DateTimeException("Unit must divide into a standard day without remainder");const nod=this.toNanoOfDay();return LocalTime.ofNanoOfDay(MathUtil.intDiv(nod,dur)*dur)}_plusUnit(amountToAdd,unit){if(requireNonNull(unit,"unit"),unit instanceof ChronoUnit){switch(unit){case ChronoUnit.NANOS:return this.plusNanos(amountToAdd);case ChronoUnit.MICROS:return this.plusNanos(1e3*MathUtil.intMod(amountToAdd,LocalTime.MICROS_PER_DAY));case ChronoUnit.MILLIS:return this.plusNanos(1e6*MathUtil.intMod(amountToAdd,LocalTime.MILLIS_PER_DAY));case ChronoUnit.SECONDS:return this.plusSeconds(amountToAdd);case ChronoUnit.MINUTES:return this.plusMinutes(amountToAdd);case ChronoUnit.HOURS:return this.plusHours(amountToAdd);case ChronoUnit.HALF_DAYS:return this.plusHours(12*MathUtil.intMod(amountToAdd,2))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.addTo(this,amountToAdd)}plusHours(hoursToAdd){if(0===hoursToAdd)return this;const newHour=MathUtil.intMod(MathUtil.intMod(hoursToAdd,LocalTime.HOURS_PER_DAY)+this._hour+LocalTime.HOURS_PER_DAY,LocalTime.HOURS_PER_DAY);return new LocalTime(newHour,this._minute,this._second,this._nano)}plusMinutes(minutesToAdd){if(0===minutesToAdd)return this;const mofd=this._hour*LocalTime.MINUTES_PER_HOUR+this._minute,newMofd=MathUtil.intMod(MathUtil.intMod(minutesToAdd,LocalTime.MINUTES_PER_DAY)+mofd+LocalTime.MINUTES_PER_DAY,LocalTime.MINUTES_PER_DAY);if(mofd===newMofd)return this;const newHour=MathUtil.intDiv(newMofd,LocalTime.MINUTES_PER_HOUR),newMinute=MathUtil.intMod(newMofd,LocalTime.MINUTES_PER_HOUR);return new LocalTime(newHour,newMinute,this._second,this._nano)}plusSeconds(secondsToAdd){if(0===secondsToAdd)return this;const sofd=this._hour*LocalTime.SECONDS_PER_HOUR+this._minute*LocalTime.SECONDS_PER_MINUTE+this._second,newSofd=MathUtil.intMod(MathUtil.intMod(secondsToAdd,LocalTime.SECONDS_PER_DAY)+sofd+LocalTime.SECONDS_PER_DAY,LocalTime.SECONDS_PER_DAY);if(sofd===newSofd)return this;const newHour=MathUtil.intDiv(newSofd,LocalTime.SECONDS_PER_HOUR),newMinute=MathUtil.intMod(MathUtil.intDiv(newSofd,LocalTime.SECONDS_PER_MINUTE),LocalTime.MINUTES_PER_HOUR),newSecond=MathUtil.intMod(newSofd,LocalTime.SECONDS_PER_MINUTE);return new LocalTime(newHour,newMinute,newSecond,this._nano)}plusNanos(nanosToAdd){if(0===nanosToAdd)return this;const nofd=this.toNanoOfDay(),newNofd=MathUtil.intMod(MathUtil.intMod(nanosToAdd,LocalTime.NANOS_PER_DAY)+nofd+LocalTime.NANOS_PER_DAY,LocalTime.NANOS_PER_DAY);if(nofd===newNofd)return this;const newHour=MathUtil.intDiv(newNofd,LocalTime.NANOS_PER_HOUR),newMinute=MathUtil.intMod(MathUtil.intDiv(newNofd,LocalTime.NANOS_PER_MINUTE),LocalTime.MINUTES_PER_HOUR),newSecond=MathUtil.intMod(MathUtil.intDiv(newNofd,LocalTime.NANOS_PER_SECOND),LocalTime.SECONDS_PER_MINUTE),newNano=MathUtil.intMod(newNofd,LocalTime.NANOS_PER_SECOND);return new LocalTime(newHour,newMinute,newSecond,newNano)}_minusUnit(amountToSubtract,unit){return requireNonNull(unit,"unit"),this._plusUnit(-1*amountToSubtract,unit)}minusHours(hoursToSubtract){return this.plusHours(-1*MathUtil.intMod(hoursToSubtract,LocalTime.HOURS_PER_DAY))}minusMinutes(minutesToSubtract){return this.plusMinutes(-1*MathUtil.intMod(minutesToSubtract,LocalTime.MINUTES_PER_DAY))}minusSeconds(secondsToSubtract){return this.plusSeconds(-1*MathUtil.intMod(secondsToSubtract,LocalTime.SECONDS_PER_DAY))}minusNanos(nanosToSubtract){return this.plusNanos(-1*MathUtil.intMod(nanosToSubtract,LocalTime.NANOS_PER_DAY))}query(query){return requireNonNull(query,"query"),query===TemporalQueries.precision()?ChronoUnit.NANOS:query===TemporalQueries.localTime()?this:query===TemporalQueries.chronology()||query===TemporalQueries.zoneId()||query===TemporalQueries.zone()||query===TemporalQueries.offset()||query===TemporalQueries.localDate()?null:query.queryFrom(this)}adjustInto(temporal){return temporal.with(LocalTime.NANO_OF_DAY,this.toNanoOfDay())}until(endExclusive,unit){requireNonNull(endExclusive,"endExclusive"),requireNonNull(unit,"unit");const end=LocalTime.from(endExclusive);if(unit instanceof ChronoUnit){const nanosUntil=end.toNanoOfDay()-this.toNanoOfDay();switch(unit){case ChronoUnit.NANOS:return nanosUntil;case ChronoUnit.MICROS:return MathUtil.intDiv(nanosUntil,1e3);case ChronoUnit.MILLIS:return MathUtil.intDiv(nanosUntil,1e6);case ChronoUnit.SECONDS:return MathUtil.intDiv(nanosUntil,LocalTime.NANOS_PER_SECOND);case ChronoUnit.MINUTES:return MathUtil.intDiv(nanosUntil,LocalTime.NANOS_PER_MINUTE);case ChronoUnit.HOURS:return MathUtil.intDiv(nanosUntil,LocalTime.NANOS_PER_HOUR);case ChronoUnit.HALF_DAYS:return MathUtil.intDiv(nanosUntil,12*LocalTime.NANOS_PER_HOUR)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.between(this,end)}atDate(date){return LocalDateTime.of(date,this)}atOffset(offset){return OffsetTime.of(this,offset)}toSecondOfDay(){let total=this._hour*LocalTime.SECONDS_PER_HOUR;return total+=this._minute*LocalTime.SECONDS_PER_MINUTE,total+=this._second,total}toNanoOfDay(){let total=this._hour*LocalTime.NANOS_PER_HOUR;return total+=this._minute*LocalTime.NANOS_PER_MINUTE,total+=this._second*LocalTime.NANOS_PER_SECOND,total+=this._nano,total}compareTo(other){requireNonNull(other,"other"),requireInstance(other,LocalTime,"other");let cmp=MathUtil.compareNumbers(this._hour,other._hour);return 0===cmp&&(cmp=MathUtil.compareNumbers(this._minute,other._minute),0===cmp&&(cmp=MathUtil.compareNumbers(this._second,other._second),0===cmp&&(cmp=MathUtil.compareNumbers(this._nano,other._nano)))),cmp}isAfter(other){return this.compareTo(other)>0}isBefore(other){return this.compareTo(other)<0}equals(other){return this===other||other instanceof LocalTime&&(this._hour===other._hour&&this._minute===other._minute&&this._second===other._second&&this._nano===other._nano)}hashCode(){const nod=this.toNanoOfDay();return MathUtil.hash(nod)}toString(){let buf="";const hourValue=this._hour,minuteValue=this._minute,secondValue=this._second,nanoValue=this._nano;return buf+=hourValue<10?"0":"",buf+=hourValue,buf+=minuteValue<10?":0":":",buf+=minuteValue,(secondValue>0||nanoValue>0)&&(buf+=secondValue<10?":0":":",buf+=secondValue,nanoValue>0&&(buf+=".",0===MathUtil.intMod(nanoValue,1e6)?buf+=`${MathUtil.intDiv(nanoValue,1e6)+1e3}`.substring(1):0===MathUtil.intMod(nanoValue,1e3)?buf+=`${MathUtil.intDiv(nanoValue,1e3)+1e6}`.substring(1):buf+=`${nanoValue+1e9}`.substring(1))),buf}toJSON(){return this.toString()}format(formatter){return requireNonNull(formatter,"formatter"),formatter.format(this)}}export function _init(){LocalTime.HOURS=[];for(let hour=0;hour<24;hour++)LocalTime.of(hour,0,0,0);LocalTime.MIN=LocalTime.HOURS[0],LocalTime.MAX=new LocalTime(23,59,59,999999999),LocalTime.MIDNIGHT=LocalTime.HOURS[0],LocalTime.NOON=LocalTime.HOURS[12],LocalTime.FROM=createTemporalQuery("LocalTime.FROM",(temporal=>LocalTime.from(temporal)))}LocalTime.HOURS_PER_DAY=24,LocalTime.MINUTES_PER_HOUR=60,LocalTime.MINUTES_PER_DAY=LocalTime.MINUTES_PER_HOUR*LocalTime.HOURS_PER_DAY,LocalTime.SECONDS_PER_MINUTE=60,LocalTime.SECONDS_PER_HOUR=LocalTime.SECONDS_PER_MINUTE*LocalTime.MINUTES_PER_HOUR,LocalTime.SECONDS_PER_DAY=LocalTime.SECONDS_PER_HOUR*LocalTime.HOURS_PER_DAY,LocalTime.MILLIS_PER_DAY=1e3*LocalTime.SECONDS_PER_DAY,LocalTime.MICROS_PER_DAY=1e6*LocalTime.SECONDS_PER_DAY,LocalTime.NANOS_PER_SECOND=1e9,LocalTime.NANOS_PER_MINUTE=LocalTime.NANOS_PER_SECOND*LocalTime.SECONDS_PER_MINUTE,LocalTime.NANOS_PER_HOUR=LocalTime.NANOS_PER_MINUTE*LocalTime.MINUTES_PER_HOUR,LocalTime.NANOS_PER_DAY=LocalTime.NANOS_PER_HOUR*LocalTime.HOURS_PER_DAY;