/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{assert,requireNonNull,requireInstance}from"./assert";import{MathUtil}from"./MathUtil";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{DateTimeException,IllegalArgumentException,UnsupportedTemporalTypeException}from"./errors";import{IsoChronology}from"./chrono/IsoChronology";import{TemporalAccessor}from"./temporal/TemporalAccessor";import{TemporalQueries}from"./temporal/TemporalQueries";export class Month extends TemporalAccessor{constructor(value,name){super(),this._value=MathUtil.safeToInt(value),this._name=name}value(){return this._value}ordinal(){return this._value-1}name(){return this._name}displayName(style,locale){throw new IllegalArgumentException("Pattern using (localized) text not implemented yet!")}isSupported(field){return null!==field&&(field instanceof ChronoField?field===ChronoField.MONTH_OF_YEAR:null!=field&&field.isSupportedBy(this))}get(field){return field===ChronoField.MONTH_OF_YEAR?this.value():this.range(field).checkValidIntValue(this.getLong(field),field)}getLong(field){if(field===ChronoField.MONTH_OF_YEAR)return this.value();if(field instanceof ChronoField)throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`);return field.getFrom(this)}plus(months){const amount=MathUtil.intMod(months,12)+12;let newMonthVal=MathUtil.intMod(this.value()+amount,12);return newMonthVal=0===newMonthVal?12:newMonthVal,Month.of(newMonthVal)}minus(months){return this.plus(-1*MathUtil.intMod(months,12))}length(leapYear){switch(this){case Month.FEBRUARY:return leapYear?29:28;case Month.APRIL:case Month.JUNE:case Month.SEPTEMBER:case Month.NOVEMBER:return 30;default:return 31}}minLength(){switch(this){case Month.FEBRUARY:return 28;case Month.APRIL:case Month.JUNE:case Month.SEPTEMBER:case Month.NOVEMBER:return 30;default:return 31}}maxLength(){switch(this){case Month.FEBRUARY:return 29;case Month.APRIL:case Month.JUNE:case Month.SEPTEMBER:case Month.NOVEMBER:return 30;default:return 31}}firstDayOfYear(leapYear){const leap=leapYear?1:0;switch(this){case Month.JANUARY:return 1;case Month.FEBRUARY:return 32;case Month.MARCH:return 60+leap;case Month.APRIL:return 91+leap;case Month.MAY:return 121+leap;case Month.JUNE:return 152+leap;case Month.JULY:return 182+leap;case Month.AUGUST:return 213+leap;case Month.SEPTEMBER:return 244+leap;case Month.OCTOBER:return 274+leap;case Month.NOVEMBER:return 305+leap;case Month.DECEMBER:default:return 335+leap}}firstMonthOfQuarter(){switch(this){case Month.JANUARY:case Month.FEBRUARY:case Month.MARCH:return Month.JANUARY;case Month.APRIL:case Month.MAY:case Month.JUNE:return Month.APRIL;case Month.JULY:case Month.AUGUST:case Month.SEPTEMBER:return Month.JULY;case Month.OCTOBER:case Month.NOVEMBER:case Month.DECEMBER:default:return Month.OCTOBER}}query(query){return assert(null!=query,"query() parameter must not be null",DateTimeException),query===TemporalQueries.chronology()?IsoChronology.INSTANCE:query===TemporalQueries.precision()?ChronoUnit.MONTHS:super.query(query)}toString(){switch(this){case Month.JANUARY:return"JANUARY";case Month.FEBRUARY:return"FEBRUARY";case Month.MARCH:return"MARCH";case Month.APRIL:return"APRIL";case Month.MAY:return"MAY";case Month.JUNE:return"JUNE";case Month.JULY:return"JULY";case Month.AUGUST:return"AUGUST";case Month.SEPTEMBER:return"SEPTEMBER";case Month.OCTOBER:return"OCTOBER";case Month.NOVEMBER:return"NOVEMBER";case Month.DECEMBER:return"DECEMBER";default:return`unknown Month, value: ${this.value()}`}}toJSON(){return this.toString()}adjustInto(temporal){return temporal.with(ChronoField.MONTH_OF_YEAR,this.value())}compareTo(other){return requireNonNull(other,"other"),requireInstance(other,Month,"other"),this._value-other._value}equals(other){return this===other}static valueOf(name){let ordinal=0;for(;ordinal<MONTHS.length&&MONTHS[ordinal].name()!==name;ordinal++);return Month.of(ordinal+1)}static values(){return MONTHS.slice()}static of(month){return(month<1||month>12)&&assert(!1,`Invalid value for MonthOfYear: ${month}`,DateTimeException),MONTHS[month-1]}static from(temporal){if(temporal instanceof Month)return temporal;try{return Month.of(temporal.get(ChronoField.MONTH_OF_YEAR))}catch(ex){throw new DateTimeException(`Unable to obtain Month from TemporalAccessor: ${temporal} of type ${temporal&&null!=temporal.constructor?temporal.constructor.name:""}`,ex)}}}let MONTHS;export function _init(){Month.JANUARY=new Month(1,"JANUARY"),Month.FEBRUARY=new Month(2,"FEBRUARY"),Month.MARCH=new Month(3,"MARCH"),Month.APRIL=new Month(4,"APRIL"),Month.MAY=new Month(5,"MAY"),Month.JUNE=new Month(6,"JUNE"),Month.JULY=new Month(7,"JULY"),Month.AUGUST=new Month(8,"AUGUST"),Month.SEPTEMBER=new Month(9,"SEPTEMBER"),Month.OCTOBER=new Month(10,"OCTOBER"),Month.NOVEMBER=new Month(11,"NOVEMBER"),Month.DECEMBER=new Month(12,"DECEMBER"),MONTHS=[Month.JANUARY,Month.FEBRUARY,Month.MARCH,Month.APRIL,Month.MAY,Month.JUNE,Month.JULY,Month.AUGUST,Month.SEPTEMBER,Month.OCTOBER,Month.NOVEMBER,Month.DECEMBER]}