/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{MathUtil}from"./MathUtil";import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException,ArithmeticException,DateTimeParseException}from"./errors";import{IsoChronology}from"./chrono/IsoChronology";import{ChronoUnit}from"./temporal/ChronoUnit";import{TemporalAmount}from"./temporal/TemporalAmount";import{LocalDate}from"./LocalDate";const PATTERN=/([-+]?)P(?:([-+]?[0-9]+)Y)?(?:([-+]?[0-9]+)M)?(?:([-+]?[0-9]+)W)?(?:([-+]?[0-9]+)D)?/;export class Period extends TemporalAmount{constructor(years,months,days){super();const _years=MathUtil.safeToInt(years),_months=MathUtil.safeToInt(months),_days=MathUtil.safeToInt(days);if(0===_years&&0===_months&&0===_days)return Period.ZERO||(this._years=_years,this._months=_months,this._days=_days,Period.ZERO=this),Period.ZERO;this._years=_years,this._months=_months,this._days=_days}static ofYears(years){return Period.create(years,0,0)}static ofMonths(months){return Period.create(0,months,0)}static ofWeeks(weeks){return Period.create(0,0,MathUtil.safeMultiply(weeks,7))}static ofDays(days){return Period.create(0,0,days)}static of(years,months,days){return Period.create(years,months,days)}static from(amount){if(amount instanceof Period)return amount;requireNonNull(amount,"amount");let years=0,months=0,days=0;const units=amount.units();for(let i=0;i<units.length;i++){const unit=units[i],unitAmount=amount.get(unit);if(unit===ChronoUnit.YEARS)years=MathUtil.safeToInt(unitAmount);else if(unit===ChronoUnit.MONTHS)months=MathUtil.safeToInt(unitAmount);else{if(unit!==ChronoUnit.DAYS)throw new DateTimeException(`Unit must be Years, Months or Days, but was ${unit}`);days=MathUtil.safeToInt(unitAmount)}}return Period.create(years,months,days)}static between(startDate,endDate){return requireNonNull(startDate,"startDate"),requireNonNull(endDate,"endDate"),requireInstance(startDate,LocalDate,"startDate"),requireInstance(endDate,LocalDate,"endDate"),startDate.until(endDate)}static parse(text){requireNonNull(text,"text");try{return Period._parse(text)}catch(ex){throw ex instanceof ArithmeticException?new DateTimeParseException("Text cannot be parsed to a Period",text,0,ex):ex}}static _parse(text){const matches=PATTERN.exec(text);if(null!=matches){const negate="-"===matches[1]?-1:1,yearMatch=matches[2],monthMatch=matches[3],weekMatch=matches[4],dayMatch=matches[5];if(null!=yearMatch||null!=monthMatch||null!=weekMatch||null!=dayMatch){const years=Period._parseNumber(text,yearMatch,negate),months=Period._parseNumber(text,monthMatch,negate),weeks=Period._parseNumber(text,weekMatch,negate);let days=Period._parseNumber(text,dayMatch,negate);return days=MathUtil.safeAdd(days,MathUtil.safeMultiply(weeks,7)),Period.create(years,months,days)}}throw new DateTimeParseException("Text cannot be parsed to a Period",text,0)}static _parseNumber(text,str,negate){if(null==str)return 0;const val=MathUtil.parseInt(str);return MathUtil.safeMultiply(val,negate)}static create(years,months,days){return new Period(years,months,days)}units(){return[ChronoUnit.YEARS,ChronoUnit.MONTHS,ChronoUnit.DAYS]}chronology(){return IsoChronology.INSTANCE}get(unit){if(unit===ChronoUnit.YEARS)return this._years;if(unit===ChronoUnit.MONTHS)return this._months;if(unit===ChronoUnit.DAYS)return this._days;throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}isZero(){return this===Period.ZERO}isNegative(){return this._years<0||this._months<0||this._days<0}years(){return this._years}months(){return this._months}days(){return this._days}withYears(years){return years===this._years?this:Period.create(years,this._months,this._days)}withMonths(months){return months===this._months?this:Period.create(this._years,months,this._days)}withDays(days){return days===this._days?this:Period.create(this._years,this._months,days)}plus(amountToAdd){const amount=Period.from(amountToAdd);return Period.create(MathUtil.safeAdd(this._years,amount._years),MathUtil.safeAdd(this._months,amount._months),MathUtil.safeAdd(this._days,amount._days))}plusYears(yearsToAdd){return 0===yearsToAdd?this:Period.create(MathUtil.safeToInt(MathUtil.safeAdd(this._years,yearsToAdd)),this._months,this._days)}plusMonths(monthsToAdd){return 0===monthsToAdd?this:Period.create(this._years,MathUtil.safeToInt(MathUtil.safeAdd(this._months,monthsToAdd)),this._days)}plusDays(daysToAdd){return 0===daysToAdd?this:Period.create(this._years,this._months,MathUtil.safeToInt(MathUtil.safeAdd(this._days,daysToAdd)))}minus(amountToSubtract){const amount=Period.from(amountToSubtract);return Period.create(MathUtil.safeSubtract(this._years,amount._years),MathUtil.safeSubtract(this._months,amount._months),MathUtil.safeSubtract(this._days,amount._days))}minusYears(yearsToSubtract){return this.plusYears(-1*yearsToSubtract)}minusMonths(monthsToSubtract){return this.plusMonths(-1*monthsToSubtract)}minusDays(daysToSubtract){return this.plusDays(-1*daysToSubtract)}multipliedBy(scalar){return this===Period.ZERO||1===scalar?this:Period.create(MathUtil.safeMultiply(this._years,scalar),MathUtil.safeMultiply(this._months,scalar),MathUtil.safeMultiply(this._days,scalar))}negated(){return this.multipliedBy(-1)}normalized(){const totalMonths=this.toTotalMonths(),splitYears=MathUtil.intDiv(totalMonths,12),splitMonths=MathUtil.intMod(totalMonths,12);return splitYears===this._years&&splitMonths===this._months?this:Period.create(MathUtil.safeToInt(splitYears),splitMonths,this._days)}toTotalMonths(){return 12*this._years+this._months}addTo(temporal){return requireNonNull(temporal,"temporal"),0!==this._years?temporal=0!==this._months?temporal.plus(this.toTotalMonths(),ChronoUnit.MONTHS):temporal.plus(this._years,ChronoUnit.YEARS):0!==this._months&&(temporal=temporal.plus(this._months,ChronoUnit.MONTHS)),0!==this._days&&(temporal=temporal.plus(this._days,ChronoUnit.DAYS)),temporal}subtractFrom(temporal){return requireNonNull(temporal,"temporal"),0!==this._years?temporal=0!==this._months?temporal.minus(this.toTotalMonths(),ChronoUnit.MONTHS):temporal.minus(this._years,ChronoUnit.YEARS):0!==this._months&&(temporal=temporal.minus(this._months,ChronoUnit.MONTHS)),0!==this._days&&(temporal=temporal.minus(this._days,ChronoUnit.DAYS)),temporal}equals(obj){if(this===obj)return!0;if(obj instanceof Period){const other=obj;return this._years===other._years&&this._months===other._months&&this._days===other._days}return!1}hashCode(){return MathUtil.hashCode(this._years,this._months,this._days)}toString(){if(this===Period.ZERO)return"P0D";{let buf="P";return 0!==this._years&&(buf+=`${this._years}Y`),0!==this._months&&(buf+=`${this._months}M`),0!==this._days&&(buf+=`${this._days}D`),buf}}toJSON(){return this.toString()}}export function _init(){Period.ofDays(0)}