/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @license BSD-3-Clause (see LICENSE.md in the root directory of this source tree)
 */
import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{MathUtil}from"./MathUtil";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{Clock}from"./Clock";import{DateTimeFormatterBuilder}from"./format/DateTimeFormatterBuilder";import{IsoChronology}from"./chrono/IsoChronology";import{LocalDate}from"./LocalDate";import{Month}from"./Month";import{SignStyle}from"./format/SignStyle";import{Temporal}from"./temporal/Temporal";import{TemporalField}from"./temporal/TemporalField";import{TemporalQueries}from"./temporal/TemporalQueries";import{TemporalQuery}from"./temporal/TemporalQuery";import{TemporalUnit}from"./temporal/TemporalUnit";import{createTemporalQuery}from"./temporal/TemporalQuery";import{ValueRange}from"./temporal/ValueRange";import{Year}from"./Year";import{ZoneId}from"./ZoneId";export class YearMonth extends Temporal{static now(zoneIdOrClock){return 0===arguments.length?YearMonth.now0():1===arguments.length&&zoneIdOrClock instanceof ZoneId?YearMonth.nowZoneId(zoneIdOrClock):YearMonth.nowClock(zoneIdOrClock)}static now0(){return YearMonth.nowClock(Clock.systemDefaultZone())}static nowZoneId(zone){return YearMonth.nowClock(Clock.system(zone))}static nowClock(clock){const now=LocalDate.now(clock);return YearMonth.of(now.year(),now.month())}static of(year,monthOrNumber){return 2===arguments.length&&monthOrNumber instanceof Month?YearMonth.ofNumberMonth(year,monthOrNumber):YearMonth.ofNumberNumber(year,monthOrNumber)}static ofNumberMonth(year,month){return requireNonNull(month,"month"),requireInstance(month,Month,"month"),YearMonth.ofNumberNumber(year,month.value())}static ofNumberNumber(year,month){return requireNonNull(year,"year"),requireNonNull(month,"month"),ChronoField.YEAR.checkValidValue(year),ChronoField.MONTH_OF_YEAR.checkValidValue(month),new YearMonth(year,month)}static from(temporal){if(requireNonNull(temporal,"temporal"),temporal instanceof YearMonth)return temporal;try{return YearMonth.of(temporal.get(ChronoField.YEAR),temporal.get(ChronoField.MONTH_OF_YEAR))}catch(ex){throw new DateTimeException(`Unable to obtain YearMonth from TemporalAccessor: ${temporal}, type ${temporal&&null!=temporal.constructor?temporal.constructor.name:""}`)}}static parse(text,formatter){return 1===arguments.length?YearMonth.parseString(text):YearMonth.parseStringFormatter(text,formatter)}static parseString(text){return YearMonth.parseStringFormatter(text,PARSER)}static parseStringFormatter(text,formatter){return requireNonNull(formatter,"formatter"),formatter.parse(text,YearMonth.FROM)}constructor(year,month){super(),this._year=MathUtil.safeToInt(year),this._month=MathUtil.safeToInt(month)}isSupported(fieldOrUnit){return 1===arguments.length&&fieldOrUnit instanceof TemporalField?this.isSupportedField(fieldOrUnit):this.isSupportedUnit(fieldOrUnit)}isSupportedField(field){return field instanceof ChronoField?field===ChronoField.YEAR||field===ChronoField.MONTH_OF_YEAR||field===ChronoField.PROLEPTIC_MONTH||field===ChronoField.YEAR_OF_ERA||field===ChronoField.ERA:null!=field&&field.isSupportedBy(this)}isSupportedUnit(unit){return unit instanceof ChronoUnit?unit===ChronoUnit.MONTHS||unit===ChronoUnit.YEARS||unit===ChronoUnit.DECADES||unit===ChronoUnit.CENTURIES||unit===ChronoUnit.MILLENNIA||unit===ChronoUnit.ERAS:null!=unit&&unit.isSupportedBy(this)}range(field){return field===ChronoField.YEAR_OF_ERA?this.year()<=0?ValueRange.of(1,Year.MAX_VALUE+1):ValueRange.of(1,Year.MAX_VALUE):super.range(field)}get(field){return requireNonNull(field,"field"),requireInstance(field,TemporalField,"field"),this.range(field).checkValidIntValue(this.getLong(field),field)}getLong(field){if(requireNonNull(field,"field"),requireInstance(field,TemporalField,"field"),field instanceof ChronoField){switch(field){case ChronoField.MONTH_OF_YEAR:return this._month;case ChronoField.PROLEPTIC_MONTH:return this._getProlepticMonth();case ChronoField.YEAR_OF_ERA:return this._year<1?1-this._year:this._year;case ChronoField.YEAR:return this._year;case ChronoField.ERA:return this._year<1?0:1}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}return field.getFrom(this)}_getProlepticMonth(){return MathUtil.safeAdd(MathUtil.safeMultiply(this._year,12),this._month-1)}year(){return this._year}monthValue(){return this._month}month(){return Month.of(this._month)}isLeapYear(){return IsoChronology.isLeapYear(this._year)}isValidDay(dayOfMonth){return dayOfMonth>=1&&dayOfMonth<=this.lengthOfMonth()}lengthOfMonth(){return this.month().length(this.isLeapYear())}lengthOfYear(){return this.isLeapYear()?366:365}with(adjusterOrField,value){return 1===arguments.length?this._withAdjuster(adjusterOrField):this._withField(adjusterOrField,value)}_withField(field,newValue){if(requireNonNull(field,"field"),requireInstance(field,TemporalField,"field"),field instanceof ChronoField){const f=field;switch(f.checkValidValue(newValue),f){case ChronoField.MONTH_OF_YEAR:return this.withMonth(newValue);case ChronoField.PROLEPTIC_MONTH:return this.plusMonths(newValue-this.getLong(ChronoField.PROLEPTIC_MONTH));case ChronoField.YEAR_OF_ERA:return this.withYear(this._year<1?1-newValue:newValue);case ChronoField.YEAR:return this.withYear(newValue);case ChronoField.ERA:return this.getLong(ChronoField.ERA)===newValue?this:this.withYear(1-this._year)}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}return field.adjustInto(this,newValue)}withYear(year){return ChronoField.YEAR.checkValidValue(year),new YearMonth(year,this._month)}withMonth(month){return ChronoField.MONTH_OF_YEAR.checkValidValue(month),new YearMonth(this._year,month)}_plusUnit(amountToAdd,unit){if(requireNonNull(unit,"unit"),requireInstance(unit,TemporalUnit,"unit"),unit instanceof ChronoUnit){switch(unit){case ChronoUnit.MONTHS:return this.plusMonths(amountToAdd);case ChronoUnit.YEARS:return this.plusYears(amountToAdd);case ChronoUnit.DECADES:return this.plusYears(MathUtil.safeMultiply(amountToAdd,10));case ChronoUnit.CENTURIES:return this.plusYears(MathUtil.safeMultiply(amountToAdd,100));case ChronoUnit.MILLENNIA:return this.plusYears(MathUtil.safeMultiply(amountToAdd,1e3));case ChronoUnit.ERAS:return this.with(ChronoField.ERA,MathUtil.safeAdd(this.getLong(ChronoField.ERA),amountToAdd))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.addTo(this,amountToAdd)}plusYears(yearsToAdd){if(0===yearsToAdd)return this;const newYear=ChronoField.YEAR.checkValidIntValue(this._year+yearsToAdd);return this.withYear(newYear)}plusMonths(monthsToAdd){if(0===monthsToAdd)return this;const calcMonths=12*this._year+(this._month-1)+monthsToAdd,newYear=ChronoField.YEAR.checkValidIntValue(MathUtil.floorDiv(calcMonths,12)),newMonth=MathUtil.floorMod(calcMonths,12)+1;return new YearMonth(newYear,newMonth)}minusYears(yearsToSubtract){return yearsToSubtract===MathUtil.MIN_SAFE_INTEGER?this.plusYears(MathUtil.MIN_SAFE_INTEGER).plusYears(1):this.plusYears(-yearsToSubtract)}minusMonths(monthsToSubtract){return monthsToSubtract===MathUtil.MIN_SAFE_INTEGER?this.plusMonths(Math.MAX_SAFE_INTEGER).plusMonths(1):this.plusMonths(-monthsToSubtract)}query(query){return requireNonNull(query,"query"),requireInstance(query,TemporalQuery,"query"),query===TemporalQueries.chronology()?IsoChronology.INSTANCE:query===TemporalQueries.precision()?ChronoUnit.MONTHS:query===TemporalQueries.localDate()||query===TemporalQueries.localTime()||query===TemporalQueries.zone()||query===TemporalQueries.zoneId()||query===TemporalQueries.offset()?null:super.query(query)}adjustInto(temporal){return requireNonNull(temporal,"temporal"),requireInstance(temporal,Temporal,"temporal"),temporal.with(ChronoField.PROLEPTIC_MONTH,this._getProlepticMonth())}until(endExclusive,unit){requireNonNull(endExclusive,"endExclusive"),requireNonNull(unit,"unit"),requireInstance(endExclusive,Temporal,"endExclusive"),requireInstance(unit,TemporalUnit,"unit");const end=YearMonth.from(endExclusive);if(unit instanceof ChronoUnit){const monthsUntil=end._getProlepticMonth()-this._getProlepticMonth();switch(unit){case ChronoUnit.MONTHS:return monthsUntil;case ChronoUnit.YEARS:return MathUtil.intDiv(monthsUntil,12);case ChronoUnit.DECADES:return MathUtil.intDiv(monthsUntil,120);case ChronoUnit.CENTURIES:return MathUtil.intDiv(monthsUntil,1200);case ChronoUnit.MILLENNIA:return MathUtil.intDiv(monthsUntil,12e3);case ChronoUnit.ERAS:return end.getLong(ChronoField.ERA)-this.getLong(ChronoField.ERA)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.between(this,end)}atDay(dayOfMonth){return requireNonNull(dayOfMonth,"dayOfMonth"),LocalDate.of(this._year,this._month,dayOfMonth)}atEndOfMonth(){return LocalDate.of(this._year,this._month,this.lengthOfMonth())}compareTo(other){requireNonNull(other,"other"),requireInstance(other,YearMonth,"other");let cmp=this._year-other.year();return 0===cmp&&(cmp=this._month-other.monthValue()),cmp}isAfter(other){return this.compareTo(other)>0}isBefore(other){return this.compareTo(other)<0}equals(obj){if(this===obj)return!0;if(obj instanceof YearMonth){const other=obj;return this.year()===other.year()&&this.monthValue()===other.monthValue()}return!1}toString(){return PARSER.format(this)}toJSON(){return this.toString()}format(formatter){return requireNonNull(formatter,"formatter"),formatter.format(this)}}let PARSER;export function _init(){PARSER=(new DateTimeFormatterBuilder).appendValue(ChronoField.YEAR,4,10,SignStyle.EXCEEDS_PAD).appendLiteral("-").appendValue(ChronoField.MONTH_OF_YEAR,2).toFormatter(),YearMonth.FROM=createTemporalQuery("YearMonth.FROM",(temporal=>YearMonth.from(temporal)))}