/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull}from"./assert";import{DateTimeException}from"./errors";import{MathUtil}from"./MathUtil";import{LocalTime}from"./LocalTime";import{ZoneId}from"./ZoneId";import{ChronoField}from"./temporal/ChronoField";import{TemporalQueries}from"./temporal/TemporalQueries";import{ZoneRules}from"./zone/ZoneRules";const SECONDS_CACHE={},ID_CACHE={};export class ZoneOffset extends ZoneId{constructor(totalSeconds){super(),ZoneOffset._validateTotalSeconds(totalSeconds),this._totalSeconds=MathUtil.safeToInt(totalSeconds),this._rules=ZoneRules.of(this),this._id=ZoneOffset._buildId(totalSeconds)}totalSeconds(){return this._totalSeconds}id(){return this._id}static _buildId(totalSeconds){if(0===totalSeconds)return"Z";{const absTotalSeconds=Math.abs(totalSeconds),absHours=MathUtil.intDiv(absTotalSeconds,LocalTime.SECONDS_PER_HOUR),absMinutes=MathUtil.intMod(MathUtil.intDiv(absTotalSeconds,LocalTime.SECONDS_PER_MINUTE),LocalTime.MINUTES_PER_HOUR);let buf=`${totalSeconds<0?"-":"+"}${absHours<10?"0":""}${absHours}${absMinutes<10?":0":":"}${absMinutes}`;const absSeconds=MathUtil.intMod(absTotalSeconds,LocalTime.SECONDS_PER_MINUTE);return 0!==absSeconds&&(buf+=(absSeconds<10?":0":":")+absSeconds),buf}}static _validateTotalSeconds(totalSeconds){if(Math.abs(totalSeconds)>ZoneOffset.MAX_SECONDS)throw new DateTimeException("Zone offset not in valid range: -18:00 to +18:00")}static _validate(hours,minutes,seconds){if(hours<-18||hours>18)throw new DateTimeException(`Zone offset hours not in valid range: value ${hours} is not in the range -18 to 18`);if(hours>0){if(minutes<0||seconds<0)throw new DateTimeException("Zone offset minutes and seconds must be positive because hours is positive")}else if(hours<0){if(minutes>0||seconds>0)throw new DateTimeException("Zone offset minutes and seconds must be negative because hours is negative")}else if(minutes>0&&seconds<0||minutes<0&&seconds>0)throw new DateTimeException("Zone offset minutes and seconds must have the same sign");if(Math.abs(minutes)>59)throw new DateTimeException(`Zone offset minutes not in valid range: abs(value) ${Math.abs(minutes)} is not in the range 0 to 59`);if(Math.abs(seconds)>59)throw new DateTimeException(`Zone offset seconds not in valid range: abs(value) ${Math.abs(seconds)} is not in the range 0 to 59`);if(18===Math.abs(hours)&&(Math.abs(minutes)>0||Math.abs(seconds)>0))throw new DateTimeException("Zone offset not in valid range: -18:00 to +18:00")}static of(offsetId){requireNonNull(offsetId,"offsetId");const offset=ID_CACHE[offsetId];if(null!=offset)return offset;let hours,minutes,seconds;switch(offsetId.length){case 2:offsetId=`${offsetId[0]}0${offsetId[1]}`;case 3:hours=ZoneOffset._parseNumber(offsetId,1,!1),minutes=0,seconds=0;break;case 5:hours=ZoneOffset._parseNumber(offsetId,1,!1),minutes=ZoneOffset._parseNumber(offsetId,3,!1),seconds=0;break;case 6:hours=ZoneOffset._parseNumber(offsetId,1,!1),minutes=ZoneOffset._parseNumber(offsetId,4,!0),seconds=0;break;case 7:hours=ZoneOffset._parseNumber(offsetId,1,!1),minutes=ZoneOffset._parseNumber(offsetId,3,!1),seconds=ZoneOffset._parseNumber(offsetId,5,!1);break;case 9:hours=ZoneOffset._parseNumber(offsetId,1,!1),minutes=ZoneOffset._parseNumber(offsetId,4,!0),seconds=ZoneOffset._parseNumber(offsetId,7,!0);break;default:throw new DateTimeException(`Invalid ID for ZoneOffset, invalid format: ${offsetId}`)}const first=offsetId[0];if("+"!==first&&"-"!==first)throw new DateTimeException(`Invalid ID for ZoneOffset, plus/minus not found when expected: ${offsetId}`);return"-"===first?ZoneOffset.ofHoursMinutesSeconds(-hours,-minutes,-seconds):ZoneOffset.ofHoursMinutesSeconds(hours,minutes,seconds)}static _parseNumber(offsetId,pos,precededByColon){if(precededByColon&&":"!==offsetId[pos-1])throw new DateTimeException(`Invalid ID for ZoneOffset, colon not found when expected: ${offsetId}`);const ch1=offsetId[pos],ch2=offsetId[pos+1];if(ch1<"0"||ch1>"9"||ch2<"0"||ch2>"9")throw new DateTimeException(`Invalid ID for ZoneOffset, non numeric characters found: ${offsetId}`);return 10*(ch1.charCodeAt(0)-48)+(ch2.charCodeAt(0)-48)}static ofHours(hours){return ZoneOffset.ofHoursMinutesSeconds(hours,0,0)}static ofHoursMinutes(hours,minutes){return ZoneOffset.ofHoursMinutesSeconds(hours,minutes,0)}static ofHoursMinutesSeconds(hours,minutes,seconds){ZoneOffset._validate(hours,minutes,seconds);const totalSeconds=hours*LocalTime.SECONDS_PER_HOUR+minutes*LocalTime.SECONDS_PER_MINUTE+seconds;return ZoneOffset.ofTotalSeconds(totalSeconds)}static ofTotalMinutes(totalMinutes){const totalSeconds=totalMinutes*LocalTime.SECONDS_PER_MINUTE;return ZoneOffset.ofTotalSeconds(totalSeconds)}static ofTotalSeconds(totalSeconds){if(totalSeconds%(15*LocalTime.SECONDS_PER_MINUTE)==0){const totalSecs=totalSeconds;let result=SECONDS_CACHE[totalSecs];return null==result&&(result=new ZoneOffset(totalSeconds),SECONDS_CACHE[totalSecs]=result,ID_CACHE[result.id()]=result),result}return new ZoneOffset(totalSeconds)}rules(){return this._rules}get(field){return this.getLong(field)}getLong(field){if(field===ChronoField.OFFSET_SECONDS)return this._totalSeconds;if(field instanceof ChronoField)throw new DateTimeException(`Unsupported field: ${field}`);return field.getFrom(this)}query(query){return requireNonNull(query,"query"),query===TemporalQueries.offset()||query===TemporalQueries.zone()?this:query===TemporalQueries.localDate()||query===TemporalQueries.localTime()||query===TemporalQueries.precision()||query===TemporalQueries.chronology()||query===TemporalQueries.zoneId()?null:query.queryFrom(this)}adjustInto(temporal){return temporal.with(ChronoField.OFFSET_SECONDS,this._totalSeconds)}compareTo(other){return requireNonNull(other,"other"),other._totalSeconds-this._totalSeconds}equals(obj){return this===obj||obj instanceof ZoneOffset&&this._totalSeconds===obj._totalSeconds}hashCode(){return this._totalSeconds}toString(){return this._id}}export function _init(){ZoneOffset.MAX_SECONDS=18*LocalTime.SECONDS_PER_HOUR,ZoneOffset.UTC=ZoneOffset.ofTotalSeconds(0),ZoneOffset.MIN=ZoneOffset.ofTotalSeconds(-ZoneOffset.MAX_SECONDS),ZoneOffset.MAX=ZoneOffset.ofTotalSeconds(ZoneOffset.MAX_SECONDS)}