/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{Enum}from"../Enum";import{requireNonNull}from"../assert";import{DateTimeException}from"../errors";import{MathUtil}from"../MathUtil";import{DayOfWeek}from"../DayOfWeek";import{LocalDate}from"../LocalDate";import{Month}from"../Month";import{Year}from"../Year";import{ChronoField}from"../temporal/ChronoField";import{ResolverStyle}from"../format/ResolverStyle";import{TemporalAdjusters}from"../temporal/TemporalAdjusters";export class IsoChronology extends Enum{static isLeapYear(prolepticYear){return 0==(3&prolepticYear)&&(prolepticYear%100!=0||prolepticYear%400==0)}_updateResolveMap(fieldValues,field,value){requireNonNull(fieldValues,"fieldValues"),requireNonNull(field,"field");const current=fieldValues.get(field);if(null!=current&&current!==value)throw new DateTimeException(`Invalid state, field: ${field} ${current} conflicts with ${field} ${value}`);fieldValues.put(field,value)}resolveDate(fieldValues,resolverStyle){if(fieldValues.containsKey(ChronoField.EPOCH_DAY))return LocalDate.ofEpochDay(fieldValues.remove(ChronoField.EPOCH_DAY));const prolepticMonth=fieldValues.remove(ChronoField.PROLEPTIC_MONTH);null!=prolepticMonth&&(resolverStyle!==ResolverStyle.LENIENT&&ChronoField.PROLEPTIC_MONTH.checkValidValue(prolepticMonth),this._updateResolveMap(fieldValues,ChronoField.MONTH_OF_YEAR,MathUtil.floorMod(prolepticMonth,12)+1),this._updateResolveMap(fieldValues,ChronoField.YEAR,MathUtil.floorDiv(prolepticMonth,12)));const yoeLong=fieldValues.remove(ChronoField.YEAR_OF_ERA);if(null!=yoeLong){resolverStyle!==ResolverStyle.LENIENT&&ChronoField.YEAR_OF_ERA.checkValidValue(yoeLong);const era=fieldValues.remove(ChronoField.ERA);if(null==era){const year=fieldValues.get(ChronoField.YEAR);resolverStyle===ResolverStyle.STRICT?null!=year?this._updateResolveMap(fieldValues,ChronoField.YEAR,year>0?yoeLong:MathUtil.safeSubtract(1,yoeLong)):fieldValues.put(ChronoField.YEAR_OF_ERA,yoeLong):this._updateResolveMap(fieldValues,ChronoField.YEAR,null==year||year>0?yoeLong:MathUtil.safeSubtract(1,yoeLong))}else if(1===era)this._updateResolveMap(fieldValues,ChronoField.YEAR,yoeLong);else{if(0!==era)throw new DateTimeException(`Invalid value for era: ${era}`);this._updateResolveMap(fieldValues,ChronoField.YEAR,MathUtil.safeSubtract(1,yoeLong))}}else fieldValues.containsKey(ChronoField.ERA)&&ChronoField.ERA.checkValidValue(fieldValues.get(ChronoField.ERA));if(fieldValues.containsKey(ChronoField.YEAR)){if(fieldValues.containsKey(ChronoField.MONTH_OF_YEAR)&&fieldValues.containsKey(ChronoField.DAY_OF_MONTH)){const y=ChronoField.YEAR.checkValidIntValue(fieldValues.remove(ChronoField.YEAR)),moy=fieldValues.remove(ChronoField.MONTH_OF_YEAR);let dom=fieldValues.remove(ChronoField.DAY_OF_MONTH);if(resolverStyle===ResolverStyle.LENIENT){const months=moy-1,days=dom-1;return LocalDate.of(y,1,1).plusMonths(months).plusDays(days)}return resolverStyle===ResolverStyle.SMART?(ChronoField.DAY_OF_MONTH.checkValidValue(dom),4===moy||6===moy||9===moy||11===moy?dom=Math.min(dom,30):2===moy&&(dom=Math.min(dom,Month.FEBRUARY.length(Year.isLeap(y)))),LocalDate.of(y,moy,dom)):LocalDate.of(y,moy,dom)}if(fieldValues.containsKey(ChronoField.DAY_OF_YEAR)){const y=ChronoField.YEAR.checkValidIntValue(fieldValues.remove(ChronoField.YEAR));if(resolverStyle===ResolverStyle.LENIENT){const days=MathUtil.safeSubtract(fieldValues.remove(ChronoField.DAY_OF_YEAR),1);return LocalDate.ofYearDay(y,1).plusDays(days)}const doy=ChronoField.DAY_OF_YEAR.checkValidIntValue(fieldValues.remove(ChronoField.DAY_OF_YEAR));return LocalDate.ofYearDay(y,doy)}if(fieldValues.containsKey(ChronoField.ALIGNED_WEEK_OF_YEAR)){if(fieldValues.containsKey(ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR)){const y=ChronoField.YEAR.checkValidIntValue(fieldValues.remove(ChronoField.YEAR));if(resolverStyle===ResolverStyle.LENIENT){const weeks=MathUtil.safeSubtract(fieldValues.remove(ChronoField.ALIGNED_WEEK_OF_YEAR),1),days=MathUtil.safeSubtract(fieldValues.remove(ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR),1);return LocalDate.of(y,1,1).plusWeeks(weeks).plusDays(days)}const aw=ChronoField.ALIGNED_WEEK_OF_YEAR.checkValidIntValue(fieldValues.remove(ChronoField.ALIGNED_WEEK_OF_YEAR)),ad=ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR.checkValidIntValue(fieldValues.remove(ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR)),date=LocalDate.of(y,1,1).plusDays(7*(aw-1)+(ad-1));if(resolverStyle===ResolverStyle.STRICT&&date.get(ChronoField.YEAR)!==y)throw new DateTimeException("Strict mode rejected date parsed to a different year");return date}if(fieldValues.containsKey(ChronoField.DAY_OF_WEEK)){const y=ChronoField.YEAR.checkValidIntValue(fieldValues.remove(ChronoField.YEAR));if(resolverStyle===ResolverStyle.LENIENT){const weeks=MathUtil.safeSubtract(fieldValues.remove(ChronoField.ALIGNED_WEEK_OF_YEAR),1),days=MathUtil.safeSubtract(fieldValues.remove(ChronoField.DAY_OF_WEEK),1);return LocalDate.of(y,1,1).plusWeeks(weeks).plusDays(days)}const aw=ChronoField.ALIGNED_WEEK_OF_YEAR.checkValidIntValue(fieldValues.remove(ChronoField.ALIGNED_WEEK_OF_YEAR)),dow=ChronoField.DAY_OF_WEEK.checkValidIntValue(fieldValues.remove(ChronoField.DAY_OF_WEEK)),date=LocalDate.of(y,1,1).plusWeeks(aw-1).with(TemporalAdjusters.nextOrSame(DayOfWeek.of(dow)));if(resolverStyle===ResolverStyle.STRICT&&date.get(ChronoField.YEAR)!==y)throw new DateTimeException("Strict mode rejected date parsed to a different month");return date}}}return null}date(temporal){return LocalDate.from(temporal)}}export function _init(){IsoChronology.INSTANCE=new IsoChronology("IsoChronology")}