/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{assert,requireNonNull}from"../assert";import{DateTimeParseException,NullPointerException}from"../errors";import{Period}from"../Period";import{ParsePosition}from"./ParsePosition";import{DateTimeBuilder}from"./DateTimeBuilder";import{DateTimeParseContext}from"./DateTimeParseContext";import{DateTimePrintContext}from"./DateTimePrintContext";import{DateTimeFormatterBuilder}from"./DateTimeFormatterBuilder";import{SignStyle}from"./SignStyle";import{StringBuilder}from"./StringBuilder";import{ResolverStyle}from"./ResolverStyle";import{IsoChronology}from"../chrono/IsoChronology";import{ChronoField}from"../temporal/ChronoField";import{createTemporalQuery}from"../temporal/TemporalQuery";export class DateTimeFormatter{static parsedExcessDays(){return DateTimeFormatter.PARSED_EXCESS_DAYS}static parsedLeapSecond(){return DateTimeFormatter.PARSED_LEAP_SECOND}static ofPattern(pattern){return(new DateTimeFormatterBuilder).appendPattern(pattern).toFormatter()}constructor(printerParser,locale,decimalStyle,resolverStyle,resolverFields,chrono=IsoChronology.INSTANCE,zone){assert(null!=printerParser),assert(null!=decimalStyle),assert(null!=resolverStyle),this._printerParser=printerParser,this._locale=locale,this._decimalStyle=decimalStyle,this._resolverStyle=resolverStyle,this._resolverFields=resolverFields,this._chrono=chrono,this._zone=zone}locale(){return this._locale}decimalStyle(){return this._decimalStyle}chronology(){return this._chrono}withChronology(chrono){return null!=this._chrono&&this._chrono.equals(chrono)?this:new DateTimeFormatter(this._printerParser,this._locale,this._decimalStyle,this._resolverStyle,this._resolverFields,chrono,this._zone)}withLocale(){return this}withResolverStyle(resolverStyle){return requireNonNull(resolverStyle,"resolverStyle"),resolverStyle.equals(this._resolverStyle)?this:new DateTimeFormatter(this._printerParser,this._locale,this._decimalStyle,resolverStyle,this._resolverFields,this._chrono,this._zone)}format(temporal){const buf=new StringBuilder(32);return this._formatTo(temporal,buf),buf.toString()}_formatTo(temporal,appendable){requireNonNull(temporal,"temporal"),requireNonNull(appendable,"appendable");const context=new DateTimePrintContext(temporal,this);this._printerParser.print(context,appendable)}parse(text,type){return 1===arguments.length?this.parse1(text):this.parse2(text,type)}parse1(text){requireNonNull(text,"text");try{return this._parseToBuilder(text,null).resolve(this._resolverStyle,this._resolverFields)}catch(ex){throw ex instanceof DateTimeParseException?ex:this._createError(text,ex)}}parse2(text,type){requireNonNull(text,"text"),requireNonNull(type,"type");try{return this._parseToBuilder(text,null).resolve(this._resolverStyle,this._resolverFields).build(type)}catch(ex){throw ex instanceof DateTimeParseException?ex:this._createError(text,ex)}}_createError(text,ex){let abbr="";return abbr=text.length>64?`${text.substring(0,64)}...`:text,new DateTimeParseException(`Text '${abbr}' could not be parsed: ${ex.message}`,text,0,ex)}_parseToBuilder(text,position){const pos=null!=position?position:new ParsePosition(0),result=this._parseUnresolved0(text,pos);if(null==result||pos.getErrorIndex()>=0||null==position&&pos.getIndex()<text.length){let abbr="";throw abbr=text.length>64?`${text.substr(0,64).toString()}...`:text,pos.getErrorIndex()>=0?new DateTimeParseException(`Text '${abbr}' could not be parsed at index ${pos.getErrorIndex()}`,text,pos.getErrorIndex()):new DateTimeParseException(`Text '${abbr}' could not be parsed, unparsed text found at index ${pos.getIndex()}`,text,pos.getIndex())}return result.toBuilder()}parseUnresolved(text,position){return this._parseUnresolved0(text,position)}_parseUnresolved0(text,position){assert(null!=text,"text",NullPointerException),assert(null!=position,"position",NullPointerException);const context=new DateTimeParseContext(this);let pos=position.getIndex();return pos=this._printerParser.parse(context,text,pos),pos<0?(position.setErrorIndex(~pos),null):(position.setIndex(pos),context.toParsed())}_toPrinterParser(optional){return this._printerParser.withOptional(optional)}toString(){const pattern=this._printerParser.toString();return 0===pattern.indexOf("[")?pattern:pattern.substring(1,pattern.length-1)}}export function _init(){DateTimeFormatter.ISO_LOCAL_DATE=(new DateTimeFormatterBuilder).appendValue(ChronoField.YEAR,4,10,SignStyle.EXCEEDS_PAD).appendLiteral("-").appendValue(ChronoField.MONTH_OF_YEAR,2).appendLiteral("-").appendValue(ChronoField.DAY_OF_MONTH,2).toFormatter(ResolverStyle.STRICT).withChronology(IsoChronology.INSTANCE),DateTimeFormatter.ISO_LOCAL_TIME=(new DateTimeFormatterBuilder).appendValue(ChronoField.HOUR_OF_DAY,2).appendLiteral(":").appendValue(ChronoField.MINUTE_OF_HOUR,2).optionalStart().appendLiteral(":").appendValue(ChronoField.SECOND_OF_MINUTE,2).optionalStart().appendFraction(ChronoField.NANO_OF_SECOND,0,9,!0).toFormatter(ResolverStyle.STRICT),DateTimeFormatter.ISO_LOCAL_DATE_TIME=(new DateTimeFormatterBuilder).parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_DATE).appendLiteral("T").append(DateTimeFormatter.ISO_LOCAL_TIME).toFormatter(ResolverStyle.STRICT).withChronology(IsoChronology.INSTANCE),DateTimeFormatter.ISO_INSTANT=(new DateTimeFormatterBuilder).parseCaseInsensitive().appendInstant().toFormatter(ResolverStyle.STRICT),DateTimeFormatter.ISO_OFFSET_DATE_TIME=(new DateTimeFormatterBuilder).parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_DATE_TIME).appendOffsetId().toFormatter(ResolverStyle.STRICT).withChronology(IsoChronology.INSTANCE),DateTimeFormatter.ISO_ZONED_DATE_TIME=(new DateTimeFormatterBuilder).append(DateTimeFormatter.ISO_OFFSET_DATE_TIME).optionalStart().appendLiteral("[").parseCaseSensitive().appendZoneId().appendLiteral("]").toFormatter(ResolverStyle.STRICT).withChronology(IsoChronology.INSTANCE),DateTimeFormatter.BASIC_ISO_DATE=(new DateTimeFormatterBuilder).appendValue(ChronoField.YEAR,4,10,SignStyle.EXCEEDS_PAD).appendValue(ChronoField.MONTH_OF_YEAR,2).appendValue(ChronoField.DAY_OF_MONTH,2).toFormatter(ResolverStyle.STRICT).withChronology(IsoChronology.INSTANCE),DateTimeFormatter.ISO_OFFSET_DATE=(new DateTimeFormatterBuilder).parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_DATE).appendOffsetId().toFormatter(ResolverStyle.STRICT).withChronology(IsoChronology.INSTANCE),DateTimeFormatter.ISO_OFFSET_TIME=(new DateTimeFormatterBuilder).parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_TIME).appendOffsetId().toFormatter(ResolverStyle.STRICT).withChronology(IsoChronology.INSTANCE),DateTimeFormatter.ISO_ORDINAL_DATE=(new DateTimeFormatterBuilder).appendValue(ChronoField.YEAR,4,10,SignStyle.EXCEEDS_PAD).appendLiteral("-").appendValue(ChronoField.DAY_OF_YEAR).toFormatter(ResolverStyle.STRICT),DateTimeFormatter.ISO_WEEK_DATE=(new DateTimeFormatterBuilder).appendValue(ChronoField.YEAR,4,10,SignStyle.EXCEEDS_PAD).appendLiteral("-W").appendValue(ChronoField.ALIGNED_WEEK_OF_YEAR).appendLiteral("-").appendValue(ChronoField.DAY_OF_WEEK).toFormatter(ResolverStyle.STRICT),DateTimeFormatter.ISO_DATE=(new DateTimeFormatterBuilder).parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_DATE).optionalStart().appendOffsetId().optionalEnd().toFormatter(ResolverStyle.STRICT).withChronology(IsoChronology.INSTANCE),DateTimeFormatter.ISO_TIME=(new DateTimeFormatterBuilder).parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_TIME).optionalStart().appendOffsetId().optionalEnd().toFormatter(ResolverStyle.STRICT),DateTimeFormatter.ISO_DATE_TIME=(new DateTimeFormatterBuilder).append(DateTimeFormatter.ISO_LOCAL_DATE_TIME).optionalStart().appendOffsetId().optionalEnd().toFormatter(ResolverStyle.STRICT).withChronology(IsoChronology.INSTANCE),DateTimeFormatter.PARSED_EXCESS_DAYS=createTemporalQuery("PARSED_EXCESS_DAYS",(temporal=>temporal instanceof DateTimeBuilder?temporal.excessDays:Period.ZERO)),DateTimeFormatter.PARSED_LEAP_SECOND=createTemporalQuery("PARSED_LEAP_SECOND",(temporal=>temporal instanceof DateTimeBuilder&&temporal.leapSecond))}