/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{UnsupportedTemporalTypeException,IllegalStateException}from"../errors";import{DayOfWeek}from"../DayOfWeek";import{Duration}from"../Duration";import{MathUtil}from"../MathUtil";import{LocalDate}from"../LocalDate";import{ChronoField}from"./ChronoField";import{ChronoUnit}from"./ChronoUnit";import{TemporalField}from"./TemporalField";import{TemporalUnit}from"./TemporalUnit";import{ValueRange}from"./ValueRange";import{IsoChronology}from"../chrono/IsoChronology";import{ResolverStyle}from"../format/ResolverStyle";export const IsoFields={};const QUARTER_DAYS=[0,90,181,273,0,91,182,274];class Field extends TemporalField{isDateBased(){return!0}isTimeBased(){return!1}_isIso(){return!0}static _getWeekRangeByLocalDate(date){const wby=Field._getWeekBasedYear(date);return ValueRange.of(1,Field._getWeekRangeByYear(wby))}static _getWeekRangeByYear(wby){const date=LocalDate.of(wby,1,1);return date.dayOfWeek()===DayOfWeek.THURSDAY||date.dayOfWeek()===DayOfWeek.WEDNESDAY&&date.isLeapYear()?53:52}static _getWeek(date){const dow0=date.dayOfWeek().ordinal(),doy0=date.dayOfYear()-1,doyThu0=doy0+(3-dow0);let firstMonDoy0=doyThu0-7*MathUtil.intDiv(doyThu0,7)-3;if(firstMonDoy0<-3&&(firstMonDoy0+=7),doy0<firstMonDoy0)return Field._getWeekRangeByLocalDate(date.withDayOfYear(180).minusYears(1)).maximum();let week=MathUtil.intDiv(doy0-firstMonDoy0,7)+1;return 53===week&&!1===(-3===firstMonDoy0||-2===firstMonDoy0&&date.isLeapYear())&&(week=1),week}static _getWeekBasedYear(date){let year=date.year(),doy=date.dayOfYear();if(doy<=3){doy-date.dayOfWeek().ordinal()<-2&&year--}else if(doy>=363){const dow=date.dayOfWeek().ordinal();doy=doy-363-(date.isLeapYear()?1:0),doy-dow>=0&&year++}return year}displayName(){return this.toString()}resolve(){return null}name(){return this.toString()}}class DAY_OF_QUARTER_FIELD extends Field{toString(){return"DayOfQuarter"}baseUnit(){return ChronoUnit.DAYS}rangeUnit(){return QUARTER_YEARS}range(){return ValueRange.of(1,90,92)}isSupportedBy(temporal){return temporal.isSupported(ChronoField.DAY_OF_YEAR)&&temporal.isSupported(ChronoField.MONTH_OF_YEAR)&&temporal.isSupported(ChronoField.YEAR)&&this._isIso(temporal)}rangeRefinedBy(temporal){if(!1===temporal.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: DayOfQuarter");const qoy=temporal.getLong(QUARTER_OF_YEAR);if(1===qoy){const year=temporal.getLong(ChronoField.YEAR);return IsoChronology.isLeapYear(year)?ValueRange.of(1,91):ValueRange.of(1,90)}return 2===qoy?ValueRange.of(1,91):3===qoy||4===qoy?ValueRange.of(1,92):this.range()}getFrom(temporal){if(!1===temporal.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: DayOfQuarter");const doy=temporal.get(ChronoField.DAY_OF_YEAR),moy=temporal.get(ChronoField.MONTH_OF_YEAR),year=temporal.getLong(ChronoField.YEAR);return doy-QUARTER_DAYS[MathUtil.intDiv(moy-1,3)+(IsoChronology.isLeapYear(year)?4:0)]}adjustInto(temporal,newValue){const curValue=this.getFrom(temporal);return this.range().checkValidValue(newValue,this),temporal.with(ChronoField.DAY_OF_YEAR,temporal.getLong(ChronoField.DAY_OF_YEAR)+(newValue-curValue))}resolve(fieldValues,partialTemporal,resolverStyle){const yearLong=fieldValues.get(ChronoField.YEAR),qoyLong=fieldValues.get(QUARTER_OF_YEAR);if(null==yearLong||null==qoyLong)return null;const y=ChronoField.YEAR.checkValidIntValue(yearLong),doq=fieldValues.get(DAY_OF_QUARTER);let date;if(resolverStyle===ResolverStyle.LENIENT){const qoy=qoyLong;date=LocalDate.of(y,1,1),date=date.plusMonths(MathUtil.safeMultiply(MathUtil.safeSubtract(qoy,1),3)),date=date.plusDays(MathUtil.safeSubtract(doq,1))}else{const qoy=QUARTER_OF_YEAR.range().checkValidIntValue(qoyLong,QUARTER_OF_YEAR);if(resolverStyle===ResolverStyle.STRICT){let max=92;1===qoy?max=IsoChronology.isLeapYear(y)?91:90:2===qoy&&(max=91),ValueRange.of(1,max).checkValidValue(doq,this)}else this.range().checkValidValue(doq,this);date=LocalDate.of(y,3*(qoy-1)+1,1).plusDays(doq-1)}return fieldValues.remove(this),fieldValues.remove(ChronoField.YEAR),fieldValues.remove(QUARTER_OF_YEAR),date}}class QUARTER_OF_YEAR_FIELD extends Field{toString(){return"QuarterOfYear"}baseUnit(){return QUARTER_YEARS}rangeUnit(){return ChronoUnit.YEARS}range(){return ValueRange.of(1,4)}isSupportedBy(temporal){return temporal.isSupported(ChronoField.MONTH_OF_YEAR)&&this._isIso(temporal)}rangeRefinedBy(temporal){return this.range()}getFrom(temporal){if(!1===temporal.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: QuarterOfYear");const moy=temporal.getLong(ChronoField.MONTH_OF_YEAR);return MathUtil.intDiv(moy+2,3)}adjustInto(temporal,newValue){const curValue=this.getFrom(temporal);return this.range().checkValidValue(newValue,this),temporal.with(ChronoField.MONTH_OF_YEAR,temporal.getLong(ChronoField.MONTH_OF_YEAR)+3*(newValue-curValue))}}class WEEK_OF_WEEK_BASED_YEAR_FIELD extends Field{toString(){return"WeekOfWeekBasedYear"}baseUnit(){return ChronoUnit.WEEKS}rangeUnit(){return WEEK_BASED_YEARS}range(){return ValueRange.of(1,52,53)}isSupportedBy(temporal){return temporal.isSupported(ChronoField.EPOCH_DAY)&&this._isIso(temporal)}rangeRefinedBy(temporal){if(!1===temporal.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: WeekOfWeekBasedYear");return Field._getWeekRangeByLocalDate(LocalDate.from(temporal))}getFrom(temporal){if(!1===temporal.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: WeekOfWeekBasedYear");return Field._getWeek(LocalDate.from(temporal))}adjustInto(temporal,newValue){return this.range().checkValidValue(newValue,this),temporal.plus(MathUtil.safeSubtract(newValue,this.getFrom(temporal)),ChronoUnit.WEEKS)}resolve(fieldValues,partialTemporal,resolverStyle){const wbyLong=fieldValues.get(WEEK_BASED_YEAR),dowLong=fieldValues.get(ChronoField.DAY_OF_WEEK);if(null==wbyLong||null==dowLong)return null;const wby=WEEK_BASED_YEAR.range().checkValidIntValue(wbyLong,WEEK_BASED_YEAR),wowby=fieldValues.get(WEEK_OF_WEEK_BASED_YEAR);let date;if(resolverStyle===ResolverStyle.LENIENT){let dow=dowLong,weeks=0;dow>7?(weeks=MathUtil.intDiv(dow-1,7),dow=MathUtil.intMod(dow-1,7)+1):dow<1&&(weeks=MathUtil.intDiv(dow,7)-1,dow=MathUtil.intMod(dow,7)+7),date=LocalDate.of(wby,1,4).plusWeeks(wowby-1).plusWeeks(weeks).with(ChronoField.DAY_OF_WEEK,dow)}else{const dow=ChronoField.DAY_OF_WEEK.checkValidIntValue(dowLong);if(resolverStyle===ResolverStyle.STRICT){const temp=LocalDate.of(wby,1,4);Field._getWeekRangeByLocalDate(temp).checkValidValue(wowby,this)}else this.range().checkValidValue(wowby,this);date=LocalDate.of(wby,1,4).plusWeeks(wowby-1).with(ChronoField.DAY_OF_WEEK,dow)}return fieldValues.remove(this),fieldValues.remove(WEEK_BASED_YEAR),fieldValues.remove(ChronoField.DAY_OF_WEEK),date}displayName(){return"Week"}}class WEEK_BASED_YEAR_FIELD extends Field{toString(){return"WeekBasedYear"}baseUnit(){return WEEK_BASED_YEARS}rangeUnit(){return ChronoUnit.FOREVER}range(){return ChronoField.YEAR.range()}isSupportedBy(temporal){return temporal.isSupported(ChronoField.EPOCH_DAY)&&this._isIso(temporal)}rangeRefinedBy(temporal){return ChronoField.YEAR.range()}getFrom(temporal){if(!1===temporal.isSupported(this))throw new UnsupportedTemporalTypeException("Unsupported field: WeekBasedYear");return Field._getWeekBasedYear(LocalDate.from(temporal))}adjustInto(temporal,newValue){if(!1===this.isSupportedBy(temporal))throw new UnsupportedTemporalTypeException("Unsupported field: WeekBasedYear");const newWby=this.range().checkValidIntValue(newValue,WEEK_BASED_YEAR),date=LocalDate.from(temporal),dow=date.get(ChronoField.DAY_OF_WEEK);let week=Field._getWeek(date);53===week&&52===Field._getWeekRangeByYear(newWby)&&(week=52);let resolved=LocalDate.of(newWby,1,4);const days=dow-resolved.get(ChronoField.DAY_OF_WEEK)+7*(week-1);return resolved=resolved.plusDays(days),temporal.with(resolved)}}class Unit extends TemporalUnit{constructor(name,estimatedDuration){super(),this._name=name,this._duration=estimatedDuration}duration(){return this._duration}isDurationEstimated(){return!0}isDateBased(){return!0}isTimeBased(){return!1}isSupportedBy(temporal){return temporal.isSupported(ChronoField.EPOCH_DAY)}addTo(temporal,periodToAdd){switch(this){case WEEK_BASED_YEARS:{const added=MathUtil.safeAdd(temporal.get(WEEK_BASED_YEAR),periodToAdd);return temporal.with(WEEK_BASED_YEAR,added)}case QUARTER_YEARS:return temporal.plus(MathUtil.intDiv(periodToAdd,256),ChronoUnit.YEARS).plus(3*MathUtil.intMod(periodToAdd,256),ChronoUnit.MONTHS);default:throw new IllegalStateException("Unreachable")}}between(temporal1,temporal2){switch(this){case WEEK_BASED_YEARS:return MathUtil.safeSubtract(temporal2.getLong(WEEK_BASED_YEAR),temporal1.getLong(WEEK_BASED_YEAR));case QUARTER_YEARS:return MathUtil.intDiv(temporal1.until(temporal2,ChronoUnit.MONTHS),3);default:throw new IllegalStateException("Unreachable")}}toString(){return this._name}}let DAY_OF_QUARTER=null,QUARTER_OF_YEAR=null,WEEK_OF_WEEK_BASED_YEAR=null,WEEK_BASED_YEAR=null,WEEK_BASED_YEARS=null,QUARTER_YEARS=null;export function _init(){DAY_OF_QUARTER=new DAY_OF_QUARTER_FIELD,QUARTER_OF_YEAR=new QUARTER_OF_YEAR_FIELD,WEEK_OF_WEEK_BASED_YEAR=new WEEK_OF_WEEK_BASED_YEAR_FIELD,WEEK_BASED_YEAR=new WEEK_BASED_YEAR_FIELD,WEEK_BASED_YEARS=new Unit("WeekBasedYears",Duration.ofSeconds(31556952)),QUARTER_YEARS=new Unit("QuarterYears",Duration.ofSeconds(7889238)),IsoFields.DAY_OF_QUARTER=DAY_OF_QUARTER,IsoFields.QUARTER_OF_YEAR=QUARTER_OF_YEAR,IsoFields.WEEK_OF_WEEK_BASED_YEAR=WEEK_OF_WEEK_BASED_YEAR,IsoFields.WEEK_BASED_YEAR=WEEK_BASED_YEAR,IsoFields.WEEK_BASED_YEARS=WEEK_BASED_YEARS,IsoFields.QUARTER_YEARS=QUARTER_YEARS,LocalDate.prototype.isoWeekOfWeekyear=function(){return this.get(IsoFields.WEEK_OF_WEEK_BASED_YEAR)},LocalDate.prototype.isoWeekyear=function(){return this.get(IsoFields.WEEK_BASED_YEAR)}}