/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull}from"../assert";import{IllegalStateException}from"../errors";import{TemporalAdjuster}from"./TemporalAdjuster";import{ChronoField}from"../temporal/ChronoField";import{ChronoUnit}from"../temporal/ChronoUnit";import{MathUtil}from"../MathUtil";export class TemporalAdjusters{static firstDayOfMonth(){return Impl.FIRST_DAY_OF_MONTH}static lastDayOfMonth(){return Impl.LAST_DAY_OF_MONTH}static firstDayOfNextMonth(){return Impl.FIRST_DAY_OF_NEXT_MONTH}static firstDayOfYear(){return Impl.FIRST_DAY_OF_YEAR}static lastDayOfYear(){return Impl.LAST_DAY_OF_YEAR}static firstDayOfNextYear(){return Impl.FIRST_DAY_OF_NEXT_YEAR}static firstInMonth(dayOfWeek){return requireNonNull(dayOfWeek,"dayOfWeek"),new DayOfWeekInMonth(1,dayOfWeek)}static lastInMonth(dayOfWeek){return requireNonNull(dayOfWeek,"dayOfWeek"),new DayOfWeekInMonth(-1,dayOfWeek)}static dayOfWeekInMonth(ordinal,dayOfWeek){return requireNonNull(dayOfWeek,"dayOfWeek"),new DayOfWeekInMonth(ordinal,dayOfWeek)}static next(dayOfWeek){return new RelativeDayOfWeek(2,dayOfWeek)}static nextOrSame(dayOfWeek){return new RelativeDayOfWeek(0,dayOfWeek)}static previous(dayOfWeek){return new RelativeDayOfWeek(3,dayOfWeek)}static previousOrSame(dayOfWeek){return new RelativeDayOfWeek(1,dayOfWeek)}}class Impl extends TemporalAdjuster{constructor(ordinal){super(),this._ordinal=ordinal}adjustInto(temporal){switch(this._ordinal){case 0:return temporal.with(ChronoField.DAY_OF_MONTH,1);case 1:return temporal.with(ChronoField.DAY_OF_MONTH,temporal.range(ChronoField.DAY_OF_MONTH).maximum());case 2:return temporal.with(ChronoField.DAY_OF_MONTH,1).plus(1,ChronoUnit.MONTHS);case 3:return temporal.with(ChronoField.DAY_OF_YEAR,1);case 4:return temporal.with(ChronoField.DAY_OF_YEAR,temporal.range(ChronoField.DAY_OF_YEAR).maximum());case 5:return temporal.with(ChronoField.DAY_OF_YEAR,1).plus(1,ChronoUnit.YEARS)}throw new IllegalStateException("Unreachable")}}Impl.FIRST_DAY_OF_MONTH=new Impl(0),Impl.LAST_DAY_OF_MONTH=new Impl(1),Impl.FIRST_DAY_OF_NEXT_MONTH=new Impl(2),Impl.FIRST_DAY_OF_YEAR=new Impl(3),Impl.LAST_DAY_OF_YEAR=new Impl(4),Impl.FIRST_DAY_OF_NEXT_YEAR=new Impl(5);class DayOfWeekInMonth extends TemporalAdjuster{constructor(ordinal,dow){super(),this._ordinal=ordinal,this._dowValue=dow.value()}adjustInto(temporal){if(this._ordinal>=0){const temp=temporal.with(ChronoField.DAY_OF_MONTH,1),curDow=temp.get(ChronoField.DAY_OF_WEEK);let dowDiff=MathUtil.intMod(this._dowValue-curDow+7,7);return dowDiff+=7*(this._ordinal-1),temp.plus(dowDiff,ChronoUnit.DAYS)}{const temp=temporal.with(ChronoField.DAY_OF_MONTH,temporal.range(ChronoField.DAY_OF_MONTH).maximum()),curDow=temp.get(ChronoField.DAY_OF_WEEK);let daysDiff=this._dowValue-curDow;return daysDiff=0===daysDiff?0:daysDiff>0?daysDiff-7:daysDiff,daysDiff-=7*(-this._ordinal-1),temp.plus(daysDiff,ChronoUnit.DAYS)}}}class RelativeDayOfWeek extends TemporalAdjuster{constructor(relative,dayOfWeek){super(),requireNonNull(dayOfWeek,"dayOfWeek"),this._relative=relative,this._dowValue=dayOfWeek.value()}adjustInto(temporal){const calDow=temporal.get(ChronoField.DAY_OF_WEEK);if(this._relative<2&&calDow===this._dowValue)return temporal;if(0==(1&this._relative)){const daysDiff=calDow-this._dowValue;return temporal.plus(daysDiff>=0?7-daysDiff:-daysDiff,ChronoUnit.DAYS)}{const daysDiff=this._dowValue-calDow;return temporal.minus(daysDiff>=0?7-daysDiff:-daysDiff,ChronoUnit.DAYS)}}}