const JsonWebTokenError=require("./lib/JsonWebTokenError"),NotBeforeError=require("./lib/NotBeforeError"),TokenExpiredError=require("./lib/TokenExpiredError"),decode=require("./decode"),timespan=require("./lib/timespan"),validateAsymmetricKey=require("./lib/validateAsymmetricKey"),PS_SUPPORTED=require("./lib/psSupported"),jws=require("jws"),{KeyObject:KeyObject,createSecretKey:createSecretKey,createPublicKey:createPublicKey}=require("crypto"),PUB_KEY_ALGS=["RS256","RS384","RS512"],EC_KEY_ALGS=["ES256","ES384","ES512"],RSA_KEY_ALGS=["RS256","RS384","RS512"],HS_ALGS=["HS256","HS384","HS512"];PS_SUPPORTED&&(PUB_KEY_ALGS.splice(PUB_KEY_ALGS.length,0,"PS256","PS384","PS512"),RSA_KEY_ALGS.splice(RSA_KEY_ALGS.length,0,"PS256","PS384","PS512")),module.exports=function(jwtString,secretOrPublicKey,options,callback){let done;if("function"!=typeof options||callback||(callback=options,options={}),options||(options={}),options=Object.assign({},options),done=callback||function(err,data){if(err)throw err;return data},options.clockTimestamp&&"number"!=typeof options.clockTimestamp)return done(new JsonWebTokenError("clockTimestamp must be a number"));if(void 0!==options.nonce&&("string"!=typeof options.nonce||""===options.nonce.trim()))return done(new JsonWebTokenError("nonce must be a non-empty string"));if(void 0!==options.allowInvalidAsymmetricKeyTypes&&"boolean"!=typeof options.allowInvalidAsymmetricKeyTypes)return done(new JsonWebTokenError("allowInvalidAsymmetricKeyTypes must be a boolean"));const clockTimestamp=options.clockTimestamp||Math.floor(Date.now()/1e3);if(!jwtString)return done(new JsonWebTokenError("jwt must be provided"));if("string"!=typeof jwtString)return done(new JsonWebTokenError("jwt must be a string"));const parts=jwtString.split(".");if(3!==parts.length)return done(new JsonWebTokenError("jwt malformed"));let decodedToken;try{decodedToken=decode(jwtString,{complete:!0})}catch(err){return done(err)}if(!decodedToken)return done(new JsonWebTokenError("invalid token"));const header=decodedToken.header;let getSecret;if("function"==typeof secretOrPublicKey){if(!callback)return done(new JsonWebTokenError("verify must be called asynchronous if secret or public key is provided as a callback"));getSecret=secretOrPublicKey}else getSecret=function(header,secretCallback){return secretCallback(null,secretOrPublicKey)};return getSecret(header,(function(err,secretOrPublicKey){if(err)return done(new JsonWebTokenError("error in secret or public key callback: "+err.message));const hasSignature=""!==parts[2].trim();if(!hasSignature&&secretOrPublicKey)return done(new JsonWebTokenError("jwt signature is required"));if(hasSignature&&!secretOrPublicKey)return done(new JsonWebTokenError("secret or public key must be provided"));if(!hasSignature&&!options.algorithms)return done(new JsonWebTokenError('please specify "none" in "algorithms" to verify unsigned tokens'));if(null!=secretOrPublicKey&&!(secretOrPublicKey instanceof KeyObject))try{secretOrPublicKey=createPublicKey(secretOrPublicKey)}catch(_){try{secretOrPublicKey=createSecretKey("string"==typeof secretOrPublicKey?Buffer.from(secretOrPublicKey):secretOrPublicKey)}catch(_){return done(new JsonWebTokenError("secretOrPublicKey is not valid key material"))}}if(options.algorithms||("secret"===secretOrPublicKey.type?options.algorithms=HS_ALGS:["rsa","rsa-pss"].includes(secretOrPublicKey.asymmetricKeyType)?options.algorithms=RSA_KEY_ALGS:"ec"===secretOrPublicKey.asymmetricKeyType?options.algorithms=EC_KEY_ALGS:options.algorithms=PUB_KEY_ALGS),-1===options.algorithms.indexOf(decodedToken.header.alg))return done(new JsonWebTokenError("invalid algorithm"));if(header.alg.startsWith("HS")&&"secret"!==secretOrPublicKey.type)return done(new JsonWebTokenError(`secretOrPublicKey must be a symmetric key when using ${header.alg}`));if(/^(?:RS|PS|ES)/.test(header.alg)&&"public"!==secretOrPublicKey.type)return done(new JsonWebTokenError(`secretOrPublicKey must be an asymmetric key when using ${header.alg}`));if(!options.allowInvalidAsymmetricKeyTypes)try{validateAsymmetricKey(header.alg,secretOrPublicKey)}catch(e){return done(e)}let valid;try{valid=jws.verify(jwtString,decodedToken.header.alg,secretOrPublicKey)}catch(e){return done(e)}if(!valid)return done(new JsonWebTokenError("invalid signature"));const payload=decodedToken.payload;if(void 0!==payload.nbf&&!options.ignoreNotBefore){if("number"!=typeof payload.nbf)return done(new JsonWebTokenError("invalid nbf value"));if(payload.nbf>clockTimestamp+(options.clockTolerance||0))return done(new NotBeforeError("jwt not active",new Date(1e3*payload.nbf)))}if(void 0!==payload.exp&&!options.ignoreExpiration){if("number"!=typeof payload.exp)return done(new JsonWebTokenError("invalid exp value"));if(clockTimestamp>=payload.exp+(options.clockTolerance||0))return done(new TokenExpiredError("jwt expired",new Date(1e3*payload.exp)))}if(options.audience){const audiences=Array.isArray(options.audience)?options.audience:[options.audience];if(!(Array.isArray(payload.aud)?payload.aud:[payload.aud]).some((function(targetAudience){return audiences.some((function(audience){return audience instanceof RegExp?audience.test(targetAudience):audience===targetAudience}))})))return done(new JsonWebTokenError("jwt audience invalid. expected: "+audiences.join(" or ")))}if(options.issuer){if("string"==typeof options.issuer&&payload.iss!==options.issuer||Array.isArray(options.issuer)&&-1===options.issuer.indexOf(payload.iss))return done(new JsonWebTokenError("jwt issuer invalid. expected: "+options.issuer))}if(options.subject&&payload.sub!==options.subject)return done(new JsonWebTokenError("jwt subject invalid. expected: "+options.subject));if(options.jwtid&&payload.jti!==options.jwtid)return done(new JsonWebTokenError("jwt jwtid invalid. expected: "+options.jwtid));if(options.nonce&&payload.nonce!==options.nonce)return done(new JsonWebTokenError("jwt nonce invalid. expected: "+options.nonce));if(options.maxAge){if("number"!=typeof payload.iat)return done(new JsonWebTokenError("iat required when maxAge is specified"));const maxAgeTimestamp=timespan(options.maxAge,payload.iat);if(void 0===maxAgeTimestamp)return done(new JsonWebTokenError('"maxAge" should be a number of seconds or string representing a timespan eg: "1d", "20h", 60'));if(clockTimestamp>=maxAgeTimestamp+(options.clockTolerance||0))return done(new TokenExpiredError("maxAge exceeded",new Date(1e3*maxAgeTimestamp)))}if(!0===options.complete){const signature=decodedToken.signature;return done(null,{header:header,payload:payload,signature:signature})}return done(null,payload)}))};