"use strict";var __importDefault=this&&this.__importDefault||function(mod){return mod&&mod.__esModule?mod:{default:mod}};Object.defineProperty(exports,"__esModule",{value:!0}),exports.Parser=void 0;const messages_1=require("./messages"),buffer_reader_1=require("./buffer-reader"),assert_1=__importDefault(require("assert")),CODE_LENGTH=1,LEN_LENGTH=4,HEADER_LENGTH=CODE_LENGTH+LEN_LENGTH,emptyBuffer=Buffer.allocUnsafe(0);class Parser{constructor(opts){if(this.buffer=emptyBuffer,this.bufferLength=0,this.bufferOffset=0,this.reader=new buffer_reader_1.BufferReader,"binary"===(null==opts?void 0:opts.mode))throw new Error("Binary mode not supported yet");this.mode=(null==opts?void 0:opts.mode)||"text"}parse(buffer,callback){this.mergeBuffer(buffer);const bufferFullLength=this.bufferOffset+this.bufferLength;let offset=this.bufferOffset;for(;offset+HEADER_LENGTH<=bufferFullLength;){const code=this.buffer[offset],length=this.buffer.readUInt32BE(offset+CODE_LENGTH),fullMessageLength=CODE_LENGTH+length;if(!(fullMessageLength+offset<=bufferFullLength))break;callback(this.handlePacket(offset+HEADER_LENGTH,code,length,this.buffer)),offset+=fullMessageLength}offset===bufferFullLength?(this.buffer=emptyBuffer,this.bufferLength=0,this.bufferOffset=0):(this.bufferLength=bufferFullLength-offset,this.bufferOffset=offset)}mergeBuffer(buffer){if(this.bufferLength>0){const newLength=this.bufferLength+buffer.byteLength;if(newLength+this.bufferOffset>this.buffer.byteLength){let newBuffer;if(newLength<=this.buffer.byteLength&&this.bufferOffset>=this.bufferLength)newBuffer=this.buffer;else{let newBufferLength=2*this.buffer.byteLength;for(;newLength>=newBufferLength;)newBufferLength*=2;newBuffer=Buffer.allocUnsafe(newBufferLength)}this.buffer.copy(newBuffer,0,this.bufferOffset,this.bufferOffset+this.bufferLength),this.buffer=newBuffer,this.bufferOffset=0}buffer.copy(this.buffer,this.bufferOffset+this.bufferLength),this.bufferLength=newLength}else this.buffer=buffer,this.bufferOffset=0,this.bufferLength=buffer.byteLength}handlePacket(offset,code,length,bytes){switch(code){case 50:return messages_1.bindComplete;case 49:return messages_1.parseComplete;case 51:return messages_1.closeComplete;case 110:return messages_1.noData;case 115:return messages_1.portalSuspended;case 99:return messages_1.copyDone;case 87:return messages_1.replicationStart;case 73:return messages_1.emptyQuery;case 68:return this.parseDataRowMessage(offset,length,bytes);case 67:return this.parseCommandCompleteMessage(offset,length,bytes);case 90:return this.parseReadyForQueryMessage(offset,length,bytes);case 65:return this.parseNotificationMessage(offset,length,bytes);case 82:return this.parseAuthenticationResponse(offset,length,bytes);case 83:return this.parseParameterStatusMessage(offset,length,bytes);case 75:return this.parseBackendKeyData(offset,length,bytes);case 69:return this.parseErrorMessage(offset,length,bytes,"error");case 78:return this.parseErrorMessage(offset,length,bytes,"notice");case 84:return this.parseRowDescriptionMessage(offset,length,bytes);case 116:return this.parseParameterDescriptionMessage(offset,length,bytes);case 71:return this.parseCopyInMessage(offset,length,bytes);case 72:return this.parseCopyOutMessage(offset,length,bytes);case 100:return this.parseCopyData(offset,length,bytes);default:assert_1.default.fail(`unknown message code: ${code.toString(16)}`)}}parseReadyForQueryMessage(offset,length,bytes){this.reader.setBuffer(offset,bytes);const status=this.reader.string(1);return new messages_1.ReadyForQueryMessage(length,status)}parseCommandCompleteMessage(offset,length,bytes){this.reader.setBuffer(offset,bytes);const text=this.reader.cstring();return new messages_1.CommandCompleteMessage(length,text)}parseCopyData(offset,length,bytes){const chunk=bytes.slice(offset,offset+(length-4));return new messages_1.CopyDataMessage(length,chunk)}parseCopyInMessage(offset,length,bytes){return this.parseCopyMessage(offset,length,bytes,"copyInResponse")}parseCopyOutMessage(offset,length,bytes){return this.parseCopyMessage(offset,length,bytes,"copyOutResponse")}parseCopyMessage(offset,length,bytes,messageName){this.reader.setBuffer(offset,bytes);const isBinary=0!==this.reader.byte(),columnCount=this.reader.int16(),message=new messages_1.CopyResponse(length,messageName,isBinary,columnCount);for(let i=0;i<columnCount;i++)message.columnTypes[i]=this.reader.int16();return message}parseNotificationMessage(offset,length,bytes){this.reader.setBuffer(offset,bytes);const processId=this.reader.int32(),channel=this.reader.cstring(),payload=this.reader.cstring();return new messages_1.NotificationResponseMessage(length,processId,channel,payload)}parseRowDescriptionMessage(offset,length,bytes){this.reader.setBuffer(offset,bytes);const fieldCount=this.reader.int16(),message=new messages_1.RowDescriptionMessage(length,fieldCount);for(let i=0;i<fieldCount;i++)message.fields[i]=this.parseField();return message}parseField(){const name=this.reader.cstring(),tableID=this.reader.int32(),columnID=this.reader.int16(),dataTypeID=this.reader.int32(),dataTypeSize=this.reader.int16(),dataTypeModifier=this.reader.int32(),mode=0===this.reader.int16()?"text":"binary";return new messages_1.Field(name,tableID,columnID,dataTypeID,dataTypeSize,dataTypeModifier,mode)}parseParameterDescriptionMessage(offset,length,bytes){this.reader.setBuffer(offset,bytes);const parameterCount=this.reader.int16(),message=new messages_1.ParameterDescriptionMessage(length,parameterCount);for(let i=0;i<parameterCount;i++)message.dataTypeIDs[i]=this.reader.int32();return message}parseDataRowMessage(offset,length,bytes){this.reader.setBuffer(offset,bytes);const fieldCount=this.reader.int16(),fields=new Array(fieldCount);for(let i=0;i<fieldCount;i++){const len=this.reader.int32();fields[i]=-1===len?null:this.reader.string(len)}return new messages_1.DataRowMessage(length,fields)}parseParameterStatusMessage(offset,length,bytes){this.reader.setBuffer(offset,bytes);const name=this.reader.cstring(),value=this.reader.cstring();return new messages_1.ParameterStatusMessage(length,name,value)}parseBackendKeyData(offset,length,bytes){this.reader.setBuffer(offset,bytes);const processID=this.reader.int32(),secretKey=this.reader.int32();return new messages_1.BackendKeyDataMessage(length,processID,secretKey)}parseAuthenticationResponse(offset,length,bytes){this.reader.setBuffer(offset,bytes);const code=this.reader.int32(),message={name:"authenticationOk",length:length};switch(code){case 0:break;case 3:8===message.length&&(message.name="authenticationCleartextPassword");break;case 5:if(12===message.length){message.name="authenticationMD5Password";const salt=this.reader.bytes(4);return new messages_1.AuthenticationMD5Password(length,salt)}break;case 10:let mechanism;message.name="authenticationSASL",message.mechanisms=[];do{mechanism=this.reader.cstring(),mechanism&&message.mechanisms.push(mechanism)}while(mechanism);break;case 11:message.name="authenticationSASLContinue",message.data=this.reader.string(length-8);break;case 12:message.name="authenticationSASLFinal",message.data=this.reader.string(length-8);break;default:throw new Error("Unknown authenticationOk message type "+code)}return message}parseErrorMessage(offset,length,bytes,name){this.reader.setBuffer(offset,bytes);const fields={};let fieldType=this.reader.string(1);for(;"\0"!==fieldType;)fields[fieldType]=this.reader.cstring(),fieldType=this.reader.string(1);const messageValue=fields.M,message="notice"===name?new messages_1.NoticeMessage(length,messageValue):new messages_1.DatabaseError(messageValue,length,name);return message.severity=fields.S,message.code=fields.C,message.detail=fields.D,message.hint=fields.H,message.position=fields.P,message.internalPosition=fields.p,message.internalQuery=fields.q,message.where=fields.W,message.schema=fields.s,message.table=fields.t,message.column=fields.c,message.dataType=fields.d,message.constraint=fields.n,message.file=fields.F,message.line=fields.L,message.routine=fields.R,message}}exports.Parser=Parser;