import{ServiceClient}from"@azure/core-client";import{isNode}from"@azure/core-util";import{createHttpHeaders,createPipelineRequest}from"@azure/core-rest-pipeline";import{AbortController}from"@azure/abort-controller";import{AuthenticationError,AuthenticationErrorName}from"../errors";import{getIdentityTokenEndpointSuffix}from"../util/identityTokenEndpoint";import{DefaultAuthorityHost,SDK_VERSION}from"../constants";import{tracingClient}from"../util/tracing";import{logger}from"../util/logging";const noCorrelationId="noCorrelationId";export function getIdentityClientAuthorityHost(options){let authorityHost=null==options?void 0:options.authorityHost;return isNode&&(authorityHost=null!=authorityHost?authorityHost:process.env.AZURE_AUTHORITY_HOST),null!=authorityHost?authorityHost:DefaultAuthorityHost}export class IdentityClient extends ServiceClient{constructor(options){var _a,_b;const packageDetails=`azsdk-js-identity/${SDK_VERSION}`,userAgentPrefix=(null===(_a=null==options?void 0:options.userAgentOptions)||void 0===_a?void 0:_a.userAgentPrefix)?`${options.userAgentOptions.userAgentPrefix} ${packageDetails}`:`${packageDetails}`,baseUri=getIdentityClientAuthorityHost(options);if(!baseUri.startsWith("https:"))throw new Error("The authorityHost address must use the 'https' protocol.");super(Object.assign(Object.assign({requestContentType:"application/json; charset=utf-8",retryOptions:{maxRetries:3}},options),{userAgentOptions:{userAgentPrefix:userAgentPrefix},baseUri:baseUri})),this.authorityHost=baseUri,this.abortControllers=new Map,this.allowLoggingAccountIdentifiers=null===(_b=null==options?void 0:options.loggingOptions)||void 0===_b?void 0:_b.allowLoggingAccountIdentifiers}async sendTokenRequest(request,expiresOnParser){logger.info(`IdentityClient: sending token request to [${request.url}]`);const response=await this.sendRequest(request);if(expiresOnParser=expiresOnParser||(responseBody=>Date.now()+1e3*responseBody.expires_in),!response.bodyAsText||200!==response.status&&201!==response.status){const error=new AuthenticationError(response.status,response.bodyAsText);throw logger.warning(`IdentityClient: authentication error. HTTP status: ${response.status}, ${error.errorResponse.errorDescription}`),error}{const parsedBody=JSON.parse(response.bodyAsText);if(!parsedBody.access_token)return null;this.logIdentifiers(response);const token={accessToken:{token:parsedBody.access_token,expiresOnTimestamp:expiresOnParser(parsedBody)},refreshToken:parsedBody.refresh_token};return logger.info(`IdentityClient: [${request.url}] token acquired, expires on ${token.accessToken.expiresOnTimestamp}`),token}}async refreshAccessToken(tenantId,clientId,scopes,refreshToken,clientSecret,expiresOnParser,options={}){if(void 0===refreshToken)return null;logger.info(`IdentityClient: refreshing access token with client ID: ${clientId}, scopes: ${scopes} started`);const refreshParams={grant_type:"refresh_token",client_id:clientId,refresh_token:refreshToken,scope:scopes};void 0!==clientSecret&&(refreshParams.client_secret=clientSecret);const query=new URLSearchParams(refreshParams);return tracingClient.withSpan("IdentityClient.refreshAccessToken",options,(async updatedOptions=>{try{const urlSuffix=getIdentityTokenEndpointSuffix(tenantId),request=createPipelineRequest({url:`${this.authorityHost}/${tenantId}/${urlSuffix}`,method:"POST",body:query.toString(),abortSignal:options.abortSignal,headers:createHttpHeaders({Accept:"application/json","Content-Type":"application/x-www-form-urlencoded"}),tracingOptions:updatedOptions.tracingOptions}),response=await this.sendTokenRequest(request,expiresOnParser);return logger.info(`IdentityClient: refreshed token for client ID: ${clientId}`),response}catch(err){if(err.name===AuthenticationErrorName&&"interaction_required"===err.errorResponse.error)return logger.info(`IdentityClient: interaction required for client ID: ${clientId}`),null;throw logger.warning(`IdentityClient: failed refreshing token for client ID: ${clientId}: ${err}`),err}}))}generateAbortSignal(correlationId){const controller=new AbortController,controllers=this.abortControllers.get(correlationId)||[];controllers.push(controller),this.abortControllers.set(correlationId,controllers);const existingOnAbort=controller.signal.onabort;return controller.signal.onabort=(...params)=>{this.abortControllers.set(correlationId,void 0),existingOnAbort&&existingOnAbort(...params)},controller.signal}abortRequests(correlationId){const key=correlationId||noCorrelationId,controllers=[...this.abortControllers.get(key)||[],...this.abortControllers.get(noCorrelationId)||[]];if(controllers.length){for(const controller of controllers)controller.abort();this.abortControllers.set(key,void 0)}}getCorrelationId(options){var _a;const parameter=null===(_a=null==options?void 0:options.body)||void 0===_a?void 0:_a.split("&").map((part=>part.split("="))).find((([key])=>"client-request-id"===key));return parameter&&parameter.length&&parameter[1]||noCorrelationId}async sendGetRequestAsync(url,options){const request=createPipelineRequest({url:url,method:"GET",body:null==options?void 0:options.body,headers:createHttpHeaders(null==options?void 0:options.headers),abortSignal:this.generateAbortSignal(noCorrelationId)}),response=await this.sendRequest(request);return this.logIdentifiers(response),{body:response.bodyAsText?JSON.parse(response.bodyAsText):void 0,headers:response.headers.toJSON(),status:response.status}}async sendPostRequestAsync(url,options){const request=createPipelineRequest({url:url,method:"POST",body:null==options?void 0:options.body,headers:createHttpHeaders(null==options?void 0:options.headers),abortSignal:this.generateAbortSignal(this.getCorrelationId(options))}),response=await this.sendRequest(request);return this.logIdentifiers(response),{body:response.bodyAsText?JSON.parse(response.bodyAsText):void 0,headers:response.headers.toJSON(),status:response.status}}logIdentifiers(response){if(!this.allowLoggingAccountIdentifiers||!response.bodyAsText)return;const unavailableUpn="No User Principal Name available";try{const accessToken=(response.parsedBody||JSON.parse(response.bodyAsText)).access_token;if(!accessToken)return;const base64Metadata=accessToken.split(".")[1],{appid:appid,upn:upn,tid:tid,oid:oid}=JSON.parse(Buffer.from(base64Metadata,"base64").toString("utf8"));logger.info(`[Authenticated account] Client ID: ${appid}. Tenant ID: ${tid}. User Principal Name: ${upn||unavailableUpn}. Object ID (user): ${oid}`)}catch(e){logger.warning("allowLoggingAccountIdentifiers was set, but we couldn't log the account information. Error:",e.message)}}}