import{delay}from"@azure/core-util";import{RestError,createHttpHeaders,createPipelineRequest}from"@azure/core-rest-pipeline";import{credentialLogger}from"../../util/logging";import{AuthenticationError}from"../../errors";import{tracingClient}from"../../util/tracing";import{imdsApiVersion,imdsEndpointPath,imdsHost}from"./constants";import{mapScopesToResource}from"./utils";const msiName="ManagedIdentityCredential - IMDS",logger=credentialLogger(msiName);function expiresOnParser(requestBody){if(requestBody.expires_on){const expires=1e3*+requestBody.expires_on;return logger.info(`${msiName}: Using expires_on: ${expires} (original value: ${requestBody.expires_on})`),expires}{const expires=Date.now()+1e3*requestBody.expires_in;return logger.info(`${msiName}: IMDS using expires_in: ${expires} (original value: ${requestBody.expires_in})`),expires}}function prepareRequestOptions(scopes,clientId,resourceId,options){var _a;const resource=mapScopesToResource(scopes);if(!resource)throw new Error(`${msiName}: Multiple scopes are not supported.`);const{skipQuery:skipQuery,skipMetadataHeader:skipMetadataHeader}=options||{};let query="";if(!skipQuery){const queryParameters={resource:resource,"api-version":imdsApiVersion};clientId&&(queryParameters.client_id=clientId),resourceId&&(queryParameters.msi_res_id=resourceId);query=`?${new URLSearchParams(queryParameters).toString()}`}const url=new URL(imdsEndpointPath,null!==(_a=process.env.AZURE_POD_IDENTITY_AUTHORITY_HOST)&&void 0!==_a?_a:imdsHost),rawHeaders={Accept:"application/json",Metadata:"true"};return skipMetadataHeader&&delete rawHeaders.Metadata,{url:`${url}${query}`,method:"GET",headers:createHttpHeaders(rawHeaders)}}export const imdsMsiRetryConfig={maxRetries:3,startDelayInMs:800,intervalIncrement:2};export const imdsMsi={async isAvailable({scopes:scopes,identityClient:identityClient,clientId:clientId,resourceId:resourceId,getTokenOptions:getTokenOptions={}}){const resource=mapScopesToResource(scopes);if(!resource)return logger.info(`${msiName}: Unavailable. Multiple scopes are not supported.`),!1;if(process.env.AZURE_POD_IDENTITY_AUTHORITY_HOST)return!0;if(!identityClient)throw new Error("Missing IdentityClient");const requestOptions=prepareRequestOptions(resource,clientId,resourceId,{skipMetadataHeader:!0,skipQuery:!0});return tracingClient.withSpan("ManagedIdentityCredential-pingImdsEndpoint",getTokenOptions,(async options=>{var _a,_b;requestOptions.tracingOptions=options.tracingOptions;try{const request=createPipelineRequest(requestOptions);request.timeout=null!==(_b=null===(_a=options.requestOptions)||void 0===_a?void 0:_a.timeout)&&void 0!==_b?_b:300,request.allowInsecureConnection=!0;try{logger.info(`${msiName}: Pinging the Azure IMDS endpoint`),await identityClient.sendRequest(request)}catch(err){if("RestError"===err.name&&err.code===RestError.REQUEST_SEND_ERROR||"AbortError"===err.name||"ENETUNREACH"===err.code||"ECONNREFUSED"===err.code||"EHOSTDOWN"===err.code)return logger.info(`${msiName}: The Azure IMDS endpoint is unavailable`),!1}return logger.info(`${msiName}: The Azure IMDS endpoint is available`),!0}catch(err){throw logger.info(`${msiName}: Error when creating the WebResource for the Azure IMDS endpoint: ${err.message}`),err}}))},async getToken(configuration,getTokenOptions={}){const{identityClient:identityClient,scopes:scopes,clientId:clientId,resourceId:resourceId}=configuration;logger.info(`${msiName}: Using the Azure IMDS endpoint coming from the environment variable MSI_ENDPOINT=${process.env.MSI_ENDPOINT}, and using the cloud shell to proceed with the authentication.`);let nextDelayInMs=imdsMsiRetryConfig.startDelayInMs;for(let retries=0;retries<imdsMsiRetryConfig.maxRetries;retries++)try{const request=createPipelineRequest(Object.assign(Object.assign({abortSignal:getTokenOptions.abortSignal},prepareRequestOptions(scopes,clientId,resourceId)),{allowInsecureConnection:!0})),tokenResponse=await identityClient.sendTokenRequest(request,expiresOnParser);return tokenResponse&&tokenResponse.accessToken||null}catch(error){if(404===error.statusCode){await delay(nextDelayInMs),nextDelayInMs*=imdsMsiRetryConfig.intervalIncrement;continue}throw error}throw new AuthenticationError(404,`${msiName}: Failed to retrieve IMDS token after ${imdsMsiRetryConfig.maxRetries} retries.`)}};