import{IdentityClient}from"../../client/identityClient";import{AuthenticationError,CredentialUnavailableError}from"../../errors";import{credentialLogger,formatError,formatSuccess}from"../../util/logging";import{appServiceMsi2017}from"./appServiceMsi2017";import{tracingClient}from"../../util/tracing";import{cloudShellMsi}from"./cloudShellMsi";import{imdsMsi}from"./imdsMsi";import{arcMsi}from"./arcMsi";import{tokenExchangeMsi}from"./tokenExchangeMsi";import{fabricMsi}from"./fabricMsi";import{appServiceMsi2019}from"./appServiceMsi2019";const logger=credentialLogger("ManagedIdentityCredential");export class ManagedIdentityCredential{constructor(clientIdOrOptions,options){let _options;if(this.isEndpointUnavailable=null,"string"==typeof clientIdOrOptions?(this.clientId=clientIdOrOptions,_options=options):(this.clientId=null==clientIdOrOptions?void 0:clientIdOrOptions.clientId,_options=clientIdOrOptions),this.resourceId=null==_options?void 0:_options.resourceId,this.clientId&&this.resourceId)throw new Error(`${ManagedIdentityCredential.name} - Client Id and Resource Id can't be provided at the same time.`);this.identityClient=new IdentityClient(_options),this.isAvailableIdentityClient=new IdentityClient(Object.assign(Object.assign({},_options),{retryOptions:{maxRetries:0}}))}async cachedAvailableMSI(scopes,getTokenOptions){if(this.cachedMSI)return this.cachedMSI;const MSIs=[arcMsi,fabricMsi,appServiceMsi2019,appServiceMsi2017,cloudShellMsi,tokenExchangeMsi(),imdsMsi];for(const msi of MSIs)if(await msi.isAvailable({scopes:scopes,identityClient:this.isAvailableIdentityClient,clientId:this.clientId,resourceId:this.resourceId,getTokenOptions:getTokenOptions}))return this.cachedMSI=msi,msi;throw new CredentialUnavailableError(`${ManagedIdentityCredential.name} - No MSI credential available`)}async authenticateManagedIdentity(scopes,getTokenOptions){const{span:span,updatedOptions:updatedOptions}=tracingClient.startSpan(`${ManagedIdentityCredential.name}.authenticateManagedIdentity`,getTokenOptions);try{return(await this.cachedAvailableMSI(scopes,updatedOptions)).getToken({identityClient:this.identityClient,scopes:scopes,clientId:this.clientId,resourceId:this.resourceId},updatedOptions)}catch(err){throw span.setStatus({status:"error",error:err}),err}finally{span.end()}}async getToken(scopes,options){let result=null;const{span:span,updatedOptions:updatedOptions}=tracingClient.startSpan(`${ManagedIdentityCredential.name}.getToken`,options);try{if(!0===this.isEndpointUnavailable){const error=new CredentialUnavailableError("The managed identity endpoint is not currently available");throw logger.getToken.info(formatError(scopes,error)),error}if(result=await this.authenticateManagedIdentity(scopes,updatedOptions),null===result){this.isEndpointUnavailable=!0;const error=new CredentialUnavailableError("The managed identity endpoint was reached, yet no tokens were received.");throw logger.getToken.info(formatError(scopes,error)),error}return this.isEndpointUnavailable=!1,logger.getToken.info(formatSuccess(scopes)),result}catch(err){if("AuthenticationRequiredError"===err.name)throw err;if(span.setStatus({status:"error",error:err}),"ENETUNREACH"===err.code){const error=new CredentialUnavailableError(`${ManagedIdentityCredential.name}: Unavailable. Network unreachable. Message: ${err.message}`);throw logger.getToken.info(formatError(scopes,error)),error}if("EHOSTUNREACH"===err.code){const error=new CredentialUnavailableError(`${ManagedIdentityCredential.name}: Unavailable. No managed identity endpoint found. Message: ${err.message}`);throw logger.getToken.info(formatError(scopes,error)),error}if(400===err.statusCode)throw new CredentialUnavailableError(`${ManagedIdentityCredential.name}: The managed identity endpoint is indicating there's no available identity. Message: ${err.message}`);if(void 0===err.statusCode)throw new CredentialUnavailableError(`${ManagedIdentityCredential.name}: Authentication failed. Message ${err.message}`);throw new AuthenticationError(err.statusCode,{error:`${ManagedIdentityCredential.name} authentication failed.`,error_description:err.message})}finally{span.end()}}}