import{KnownKeyOperations}from"./keysModels";import{RemoteCryptographyProvider}from"./cryptography/remoteCryptographyProvider";import{randomBytes}from"./cryptography/crypto";import{RsaCryptographyProvider}from"./cryptography/rsaCryptographyProvider";import{AesCryptographyProvider}from"./cryptography/aesCryptographyProvider";import{tracingClient}from"./tracing";import{isRestError}from"@azure/core-rest-pipeline";import{logger}from"./log";export class CryptographyClient{constructor(key,credential,pipelineOptions={}){"string"==typeof key?(this.key={kind:"identifier",value:key},this.remoteProvider=new RemoteCryptographyProvider(key,credential,pipelineOptions)):"name"in key?(this.key={kind:"KeyVaultKey",value:key},this.remoteProvider=new RemoteCryptographyProvider(key,credential,pipelineOptions)):this.key={kind:"JsonWebKey",value:key}}get vaultUrl(){var _a;return(null===(_a=this.remoteProvider)||void 0===_a?void 0:_a.vaultUrl)||""}get keyID(){return"identifier"===this.key.kind||"remoteOnlyIdentifier"===this.key.kind?this.key.value:"KeyVaultKey"===this.key.kind?this.key.value.id:this.key.value.kid}encrypt(...args){const[parameters,options]=this.disambiguateEncryptArguments(args);return tracingClient.withSpan("CryptographyClient.encrypt",options,(async updatedOptions=>{this.ensureValid(await this.fetchKey(updatedOptions),KnownKeyOperations.Encrypt),this.initializeIV(parameters);const provider=await this.getProvider("encrypt",parameters.algorithm,updatedOptions);try{return provider.encrypt(parameters,updatedOptions)}catch(error){if(this.remoteProvider)return this.remoteProvider.encrypt(parameters,updatedOptions);throw error}}))}initializeIV(parameters){const algorithmsRequiringIV=["A128CBC","A128CBCPAD","A192CBC","A192CBCPAD","A256CBC","A256CBCPAD"];if(parameters.algorithm in algorithmsRequiringIV)try{const cbcParams=parameters;cbcParams.iv||(cbcParams.iv=randomBytes(16))}catch(e){throw new Error(`Unable to initialize IV for algorithm ${parameters.algorithm}. You may pass a valid IV to avoid this error. Error: ${e.message}`)}}disambiguateEncryptArguments(args){return"string"==typeof args[0]?[{algorithm:args[0],plaintext:args[1]},args[2]||{}]:[args[0],args[1]||{}]}decrypt(...args){const[parameters,options]=this.disambiguateDecryptArguments(args);return tracingClient.withSpan("CryptographyClient.decrypt",options,(async updatedOptions=>{this.ensureValid(await this.fetchKey(updatedOptions),KnownKeyOperations.Decrypt);const provider=await this.getProvider("decrypt",parameters.algorithm,updatedOptions);try{return provider.decrypt(parameters,updatedOptions)}catch(error){if(this.remoteProvider)return this.remoteProvider.decrypt(parameters,updatedOptions);throw error}}))}disambiguateDecryptArguments(args){return"string"==typeof args[0]?[{algorithm:args[0],ciphertext:args[1]},args[2]||{}]:[args[0],args[1]||{}]}wrapKey(algorithm,key,options={}){return tracingClient.withSpan("CryptographyClient.wrapKey",options,(async updatedOptions=>{this.ensureValid(await this.fetchKey(updatedOptions),KnownKeyOperations.WrapKey);const provider=await this.getProvider("wrapKey",algorithm,updatedOptions);try{return provider.wrapKey(algorithm,key,updatedOptions)}catch(err){if(this.remoteProvider)return this.remoteProvider.wrapKey(algorithm,key,options);throw err}}))}unwrapKey(algorithm,encryptedKey,options={}){return tracingClient.withSpan("CryptographyClient.unwrapKey",options,(async updatedOptions=>{this.ensureValid(await this.fetchKey(updatedOptions),KnownKeyOperations.UnwrapKey);const provider=await this.getProvider("unwrapKey",algorithm,updatedOptions);try{return provider.unwrapKey(algorithm,encryptedKey,updatedOptions)}catch(err){if(this.remoteProvider)return this.remoteProvider.unwrapKey(algorithm,encryptedKey,options);throw err}}))}sign(algorithm,digest,options={}){return tracingClient.withSpan("CryptographyClient.sign",options,(async updatedOptions=>{this.ensureValid(await this.fetchKey(updatedOptions),KnownKeyOperations.Sign);const provider=await this.getProvider("sign",algorithm,updatedOptions);try{return provider.sign(algorithm,digest,updatedOptions)}catch(err){if(this.remoteProvider)return this.remoteProvider.sign(algorithm,digest,updatedOptions);throw err}}))}verify(algorithm,digest,signature,options={}){return tracingClient.withSpan("CryptographyClient.verify",options,(async updatedOptions=>{this.ensureValid(await this.fetchKey(updatedOptions),KnownKeyOperations.Verify);const provider=await this.getProvider("verify",algorithm,updatedOptions);try{return provider.verify(algorithm,digest,signature,updatedOptions)}catch(err){if(this.remoteProvider)return this.remoteProvider.verify(algorithm,digest,signature,updatedOptions);throw err}}))}signData(algorithm,data,options={}){return tracingClient.withSpan("CryptographyClient.signData",options,(async updatedOptions=>{this.ensureValid(await this.fetchKey(updatedOptions),KnownKeyOperations.Sign);const provider=await this.getProvider("signData",algorithm,updatedOptions);try{return provider.signData(algorithm,data,updatedOptions)}catch(err){if(this.remoteProvider)return this.remoteProvider.signData(algorithm,data,options);throw err}}))}verifyData(algorithm,data,signature,options={}){return tracingClient.withSpan("CryptographyClient.verifyData",options,(async updatedOptions=>{this.ensureValid(await this.fetchKey(updatedOptions),KnownKeyOperations.Verify);const provider=await this.getProvider("verifyData",algorithm,updatedOptions);try{return provider.verifyData(algorithm,data,signature,updatedOptions)}catch(err){if(this.remoteProvider)return this.remoteProvider.verifyData(algorithm,data,signature,updatedOptions);throw err}}))}async getKeyMaterial(options){const key=await this.fetchKey(options);switch(key.kind){case"JsonWebKey":return key.value;case"KeyVaultKey":return key.value.key;default:return}}async fetchKey(options){if("identifier"===this.key.kind){let key;try{key=await this.remoteProvider.getKey(options)}catch(e){if(!isRestError(e)||403!==e.statusCode)throw e;logger.verbose(`Permission denied to get key ${this.key.value}. Falling back to remote operation.`),this.key={kind:"remoteOnlyIdentifier",value:this.key.value}}key&&(this.key={kind:"KeyVaultKey",value:key})}return this.key}async getProvider(operation,algorithm,options){if(!this.providers){const keyMaterial=await this.getKeyMaterial(options);this.providers=[],keyMaterial&&this.providers.push(new RsaCryptographyProvider(keyMaterial),new AesCryptographyProvider(keyMaterial)),this.remoteProvider&&this.providers.push(this.remoteProvider)}const providers=this.providers.filter((p=>p.isSupported(algorithm,operation)));if(0===providers.length)throw new Error(`Unable to support operation: "${operation}" with algorithm: "${algorithm}" ${"JsonWebKey"===this.key.kind?"using a local JsonWebKey":""}`);return providers[0]}ensureValid(key,operation){var _a;if("KeyVaultKey"===key.kind){const keyOps=key.value.keyOperations,{notBefore:notBefore,expiresOn:expiresOn}=key.value.properties,now=new Date;if(notBefore&&now<notBefore)throw new Error(`Key ${key.value.id} can't be used before ${notBefore.toISOString()}`);if(expiresOn&&now>expiresOn)throw new Error(`Key ${key.value.id} expired at ${expiresOn.toISOString()}`);if(operation&&keyOps&&!(null==keyOps?void 0:keyOps.includes(operation)))throw new Error(`Operation ${operation} is not supported on key ${key.value.id}`)}else if("JsonWebKey"===key.kind&&operation&&key.value.keyOps&&!(null===(_a=key.value.keyOps)||void 0===_a?void 0:_a.includes(operation)))throw new Error(`Operation ${operation} is not supported on key ${key.value.kid}`)}}