/*! @azure/msal-browser v2.37.1 2023-06-07 */
"use strict";import{__extends,__awaiter,__generator,__spread}from"../_virtual/_tslib.js";import{CredentialType,RefreshTokenEntity,CacheManager,AccessTokenEntity,IdTokenEntity,AccountEntity,ClientAuthError,AppMetadataEntity,ServerTelemetryEntity,AuthorityMetadataEntity,Constants,PersistentCacheKeys,ThrottlingEntity,StringUtils,ProtocolUtils,CcsCredentialType,IdToken,DEFAULT_CRYPTO_IMPLEMENTATION}from"@azure/msal-common";import{BrowserAuthError}from"../error/BrowserAuthError.js";import{BrowserCacheLocation,StaticCacheKeys,InMemoryCacheKeys,TemporaryCacheKeys}from"../utils/BrowserConstants.js";import{BrowserStorage}from"./BrowserStorage.js";import{MemoryStorage}from"./MemoryStorage.js";import{BrowserProtocolUtils}from"../utils/BrowserProtocolUtils.js";var BrowserCacheManager=function(_super){function BrowserCacheManager(clientId,cacheConfig,cryptoImpl,logger){var _this=_super.call(this,clientId,cryptoImpl,logger)||this;return _this.COOKIE_LIFE_MULTIPLIER=864e5,_this.cacheConfig=cacheConfig,_this.logger=logger,_this.internalStorage=new MemoryStorage,_this.browserStorage=_this.setupBrowserStorage(_this.cacheConfig.cacheLocation),_this.temporaryCacheStorage=_this.setupTemporaryCacheStorage(_this.cacheConfig.temporaryCacheLocation,_this.cacheConfig.cacheLocation),cacheConfig.cacheMigrationEnabled&&(_this.migrateCacheEntries(),_this.createKeyMaps()),_this}return __extends(BrowserCacheManager,_super),BrowserCacheManager.prototype.setupBrowserStorage=function(cacheLocation){switch(cacheLocation){case BrowserCacheLocation.LocalStorage:case BrowserCacheLocation.SessionStorage:try{return new BrowserStorage(cacheLocation)}catch(e){this.logger.verbose(e);break}}return this.cacheConfig.cacheLocation=BrowserCacheLocation.MemoryStorage,new MemoryStorage},BrowserCacheManager.prototype.setupTemporaryCacheStorage=function(temporaryCacheLocation,cacheLocation){switch(cacheLocation){case BrowserCacheLocation.LocalStorage:case BrowserCacheLocation.SessionStorage:try{return new BrowserStorage(temporaryCacheLocation||BrowserCacheLocation.SessionStorage)}catch(e){return this.logger.verbose(e),this.internalStorage}case BrowserCacheLocation.MemoryStorage:default:return this.internalStorage}},BrowserCacheManager.prototype.migrateCacheEntries=function(){var _this=this,idTokenKey=Constants.CACHE_PREFIX+"."+PersistentCacheKeys.ID_TOKEN,clientInfoKey=Constants.CACHE_PREFIX+"."+PersistentCacheKeys.CLIENT_INFO,errorKey=Constants.CACHE_PREFIX+"."+PersistentCacheKeys.ERROR,errorDescKey=Constants.CACHE_PREFIX+"."+PersistentCacheKeys.ERROR_DESC,values=[this.browserStorage.getItem(idTokenKey),this.browserStorage.getItem(clientInfoKey),this.browserStorage.getItem(errorKey),this.browserStorage.getItem(errorDescKey)];[PersistentCacheKeys.ID_TOKEN,PersistentCacheKeys.CLIENT_INFO,PersistentCacheKeys.ERROR,PersistentCacheKeys.ERROR_DESC].forEach((function(cacheKey,index){return _this.migrateCacheEntry(cacheKey,values[index])}))},BrowserCacheManager.prototype.migrateCacheEntry=function(newKey,value){value&&this.setTemporaryCache(newKey,value,!0)},BrowserCacheManager.prototype.createKeyMaps=function(){var _this=this;this.logger.trace("BrowserCacheManager - createKeyMaps called.");var accountKeys=this.getItem(StaticCacheKeys.ACCOUNT_KEYS),tokenKeys=this.getItem(StaticCacheKeys.TOKEN_KEYS+"."+this.clientId);accountKeys&&tokenKeys?this.logger.verbose("BrowserCacheManager:createKeyMaps - account and token key maps already exist, skipping migration."):this.browserStorage.getKeys().forEach((function(key){var value;if(_this.isCredentialKey(key)&&(value=_this.getItem(key))){var credObj=_this.validateAndParseJson(value);if(credObj&&credObj.hasOwnProperty("credentialType"))switch(credObj.credentialType){case CredentialType.ID_TOKEN:if(IdTokenEntity.isIdTokenEntity(credObj)){_this.logger.trace("BrowserCacheManager:createKeyMaps - idToken found, saving key to token key map"),_this.logger.tracePii("BrowserCacheManager:createKeyMaps - idToken with key: "+key+" found, saving key to token key map");var idTokenEntity=CacheManager.toObject(new IdTokenEntity,credObj),newKey=_this.updateCredentialCacheKey(key,idTokenEntity);return void _this.addTokenKey(newKey,CredentialType.ID_TOKEN)}_this.logger.trace("BrowserCacheManager:createKeyMaps - key found matching idToken schema with value containing idToken credentialType field but value failed IdTokenEntity validation, skipping."),_this.logger.tracePii("BrowserCacheManager:createKeyMaps - failed idToken validation on key: "+key);break;case CredentialType.ACCESS_TOKEN:case CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME:if(AccessTokenEntity.isAccessTokenEntity(credObj)){_this.logger.trace("BrowserCacheManager:createKeyMaps - accessToken found, saving key to token key map"),_this.logger.tracePii("BrowserCacheManager:createKeyMaps - accessToken with key: "+key+" found, saving key to token key map");var accessTokenEntity=CacheManager.toObject(new AccessTokenEntity,credObj);newKey=_this.updateCredentialCacheKey(key,accessTokenEntity);return void _this.addTokenKey(newKey,CredentialType.ACCESS_TOKEN)}_this.logger.trace("BrowserCacheManager:createKeyMaps - key found matching accessToken schema with value containing accessToken credentialType field but value failed AccessTokenEntity validation, skipping."),_this.logger.tracePii("BrowserCacheManager:createKeyMaps - failed accessToken validation on key: "+key);break;case CredentialType.REFRESH_TOKEN:if(RefreshTokenEntity.isRefreshTokenEntity(credObj)){_this.logger.trace("BrowserCacheManager:createKeyMaps - refreshToken found, saving key to token key map"),_this.logger.tracePii("BrowserCacheManager:createKeyMaps - refreshToken with key: "+key+" found, saving key to token key map");var refreshTokenEntity=CacheManager.toObject(new RefreshTokenEntity,credObj);newKey=_this.updateCredentialCacheKey(key,refreshTokenEntity);return void _this.addTokenKey(newKey,CredentialType.REFRESH_TOKEN)}_this.logger.trace("BrowserCacheManager:createKeyMaps - key found matching refreshToken schema with value containing refreshToken credentialType field but value failed RefreshTokenEntity validation, skipping."),_this.logger.tracePii("BrowserCacheManager:createKeyMaps - failed refreshToken validation on key: "+key)}}if(_this.isAccountKey(key)&&(value=_this.getItem(key))){var accountObj=_this.validateAndParseJson(value);accountObj&&AccountEntity.isAccountEntity(accountObj)&&(_this.logger.trace("BrowserCacheManager:createKeyMaps - account found, saving key to account key map"),_this.logger.tracePii("BrowserCacheManager:createKeyMaps - account with key: "+key+" found, saving key to account key map"),_this.addAccountKeyToMap(key))}}))},BrowserCacheManager.prototype.validateAndParseJson=function(jsonValue){try{var parsedJson=JSON.parse(jsonValue);return parsedJson&&"object"==typeof parsedJson?parsedJson:null}catch(error){return null}},BrowserCacheManager.prototype.getItem=function(key){return this.browserStorage.getItem(key)},BrowserCacheManager.prototype.setItem=function(key,value){this.browserStorage.setItem(key,value)},BrowserCacheManager.prototype.getAccount=function(accountKey){this.logger.trace("BrowserCacheManager.getAccount called");var account=this.getItem(accountKey);if(!account)return this.removeAccountKeyFromMap(accountKey),null;var parsedAccount=this.validateAndParseJson(account);return parsedAccount&&AccountEntity.isAccountEntity(parsedAccount)?CacheManager.toObject(new AccountEntity,parsedAccount):(this.removeAccountKeyFromMap(accountKey),null)},BrowserCacheManager.prototype.setAccount=function(account){this.logger.trace("BrowserCacheManager.setAccount called");var key=account.generateAccountKey();this.setItem(key,JSON.stringify(account)),this.addAccountKeyToMap(key)},BrowserCacheManager.prototype.getAccountKeys=function(){this.logger.trace("BrowserCacheManager.getAccountKeys called");var accountKeys=this.getItem(StaticCacheKeys.ACCOUNT_KEYS);return accountKeys?JSON.parse(accountKeys):(this.logger.verbose("BrowserCacheManager.getAccountKeys - No account keys found"),[])},BrowserCacheManager.prototype.addAccountKeyToMap=function(key){this.logger.trace("BrowserCacheManager.addAccountKeyToMap called"),this.logger.tracePii("BrowserCacheManager.addAccountKeyToMap called with key: "+key);var accountKeys=this.getAccountKeys();-1===accountKeys.indexOf(key)?(accountKeys.push(key),this.setItem(StaticCacheKeys.ACCOUNT_KEYS,JSON.stringify(accountKeys)),this.logger.verbose("BrowserCacheManager.addAccountKeyToMap account key added")):this.logger.verbose("BrowserCacheManager.addAccountKeyToMap account key already exists in map")},BrowserCacheManager.prototype.removeAccountKeyFromMap=function(key){this.logger.trace("BrowserCacheManager.removeAccountKeyFromMap called"),this.logger.tracePii("BrowserCacheManager.removeAccountKeyFromMap called with key: "+key);var accountKeys=this.getAccountKeys(),removalIndex=accountKeys.indexOf(key);removalIndex>-1?(accountKeys.splice(removalIndex,1),this.setItem(StaticCacheKeys.ACCOUNT_KEYS,JSON.stringify(accountKeys)),this.logger.trace("BrowserCacheManager.removeAccountKeyFromMap account key removed")):this.logger.trace("BrowserCacheManager.removeAccountKeyFromMap key not found in existing map")},BrowserCacheManager.prototype.removeAccount=function(key){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){return _super.prototype.removeAccount.call(this,key),this.removeAccountKeyFromMap(key),[2]}))}))},BrowserCacheManager.prototype.removeIdToken=function(key){_super.prototype.removeIdToken.call(this,key),this.removeTokenKey(key,CredentialType.ID_TOKEN)},BrowserCacheManager.prototype.removeAccessToken=function(key){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){return _super.prototype.removeAccessToken.call(this,key),this.removeTokenKey(key,CredentialType.ACCESS_TOKEN),[2]}))}))},BrowserCacheManager.prototype.removeRefreshToken=function(key){_super.prototype.removeRefreshToken.call(this,key),this.removeTokenKey(key,CredentialType.REFRESH_TOKEN)},BrowserCacheManager.prototype.getTokenKeys=function(){this.logger.trace("BrowserCacheManager.getTokenKeys called");var item=this.getItem(StaticCacheKeys.TOKEN_KEYS+"."+this.clientId);if(item){var tokenKeys=this.validateAndParseJson(item);if(tokenKeys&&tokenKeys.hasOwnProperty("idToken")&&tokenKeys.hasOwnProperty("accessToken")&&tokenKeys.hasOwnProperty("refreshToken"))return tokenKeys;this.logger.error("BrowserCacheManager.getTokenKeys - Token keys found but in an unknown format. Returning empty key map.")}else this.logger.verbose("BrowserCacheManager.getTokenKeys - No token keys found");return{idToken:[],accessToken:[],refreshToken:[]}},BrowserCacheManager.prototype.addTokenKey=function(key,type){this.logger.trace("BrowserCacheManager addTokenKey called");var tokenKeys=this.getTokenKeys();switch(type){case CredentialType.ID_TOKEN:-1===tokenKeys.idToken.indexOf(key)&&(this.logger.info("BrowserCacheManager: addTokenKey - idToken added to map"),tokenKeys.idToken.push(key));break;case CredentialType.ACCESS_TOKEN:-1===tokenKeys.accessToken.indexOf(key)&&(this.logger.info("BrowserCacheManager: addTokenKey - accessToken added to map"),tokenKeys.accessToken.push(key));break;case CredentialType.REFRESH_TOKEN:-1===tokenKeys.refreshToken.indexOf(key)&&(this.logger.info("BrowserCacheManager: addTokenKey - refreshToken added to map"),tokenKeys.refreshToken.push(key));break;default:this.logger.error("BrowserCacheManager:addTokenKey - CredentialType provided invalid. CredentialType: "+type),ClientAuthError.createUnexpectedCredentialTypeError()}this.setItem(StaticCacheKeys.TOKEN_KEYS+"."+this.clientId,JSON.stringify(tokenKeys))},BrowserCacheManager.prototype.removeTokenKey=function(key,type){this.logger.trace("BrowserCacheManager removeTokenKey called");var tokenKeys=this.getTokenKeys();switch(type){case CredentialType.ID_TOKEN:this.logger.infoPii("BrowserCacheManager: removeTokenKey - attempting to remove idToken with key: "+key+" from map");var idRemoval=tokenKeys.idToken.indexOf(key);idRemoval>-1?(this.logger.info("BrowserCacheManager: removeTokenKey - idToken removed from map"),tokenKeys.idToken.splice(idRemoval,1)):this.logger.info("BrowserCacheManager: removeTokenKey - idToken does not exist in map. Either it was previously removed or it was never added.");break;case CredentialType.ACCESS_TOKEN:this.logger.infoPii("BrowserCacheManager: removeTokenKey - attempting to remove accessToken with key: "+key+" from map");var accessRemoval=tokenKeys.accessToken.indexOf(key);accessRemoval>-1?(this.logger.info("BrowserCacheManager: removeTokenKey - accessToken removed from map"),tokenKeys.accessToken.splice(accessRemoval,1)):this.logger.info("BrowserCacheManager: removeTokenKey - accessToken does not exist in map. Either it was previously removed or it was never added.");break;case CredentialType.REFRESH_TOKEN:this.logger.infoPii("BrowserCacheManager: removeTokenKey - attempting to remove refreshToken with key: "+key+" from map");var refreshRemoval=tokenKeys.refreshToken.indexOf(key);refreshRemoval>-1?(this.logger.info("BrowserCacheManager: removeTokenKey - refreshToken removed from map"),tokenKeys.refreshToken.splice(refreshRemoval,1)):this.logger.info("BrowserCacheManager: removeTokenKey - refreshToken does not exist in map. Either it was previously removed or it was never added.");break;default:this.logger.error("BrowserCacheManager:removeTokenKey - CredentialType provided invalid. CredentialType: "+type),ClientAuthError.createUnexpectedCredentialTypeError()}this.setItem(StaticCacheKeys.TOKEN_KEYS+"."+this.clientId,JSON.stringify(tokenKeys))},BrowserCacheManager.prototype.getIdTokenCredential=function(idTokenKey){var value=this.getItem(idTokenKey);if(!value)return this.logger.trace("BrowserCacheManager.getIdTokenCredential: called, no cache hit"),this.removeTokenKey(idTokenKey,CredentialType.ID_TOKEN),null;var parsedIdToken=this.validateAndParseJson(value);return parsedIdToken&&IdTokenEntity.isIdTokenEntity(parsedIdToken)?(this.logger.trace("BrowserCacheManager.getIdTokenCredential: cache hit"),CacheManager.toObject(new IdTokenEntity,parsedIdToken)):(this.logger.trace("BrowserCacheManager.getIdTokenCredential: called, no cache hit"),this.removeTokenKey(idTokenKey,CredentialType.ID_TOKEN),null)},BrowserCacheManager.prototype.setIdTokenCredential=function(idToken){this.logger.trace("BrowserCacheManager.setIdTokenCredential called");var idTokenKey=idToken.generateCredentialKey();this.setItem(idTokenKey,JSON.stringify(idToken)),this.addTokenKey(idTokenKey,CredentialType.ID_TOKEN)},BrowserCacheManager.prototype.getAccessTokenCredential=function(accessTokenKey){var value=this.getItem(accessTokenKey);if(!value)return this.logger.trace("BrowserCacheManager.getAccessTokenCredential: called, no cache hit"),this.removeTokenKey(accessTokenKey,CredentialType.ACCESS_TOKEN),null;var parsedAccessToken=this.validateAndParseJson(value);return parsedAccessToken&&AccessTokenEntity.isAccessTokenEntity(parsedAccessToken)?(this.logger.trace("BrowserCacheManager.getAccessTokenCredential: cache hit"),CacheManager.toObject(new AccessTokenEntity,parsedAccessToken)):(this.logger.trace("BrowserCacheManager.getAccessTokenCredential: called, no cache hit"),this.removeTokenKey(accessTokenKey,CredentialType.ACCESS_TOKEN),null)},BrowserCacheManager.prototype.setAccessTokenCredential=function(accessToken){this.logger.trace("BrowserCacheManager.setAccessTokenCredential called");var accessTokenKey=accessToken.generateCredentialKey();this.setItem(accessTokenKey,JSON.stringify(accessToken)),this.addTokenKey(accessTokenKey,CredentialType.ACCESS_TOKEN)},BrowserCacheManager.prototype.getRefreshTokenCredential=function(refreshTokenKey){var value=this.getItem(refreshTokenKey);if(!value)return this.logger.trace("BrowserCacheManager.getRefreshTokenCredential: called, no cache hit"),this.removeTokenKey(refreshTokenKey,CredentialType.REFRESH_TOKEN),null;var parsedRefreshToken=this.validateAndParseJson(value);return parsedRefreshToken&&RefreshTokenEntity.isRefreshTokenEntity(parsedRefreshToken)?(this.logger.trace("BrowserCacheManager.getRefreshTokenCredential: cache hit"),CacheManager.toObject(new RefreshTokenEntity,parsedRefreshToken)):(this.logger.trace("BrowserCacheManager.getRefreshTokenCredential: called, no cache hit"),this.removeTokenKey(refreshTokenKey,CredentialType.REFRESH_TOKEN),null)},BrowserCacheManager.prototype.setRefreshTokenCredential=function(refreshToken){this.logger.trace("BrowserCacheManager.setRefreshTokenCredential called");var refreshTokenKey=refreshToken.generateCredentialKey();this.setItem(refreshTokenKey,JSON.stringify(refreshToken)),this.addTokenKey(refreshTokenKey,CredentialType.REFRESH_TOKEN)},BrowserCacheManager.prototype.getAppMetadata=function(appMetadataKey){var value=this.getItem(appMetadataKey);if(!value)return this.logger.trace("BrowserCacheManager.getAppMetadata: called, no cache hit"),null;var parsedMetadata=this.validateAndParseJson(value);return parsedMetadata&&AppMetadataEntity.isAppMetadataEntity(appMetadataKey,parsedMetadata)?(this.logger.trace("BrowserCacheManager.getAppMetadata: cache hit"),CacheManager.toObject(new AppMetadataEntity,parsedMetadata)):(this.logger.trace("BrowserCacheManager.getAppMetadata: called, no cache hit"),null)},BrowserCacheManager.prototype.setAppMetadata=function(appMetadata){this.logger.trace("BrowserCacheManager.setAppMetadata called");var appMetadataKey=appMetadata.generateAppMetadataKey();this.setItem(appMetadataKey,JSON.stringify(appMetadata))},BrowserCacheManager.prototype.getServerTelemetry=function(serverTelemetryKey){var value=this.getItem(serverTelemetryKey);if(!value)return this.logger.trace("BrowserCacheManager.getServerTelemetry: called, no cache hit"),null;var parsedMetadata=this.validateAndParseJson(value);return parsedMetadata&&ServerTelemetryEntity.isServerTelemetryEntity(serverTelemetryKey,parsedMetadata)?(this.logger.trace("BrowserCacheManager.getServerTelemetry: cache hit"),CacheManager.toObject(new ServerTelemetryEntity,parsedMetadata)):(this.logger.trace("BrowserCacheManager.getServerTelemetry: called, no cache hit"),null)},BrowserCacheManager.prototype.setServerTelemetry=function(serverTelemetryKey,serverTelemetry){this.logger.trace("BrowserCacheManager.setServerTelemetry called"),this.setItem(serverTelemetryKey,JSON.stringify(serverTelemetry))},BrowserCacheManager.prototype.getAuthorityMetadata=function(key){var value=this.internalStorage.getItem(key);if(!value)return this.logger.trace("BrowserCacheManager.getAuthorityMetadata: called, no cache hit"),null;var parsedMetadata=this.validateAndParseJson(value);return parsedMetadata&&AuthorityMetadataEntity.isAuthorityMetadataEntity(key,parsedMetadata)?(this.logger.trace("BrowserCacheManager.getAuthorityMetadata: cache hit"),CacheManager.toObject(new AuthorityMetadataEntity,parsedMetadata)):null},BrowserCacheManager.prototype.getAuthorityMetadataKeys=function(){var _this=this;return this.internalStorage.getKeys().filter((function(key){return _this.isAuthorityMetadata(key)}))},BrowserCacheManager.prototype.setWrapperMetadata=function(wrapperSKU,wrapperVersion){this.internalStorage.setItem(InMemoryCacheKeys.WRAPPER_SKU,wrapperSKU),this.internalStorage.setItem(InMemoryCacheKeys.WRAPPER_VER,wrapperVersion)},BrowserCacheManager.prototype.getWrapperMetadata=function(){return[this.internalStorage.getItem(InMemoryCacheKeys.WRAPPER_SKU)||Constants.EMPTY_STRING,this.internalStorage.getItem(InMemoryCacheKeys.WRAPPER_VER)||Constants.EMPTY_STRING]},BrowserCacheManager.prototype.setAuthorityMetadata=function(key,entity){this.logger.trace("BrowserCacheManager.setAuthorityMetadata called"),this.internalStorage.setItem(key,JSON.stringify(entity))},BrowserCacheManager.prototype.getActiveAccount=function(){var activeAccountKeyFilters=this.generateCacheKey(PersistentCacheKeys.ACTIVE_ACCOUNT_FILTERS),activeAccountValueFilters=this.getItem(activeAccountKeyFilters);if(!activeAccountValueFilters){this.logger.trace("BrowserCacheManager.getActiveAccount: No active account filters cache schema found, looking for legacy schema");var activeAccountKeyLocal=this.generateCacheKey(PersistentCacheKeys.ACTIVE_ACCOUNT),activeAccountValueLocal=this.getItem(activeAccountKeyLocal);if(!activeAccountValueLocal)return this.logger.trace("BrowserCacheManager.getActiveAccount: No active account found"),null;var activeAccount=this.getAccountInfoByFilter({localAccountId:activeAccountValueLocal})[0]||null;return activeAccount?(this.logger.trace("BrowserCacheManager.getActiveAccount: Legacy active account cache schema found"),this.logger.trace("BrowserCacheManager.getActiveAccount: Adding active account filters cache schema"),this.setActiveAccount(activeAccount),activeAccount):null}var activeAccountValueObj=this.validateAndParseJson(activeAccountValueFilters);return activeAccountValueObj?(this.logger.trace("BrowserCacheManager.getActiveAccount: Active account filters schema found"),this.getAccountInfoByFilter({homeAccountId:activeAccountValueObj.homeAccountId,localAccountId:activeAccountValueObj.localAccountId})[0]||null):(this.logger.trace("BrowserCacheManager.getActiveAccount: No active account found"),null)},BrowserCacheManager.prototype.setActiveAccount=function(account){var activeAccountKey=this.generateCacheKey(PersistentCacheKeys.ACTIVE_ACCOUNT_FILTERS),activeAccountKeyLocal=this.generateCacheKey(PersistentCacheKeys.ACTIVE_ACCOUNT);if(account){this.logger.verbose("setActiveAccount: Active account set");var activeAccountValue={homeAccountId:account.homeAccountId,localAccountId:account.localAccountId};this.browserStorage.setItem(activeAccountKey,JSON.stringify(activeAccountValue)),this.browserStorage.setItem(activeAccountKeyLocal,account.localAccountId)}else this.logger.verbose("setActiveAccount: No account passed, active account not set"),this.browserStorage.removeItem(activeAccountKey),this.browserStorage.removeItem(activeAccountKeyLocal)},BrowserCacheManager.prototype.getAccountInfoByFilter=function(accountFilter){var allAccounts=this.getAllAccounts();return this.logger.trace("BrowserCacheManager.getAccountInfoByFilter: total "+allAccounts.length+" accounts found"),allAccounts.filter((function(accountObj){return(!accountFilter.username||accountFilter.username.toLowerCase()===accountObj.username.toLowerCase())&&((!accountFilter.homeAccountId||accountFilter.homeAccountId===accountObj.homeAccountId)&&((!accountFilter.localAccountId||accountFilter.localAccountId===accountObj.localAccountId)&&((!accountFilter.tenantId||accountFilter.tenantId===accountObj.tenantId)&&(!accountFilter.environment||accountFilter.environment===accountObj.environment))))}))},BrowserCacheManager.prototype.getAccountInfoByHints=function(loginHint,sid){var matchingAccounts=this.getAllAccounts().filter((function(accountInfo){if(sid){var accountSid=accountInfo.idTokenClaims&&accountInfo.idTokenClaims.sid;return sid===accountSid}return!!loginHint&&loginHint===accountInfo.username}));if(1===matchingAccounts.length)return matchingAccounts[0];if(matchingAccounts.length>1)throw ClientAuthError.createMultipleMatchingAccountsInCacheError();return null},BrowserCacheManager.prototype.getThrottlingCache=function(throttlingCacheKey){var value=this.getItem(throttlingCacheKey);if(!value)return this.logger.trace("BrowserCacheManager.getThrottlingCache: called, no cache hit"),null;var parsedThrottlingCache=this.validateAndParseJson(value);return parsedThrottlingCache&&ThrottlingEntity.isThrottlingEntity(throttlingCacheKey,parsedThrottlingCache)?(this.logger.trace("BrowserCacheManager.getThrottlingCache: cache hit"),CacheManager.toObject(new ThrottlingEntity,parsedThrottlingCache)):(this.logger.trace("BrowserCacheManager.getThrottlingCache: called, no cache hit"),null)},BrowserCacheManager.prototype.setThrottlingCache=function(throttlingCacheKey,throttlingCache){this.logger.trace("BrowserCacheManager.setThrottlingCache called"),this.setItem(throttlingCacheKey,JSON.stringify(throttlingCache))},BrowserCacheManager.prototype.getTemporaryCache=function(cacheKey,generateKey){var key=generateKey?this.generateCacheKey(cacheKey):cacheKey;if(this.cacheConfig.storeAuthStateInCookie){var itemCookie=this.getItemCookie(key);if(itemCookie)return this.logger.trace("BrowserCacheManager.getTemporaryCache: storeAuthStateInCookies set to true, retrieving from cookies"),itemCookie}var value=this.temporaryCacheStorage.getItem(key);if(!value){if(this.cacheConfig.cacheLocation===BrowserCacheLocation.LocalStorage){var item=this.browserStorage.getItem(key);if(item)return this.logger.trace("BrowserCacheManager.getTemporaryCache: Temporary cache item found in local storage"),item}return this.logger.trace("BrowserCacheManager.getTemporaryCache: No cache item found in local storage"),null}return this.logger.trace("BrowserCacheManager.getTemporaryCache: Temporary cache item returned"),value},BrowserCacheManager.prototype.setTemporaryCache=function(cacheKey,value,generateKey){var key=generateKey?this.generateCacheKey(cacheKey):cacheKey;this.temporaryCacheStorage.setItem(key,value),this.cacheConfig.storeAuthStateInCookie&&(this.logger.trace("BrowserCacheManager.setTemporaryCache: storeAuthStateInCookie set to true, setting item cookie"),this.setItemCookie(key,value))},BrowserCacheManager.prototype.removeItem=function(key){this.browserStorage.removeItem(key),this.temporaryCacheStorage.removeItem(key),this.cacheConfig.storeAuthStateInCookie&&(this.logger.trace("BrowserCacheManager.removeItem: storeAuthStateInCookie is true, clearing item cookie"),this.clearItemCookie(key))},BrowserCacheManager.prototype.containsKey=function(key){return this.browserStorage.containsKey(key)||this.temporaryCacheStorage.containsKey(key)},BrowserCacheManager.prototype.getKeys=function(){return __spread(this.browserStorage.getKeys(),this.temporaryCacheStorage.getKeys())},BrowserCacheManager.prototype.clear=function(){return __awaiter(this,void 0,void 0,(function(){var _this=this;return __generator(this,(function(_a){switch(_a.label){case 0:return[4,this.removeAllAccounts()];case 1:return _a.sent(),this.removeAppMetadata(),this.getKeys().forEach((function(cacheKey){!_this.browserStorage.containsKey(cacheKey)&&!_this.temporaryCacheStorage.containsKey(cacheKey)||-1===cacheKey.indexOf(Constants.CACHE_PREFIX)&&-1===cacheKey.indexOf(_this.clientId)||_this.removeItem(cacheKey)})),this.internalStorage.clear(),[2]}}))}))},BrowserCacheManager.prototype.setItemCookie=function(cookieName,cookieValue,expires){var cookieStr=encodeURIComponent(cookieName)+"="+encodeURIComponent(cookieValue)+";path=/;SameSite=Lax;";expires&&(cookieStr+="expires="+this.getCookieExpirationTime(expires)+";");this.cacheConfig.secureCookies&&(cookieStr+="Secure;"),document.cookie=cookieStr},BrowserCacheManager.prototype.getItemCookie=function(cookieName){for(var name=encodeURIComponent(cookieName)+"=",cookieList=document.cookie.split(";"),i=0;i<cookieList.length;i++){for(var cookie=cookieList[i];" "===cookie.charAt(0);)cookie=cookie.substring(1);if(0===cookie.indexOf(name))return decodeURIComponent(cookie.substring(name.length,cookie.length))}return Constants.EMPTY_STRING},BrowserCacheManager.prototype.clearMsalCookies=function(){var _this=this,cookiePrefix=Constants.CACHE_PREFIX+"."+this.clientId;document.cookie.split(";").forEach((function(cookie){for(;" "===cookie.charAt(0);)cookie=cookie.substring(1);if(0===cookie.indexOf(cookiePrefix)){var cookieKey=cookie.split("=")[0];_this.clearItemCookie(cookieKey)}}))},BrowserCacheManager.prototype.clearItemCookie=function(cookieName){this.setItemCookie(cookieName,Constants.EMPTY_STRING,-1)},BrowserCacheManager.prototype.getCookieExpirationTime=function(cookieLifeDays){var today=new Date;return new Date(today.getTime()+cookieLifeDays*this.COOKIE_LIFE_MULTIPLIER).toUTCString()},BrowserCacheManager.prototype.getCache=function(){return this.browserStorage},BrowserCacheManager.prototype.setCache=function(){},BrowserCacheManager.prototype.generateCacheKey=function(key){return this.validateAndParseJson(key)?JSON.stringify(key):StringUtils.startsWith(key,Constants.CACHE_PREFIX)||StringUtils.startsWith(key,PersistentCacheKeys.ADAL_ID_TOKEN)?key:Constants.CACHE_PREFIX+"."+this.clientId+"."+key},BrowserCacheManager.prototype.generateAuthorityKey=function(stateString){var stateId=ProtocolUtils.parseRequestState(this.cryptoImpl,stateString).libraryState.id;return this.generateCacheKey(TemporaryCacheKeys.AUTHORITY+"."+stateId)},BrowserCacheManager.prototype.generateNonceKey=function(stateString){var stateId=ProtocolUtils.parseRequestState(this.cryptoImpl,stateString).libraryState.id;return this.generateCacheKey(TemporaryCacheKeys.NONCE_IDTOKEN+"."+stateId)},BrowserCacheManager.prototype.generateStateKey=function(stateString){var stateId=ProtocolUtils.parseRequestState(this.cryptoImpl,stateString).libraryState.id;return this.generateCacheKey(TemporaryCacheKeys.REQUEST_STATE+"."+stateId)},BrowserCacheManager.prototype.getCachedAuthority=function(cachedState){var stateCacheKey=this.generateStateKey(cachedState),state=this.getTemporaryCache(stateCacheKey);if(!state)return null;var authorityCacheKey=this.generateAuthorityKey(state);return this.getTemporaryCache(authorityCacheKey)},BrowserCacheManager.prototype.updateCacheEntries=function(state,nonce,authorityInstance,loginHint,account){this.logger.trace("BrowserCacheManager.updateCacheEntries called");var stateCacheKey=this.generateStateKey(state);this.setTemporaryCache(stateCacheKey,state,!1);var nonceCacheKey=this.generateNonceKey(state);this.setTemporaryCache(nonceCacheKey,nonce,!1);var authorityCacheKey=this.generateAuthorityKey(state);if(this.setTemporaryCache(authorityCacheKey,authorityInstance,!1),account){var ccsCredential={credential:account.homeAccountId,type:CcsCredentialType.HOME_ACCOUNT_ID};this.setTemporaryCache(TemporaryCacheKeys.CCS_CREDENTIAL,JSON.stringify(ccsCredential),!0)}else if(!StringUtils.isEmpty(loginHint)){ccsCredential={credential:loginHint,type:CcsCredentialType.UPN};this.setTemporaryCache(TemporaryCacheKeys.CCS_CREDENTIAL,JSON.stringify(ccsCredential),!0)}},BrowserCacheManager.prototype.resetRequestCache=function(state){var _this=this;this.logger.trace("BrowserCacheManager.resetRequestCache called"),StringUtils.isEmpty(state)||this.getKeys().forEach((function(key){-1!==key.indexOf(state)&&_this.removeItem(key)})),state&&(this.removeItem(this.generateStateKey(state)),this.removeItem(this.generateNonceKey(state)),this.removeItem(this.generateAuthorityKey(state))),this.removeItem(this.generateCacheKey(TemporaryCacheKeys.REQUEST_PARAMS)),this.removeItem(this.generateCacheKey(TemporaryCacheKeys.ORIGIN_URI)),this.removeItem(this.generateCacheKey(TemporaryCacheKeys.URL_HASH)),this.removeItem(this.generateCacheKey(TemporaryCacheKeys.CORRELATION_ID)),this.removeItem(this.generateCacheKey(TemporaryCacheKeys.CCS_CREDENTIAL)),this.removeItem(this.generateCacheKey(TemporaryCacheKeys.NATIVE_REQUEST)),this.setInteractionInProgress(!1)},BrowserCacheManager.prototype.cleanRequestByState=function(stateString){if(this.logger.trace("BrowserCacheManager.cleanRequestByState called"),stateString){var stateKey=this.generateStateKey(stateString),cachedState=this.temporaryCacheStorage.getItem(stateKey);this.logger.infoPii("BrowserCacheManager.cleanRequestByState: Removing temporary cache items for state: "+cachedState),this.resetRequestCache(cachedState||Constants.EMPTY_STRING)}this.clearMsalCookies()},BrowserCacheManager.prototype.cleanRequestByInteractionType=function(interactionType){var _this=this;this.logger.trace("BrowserCacheManager.cleanRequestByInteractionType called"),this.getKeys().forEach((function(key){if(-1!==key.indexOf(TemporaryCacheKeys.REQUEST_STATE)){var stateValue=_this.temporaryCacheStorage.getItem(key);if(stateValue){var parsedState=BrowserProtocolUtils.extractBrowserRequestState(_this.cryptoImpl,stateValue);parsedState&&parsedState.interactionType===interactionType&&(_this.logger.infoPii("BrowserCacheManager.cleanRequestByInteractionType: Removing temporary cache items for state: "+stateValue),_this.resetRequestCache(stateValue))}}})),this.clearMsalCookies(),this.setInteractionInProgress(!1)},BrowserCacheManager.prototype.cacheCodeRequest=function(authCodeRequest,browserCrypto){this.logger.trace("BrowserCacheManager.cacheCodeRequest called");var encodedValue=browserCrypto.base64Encode(JSON.stringify(authCodeRequest));this.setTemporaryCache(TemporaryCacheKeys.REQUEST_PARAMS,encodedValue,!0)},BrowserCacheManager.prototype.getCachedRequest=function(state,browserCrypto){this.logger.trace("BrowserCacheManager.getCachedRequest called");var encodedTokenRequest=this.getTemporaryCache(TemporaryCacheKeys.REQUEST_PARAMS,!0);if(!encodedTokenRequest)throw BrowserAuthError.createNoTokenRequestCacheError();var parsedRequest=this.validateAndParseJson(browserCrypto.base64Decode(encodedTokenRequest));if(!parsedRequest)throw BrowserAuthError.createUnableToParseTokenRequestCacheError();if(this.removeItem(this.generateCacheKey(TemporaryCacheKeys.REQUEST_PARAMS)),StringUtils.isEmpty(parsedRequest.authority)){var authorityCacheKey=this.generateAuthorityKey(state),cachedAuthority=this.getTemporaryCache(authorityCacheKey);if(!cachedAuthority)throw BrowserAuthError.createNoCachedAuthorityError();parsedRequest.authority=cachedAuthority}return parsedRequest},BrowserCacheManager.prototype.getCachedNativeRequest=function(){this.logger.trace("BrowserCacheManager.getCachedNativeRequest called");var cachedRequest=this.getTemporaryCache(TemporaryCacheKeys.NATIVE_REQUEST,!0);if(!cachedRequest)return this.logger.trace("BrowserCacheManager.getCachedNativeRequest: No cached native request found"),null;var parsedRequest=this.validateAndParseJson(cachedRequest);return parsedRequest||(this.logger.error("BrowserCacheManager.getCachedNativeRequest: Unable to parse native request"),null)},BrowserCacheManager.prototype.isInteractionInProgress=function(matchClientId){var clientId=this.getInteractionInProgress();return matchClientId?clientId===this.clientId:!!clientId},BrowserCacheManager.prototype.getInteractionInProgress=function(){var key=Constants.CACHE_PREFIX+"."+TemporaryCacheKeys.INTERACTION_STATUS_KEY;return this.getTemporaryCache(key,!1)},BrowserCacheManager.prototype.setInteractionInProgress=function(inProgress){var key=Constants.CACHE_PREFIX+"."+TemporaryCacheKeys.INTERACTION_STATUS_KEY;if(inProgress){if(this.getInteractionInProgress())throw BrowserAuthError.createInteractionInProgressError();this.setTemporaryCache(key,this.clientId,!1)}else inProgress||this.getInteractionInProgress()!==this.clientId||this.removeItem(key)},BrowserCacheManager.prototype.getLegacyLoginHint=function(){var adalIdTokenString=this.getTemporaryCache(PersistentCacheKeys.ADAL_ID_TOKEN);adalIdTokenString&&(this.browserStorage.removeItem(PersistentCacheKeys.ADAL_ID_TOKEN),this.logger.verbose("Cached ADAL id token retrieved."));var msalIdTokenString=this.getTemporaryCache(PersistentCacheKeys.ID_TOKEN,!0);msalIdTokenString&&(this.removeItem(this.generateCacheKey(PersistentCacheKeys.ID_TOKEN)),this.logger.verbose("Cached MSAL.js v1 id token retrieved"));var cachedIdTokenString=msalIdTokenString||adalIdTokenString;if(cachedIdTokenString){var cachedIdToken=new IdToken(cachedIdTokenString,this.cryptoImpl);if(cachedIdToken.claims&&cachedIdToken.claims.preferred_username)return this.logger.verbose("No SSO params used and ADAL/MSAL v1 token retrieved, setting ADAL/MSAL v1 preferred_username as loginHint"),cachedIdToken.claims.preferred_username;if(cachedIdToken.claims&&cachedIdToken.claims.upn)return this.logger.verbose("No SSO params used and ADAL/MSAL v1 token retrieved, setting ADAL/MSAL v1 upn as loginHint"),cachedIdToken.claims.upn;this.logger.verbose("No SSO params used and ADAL/MSAL v1 token retrieved, however, no account hint claim found. Enable preferred_username or upn id token claim to get SSO.")}return null},BrowserCacheManager.prototype.updateCredentialCacheKey=function(currentCacheKey,credential){var updatedCacheKey=credential.generateCredentialKey();if(currentCacheKey!==updatedCacheKey){var cacheItem=this.getItem(currentCacheKey);if(cacheItem)return this.removeItem(currentCacheKey),this.setItem(updatedCacheKey,cacheItem),this.logger.verbose("Updated an outdated "+credential.credentialType+" cache key"),updatedCacheKey;this.logger.error("Attempted to update an outdated "+credential.credentialType+" cache key but no item matching the outdated key was found in storage")}return currentCacheKey},BrowserCacheManager.prototype.getRedirectRequestContext=function(){return this.getTemporaryCache(TemporaryCacheKeys.REDIRECT_CONTEXT,!0)},BrowserCacheManager.prototype.setRedirectRequestContext=function(value){this.setTemporaryCache(TemporaryCacheKeys.REDIRECT_CONTEXT,value,!0)},BrowserCacheManager}(CacheManager),DEFAULT_BROWSER_CACHE_MANAGER=function(clientId,logger){var cacheOptions={cacheLocation:BrowserCacheLocation.MemoryStorage,temporaryCacheLocation:BrowserCacheLocation.MemoryStorage,storeAuthStateInCookie:!1,secureCookies:!1,cacheMigrationEnabled:!1};return new BrowserCacheManager(clientId,cacheOptions,DEFAULT_CRYPTO_IMPLEMENTATION,logger)};export{BrowserCacheManager,DEFAULT_BROWSER_CACHE_MANAGER};