/*! @azure/msal-browser v2.37.1 2023-06-07 */
"use strict";import{AuthToken,CacheRecord,Authority,AccountEntity,IdTokenEntity,ScopeSet,AccessTokenEntity,RefreshTokenEntity,Constants}from"@azure/msal-common";import{BrowserAuthError}from"../error/BrowserAuthError.js";var TokenCache=function(){function TokenCache(configuration,storage,logger,cryptoObj){this.isBrowserEnvironment="undefined"!=typeof window,this.config=configuration,this.storage=storage,this.logger=logger,this.cryptoObj=cryptoObj}return TokenCache.prototype.loadExternalTokens=function(request,response,options){if(this.logger.info("TokenCache - loadExternalTokens called"),!response.id_token)throw BrowserAuthError.createUnableToLoadTokenError("Please ensure server response includes id token.");var cacheRecord,authority,idToken=new AuthToken(response.id_token,this.cryptoObj);if(request.account){var cacheRecordAccount=this.loadAccount(idToken,request.account.environment,void 0,void 0,request.account.homeAccountId);cacheRecord=new CacheRecord(cacheRecordAccount,this.loadIdToken(idToken,cacheRecordAccount.homeAccountId,request.account.environment,request.account.tenantId),this.loadAccessToken(request,response,cacheRecordAccount.homeAccountId,request.account.environment,request.account.tenantId,options),this.loadRefreshToken(request,response,cacheRecordAccount.homeAccountId,request.account.environment))}else{if(!request.authority)throw BrowserAuthError.createUnableToLoadTokenError("Please provide a request with an account or a request with authority.");var authorityUrl=Authority.generateAuthority(request.authority,request.azureCloudOptions),authorityOptions={protocolMode:this.config.auth.protocolMode,knownAuthorities:this.config.auth.knownAuthorities,cloudDiscoveryMetadata:this.config.auth.cloudDiscoveryMetadata,authorityMetadata:this.config.auth.authorityMetadata,skipAuthorityMetadataCache:this.config.auth.skipAuthorityMetadataCache};if(authority=new Authority(authorityUrl,this.config.system.networkClient,this.storage,authorityOptions,this.logger),options.clientInfo){this.logger.trace("TokenCache - homeAccountId from options");cacheRecordAccount=this.loadAccount(idToken,authority.hostnameAndPort,options.clientInfo,authority.authorityType);cacheRecord=new CacheRecord(cacheRecordAccount,this.loadIdToken(idToken,cacheRecordAccount.homeAccountId,authority.hostnameAndPort,authority.tenant),this.loadAccessToken(request,response,cacheRecordAccount.homeAccountId,authority.hostnameAndPort,authority.tenant,options),this.loadRefreshToken(request,response,cacheRecordAccount.homeAccountId,authority.hostnameAndPort))}else{if(!response.client_info)throw BrowserAuthError.createUnableToLoadTokenError("Please provide clientInfo in the response or options.");this.logger.trace("TokenCache - homeAccountId from response");cacheRecordAccount=this.loadAccount(idToken,authority.hostnameAndPort,response.client_info,authority.authorityType);cacheRecord=new CacheRecord(cacheRecordAccount,this.loadIdToken(idToken,cacheRecordAccount.homeAccountId,authority.hostnameAndPort,authority.tenant),this.loadAccessToken(request,response,cacheRecordAccount.homeAccountId,authority.hostnameAndPort,authority.tenant,options),this.loadRefreshToken(request,response,cacheRecordAccount.homeAccountId,authority.hostnameAndPort))}}return this.generateAuthenticationResult(request,idToken,cacheRecord,authority)},TokenCache.prototype.loadAccount=function(idToken,environment,clientInfo,authorityType,requestHomeAccountId){var homeAccountId;if(requestHomeAccountId?homeAccountId=requestHomeAccountId:void 0!==authorityType&&clientInfo&&(homeAccountId=AccountEntity.generateHomeAccountId(clientInfo,authorityType,this.logger,this.cryptoObj,idToken)),!homeAccountId)throw BrowserAuthError.createUnableToLoadTokenError("Unexpected missing homeAccountId");var accountEntity=clientInfo?AccountEntity.createAccount(clientInfo,homeAccountId,idToken,void 0,void 0,void 0,environment):AccountEntity.createGenericAccount(homeAccountId,idToken,void 0,void 0,void 0,environment);if(this.isBrowserEnvironment)return this.logger.verbose("TokenCache - loading account"),this.storage.setAccount(accountEntity),accountEntity;throw BrowserAuthError.createUnableToLoadTokenError("loadExternalTokens is designed to work in browser environments only.")},TokenCache.prototype.loadIdToken=function(idToken,homeAccountId,environment,tenantId){var idTokenEntity=IdTokenEntity.createIdTokenEntity(homeAccountId,environment,idToken.rawToken,this.config.auth.clientId,tenantId);if(this.isBrowserEnvironment)return this.logger.verbose("TokenCache - loading id token"),this.storage.setIdTokenCredential(idTokenEntity),idTokenEntity;throw BrowserAuthError.createUnableToLoadTokenError("loadExternalTokens is designed to work in browser environments only.")},TokenCache.prototype.loadAccessToken=function(request,response,homeAccountId,environment,tenantId,options){if(!response.access_token)return this.logger.verbose("TokenCache - No access token provided for caching"),null;if(!response.expires_in)throw BrowserAuthError.createUnableToLoadTokenError("Please ensure server response includes expires_in value.");if(!options.extendedExpiresOn)throw BrowserAuthError.createUnableToLoadTokenError("Please provide an extendedExpiresOn value in the options.");var scopes=new ScopeSet(request.scopes).printScopes(),expiresOn=options.expiresOn||response.expires_in+(new Date).getTime()/1e3,extendedExpiresOn=options.extendedExpiresOn,accessTokenEntity=AccessTokenEntity.createAccessTokenEntity(homeAccountId,environment,response.access_token,this.config.auth.clientId,tenantId,scopes,expiresOn,extendedExpiresOn,this.cryptoObj);if(this.isBrowserEnvironment)return this.logger.verbose("TokenCache - loading access token"),this.storage.setAccessTokenCredential(accessTokenEntity),accessTokenEntity;throw BrowserAuthError.createUnableToLoadTokenError("loadExternalTokens is designed to work in browser environments only.")},TokenCache.prototype.loadRefreshToken=function(request,response,homeAccountId,environment){if(!response.refresh_token)return this.logger.verbose("TokenCache - No refresh token provided for caching"),null;var refreshTokenEntity=RefreshTokenEntity.createRefreshTokenEntity(homeAccountId,environment,response.refresh_token,this.config.auth.clientId);if(this.isBrowserEnvironment)return this.logger.verbose("TokenCache - loading refresh token"),this.storage.setRefreshTokenCredential(refreshTokenEntity),refreshTokenEntity;throw BrowserAuthError.createUnableToLoadTokenError("loadExternalTokens is designed to work in browser environments only.")},TokenCache.prototype.generateAuthenticationResult=function(request,idTokenObj,cacheRecord,authority){var _a,_b,_c,extExpiresOn,accessToken=Constants.EMPTY_STRING,responseScopes=[],expiresOn=null;(null==cacheRecord?void 0:cacheRecord.accessToken)&&(accessToken=cacheRecord.accessToken.secret,responseScopes=ScopeSet.fromString(cacheRecord.accessToken.target).asArray(),expiresOn=new Date(1e3*Number(cacheRecord.accessToken.expiresOn)),extExpiresOn=new Date(1e3*Number(cacheRecord.accessToken.extendedExpiresOn)));var uid=(null==idTokenObj?void 0:idTokenObj.claims.oid)||(null==idTokenObj?void 0:idTokenObj.claims.sub)||Constants.EMPTY_STRING,tid=(null==idTokenObj?void 0:idTokenObj.claims.tid)||Constants.EMPTY_STRING;return{authority:authority?authority.canonicalAuthority:Constants.EMPTY_STRING,uniqueId:uid,tenantId:tid,scopes:responseScopes,account:(null==cacheRecord?void 0:cacheRecord.account)?cacheRecord.account.getAccountInfo():null,idToken:idTokenObj?idTokenObj.rawToken:Constants.EMPTY_STRING,idTokenClaims:idTokenObj?idTokenObj.claims:{},accessToken:accessToken,fromCache:!0,expiresOn:expiresOn,correlationId:request.correlationId||Constants.EMPTY_STRING,requestId:Constants.EMPTY_STRING,extExpiresOn:extExpiresOn,familyId:Constants.EMPTY_STRING,tokenType:(null===(_a=null==cacheRecord?void 0:cacheRecord.accessToken)||void 0===_a?void 0:_a.tokenType)||Constants.EMPTY_STRING,state:Constants.EMPTY_STRING,cloudGraphHostName:(null===(_b=null==cacheRecord?void 0:cacheRecord.account)||void 0===_b?void 0:_b.cloudGraphHostName)||Constants.EMPTY_STRING,msGraphHost:(null===(_c=null==cacheRecord?void 0:cacheRecord.account)||void 0===_c?void 0:_c.msGraphHost)||Constants.EMPTY_STRING,code:void 0,fromNativeBroker:!1}},TokenCache}();export{TokenCache};