/*! @azure/msal-browser v2.37.1 2023-06-07 */
"use strict";import{__awaiter,__generator,__spread,__assign}from"../_virtual/_tslib.js";import{AccountEntity,AuthenticationScheme,ClientConfigurationError,StringUtils,PerformanceEvents,UrlString,ServerTelemetryManager,AuthorityFactory}from"@azure/msal-common";import{version}from"../packageMetadata.js";import{BrowserConstants}from"../utils/BrowserConstants.js";import{BrowserUtils}from"../utils/BrowserUtils.js";var BaseInteractionClient=function(){function BaseInteractionClient(config,storageImpl,browserCrypto,logger,eventHandler,navigationClient,performanceClient,nativeMessageHandler,correlationId){this.config=config,this.browserStorage=storageImpl,this.browserCrypto=browserCrypto,this.networkClient=this.config.system.networkClient,this.eventHandler=eventHandler,this.navigationClient=navigationClient,this.nativeMessageHandler=nativeMessageHandler,this.correlationId=correlationId||this.browserCrypto.createNewGuid(),this.logger=logger.clone(BrowserConstants.MSAL_SKU,version,this.correlationId),this.performanceClient=performanceClient}return BaseInteractionClient.prototype.clearCacheOnLogout=function(account){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){switch(_a.label){case 0:if(!account)return[3,5];AccountEntity.accountInfoIsEqual(account,this.browserStorage.getActiveAccount(),!1)&&(this.logger.verbose("Setting active account to null"),this.browserStorage.setActiveAccount(null)),_a.label=1;case 1:return _a.trys.push([1,3,,4]),[4,this.browserStorage.removeAccount(AccountEntity.generateAccountCacheKey(account))];case 2:return _a.sent(),this.logger.verbose("Cleared cache items belonging to the account provided in the logout request."),[3,4];case 3:return _a.sent(),this.logger.error("Account provided in logout request was not found. Local cache unchanged."),[3,4];case 4:return[3,9];case 5:return _a.trys.push([5,8,,9]),this.logger.verbose("No account provided in logout request, clearing all cache items.",this.correlationId),[4,this.browserStorage.clear()];case 6:return _a.sent(),[4,this.browserCrypto.clearKeystore()];case 7:return _a.sent(),[3,9];case 8:return _a.sent(),this.logger.error("Attempted to clear all MSAL cache items and failed. Local cache unchanged."),[3,9];case 9:return[2]}}))}))},BaseInteractionClient.prototype.initializeBaseRequest=function(request,account){return __awaiter(this,void 0,void 0,(function(){var authority,scopes,validatedRequest,_a;return __generator(this,(function(_b){switch(_b.label){case 0:return this.performanceClient.addQueueMeasurement(PerformanceEvents.InitializeBaseRequest,request.correlationId),this.logger.verbose("Initializing BaseAuthRequest"),authority=request.authority||this.config.auth.authority,account?[4,this.validateRequestAuthority(authority,account)]:[3,2];case 1:_b.sent(),_b.label=2;case 2:if(scopes=__spread(request&&request.scopes||[]),(validatedRequest=__assign(__assign({},request),{correlationId:this.correlationId,authority:authority,scopes:scopes})).authenticationScheme){if(validatedRequest.authenticationScheme===AuthenticationScheme.SSH){if(!request.sshJwk)throw ClientConfigurationError.createMissingSshJwkError();if(!request.sshKid)throw ClientConfigurationError.createMissingSshKidError()}this.logger.verbose('Authentication Scheme set to "'+validatedRequest.authenticationScheme+'" as configured in Auth request')}else validatedRequest.authenticationScheme=AuthenticationScheme.BEARER,this.logger.verbose('Authentication Scheme wasn\'t explicitly set in request, defaulting to "Bearer" request');return!request.claims||StringUtils.isEmpty(request.claims)?[3,4]:(_a=validatedRequest,[4,this.browserCrypto.hashString(request.claims)]);case 3:_a.requestedClaimsHash=_b.sent(),_b.label=4;case 4:return[2,validatedRequest]}}))}))},BaseInteractionClient.prototype.getRedirectUri=function(requestRedirectUri){this.logger.verbose("getRedirectUri called");var redirectUri=requestRedirectUri||this.config.auth.redirectUri||BrowserUtils.getCurrentUri();return UrlString.getAbsoluteUrl(redirectUri,BrowserUtils.getCurrentUri())},BaseInteractionClient.prototype.validateRequestAuthority=function(authority,account){return __awaiter(this,void 0,void 0,(function(){return __generator(this,(function(_a){switch(_a.label){case 0:return[4,this.getDiscoveredAuthority(authority)];case 1:if(!_a.sent().isAlias(account.environment))throw ClientConfigurationError.createAuthorityMismatchError();return[2]}}))}))},BaseInteractionClient.prototype.initializeServerTelemetryManager=function(apiId,forceRefresh){this.logger.verbose("initializeServerTelemetryManager called");var telemetryPayload={clientId:this.config.auth.clientId,correlationId:this.correlationId,apiId:apiId,forceRefresh:forceRefresh||!1,wrapperSKU:this.browserStorage.getWrapperMetadata()[0],wrapperVer:this.browserStorage.getWrapperMetadata()[1]};return new ServerTelemetryManager(telemetryPayload,this.browserStorage)},BaseInteractionClient.prototype.getDiscoveredAuthority=function(requestAuthority){return __awaiter(this,void 0,void 0,(function(){var authorityOptions;return __generator(this,(function(_a){switch(_a.label){case 0:return this.logger.verbose("getDiscoveredAuthority called"),authorityOptions={protocolMode:this.config.auth.protocolMode,knownAuthorities:this.config.auth.knownAuthorities,cloudDiscoveryMetadata:this.config.auth.cloudDiscoveryMetadata,authorityMetadata:this.config.auth.authorityMetadata},requestAuthority?(this.logger.verbose("Creating discovered authority with request authority"),[4,AuthorityFactory.createDiscoveredInstance(requestAuthority,this.config.system.networkClient,this.browserStorage,authorityOptions,this.logger)]):[3,2];case 1:case 3:return[2,_a.sent()];case 2:return this.logger.verbose("Creating discovered authority with configured authority"),[4,AuthorityFactory.createDiscoveredInstance(this.config.auth.authority,this.config.system.networkClient,this.browserStorage,authorityOptions,this.logger)]}}))}))},BaseInteractionClient}();export{BaseInteractionClient};