/*! @azure/msal-browser v2.37.1 2023-06-07 */
"use strict";import{__extends,__awaiter,__generator,__rest,__assign}from"../_virtual/_tslib.js";import{ScopeSet,AuthToken,Constants,AccountEntity,AuthorityType,IdTokenEntity,AccessTokenEntity,CacheRecord,PerformanceEvents,TimeUtils,ClientAuthError,AuthenticationScheme,PopTokenGenerator,UrlString,OIDC_DEFAULT_SCOPES,PromptValue}from"@azure/msal-common";import{BaseInteractionClient}from"./BaseInteractionClient.js";import{TemporaryCacheKeys,NativeExtensionMethod,NativeConstants,ApiId}from"../utils/BrowserConstants.js";import{NativeAuthError}from"../error/NativeAuthError.js";import{BrowserAuthError}from"../error/BrowserAuthError.js";import{SilentCacheClient}from"./SilentCacheClient.js";var NativeInteractionClient=function(_super){function NativeInteractionClient(config,browserStorage,browserCrypto,logger,eventHandler,navigationClient,apiId,performanceClient,provider,accountId,nativeStorageImpl,correlationId){var _this=_super.call(this,config,browserStorage,browserCrypto,logger,eventHandler,navigationClient,performanceClient,provider,correlationId)||this;return _this.apiId=apiId,_this.accountId=accountId,_this.nativeMessageHandler=provider,_this.nativeStorageManager=nativeStorageImpl,_this.silentCacheClient=new SilentCacheClient(config,_this.nativeStorageManager,browserCrypto,logger,eventHandler,navigationClient,performanceClient,provider,correlationId),_this}return __extends(NativeInteractionClient,_super),NativeInteractionClient.prototype.acquireToken=function(request){return __awaiter(this,void 0,void 0,(function(){var nativeATMeasurement,reqTimestamp,nativeRequest,result,messageBody,response,validatedResponse;return __generator(this,(function(_a){switch(_a.label){case 0:return this.logger.trace("NativeInteractionClient - acquireToken called."),nativeATMeasurement=this.performanceClient.startMeasurement(PerformanceEvents.NativeInteractionClientAcquireToken,request.correlationId),reqTimestamp=TimeUtils.nowSeconds(),[4,this.initializeNativeRequest(request)];case 1:nativeRequest=_a.sent(),_a.label=2;case 2:return _a.trys.push([2,4,,5]),[4,this.acquireTokensFromCache(this.accountId,nativeRequest)];case 3:return result=_a.sent(),nativeATMeasurement.endMeasurement({success:!0,isNativeBroker:!1,fromCache:!0}),[2,result];case 4:return _a.sent(),this.logger.info("MSAL internal Cache does not contain tokens, proceed to make a native call"),[3,5];case 5:return messageBody={method:NativeExtensionMethod.GetToken,request:nativeRequest},[4,this.nativeMessageHandler.sendMessage(messageBody)];case 6:return response=_a.sent(),validatedResponse=this.validateNativeResponse(response),[2,this.handleNativeResponse(validatedResponse,nativeRequest,reqTimestamp).then((function(result){return nativeATMeasurement.endMeasurement({success:!0,isNativeBroker:!0,requestId:result.requestId}),result})).catch((function(error){throw nativeATMeasurement.endMeasurement({success:!1,errorCode:error.errorCode,subErrorCode:error.subError,isNativeBroker:!0}),error}))]}}))}))},NativeInteractionClient.prototype.createSilentCacheRequest=function(request,cachedAccount){return{authority:request.authority,correlationId:this.correlationId,scopes:ScopeSet.fromString(request.scope).asArray(),account:cachedAccount,forceRefresh:!1}},NativeInteractionClient.prototype.acquireTokensFromCache=function(nativeAccountId,request){return __awaiter(this,void 0,void 0,(function(){var account,silentRequest;return __generator(this,(function(_a){switch(_a.label){case 0:if(!nativeAccountId)throw this.logger.warning("NativeInteractionClient:acquireTokensFromCache - No nativeAccountId provided"),ClientAuthError.createNoAccountFoundError();if(!(account=this.browserStorage.getAccountInfoFilteredBy({nativeAccountId:nativeAccountId})))throw ClientAuthError.createNoAccountFoundError();_a.label=1;case 1:return _a.trys.push([1,3,,4]),silentRequest=this.createSilentCacheRequest(request,account),[4,this.silentCacheClient.acquireToken(silentRequest)];case 2:return[2,_a.sent()];case 3:throw _a.sent();case 4:return[2]}}))}))},NativeInteractionClient.prototype.acquireTokenRedirect=function(request){return __awaiter(this,void 0,void 0,(function(){var nativeRequest,messageBody,response,e_3,navigationOptions,redirectUri;return __generator(this,(function(_a){switch(_a.label){case 0:return this.logger.trace("NativeInteractionClient - acquireTokenRedirect called."),[4,this.initializeNativeRequest(request)];case 1:nativeRequest=_a.sent(),messageBody={method:NativeExtensionMethod.GetToken,request:nativeRequest},_a.label=2;case 2:return _a.trys.push([2,4,,5]),[4,this.nativeMessageHandler.sendMessage(messageBody)];case 3:return response=_a.sent(),this.validateNativeResponse(response),[3,5];case 4:if((e_3=_a.sent())instanceof NativeAuthError&&e_3.isFatal())throw e_3;return[3,5];case 5:return this.browserStorage.setTemporaryCache(TemporaryCacheKeys.NATIVE_REQUEST,JSON.stringify(nativeRequest),!0),navigationOptions={apiId:ApiId.acquireTokenRedirect,timeout:this.config.system.redirectNavigationTimeout,noHistory:!1},redirectUri=this.config.auth.navigateToLoginRequestUrl?window.location.href:this.getRedirectUri(request.redirectUri),[4,this.navigationClient.navigateExternal(redirectUri,navigationOptions)];case 6:return _a.sent(),[2]}}))}))},NativeInteractionClient.prototype.handleRedirectPromise=function(){return __awaiter(this,void 0,void 0,(function(){var cachedRequest,prompt,request,messageBody,reqTimestamp,response,result,e_4;return __generator(this,(function(_a){switch(_a.label){case 0:if(this.logger.trace("NativeInteractionClient - handleRedirectPromise called."),!this.browserStorage.isInteractionInProgress(!0))return this.logger.info("handleRedirectPromise called but there is no interaction in progress, returning null."),[2,null];if(!(cachedRequest=this.browserStorage.getCachedNativeRequest()))return this.logger.verbose("NativeInteractionClient - handleRedirectPromise called but there is no cached request, returning null."),[2,null];prompt=cachedRequest.prompt,request=__rest(cachedRequest,["prompt"]),prompt&&this.logger.verbose("NativeInteractionClient - handleRedirectPromise called and prompt was included in the original request, removing prompt from cached request to prevent second interaction with native broker window."),this.browserStorage.removeItem(this.browserStorage.generateCacheKey(TemporaryCacheKeys.NATIVE_REQUEST)),messageBody={method:NativeExtensionMethod.GetToken,request:request},reqTimestamp=TimeUtils.nowSeconds(),_a.label=1;case 1:return _a.trys.push([1,3,,4]),this.logger.verbose("NativeInteractionClient - handleRedirectPromise sending message to native broker."),[4,this.nativeMessageHandler.sendMessage(messageBody)];case 2:return response=_a.sent(),this.validateNativeResponse(response),result=this.handleNativeResponse(response,request,reqTimestamp),this.browserStorage.setInteractionInProgress(!1),[2,result];case 3:throw e_4=_a.sent(),this.browserStorage.setInteractionInProgress(!1),e_4;case 4:return[2]}}))}))},NativeInteractionClient.prototype.logout=function(){return this.logger.trace("NativeInteractionClient - logout called."),Promise.reject("Logout not implemented yet")},NativeInteractionClient.prototype.handleNativeResponse=function(response,request,reqTimestamp){return __awaiter(this,void 0,void 0,(function(){var authority,authorityPreferredCache,idTokenObj,homeAccountIdentifier,accountEntity,result;return __generator(this,(function(_a){switch(_a.label){case 0:if(this.logger.trace("NativeInteractionClient - handleNativeResponse called."),response.account.id!==request.accountId)throw NativeAuthError.createUserSwitchError();return[4,this.getDiscoveredAuthority(request.authority)];case 1:return authority=_a.sent(),authorityPreferredCache=authority.getPreferredCache(),idTokenObj=this.createIdTokenObj(response),homeAccountIdentifier=this.createHomeAccountIdentifier(response,idTokenObj),accountEntity=this.createAccountEntity(response,homeAccountIdentifier,idTokenObj,authorityPreferredCache),[4,this.generateAuthenticationResult(response,request,idTokenObj,accountEntity,authority.canonicalAuthority,reqTimestamp)];case 2:return result=_a.sent(),this.cacheAccount(accountEntity),this.cacheNativeTokens(response,request,homeAccountIdentifier,accountEntity,idTokenObj,result.accessToken,result.tenantId,reqTimestamp),[2,result]}}))}))},NativeInteractionClient.prototype.createIdTokenObj=function(response){return new AuthToken(response.id_token||Constants.EMPTY_STRING,this.browserCrypto)},NativeInteractionClient.prototype.createHomeAccountIdentifier=function(response,idTokenObj){return AccountEntity.generateHomeAccountId(response.client_info||Constants.EMPTY_STRING,AuthorityType.Default,this.logger,this.browserCrypto,idTokenObj)},NativeInteractionClient.prototype.createAccountEntity=function(response,homeAccountIdentifier,idTokenObj,authority){return AccountEntity.createAccount(response.client_info,homeAccountIdentifier,idTokenObj,void 0,void 0,void 0,authority,response.account.id)},NativeInteractionClient.prototype.generateScopes=function(response,request){return response.scope?ScopeSet.fromString(response.scope):ScopeSet.fromString(request.scope)},NativeInteractionClient.prototype.generatePopAccessToken=function(response,request){return __awaiter(this,void 0,void 0,(function(){var popTokenGenerator,shrParameters;return __generator(this,(function(_a){switch(_a.label){case 0:if(request.tokenType!==AuthenticationScheme.POP)return[3,2];if(response.shr)return this.logger.trace("handleNativeServerResponse: SHR is enabled in native layer"),[2,response.shr];if(popTokenGenerator=new PopTokenGenerator(this.browserCrypto),shrParameters={resourceRequestMethod:request.resourceRequestMethod,resourceRequestUri:request.resourceRequestUri,shrClaims:request.shrClaims,shrNonce:request.shrNonce},!request.keyId)throw ClientAuthError.createKeyIdMissingError();return[4,popTokenGenerator.signPopToken(response.access_token,request.keyId,shrParameters)];case 1:return[2,_a.sent()];case 2:return[2,response.access_token]}}))}))},NativeInteractionClient.prototype.generateAuthenticationResult=function(response,request,idTokenObj,accountEntity,authority,reqTimestamp){return __awaiter(this,void 0,void 0,(function(){var mats,responseScopes,accountProperties,uid,tid,responseAccessToken,tokenType;return __generator(this,(function(_a){switch(_a.label){case 0:return mats=this.addTelemetryFromNativeResponse(response),responseScopes=response.scope?ScopeSet.fromString(response.scope):ScopeSet.fromString(request.scope),accountProperties=response.account.properties||{},uid=accountProperties.UID||idTokenObj.claims.oid||idTokenObj.claims.sub||Constants.EMPTY_STRING,tid=accountProperties.TenantId||idTokenObj.claims.tid||Constants.EMPTY_STRING,[4,this.generatePopAccessToken(response,request)];case 1:return responseAccessToken=_a.sent(),tokenType=request.tokenType===AuthenticationScheme.POP?AuthenticationScheme.POP:AuthenticationScheme.BEARER,[2,{authority:authority,uniqueId:uid,tenantId:tid,scopes:responseScopes.asArray(),account:accountEntity.getAccountInfo(),idToken:response.id_token,idTokenClaims:idTokenObj.claims,accessToken:responseAccessToken,fromCache:!!mats&&this.isResponseFromCache(mats),expiresOn:new Date(1e3*Number(reqTimestamp+response.expires_in)),tokenType:tokenType,correlationId:this.correlationId,state:response.state,fromNativeBroker:!0}]}}))}))},NativeInteractionClient.prototype.cacheAccount=function(accountEntity){var _this=this;this.browserStorage.setAccount(accountEntity),this.browserStorage.removeAccountContext(accountEntity).catch((function(e){_this.logger.error("Error occurred while removing account context from browser storage. "+e)}))},NativeInteractionClient.prototype.cacheNativeTokens=function(response,request,homeAccountIdentifier,accountEntity,idTokenObj,responseAccessToken,tenantId,reqTimestamp){var cachedIdToken=IdTokenEntity.createIdTokenEntity(homeAccountIdentifier,request.authority,response.id_token||Constants.EMPTY_STRING,request.clientId,idTokenObj.claims.tid||Constants.EMPTY_STRING),tokenExpirationSeconds=reqTimestamp+(request.tokenType===AuthenticationScheme.POP?Constants.SHR_NONCE_VALIDITY:("string"==typeof response.expires_in?parseInt(response.expires_in,10):response.expires_in)||0),responseScopes=this.generateScopes(response,request),cachedAccessToken=AccessTokenEntity.createAccessTokenEntity(homeAccountIdentifier,request.authority,responseAccessToken,request.clientId,idTokenObj?idTokenObj.claims.tid||Constants.EMPTY_STRING:tenantId,responseScopes.printScopes(),tokenExpirationSeconds,0,this.browserCrypto),nativeCacheRecord=new CacheRecord(accountEntity,cachedIdToken,cachedAccessToken);this.nativeStorageManager.saveCacheRecord(nativeCacheRecord)},NativeInteractionClient.prototype.addTelemetryFromNativeResponse=function(response){var mats=this.getMATSFromResponse(response);return mats?(this.performanceClient.addStaticFields({extensionId:this.nativeMessageHandler.getExtensionId(),extensionVersion:this.nativeMessageHandler.getExtensionVersion(),matsBrokerVersion:mats.broker_version,matsAccountJoinOnStart:mats.account_join_on_start,matsAccountJoinOnEnd:mats.account_join_on_end,matsDeviceJoin:mats.device_join,matsPromptBehavior:mats.prompt_behavior,matsApiErrorCode:mats.api_error_code,matsUiVisible:mats.ui_visible,matsSilentCode:mats.silent_code,matsSilentBiSubCode:mats.silent_bi_sub_code,matsSilentMessage:mats.silent_message,matsSilentStatus:mats.silent_status,matsHttpStatus:mats.http_status,matsHttpEventCount:mats.http_event_count},this.correlationId),mats):null},NativeInteractionClient.prototype.validateNativeResponse=function(response){if(response.hasOwnProperty("access_token")&&response.hasOwnProperty("id_token")&&response.hasOwnProperty("client_info")&&response.hasOwnProperty("account")&&response.hasOwnProperty("scope")&&response.hasOwnProperty("expires_in"))return response;throw NativeAuthError.createUnexpectedError("Response missing expected properties.")},NativeInteractionClient.prototype.getMATSFromResponse=function(response){if(response.properties.MATS)try{return JSON.parse(response.properties.MATS)}catch(e){this.logger.error("NativeInteractionClient - Error parsing MATS telemetry, returning null instead")}return null},NativeInteractionClient.prototype.isResponseFromCache=function(mats){return void 0===mats.is_cached?(this.logger.verbose("NativeInteractionClient - MATS telemetry does not contain field indicating if response was served from cache. Returning false."),!1):!!mats.is_cached},NativeInteractionClient.prototype.initializeNativeRequest=function(request){return __awaiter(this,void 0,void 0,(function(){var authority,canonicalAuthority,scopes,remainingProperties,scopeSet,getPrompt,validatedRequest,shrParameters,reqCnfData,_this=this;return __generator(this,(function(_a){switch(_a.label){case 0:return this.logger.trace("NativeInteractionClient - initializeNativeRequest called"),authority=request.authority||this.config.auth.authority,request.account?[4,this.validateRequestAuthority(authority,request.account)]:[3,2];case 1:_a.sent(),_a.label=2;case 2:return(canonicalAuthority=new UrlString(authority)).validateAsUri(),scopes=request.scopes,remainingProperties=__rest(request,["scopes"]),(scopeSet=new ScopeSet(scopes||[])).appendScopes(OIDC_DEFAULT_SCOPES),getPrompt=function(){switch(_this.apiId){case ApiId.ssoSilent:case ApiId.acquireTokenSilent_silentFlow:return _this.logger.trace("initializeNativeRequest: silent request sets prompt to none"),PromptValue.NONE}if(request.prompt)switch(request.prompt){case PromptValue.NONE:case PromptValue.CONSENT:case PromptValue.LOGIN:return _this.logger.trace("initializeNativeRequest: prompt is compatible with native flow"),request.prompt;default:throw _this.logger.trace("initializeNativeRequest: prompt = "+request.prompt+" is not compatible with native flow"),BrowserAuthError.createNativePromptParameterNotSupportedError()}else _this.logger.trace("initializeNativeRequest: prompt was not provided")},validatedRequest=__assign(__assign({},remainingProperties),{accountId:this.accountId,clientId:this.config.auth.clientId,authority:canonicalAuthority.urlString,scope:scopeSet.printScopes(),redirectUri:this.getRedirectUri(request.redirectUri),prompt:getPrompt(),correlationId:this.correlationId,tokenType:request.authenticationScheme,windowTitleSubstring:document.title,extraParameters:__assign(__assign(__assign({},request.extraQueryParameters),request.tokenQueryParameters),{telemetry:NativeConstants.MATS_TELEMETRY}),extendedExpiryToken:!1}),request.authenticationScheme!==AuthenticationScheme.POP?[3,4]:(shrParameters={resourceRequestUri:request.resourceRequestUri,resourceRequestMethod:request.resourceRequestMethod,shrClaims:request.shrClaims,shrNonce:request.shrNonce},[4,new PopTokenGenerator(this.browserCrypto).generateCnf(shrParameters)]);case 3:reqCnfData=_a.sent(),validatedRequest.reqCnf=reqCnfData.reqCnfString,validatedRequest.keyId=reqCnfData.kid,_a.label=4;case 4:return[2,validatedRequest]}}))}))},NativeInteractionClient}(BaseInteractionClient);export{NativeInteractionClient};