/*! @azure/msal-browser v2.37.1 2023-06-07 */
"use strict";import{__extends,__awaiter,__generator,__assign}from"../_virtual/_tslib.js";import{UrlString,Constants,PerformanceEvents,AuthorizationCodeClient,Authority,AuthorityFactory,ResponseMode,StringUtils,ProtocolUtils}from"@azure/msal-common";import{BaseInteractionClient}from"./BaseInteractionClient.js";import{BrowserConstants}from"../utils/BrowserConstants.js";import{version}from"../packageMetadata.js";import{BrowserAuthError}from"../error/BrowserAuthError.js";import{BrowserProtocolUtils}from"../utils/BrowserProtocolUtils.js";import{BrowserUtils}from"../utils/BrowserUtils.js";var StandardInteractionClient=function(_super){function StandardInteractionClient(){return null!==_super&&_super.apply(this,arguments)||this}return __extends(StandardInteractionClient,_super),StandardInteractionClient.prototype.initializeAuthorizationCodeRequest=function(request){return __awaiter(this,void 0,void 0,(function(){var generatedPkceParams,authCodeRequest;return __generator(this,(function(_a){switch(_a.label){case 0:return this.performanceClient.addQueueMeasurement(PerformanceEvents.StandardInteractionClientInitializeAuthorizationCodeRequest,request.correlationId),this.logger.verbose("initializeAuthorizationRequest called",request.correlationId),[4,this.browserCrypto.generatePkceCodes()];case 1:return generatedPkceParams=_a.sent(),authCodeRequest=__assign(__assign({},request),{redirectUri:request.redirectUri,code:Constants.EMPTY_STRING,codeVerifier:generatedPkceParams.verifier}),request.codeChallenge=generatedPkceParams.challenge,request.codeChallengeMethod=Constants.S256_CODE_CHALLENGE_METHOD,[2,authCodeRequest]}}))}))},StandardInteractionClient.prototype.initializeLogoutRequest=function(logoutRequest){this.logger.verbose("initializeLogoutRequest called",null==logoutRequest?void 0:logoutRequest.correlationId);var validLogoutRequest=__assign({correlationId:this.correlationId||this.browserCrypto.createNewGuid()},logoutRequest);if(logoutRequest)if(logoutRequest.logoutHint)this.logger.verbose("logoutHint has already been set in logoutRequest");else if(logoutRequest.account){var logoutHint=this.getLogoutHintFromIdTokenClaims(logoutRequest.account);logoutHint&&(this.logger.verbose("Setting logoutHint to login_hint ID Token Claim value for the account provided"),validLogoutRequest.logoutHint=logoutHint)}else this.logger.verbose("logoutHint was not set and account was not passed into logout request, logoutHint will not be set");else this.logger.verbose("logoutHint will not be set since no logout request was configured");return logoutRequest&&null===logoutRequest.postLogoutRedirectUri?this.logger.verbose("postLogoutRedirectUri passed as null, not setting post logout redirect uri",validLogoutRequest.correlationId):logoutRequest&&logoutRequest.postLogoutRedirectUri?(this.logger.verbose("Setting postLogoutRedirectUri to uri set on logout request",validLogoutRequest.correlationId),validLogoutRequest.postLogoutRedirectUri=UrlString.getAbsoluteUrl(logoutRequest.postLogoutRedirectUri,BrowserUtils.getCurrentUri())):null===this.config.auth.postLogoutRedirectUri?this.logger.verbose("postLogoutRedirectUri configured as null and no uri set on request, not passing post logout redirect",validLogoutRequest.correlationId):this.config.auth.postLogoutRedirectUri?(this.logger.verbose("Setting postLogoutRedirectUri to configured uri",validLogoutRequest.correlationId),validLogoutRequest.postLogoutRedirectUri=UrlString.getAbsoluteUrl(this.config.auth.postLogoutRedirectUri,BrowserUtils.getCurrentUri())):(this.logger.verbose("Setting postLogoutRedirectUri to current page",validLogoutRequest.correlationId),validLogoutRequest.postLogoutRedirectUri=UrlString.getAbsoluteUrl(BrowserUtils.getCurrentUri(),BrowserUtils.getCurrentUri())),validLogoutRequest},StandardInteractionClient.prototype.getLogoutHintFromIdTokenClaims=function(account){var idTokenClaims=account.idTokenClaims;if(idTokenClaims){if(idTokenClaims.login_hint)return idTokenClaims.login_hint;this.logger.verbose("The ID Token Claims tied to the provided account do not contain a login_hint claim, logoutHint will not be added to logout request")}else this.logger.verbose("The provided account does not contain ID Token Claims, logoutHint will not be added to logout request");return null},StandardInteractionClient.prototype.createAuthCodeClient=function(serverTelemetryManager,authorityUrl,requestAzureCloudOptions){return __awaiter(this,void 0,void 0,(function(){var clientConfig;return __generator(this,(function(_a){switch(_a.label){case 0:return this.performanceClient.addQueueMeasurement(PerformanceEvents.StandardInteractionClientCreateAuthCodeClient,this.correlationId),this.performanceClient.setPreQueueTime(PerformanceEvents.StandardInteractionClientGetClientConfiguration,this.correlationId),[4,this.getClientConfiguration(serverTelemetryManager,authorityUrl,requestAzureCloudOptions)];case 1:return clientConfig=_a.sent(),[2,new AuthorizationCodeClient(clientConfig,this.performanceClient)]}}))}))},StandardInteractionClient.prototype.getClientConfiguration=function(serverTelemetryManager,requestAuthority,requestAzureCloudOptions){return __awaiter(this,void 0,void 0,(function(){var discoveredAuthority,logger;return __generator(this,(function(_a){switch(_a.label){case 0:return this.performanceClient.addQueueMeasurement(PerformanceEvents.StandardInteractionClientGetClientConfiguration,this.correlationId),this.logger.verbose("getClientConfiguration called",this.correlationId),this.performanceClient.setPreQueueTime(PerformanceEvents.StandardInteractionClientGetDiscoveredAuthority,this.correlationId),[4,this.getDiscoveredAuthority(requestAuthority,requestAzureCloudOptions)];case 1:return discoveredAuthority=_a.sent(),logger=this.config.system.loggerOptions,[2,{authOptions:{clientId:this.config.auth.clientId,authority:discoveredAuthority,clientCapabilities:this.config.auth.clientCapabilities},systemOptions:{tokenRenewalOffsetSeconds:this.config.system.tokenRenewalOffsetSeconds,preventCorsPreflight:!0},loggerOptions:{loggerCallback:logger.loggerCallback,piiLoggingEnabled:logger.piiLoggingEnabled,logLevel:logger.logLevel,correlationId:this.correlationId},cryptoInterface:this.browserCrypto,networkInterface:this.networkClient,storageInterface:this.browserStorage,serverTelemetryManager:serverTelemetryManager,libraryInfo:{sku:BrowserConstants.MSAL_SKU,version:version,cpu:Constants.EMPTY_STRING,os:Constants.EMPTY_STRING},telemetry:this.config.telemetry}]}}))}))},StandardInteractionClient.prototype.validateAndExtractStateFromHash=function(serverParams,interactionType,requestCorrelationId){if(this.logger.verbose("validateAndExtractStateFromHash called",requestCorrelationId),!serverParams.state)throw BrowserAuthError.createHashDoesNotContainStateError();var platformStateObj=BrowserProtocolUtils.extractBrowserRequestState(this.browserCrypto,serverParams.state);if(!platformStateObj)throw BrowserAuthError.createUnableToParseStateError();if(platformStateObj.interactionType!==interactionType)throw BrowserAuthError.createStateInteractionTypeMismatchError();return this.logger.verbose("Returning state from hash",requestCorrelationId),serverParams.state},StandardInteractionClient.prototype.getDiscoveredAuthority=function(requestAuthority,requestAzureCloudOptions){var _a;return __awaiter(this,void 0,void 0,(function(){var getAuthorityMeasurement,authorityOptions,userAuthority,builtAuthority;return __generator(this,(function(_b){switch(_b.label){case 0:return this.performanceClient.addQueueMeasurement(PerformanceEvents.StandardInteractionClientGetDiscoveredAuthority,this.correlationId),this.logger.verbose("getDiscoveredAuthority called",this.correlationId),getAuthorityMeasurement=null===(_a=this.performanceClient)||void 0===_a?void 0:_a.startMeasurement(PerformanceEvents.StandardInteractionClientGetDiscoveredAuthority,this.correlationId),authorityOptions={protocolMode:this.config.auth.protocolMode,knownAuthorities:this.config.auth.knownAuthorities,cloudDiscoveryMetadata:this.config.auth.cloudDiscoveryMetadata,authorityMetadata:this.config.auth.authorityMetadata,skipAuthorityMetadataCache:this.config.auth.skipAuthorityMetadataCache},userAuthority=requestAuthority||this.config.auth.authority,builtAuthority=Authority.generateAuthority(userAuthority,requestAzureCloudOptions||this.config.auth.azureCloudOptions),this.logger.verbose("Creating discovered authority with configured authority",this.correlationId),this.performanceClient.setPreQueueTime(PerformanceEvents.AuthorityFactoryCreateDiscoveredInstance,this.correlationId),[4,AuthorityFactory.createDiscoveredInstance(builtAuthority,this.config.system.networkClient,this.browserStorage,authorityOptions,this.logger,this.performanceClient,this.correlationId).then((function(result){return getAuthorityMeasurement.endMeasurement({success:!0}),result})).catch((function(error){throw getAuthorityMeasurement.endMeasurement({errorCode:error.errorCode,subErrorCode:error.subError,success:!1}),error}))];case 1:return[2,_b.sent()]}}))}))},StandardInteractionClient.prototype.initializeAuthorizationRequest=function(request,interactionType){return __awaiter(this,void 0,void 0,(function(){var redirectUri,browserState,state,validatedRequest,_a,account,legacyLoginHint;return __generator(this,(function(_b){switch(_b.label){case 0:return this.performanceClient.addQueueMeasurement(PerformanceEvents.StandardInteractionClientInitializeAuthorizationRequest,this.correlationId),this.logger.verbose("initializeAuthorizationRequest called",this.correlationId),redirectUri=this.getRedirectUri(request.redirectUri),browserState={interactionType:interactionType},state=ProtocolUtils.setRequestState(this.browserCrypto,request&&request.state||Constants.EMPTY_STRING,browserState),this.performanceClient.setPreQueueTime(PerformanceEvents.InitializeBaseRequest,this.correlationId),_a=[{}],[4,this.initializeBaseRequest(request)];case 1:return validatedRequest=__assign.apply(void 0,[__assign.apply(void 0,_a.concat([_b.sent()])),{redirectUri:redirectUri,state:state,nonce:request.nonce||this.browserCrypto.createNewGuid(),responseMode:ResponseMode.FRAGMENT}]),(account=request.account||this.browserStorage.getActiveAccount())&&(this.logger.verbose("Setting validated request account",this.correlationId),this.logger.verbosePii("Setting validated request account: "+account.homeAccountId,this.correlationId),validatedRequest.account=account),StringUtils.isEmpty(validatedRequest.loginHint)&&!account&&(legacyLoginHint=this.browserStorage.getLegacyLoginHint())&&(validatedRequest.loginHint=legacyLoginHint),[2,validatedRequest]}}))}))},StandardInteractionClient}(BaseInteractionClient);export{StandardInteractionClient};