/*! @azure/msal-common v7.6.0 2022-10-10 */
"use strict";import{__awaiter,__generator}from"../_virtual/_tslib.js";import{buildClientInfo}from"../account/ClientInfo.js";import{ClientAuthError}from"../error/ClientAuthError.js";import{StringUtils}from"../utils/StringUtils.js";import{ServerError}from"../error/ServerError.js";import{AuthToken}from"../account/AuthToken.js";import{ScopeSet}from"../request/ScopeSet.js";import{AccountEntity}from"../cache/entities/AccountEntity.js";import{AuthorityType}from"../authority/AuthorityType.js";import{IdTokenEntity}from"../cache/entities/IdTokenEntity.js";import{AccessTokenEntity}from"../cache/entities/AccessTokenEntity.js";import{RefreshTokenEntity}from"../cache/entities/RefreshTokenEntity.js";import{InteractionRequiredAuthError}from"../error/InteractionRequiredAuthError.js";import{CacheRecord}from"../cache/entities/CacheRecord.js";import{ProtocolUtils}from"../utils/ProtocolUtils.js";import{Constants,AuthenticationScheme,THE_FAMILY_ID}from"../utils/Constants.js";import{PopTokenGenerator}from"../crypto/PopTokenGenerator.js";import{AppMetadataEntity}from"../cache/entities/AppMetadataEntity.js";import{TokenCacheContext}from"../cache/persistence/TokenCacheContext.js";var ResponseHandler=function(){function ResponseHandler(clientId,cacheStorage,cryptoObj,logger,serializableCache,persistencePlugin){this.clientId=clientId,this.cacheStorage=cacheStorage,this.cryptoObj=cryptoObj,this.logger=logger,this.serializableCache=serializableCache,this.persistencePlugin=persistencePlugin}return ResponseHandler.prototype.validateServerAuthorizationCodeResponse=function(serverResponseHash,cachedState,cryptoObj){if(!serverResponseHash.state||!cachedState)throw serverResponseHash.state?ClientAuthError.createStateNotFoundError("Cached State"):ClientAuthError.createStateNotFoundError("Server State");if(decodeURIComponent(serverResponseHash.state)!==decodeURIComponent(cachedState))throw ClientAuthError.createStateMismatchError();if(serverResponseHash.error||serverResponseHash.error_description||serverResponseHash.suberror){if(InteractionRequiredAuthError.isInteractionRequiredError(serverResponseHash.error,serverResponseHash.error_description,serverResponseHash.suberror))throw new InteractionRequiredAuthError(serverResponseHash.error||Constants.EMPTY_STRING,serverResponseHash.error_description,serverResponseHash.suberror);throw new ServerError(serverResponseHash.error||Constants.EMPTY_STRING,serverResponseHash.error_description,serverResponseHash.suberror)}serverResponseHash.client_info&&buildClientInfo(serverResponseHash.client_info,cryptoObj)},ResponseHandler.prototype.validateTokenResponse=function(serverResponse){if(serverResponse.error||serverResponse.error_description||serverResponse.suberror){if(InteractionRequiredAuthError.isInteractionRequiredError(serverResponse.error,serverResponse.error_description,serverResponse.suberror))throw new InteractionRequiredAuthError(serverResponse.error,serverResponse.error_description,serverResponse.suberror);var errString=serverResponse.error_codes+" - ["+serverResponse.timestamp+"]: "+serverResponse.error_description+" - Correlation ID: "+serverResponse.correlation_id+" - Trace ID: "+serverResponse.trace_id;throw new ServerError(serverResponse.error,errString,serverResponse.suberror)}},ResponseHandler.prototype.handleServerTokenResponse=function(serverTokenResponse,authority,reqTimestamp,request,authCodePayload,userAssertionHash,handlingRefreshTokenResponse,forceCacheRefreshTokenResponse,serverRequestId){return __awaiter(this,void 0,void 0,(function(){var idTokenObj,authTime,requestStateObj,cacheRecord,cacheContext,key;return __generator(this,(function(_a){switch(_a.label){case 0:if(serverTokenResponse.id_token){if(idTokenObj=new AuthToken(serverTokenResponse.id_token||Constants.EMPTY_STRING,this.cryptoObj),authCodePayload&&!StringUtils.isEmpty(authCodePayload.nonce)&&idTokenObj.claims.nonce!==authCodePayload.nonce)throw ClientAuthError.createNonceMismatchError();if(request.maxAge||0===request.maxAge){if(!(authTime=idTokenObj.claims.auth_time))throw ClientAuthError.createAuthTimeNotFoundError();AuthToken.checkMaxAge(authTime,request.maxAge)}}this.homeAccountIdentifier=AccountEntity.generateHomeAccountId(serverTokenResponse.client_info||Constants.EMPTY_STRING,authority.authorityType,this.logger,this.cryptoObj,idTokenObj),authCodePayload&&authCodePayload.state&&(requestStateObj=ProtocolUtils.parseRequestState(this.cryptoObj,authCodePayload.state)),serverTokenResponse.key_id=serverTokenResponse.key_id||request.sshKid||void 0,cacheRecord=this.generateCacheRecord(serverTokenResponse,authority,reqTimestamp,request,idTokenObj,userAssertionHash,authCodePayload),_a.label=1;case 1:return _a.trys.push([1,,5,8]),this.persistencePlugin&&this.serializableCache?(this.logger.verbose("Persistence enabled, calling beforeCacheAccess"),cacheContext=new TokenCacheContext(this.serializableCache,!0),[4,this.persistencePlugin.beforeCacheAccess(cacheContext)]):[3,3];case 2:_a.sent(),_a.label=3;case 3:return!handlingRefreshTokenResponse||forceCacheRefreshTokenResponse||!cacheRecord.account||(key=cacheRecord.account.generateAccountKey(),this.cacheStorage.getAccount(key))?[4,this.cacheStorage.saveCacheRecord(cacheRecord)]:(this.logger.warning("Account used to refresh tokens not in persistence, refreshed tokens will not be stored in the cache"),[2,ResponseHandler.generateAuthenticationResult(this.cryptoObj,authority,cacheRecord,!1,request,idTokenObj,requestStateObj,void 0,serverRequestId)]);case 4:return _a.sent(),[3,8];case 5:return this.persistencePlugin&&this.serializableCache&&cacheContext?(this.logger.verbose("Persistence enabled, calling afterCacheAccess"),[4,this.persistencePlugin.afterCacheAccess(cacheContext)]):[3,7];case 6:_a.sent(),_a.label=7;case 7:return[7];case 8:return[2,ResponseHandler.generateAuthenticationResult(this.cryptoObj,authority,cacheRecord,!1,request,idTokenObj,requestStateObj,serverTokenResponse.spa_code,serverRequestId)]}}))}))},ResponseHandler.prototype.generateCacheRecord=function(serverTokenResponse,authority,reqTimestamp,request,idTokenObj,userAssertionHash,authCodePayload){var cachedIdToken,cachedAccount,env=authority.getPreferredCache();if(StringUtils.isEmpty(env))throw ClientAuthError.createInvalidCacheEnvironmentError();!StringUtils.isEmpty(serverTokenResponse.id_token)&&idTokenObj&&(cachedIdToken=IdTokenEntity.createIdTokenEntity(this.homeAccountIdentifier,env,serverTokenResponse.id_token||Constants.EMPTY_STRING,this.clientId,idTokenObj.claims.tid||Constants.EMPTY_STRING),cachedAccount=this.generateAccountEntity(serverTokenResponse,idTokenObj,authority,authCodePayload));var cachedAccessToken=null;if(!StringUtils.isEmpty(serverTokenResponse.access_token)){var responseScopes=serverTokenResponse.scope?ScopeSet.fromString(serverTokenResponse.scope):new ScopeSet(request.scopes||[]),expiresIn=("string"==typeof serverTokenResponse.expires_in?parseInt(serverTokenResponse.expires_in,10):serverTokenResponse.expires_in)||0,extExpiresIn=("string"==typeof serverTokenResponse.ext_expires_in?parseInt(serverTokenResponse.ext_expires_in,10):serverTokenResponse.ext_expires_in)||0,refreshIn=("string"==typeof serverTokenResponse.refresh_in?parseInt(serverTokenResponse.refresh_in,10):serverTokenResponse.refresh_in)||void 0,tokenExpirationSeconds=reqTimestamp+expiresIn,extendedTokenExpirationSeconds=tokenExpirationSeconds+extExpiresIn,refreshOnSeconds=refreshIn&&refreshIn>0?reqTimestamp+refreshIn:void 0;cachedAccessToken=AccessTokenEntity.createAccessTokenEntity(this.homeAccountIdentifier,env,serverTokenResponse.access_token||Constants.EMPTY_STRING,this.clientId,idTokenObj?idTokenObj.claims.tid||Constants.EMPTY_STRING:authority.tenant,responseScopes.printScopes(),tokenExpirationSeconds,extendedTokenExpirationSeconds,this.cryptoObj,refreshOnSeconds,serverTokenResponse.token_type,userAssertionHash,serverTokenResponse.key_id,request.claims,request.requestedClaimsHash)}var cachedRefreshToken=null;StringUtils.isEmpty(serverTokenResponse.refresh_token)||(cachedRefreshToken=RefreshTokenEntity.createRefreshTokenEntity(this.homeAccountIdentifier,env,serverTokenResponse.refresh_token||Constants.EMPTY_STRING,this.clientId,serverTokenResponse.foci,userAssertionHash));var cachedAppMetadata=null;return StringUtils.isEmpty(serverTokenResponse.foci)||(cachedAppMetadata=AppMetadataEntity.createAppMetadataEntity(this.clientId,env,serverTokenResponse.foci)),new CacheRecord(cachedAccount,cachedIdToken,cachedAccessToken,cachedRefreshToken,cachedAppMetadata)},ResponseHandler.prototype.generateAccountEntity=function(serverTokenResponse,idToken,authority,authCodePayload){var authorityType=authority.authorityType,cloudGraphHostName=authCodePayload?authCodePayload.cloud_graph_host_name:Constants.EMPTY_STRING,msGraphhost=authCodePayload?authCodePayload.msgraph_host:Constants.EMPTY_STRING;if(authorityType===AuthorityType.Adfs)return this.logger.verbose("Authority type is ADFS, creating ADFS account"),AccountEntity.createGenericAccount(this.homeAccountIdentifier,idToken,authority,cloudGraphHostName,msGraphhost);if(StringUtils.isEmpty(serverTokenResponse.client_info)&&"AAD"===authority.protocolMode)throw ClientAuthError.createClientInfoEmptyError();return serverTokenResponse.client_info?AccountEntity.createAccount(serverTokenResponse.client_info,this.homeAccountIdentifier,idToken,authority,cloudGraphHostName,msGraphhost):AccountEntity.createGenericAccount(this.homeAccountIdentifier,idToken,authority,cloudGraphHostName,msGraphhost)},ResponseHandler.generateAuthenticationResult=function(cryptoObj,authority,cacheRecord,fromTokenCache,request,idTokenObj,requestState,code,requestId){var _a,_b,_c;return __awaiter(this,void 0,void 0,(function(){var accessToken,responseScopes,expiresOn,extExpiresOn,familyId,popTokenGenerator,_d,secret,keyId,uid,tid;return __generator(this,(function(_e){switch(_e.label){case 0:if(accessToken=Constants.EMPTY_STRING,responseScopes=[],expiresOn=null,familyId=Constants.EMPTY_STRING,!cacheRecord.accessToken)return[3,4];if(cacheRecord.accessToken.tokenType!==AuthenticationScheme.POP)return[3,2];if(popTokenGenerator=new PopTokenGenerator(cryptoObj),_d=cacheRecord.accessToken,secret=_d.secret,!(keyId=_d.keyId))throw ClientAuthError.createKeyIdMissingError();return[4,popTokenGenerator.signPopToken(secret,keyId,request)];case 1:return accessToken=_e.sent(),[3,3];case 2:accessToken=cacheRecord.accessToken.secret,_e.label=3;case 3:responseScopes=ScopeSet.fromString(cacheRecord.accessToken.target).asArray(),expiresOn=new Date(1e3*Number(cacheRecord.accessToken.expiresOn)),extExpiresOn=new Date(1e3*Number(cacheRecord.accessToken.extendedExpiresOn)),_e.label=4;case 4:return cacheRecord.appMetadata&&(familyId=cacheRecord.appMetadata.familyId===THE_FAMILY_ID?THE_FAMILY_ID:Constants.EMPTY_STRING),uid=(null==idTokenObj?void 0:idTokenObj.claims.oid)||(null==idTokenObj?void 0:idTokenObj.claims.sub)||Constants.EMPTY_STRING,tid=(null==idTokenObj?void 0:idTokenObj.claims.tid)||Constants.EMPTY_STRING,[2,{authority:authority.canonicalAuthority,uniqueId:uid,tenantId:tid,scopes:responseScopes,account:cacheRecord.account?cacheRecord.account.getAccountInfo():null,idToken:idTokenObj?idTokenObj.rawToken:Constants.EMPTY_STRING,idTokenClaims:idTokenObj?idTokenObj.claims:{},accessToken:accessToken,fromCache:fromTokenCache,expiresOn:expiresOn,correlationId:request.correlationId,requestId:requestId||Constants.EMPTY_STRING,extExpiresOn:extExpiresOn,familyId:familyId,tokenType:(null===(_a=cacheRecord.accessToken)||void 0===_a?void 0:_a.tokenType)||Constants.EMPTY_STRING,state:requestState?requestState.userRequestState:Constants.EMPTY_STRING,cloudGraphHostName:(null===(_b=cacheRecord.account)||void 0===_b?void 0:_b.cloudGraphHostName)||Constants.EMPTY_STRING,msGraphHost:(null===(_c=cacheRecord.account)||void 0===_c?void 0:_c.msGraphHost)||Constants.EMPTY_STRING,code:code,fromNativeBroker:!1}]}}))}))},ResponseHandler}();export{ResponseHandler};