/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{assert,requireNonNull,requireInstance}from"./assert";import{MathUtil}from"./MathUtil";import{DateTimeException,UnsupportedTemporalTypeException,NullPointerException,IllegalArgumentException}from"./errors";import{IsoChronology}from"./chrono/IsoChronology";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{ChronoLocalDate}from"./chrono/ChronoLocalDate";import{TemporalQueries}from"./temporal/TemporalQueries";import{createTemporalQuery}from"./temporal/TemporalQuery";import{ValueRange}from"./temporal/ValueRange";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{Clock}from"./Clock";import{DayOfWeek}from"./DayOfWeek";import{OffsetDateTime}from"./OffsetDateTime";import{OffsetTime}from"./OffsetTime";import{Month}from"./Month";import{Period}from"./Period";import{YearConstants}from"./YearConstants";import{LocalTime}from"./LocalTime";import{LocalDateTime}from"./LocalDateTime";import{Year}from"./Year";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";import{ZonedDateTime}from"./ZonedDateTime";const DAYS_PER_CYCLE=146097,DAYS_0000_TO_1970=5*DAYS_PER_CYCLE-10957;export class LocalDate extends ChronoLocalDate{static now(clockOrZone){let clock;return clock=null==clockOrZone?Clock.systemDefaultZone():clockOrZone instanceof ZoneId?Clock.system(clockOrZone):clockOrZone,LocalDate.ofInstant(clock.instant(),clock.zone())}static ofInstant(instant,zone=ZoneId.systemDefault()){requireNonNull(instant,"instant");const offset=zone.rules().offset(instant),epochSec=instant.epochSecond()+offset.totalSeconds(),epochDay=MathUtil.floorDiv(epochSec,LocalTime.SECONDS_PER_DAY);return LocalDate.ofEpochDay(epochDay)}static of(year,month,dayOfMonth){return new LocalDate(year,month,dayOfMonth)}static ofYearDay(year,dayOfYear){ChronoField.YEAR.checkValidValue(year);const leap=IsoChronology.isLeapYear(year);366===dayOfYear&&!1===leap&&assert(!1,`Invalid date 'DayOfYear 366' as '${year}' is not a leap year`,DateTimeException);let moy=Month.of(Math.floor((dayOfYear-1)/31+1));dayOfYear>moy.firstDayOfYear(leap)+moy.length(leap)-1&&(moy=moy.plus(1));const dom=dayOfYear-moy.firstDayOfYear(leap)+1;return new LocalDate(year,moy.value(),dom)}static ofEpochDay(epochDay=0){let adjust,adjustCycles,doyEst,yearEst,zeroDay;zeroDay=epochDay+DAYS_0000_TO_1970,zeroDay-=60,adjust=0,zeroDay<0&&(adjustCycles=MathUtil.intDiv(zeroDay+1,DAYS_PER_CYCLE)-1,adjust=400*adjustCycles,zeroDay+=-adjustCycles*DAYS_PER_CYCLE),yearEst=MathUtil.intDiv(400*zeroDay+591,DAYS_PER_CYCLE),doyEst=zeroDay-(365*yearEst+MathUtil.intDiv(yearEst,4)-MathUtil.intDiv(yearEst,100)+MathUtil.intDiv(yearEst,400)),doyEst<0&&(yearEst--,doyEst=zeroDay-(365*yearEst+MathUtil.intDiv(yearEst,4)-MathUtil.intDiv(yearEst,100)+MathUtil.intDiv(yearEst,400))),yearEst+=adjust;const marchDoy0=doyEst,marchMonth0=MathUtil.intDiv(5*marchDoy0+2,153),month=(marchMonth0+2)%12+1,dom=marchDoy0-MathUtil.intDiv(306*marchMonth0+5,10)+1;yearEst+=MathUtil.intDiv(marchMonth0,10);return new LocalDate(yearEst,month,dom)}static from(temporal){requireNonNull(temporal,"temporal");const date=temporal.query(TemporalQueries.localDate());if(null==date)throw new DateTimeException(`Unable to obtain LocalDate from TemporalAccessor: ${temporal}, type ${null!=temporal.constructor?temporal.constructor.name:""}`);return date}static parse(text,formatter=DateTimeFormatter.ISO_LOCAL_DATE){return assert(null!=formatter,"formatter",NullPointerException),formatter.parse(text,LocalDate.FROM)}static _resolvePreviousValid(year,month,day){switch(month){case 2:day=Math.min(day,IsoChronology.isLeapYear(year)?29:28);break;case 4:case 6:case 9:case 11:day=Math.min(day,30)}return LocalDate.of(year,month,day)}constructor(year,month,dayOfMonth){super(),requireNonNull(year,"year"),requireNonNull(month,"month"),requireNonNull(dayOfMonth,"dayOfMonth"),month instanceof Month&&(month=month.value()),this._year=MathUtil.safeToInt(year),this._month=MathUtil.safeToInt(month),this._day=MathUtil.safeToInt(dayOfMonth),LocalDate._validate(this._year,this._month,this._day)}static _validate(year,month,dayOfMonth){let dom;if(ChronoField.YEAR.checkValidValue(year),ChronoField.MONTH_OF_YEAR.checkValidValue(month),ChronoField.DAY_OF_MONTH.checkValidValue(dayOfMonth),dayOfMonth>28){switch(dom=31,month){case 2:dom=IsoChronology.isLeapYear(year)?29:28;break;case 4:case 6:case 9:case 11:dom=30}dayOfMonth>dom&&assert(!1,29===dayOfMonth?`Invalid date 'February 29' as '${year}' is not a leap year`:`Invalid date '${year}' '${month}' '${dayOfMonth}'`,DateTimeException)}}isSupported(field){return super.isSupported(field)}range(field){if(field instanceof ChronoField){if(field.isDateBased()){switch(field){case ChronoField.DAY_OF_MONTH:return ValueRange.of(1,this.lengthOfMonth());case ChronoField.DAY_OF_YEAR:return ValueRange.of(1,this.lengthOfYear());case ChronoField.ALIGNED_WEEK_OF_MONTH:return ValueRange.of(1,this.month()===Month.FEBRUARY&&!1===this.isLeapYear()?4:5);case ChronoField.YEAR_OF_ERA:return this._year<=0?ValueRange.of(1,Year.MAX_VALUE+1):ValueRange.of(1,Year.MAX_VALUE)}return field.range()}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}return field.rangeRefinedBy(this)}get(field){return this.getLong(field)}getLong(field){return assert(null!=field,"",NullPointerException),field instanceof ChronoField?this._get0(field):field.getFrom(this)}_get0(field){switch(field){case ChronoField.DAY_OF_WEEK:return this.dayOfWeek().value();case ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH:return MathUtil.intMod(this._day-1,7)+1;case ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR:return MathUtil.intMod(this.dayOfYear()-1,7)+1;case ChronoField.DAY_OF_MONTH:return this._day;case ChronoField.DAY_OF_YEAR:return this.dayOfYear();case ChronoField.EPOCH_DAY:return this.toEpochDay();case ChronoField.ALIGNED_WEEK_OF_MONTH:return MathUtil.intDiv(this._day-1,7)+1;case ChronoField.ALIGNED_WEEK_OF_YEAR:return MathUtil.intDiv(this.dayOfYear()-1,7)+1;case ChronoField.MONTH_OF_YEAR:return this._month;case ChronoField.PROLEPTIC_MONTH:return this._prolepticMonth();case ChronoField.YEAR_OF_ERA:return this._year>=1?this._year:1-this._year;case ChronoField.YEAR:return this._year;case ChronoField.ERA:return this._year>=1?1:0}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}_prolepticMonth(){return 12*this._year+(this._month-1)}chronology(){return IsoChronology.INSTANCE}year(){return this._year}monthValue(){return this._month}month(){return Month.of(this._month)}dayOfMonth(){return this._day}dayOfYear(){return this.month().firstDayOfYear(this.isLeapYear())+this._day-1}dayOfWeek(){const dow0=MathUtil.floorMod(this.toEpochDay()+3,7);return DayOfWeek.of(dow0+1)}isLeapYear(){return IsoChronology.isLeapYear(this._year)}lengthOfMonth(){switch(this._month){case 2:return this.isLeapYear()?29:28;case 4:case 6:case 9:case 11:return 30;default:return 31}}lengthOfYear(){return this.isLeapYear()?366:365}_withAdjuster(adjuster){return requireNonNull(adjuster,"adjuster"),adjuster instanceof LocalDate?adjuster:super._withAdjuster(adjuster)}_withField(field,newValue){if(assert(null!=field,"field",NullPointerException),field instanceof ChronoField){const f=field;switch(f.checkValidValue(newValue),f){case ChronoField.DAY_OF_WEEK:return this.plusDays(newValue-this.dayOfWeek().value());case ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH:return this.plusDays(newValue-this.getLong(ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH));case ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR:return this.plusDays(newValue-this.getLong(ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR));case ChronoField.DAY_OF_MONTH:return this.withDayOfMonth(newValue);case ChronoField.DAY_OF_YEAR:return this.withDayOfYear(newValue);case ChronoField.EPOCH_DAY:return LocalDate.ofEpochDay(newValue);case ChronoField.ALIGNED_WEEK_OF_MONTH:return this.plusWeeks(newValue-this.getLong(ChronoField.ALIGNED_WEEK_OF_MONTH));case ChronoField.ALIGNED_WEEK_OF_YEAR:return this.plusWeeks(newValue-this.getLong(ChronoField.ALIGNED_WEEK_OF_YEAR));case ChronoField.MONTH_OF_YEAR:return this.withMonth(newValue);case ChronoField.PROLEPTIC_MONTH:return this.plusMonths(newValue-this.getLong(ChronoField.PROLEPTIC_MONTH));case ChronoField.YEAR_OF_ERA:return this.withYear(this._year>=1?newValue:1-newValue);case ChronoField.YEAR:return this.withYear(newValue);case ChronoField.ERA:return this.getLong(ChronoField.ERA)===newValue?this:this.withYear(1-this._year)}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}return field.adjustInto(this,newValue)}withYear(year){return this._year===year?this:(ChronoField.YEAR.checkValidValue(year),LocalDate._resolvePreviousValid(year,this._month,this._day))}withMonth(month){const m=month instanceof Month?month.value():month;return this._month===m?this:(ChronoField.MONTH_OF_YEAR.checkValidValue(m),LocalDate._resolvePreviousValid(this._year,m,this._day))}withDayOfMonth(dayOfMonth){return this._day===dayOfMonth?this:LocalDate.of(this._year,this._month,dayOfMonth)}withDayOfYear(dayOfYear){return this.dayOfYear()===dayOfYear?this:LocalDate.ofYearDay(this._year,dayOfYear)}_plusUnit(amountToAdd,unit){if(requireNonNull(amountToAdd,"amountToAdd"),requireNonNull(unit,"unit"),unit instanceof ChronoUnit){switch(unit){case ChronoUnit.DAYS:return this.plusDays(amountToAdd);case ChronoUnit.WEEKS:return this.plusWeeks(amountToAdd);case ChronoUnit.MONTHS:return this.plusMonths(amountToAdd);case ChronoUnit.YEARS:return this.plusYears(amountToAdd);case ChronoUnit.DECADES:return this.plusYears(MathUtil.safeMultiply(amountToAdd,10));case ChronoUnit.CENTURIES:return this.plusYears(MathUtil.safeMultiply(amountToAdd,100));case ChronoUnit.MILLENNIA:return this.plusYears(MathUtil.safeMultiply(amountToAdd,1e3));case ChronoUnit.ERAS:return this.with(ChronoField.ERA,MathUtil.safeAdd(this.getLong(ChronoField.ERA),amountToAdd))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.addTo(this,amountToAdd)}plusYears(yearsToAdd){if(0===yearsToAdd)return this;const newYear=ChronoField.YEAR.checkValidIntValue(this._year+yearsToAdd);return LocalDate._resolvePreviousValid(newYear,this._month,this._day)}plusMonths(monthsToAdd){if(0===monthsToAdd)return this;const calcMonths=12*this._year+(this._month-1)+monthsToAdd,newYear=ChronoField.YEAR.checkValidIntValue(MathUtil.floorDiv(calcMonths,12)),newMonth=MathUtil.floorMod(calcMonths,12)+1;return LocalDate._resolvePreviousValid(newYear,newMonth,this._day)}plusWeeks(weeksToAdd){return this.plusDays(MathUtil.safeMultiply(weeksToAdd,7))}plusDays(daysToAdd){if(0===daysToAdd)return this;const mjDay=MathUtil.safeAdd(this.toEpochDay(),daysToAdd);return LocalDate.ofEpochDay(mjDay)}_minusUnit(amountToSubtract,unit){return requireNonNull(amountToSubtract,"amountToSubtract"),requireNonNull(unit,"unit"),this._plusUnit(-1*amountToSubtract,unit)}minusYears(yearsToSubtract){return this.plusYears(-1*yearsToSubtract)}minusMonths(monthsToSubtract){return this.plusMonths(-1*monthsToSubtract)}minusWeeks(weeksToSubtract){return this.plusWeeks(-1*weeksToSubtract)}minusDays(daysToSubtract){return this.plusDays(-1*daysToSubtract)}query(query){return requireNonNull(query,"query"),query===TemporalQueries.localDate()?this:super.query(query)}adjustInto(temporal){return super.adjustInto(temporal)}until(p1,p2){return arguments.length<2?this.until1(p1):this.until2(p1,p2)}until2(endExclusive,unit){const end=LocalDate.from(endExclusive);if(unit instanceof ChronoUnit){switch(unit){case ChronoUnit.DAYS:return this.daysUntil(end);case ChronoUnit.WEEKS:return MathUtil.intDiv(this.daysUntil(end),7);case ChronoUnit.MONTHS:return this._monthsUntil(end);case ChronoUnit.YEARS:return MathUtil.intDiv(this._monthsUntil(end),12);case ChronoUnit.DECADES:return MathUtil.intDiv(this._monthsUntil(end),120);case ChronoUnit.CENTURIES:return MathUtil.intDiv(this._monthsUntil(end),1200);case ChronoUnit.MILLENNIA:return MathUtil.intDiv(this._monthsUntil(end),12e3);case ChronoUnit.ERAS:return end.getLong(ChronoField.ERA)-this.getLong(ChronoField.ERA)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.between(this,end)}daysUntil(end){return end.toEpochDay()-this.toEpochDay()}_monthsUntil(end){const packed1=32*this._prolepticMonth()+this.dayOfMonth(),packed2=32*end._prolepticMonth()+end.dayOfMonth();return MathUtil.intDiv(packed2-packed1,32)}until1(endDate){const end=LocalDate.from(endDate);let totalMonths=end._prolepticMonth()-this._prolepticMonth(),days=end._day-this._day;if(totalMonths>0&&days<0){totalMonths--;const calcDate=this.plusMonths(totalMonths);days=end.toEpochDay()-calcDate.toEpochDay()}else totalMonths<0&&days>0&&(totalMonths++,days-=end.lengthOfMonth());const years=MathUtil.intDiv(totalMonths,12),months=MathUtil.intMod(totalMonths,12);return Period.of(years,months,days)}atTime(){return 1===arguments.length?this.atTime1.apply(this,arguments):this.atTime4.apply(this,arguments)}atTime1(time){if(requireNonNull(time,"time"),time instanceof LocalTime)return LocalDateTime.of(this,time);if(time instanceof OffsetTime)return this._atTimeOffsetTime(time);throw new IllegalArgumentException("time must be an instance of LocalTime or OffsetTime"+(time&&time.constructor&&time.constructor.name?`, but is ${time.constructor.name}`:""))}atTime4(hour,minute,second=0,nanoOfSecond=0){return this.atTime1(LocalTime.of(hour,minute,second,nanoOfSecond))}_atTimeOffsetTime(time){return OffsetDateTime.of(LocalDateTime.of(this,time.toLocalTime()),time.offset())}atStartOfDay(zone){return null!=zone?this._atStartOfDayWithZone(zone):LocalDateTime.of(this,LocalTime.MIDNIGHT)}_atStartOfDayWithZone(zone){requireNonNull(zone,"zone");let ldt=this.atTime(LocalTime.MIDNIGHT);if(zone instanceof ZoneOffset==!1){const trans=zone.rules().transition(ldt);null!=trans&&trans.isGap()&&(ldt=trans.dateTimeAfter())}return ZonedDateTime.of(ldt,zone)}toEpochDay(){const y=this._year,m=this._month;let total=0;return total+=365*y,y>=0?total+=MathUtil.intDiv(y+3,4)-MathUtil.intDiv(y+99,100)+MathUtil.intDiv(y+399,400):total-=MathUtil.intDiv(y,-4)-MathUtil.intDiv(y,-100)+MathUtil.intDiv(y,-400),total+=MathUtil.intDiv(367*m-362,12),total+=this.dayOfMonth()-1,m>2&&(total--,IsoChronology.isLeapYear(y)||total--),total-DAYS_0000_TO_1970}compareTo(other){return requireNonNull(other,"other"),requireInstance(other,LocalDate,"other"),this._compareTo0(other)}_compareTo0(otherDate){let cmp=this._year-otherDate._year;return 0===cmp&&(cmp=this._month-otherDate._month,0===cmp&&(cmp=this._day-otherDate._day)),cmp}isAfter(other){return this.compareTo(other)>0}isBefore(other){return this.compareTo(other)<0}isEqual(other){return 0===this.compareTo(other)}equals(other){return this===other||other instanceof LocalDate&&0===this._compareTo0(other)}hashCode(){const yearValue=this._year,monthValue=this._month,dayValue=this._day;return MathUtil.hash(4294965248&yearValue^(yearValue<<11)+(monthValue<<6)+dayValue)}toString(){let dayString,monthString,yearString;const yearValue=this._year,monthValue=this._month,dayValue=this._day;return yearString=Math.abs(yearValue)<1e3?yearValue<0?`-${(""+(yearValue-1e4)).slice(-4)}`:`${yearValue+1e4}`.slice(-4):yearValue>9999?`+${yearValue}`:`${yearValue}`,monthString=monthValue<10?`-0${monthValue}`:`-${monthValue}`,dayString=dayValue<10?`-0${dayValue}`:`-${dayValue}`,yearString+monthString+dayString}toJSON(){return this.toString()}format(formatter){return requireNonNull(formatter,"formatter"),requireInstance(formatter,DateTimeFormatter,"formatter"),super.format(formatter)}}export function _init(){LocalDate.MIN=LocalDate.of(YearConstants.MIN_VALUE,1,1),LocalDate.MAX=LocalDate.of(YearConstants.MAX_VALUE,12,31),LocalDate.EPOCH_0=LocalDate.ofEpochDay(0),LocalDate.FROM=createTemporalQuery("LocalDate.FROM",(temporal=>LocalDate.from(temporal)))}