/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @license BSD-3-Clause (see LICENSE.md in the root directory of this source tree)
 */
import{requireNonNull,requireInstance}from"./assert";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{MathUtil}from"./MathUtil";import{ChronoField}from"./temporal/ChronoField";import{Clock}from"./Clock";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{DateTimeFormatterBuilder}from"./format/DateTimeFormatterBuilder";import{IsoChronology}from"./chrono/IsoChronology";import{LocalDate}from"./LocalDate";import{Month}from"./Month";import{TemporalAccessor}from"./temporal/TemporalAccessor";import{TemporalQuery,createTemporalQuery}from"./temporal/TemporalQuery";import{TemporalQueries}from"./temporal/TemporalQueries";import{ValueRange}from"./temporal/ValueRange";import{Year}from"./Year";import{ZoneId}from"./ZoneId";export class MonthDay extends TemporalAccessor{static now(zoneIdOrClock){return 0===arguments.length?MonthDay.now0():1===arguments.length&&zoneIdOrClock instanceof ZoneId?MonthDay.nowZoneId(zoneIdOrClock):MonthDay.nowClock(zoneIdOrClock)}static now0(){return this.nowClock(Clock.systemDefaultZone())}static nowZoneId(zone){return requireNonNull(zone,"zone"),this.nowClock(Clock.system(zone))}static nowClock(clock){requireNonNull(clock,"clock");const now=LocalDate.now(clock);return MonthDay.of(now.month(),now.dayOfMonth())}static of(monthOrNumber,number){return 2===arguments.length&&monthOrNumber instanceof Month?MonthDay.ofMonthNumber(monthOrNumber,number):MonthDay.ofNumberNumber(monthOrNumber,number)}static ofMonthNumber(month,dayOfMonth){if(requireNonNull(month,"month"),ChronoField.DAY_OF_MONTH.checkValidValue(dayOfMonth),dayOfMonth>month.maxLength())throw new DateTimeException(`Illegal value for DayOfMonth field, value ${dayOfMonth} is not valid for month ${month.toString()}`);return new MonthDay(month.value(),dayOfMonth)}static ofNumberNumber(month,dayOfMonth){return requireNonNull(month,"month"),requireNonNull(dayOfMonth,"dayOfMonth"),MonthDay.of(Month.of(month),dayOfMonth)}static from(temporal){if(requireNonNull(temporal,"temporal"),requireInstance(temporal,TemporalAccessor,"temporal"),temporal instanceof MonthDay)return temporal;try{return MonthDay.of(temporal.get(ChronoField.MONTH_OF_YEAR),temporal.get(ChronoField.DAY_OF_MONTH))}catch(ex){throw new DateTimeException(`Unable to obtain MonthDay from TemporalAccessor: ${temporal}, type ${temporal&&null!=temporal.constructor?temporal.constructor.name:""}`)}}static parse(text,formatter){return 1===arguments.length?MonthDay.parseString(text):MonthDay.parseStringFormatter(text,formatter)}static parseString(text){return MonthDay.parseStringFormatter(text,PARSER)}static parseStringFormatter(text,formatter){return requireNonNull(text,"text"),requireNonNull(formatter,"formatter"),requireInstance(formatter,DateTimeFormatter,"formatter"),formatter.parse(text,MonthDay.FROM)}constructor(month,dayOfMonth){super(),this._month=MathUtil.safeToInt(month),this._day=MathUtil.safeToInt(dayOfMonth)}monthValue(){return this._month}month(){return Month.of(this._month)}dayOfMonth(){return this._day}isSupported(field){return field instanceof ChronoField?field===ChronoField.MONTH_OF_YEAR||field===ChronoField.DAY_OF_MONTH:null!=field&&field.isSupportedBy(this)}range(field){return field===ChronoField.MONTH_OF_YEAR?field.range():field===ChronoField.DAY_OF_MONTH?ValueRange.of(1,this.month().minLength(),this.month().maxLength()):super.range(field)}get(field){return this.range(field).checkValidIntValue(this.getLong(field),field)}getLong(field){if(requireNonNull(field,"field"),field instanceof ChronoField){switch(field){case ChronoField.DAY_OF_MONTH:return this._day;case ChronoField.MONTH_OF_YEAR:return this._month}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}return field.getFrom(this)}isValidYear(year){return!1==(29===this._day&&2===this._month&&!1===Year.isLeap(year))}withMonth(month){return this.with(Month.of(month))}with(month){if(requireNonNull(month,"month"),month.value()===this._month)return this;const day=Math.min(this._day,month.maxLength());return new MonthDay(month.value(),day)}withDayOfMonth(dayOfMonth){return dayOfMonth===this._day?this:MonthDay.of(this._month,dayOfMonth)}query(query){return requireNonNull(query,"query"),requireInstance(query,TemporalQuery,"query"),query===TemporalQueries.chronology()?IsoChronology.INSTANCE:super.query(query)}adjustInto(temporal){return requireNonNull(temporal,"temporal"),(temporal=temporal.with(ChronoField.MONTH_OF_YEAR,this._month)).with(ChronoField.DAY_OF_MONTH,Math.min(temporal.range(ChronoField.DAY_OF_MONTH).maximum(),this._day))}atYear(year){return LocalDate.of(year,this._month,this.isValidYear(year)?this._day:28)}compareTo(other){requireNonNull(other,"other"),requireInstance(other,MonthDay,"other");let cmp=this._month-other.monthValue();return 0===cmp&&(cmp=this._day-other.dayOfMonth()),cmp}isAfter(other){return requireNonNull(other,"other"),requireInstance(other,MonthDay,"other"),this.compareTo(other)>0}isBefore(other){return requireNonNull(other,"other"),requireInstance(other,MonthDay,"other"),this.compareTo(other)<0}equals(obj){if(this===obj)return!0;if(obj instanceof MonthDay){const other=obj;return this.monthValue()===other.monthValue()&&this.dayOfMonth()===other.dayOfMonth()}return!1}toString(){return`--${this._month<10?"0":""}${this._month}${this._day<10?"-0":"-"}${this._day}`}toJSON(){return this.toString()}format(formatter){return requireNonNull(formatter,"formatter"),requireInstance(formatter,DateTimeFormatter,"formatter"),formatter.format(this)}}let PARSER;export function _init(){PARSER=(new DateTimeFormatterBuilder).appendLiteral("--").appendValue(ChronoField.MONTH_OF_YEAR,2).appendLiteral("-").appendValue(ChronoField.DAY_OF_MONTH,2).toFormatter(),MonthDay.FROM=createTemporalQuery("MonthDay.FROM",(temporal=>MonthDay.from(temporal)))}