/**
 * @copyright (c) 2016-present, Philipp Thürwächter & Pattrick Hüper  & js-joda contributors
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{Temporal}from"./temporal/Temporal";import{Clock}from"./Clock";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{Instant}from"./Instant";import{IsoChronology}from"./chrono/IsoChronology";import{LocalDateTime}from"./LocalDateTime";import{LocalDate}from"./LocalDate";import{LocalTime}from"./LocalTime";import{MathUtil}from"./MathUtil";import{OffsetTime}from"./OffsetTime";import{TemporalQueries}from"./temporal/TemporalQueries";import{ZonedDateTime}from"./ZonedDateTime";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";import{DateTimeException,IllegalArgumentException}from"./errors";import{createTemporalQuery}from"./temporal/TemporalQuery";import{requireInstance,requireNonNull}from"./assert";export class OffsetDateTime extends Temporal{static from(temporal){if(requireNonNull(temporal,"temporal"),temporal instanceof OffsetDateTime)return temporal;try{const offset=ZoneOffset.from(temporal);try{const ldt=LocalDateTime.from(temporal);return OffsetDateTime.of(ldt,offset)}catch(_){const instant=Instant.from(temporal);return OffsetDateTime.ofInstant(instant,offset)}}catch(ex){throw new DateTimeException(`Unable to obtain OffsetDateTime TemporalAccessor: ${temporal}, type ${null!=temporal.constructor?temporal.constructor.name:""}`)}}static now(clockOrZone){if(0===arguments.length)return OffsetDateTime.now(Clock.systemDefaultZone());if(requireNonNull(clockOrZone,"clockOrZone"),clockOrZone instanceof ZoneId)return OffsetDateTime.now(Clock.system(clockOrZone));if(clockOrZone instanceof Clock){const now=clockOrZone.instant();return OffsetDateTime.ofInstant(now,clockOrZone.zone().rules().offset(now))}throw new IllegalArgumentException("clockOrZone must be an instance of ZoneId or Clock")}static of(){return arguments.length<=2?OffsetDateTime.ofDateTime.apply(this,arguments):3===arguments.length?OffsetDateTime.ofDateAndTime.apply(this,arguments):OffsetDateTime.ofNumbers.apply(this,arguments)}static ofDateTime(dateTime,offset){return new OffsetDateTime(dateTime,offset)}static ofDateAndTime(date,time,offset){const dt=LocalDateTime.of(date,time);return new OffsetDateTime(dt,offset)}static ofNumbers(year,month,dayOfMonth,hour=0,minute=0,second=0,nanoOfSecond=0,offset){const dt=LocalDateTime.of(year,month,dayOfMonth,hour,minute,second,nanoOfSecond);return new OffsetDateTime(dt,offset)}static ofInstant(instant,zone){requireNonNull(instant,"instant"),requireNonNull(zone,"zone");const offset=zone.rules().offset(instant),ldt=LocalDateTime.ofEpochSecond(instant.epochSecond(),instant.nano(),offset);return new OffsetDateTime(ldt,offset)}static parse(text,formatter=DateTimeFormatter.ISO_OFFSET_DATE_TIME){return requireNonNull(formatter,"formatter"),formatter.parse(text,OffsetDateTime.FROM)}constructor(dateTime,offset){super(),requireNonNull(dateTime,"dateTime"),requireInstance(dateTime,LocalDateTime,"dateTime"),requireNonNull(offset,"offset"),requireInstance(offset,ZoneOffset,"offset"),this._dateTime=dateTime,this._offset=offset}adjustInto(temporal){return temporal.with(ChronoField.EPOCH_DAY,this.toLocalDate().toEpochDay()).with(ChronoField.NANO_OF_DAY,this.toLocalTime().toNanoOfDay()).with(ChronoField.OFFSET_SECONDS,this.offset().totalSeconds())}until(endExclusive,unit){let end=OffsetDateTime.from(endExclusive);return unit instanceof ChronoUnit?(end=end.withOffsetSameInstant(this._offset),this._dateTime.until(end._dateTime,unit)):unit.between(this,end)}atZoneSameInstant(zone){return ZonedDateTime.ofInstant(this._dateTime,this._offset,zone)}atZoneSimilarLocal(zone){return ZonedDateTime.ofLocal(this._dateTime,zone,this._offset)}query(query){return requireNonNull(query,"query"),query===TemporalQueries.chronology()?IsoChronology.INSTANCE:query===TemporalQueries.precision()?ChronoUnit.NANOS:query===TemporalQueries.offset()||query===TemporalQueries.zone()?this.offset():query===TemporalQueries.localDate()?this.toLocalDate():query===TemporalQueries.localTime()?this.toLocalTime():query===TemporalQueries.zoneId()?null:super.query(query)}get(field){if(field instanceof ChronoField){switch(field){case ChronoField.INSTANT_SECONDS:throw new DateTimeException(`Field too large for an int: ${field}`);case ChronoField.OFFSET_SECONDS:return this.offset().totalSeconds()}return this._dateTime.get(field)}return super.get(field)}getLong(field){if(field instanceof ChronoField){switch(field){case ChronoField.INSTANT_SECONDS:return this.toEpochSecond();case ChronoField.OFFSET_SECONDS:return this.offset().totalSeconds()}return this._dateTime.getLong(field)}return field.getFrom(this)}offset(){return this._offset}year(){return this._dateTime.year()}monthValue(){return this._dateTime.monthValue()}month(){return this._dateTime.month()}dayOfMonth(){return this._dateTime.dayOfMonth()}dayOfYear(){return this._dateTime.dayOfYear()}dayOfWeek(){return this._dateTime.dayOfWeek()}hour(){return this._dateTime.hour()}minute(){return this._dateTime.minute()}second(){return this._dateTime.second()}nano(){return this._dateTime.nano()}toLocalDateTime(){return this._dateTime}toLocalDate(){return this._dateTime.toLocalDate()}toLocalTime(){return this._dateTime.toLocalTime()}toOffsetTime(){return OffsetTime.of(this._dateTime.toLocalTime(),this._offset)}toZonedDateTime(){return ZonedDateTime.of(this._dateTime,this._offset)}toInstant(){return this._dateTime.toInstant(this._offset)}toEpochSecond(){return this._dateTime.toEpochSecond(this._offset)}isSupported(fieldOrUnit){return fieldOrUnit instanceof ChronoField||fieldOrUnit instanceof ChronoUnit?fieldOrUnit.isDateBased()||fieldOrUnit.isTimeBased():null!=fieldOrUnit&&fieldOrUnit.isSupportedBy(this)}range(field){return field instanceof ChronoField?field===ChronoField.INSTANT_SECONDS||field===ChronoField.OFFSET_SECONDS?field.range():this._dateTime.range(field):field.rangeRefinedBy(this)}_withAdjuster(adjuster){return requireNonNull(adjuster),adjuster instanceof LocalDate||adjuster instanceof LocalTime||adjuster instanceof LocalDateTime?this._withDateTimeOffset(this._dateTime.with(adjuster),this._offset):adjuster instanceof Instant?OffsetDateTime.ofInstant(adjuster,this._offset):adjuster instanceof ZoneOffset?this._withDateTimeOffset(this._dateTime,adjuster):adjuster instanceof OffsetDateTime?adjuster:adjuster.adjustInto(this)}_withField(field,newValue){if(requireNonNull(field),field instanceof ChronoField){const f=field;switch(f){case ChronoField.INSTANT_SECONDS:return OffsetDateTime.ofInstant(Instant.ofEpochSecond(newValue,this.nano()),this._offset);case ChronoField.OFFSET_SECONDS:return this._withDateTimeOffset(this._dateTime,ZoneOffset.ofTotalSeconds(f.checkValidIntValue(newValue)))}return this._withDateTimeOffset(this._dateTime.with(field,newValue),this._offset)}return field.adjustInto(this,newValue)}_withDateTimeOffset(dateTime,offset){return this._dateTime===dateTime&&this._offset.equals(offset)?this:new OffsetDateTime(dateTime,offset)}withYear(year){return this._withDateTimeOffset(this._dateTime.withYear(year),this._offset)}withMonth(month){return this._withDateTimeOffset(this._dateTime.withMonth(month),this._offset)}withDayOfMonth(dayOfMonth){return this._withDateTimeOffset(this._dateTime.withDayOfMonth(dayOfMonth),this._offset)}withDayOfYear(dayOfYear){return this._withDateTimeOffset(this._dateTime.withDayOfYear(dayOfYear),this._offset)}withHour(hour){return this._withDateTimeOffset(this._dateTime.withHour(hour),this._offset)}withMinute(minute){return this._withDateTimeOffset(this._dateTime.withMinute(minute),this._offset)}withSecond(second){return this._withDateTimeOffset(this._dateTime.withSecond(second),this._offset)}withNano(nanoOfSecond){return this._withDateTimeOffset(this._dateTime.withNano(nanoOfSecond),this._offset)}withOffsetSameLocal(offset){return requireNonNull(offset,"offset"),this._withDateTimeOffset(this._dateTime,offset)}withOffsetSameInstant(offset){if(requireNonNull(offset,"offset"),offset.equals(this._offset))return this;const difference=offset.totalSeconds()-this._offset.totalSeconds(),adjusted=this._dateTime.plusSeconds(difference);return new OffsetDateTime(adjusted,offset)}truncatedTo(unit){return this._withDateTimeOffset(this._dateTime.truncatedTo(unit),this._offset)}_plusAmount(amount){return requireNonNull(amount,"amount"),amount.addTo(this)}_plusUnit(amountToAdd,unit){return unit instanceof ChronoUnit?this._withDateTimeOffset(this._dateTime.plus(amountToAdd,unit),this._offset):unit.addTo(this,amountToAdd)}plusYears(years){return this._withDateTimeOffset(this._dateTime.plusYears(years),this._offset)}plusMonths(months){return this._withDateTimeOffset(this._dateTime.plusMonths(months),this._offset)}plusWeeks(weeks){return this._withDateTimeOffset(this._dateTime.plusWeeks(weeks),this._offset)}plusDays(days){return this._withDateTimeOffset(this._dateTime.plusDays(days),this._offset)}plusHours(hours){return this._withDateTimeOffset(this._dateTime.plusHours(hours),this._offset)}plusMinutes(minutes){return this._withDateTimeOffset(this._dateTime.plusMinutes(minutes),this._offset)}plusSeconds(seconds){return this._withDateTimeOffset(this._dateTime.plusSeconds(seconds),this._offset)}plusNanos(nanos){return this._withDateTimeOffset(this._dateTime.plusNanos(nanos),this._offset)}_minusAmount(amount){return requireNonNull(amount),amount.subtractFrom(this)}_minusUnit(amountToSubtract,unit){return this.plus(-1*amountToSubtract,unit)}minusYears(years){return this._withDateTimeOffset(this._dateTime.minusYears(years),this._offset)}minusMonths(months){return this._withDateTimeOffset(this._dateTime.minusMonths(months),this._offset)}minusWeeks(weeks){return this._withDateTimeOffset(this._dateTime.minusWeeks(weeks),this._offset)}minusDays(days){return this._withDateTimeOffset(this._dateTime.minusDays(days),this._offset)}minusHours(hours){return this._withDateTimeOffset(this._dateTime.minusHours(hours),this._offset)}minusMinutes(minutes){return this._withDateTimeOffset(this._dateTime.minusMinutes(minutes),this._offset)}minusSeconds(seconds){return this._withDateTimeOffset(this._dateTime.minusSeconds(seconds),this._offset)}minusNanos(nanos){return this._withDateTimeOffset(this._dateTime.minusNanos(nanos),this._offset)}compareTo(other){if(requireNonNull(other,"other"),requireInstance(other,OffsetDateTime,"other"),this.offset().equals(other.offset()))return this.toLocalDateTime().compareTo(other.toLocalDateTime());let cmp=MathUtil.compareNumbers(this.toEpochSecond(),other.toEpochSecond());return 0===cmp&&(cmp=this.toLocalTime().nano()-other.toLocalTime().nano(),0===cmp&&(cmp=this.toLocalDateTime().compareTo(other.toLocalDateTime()))),cmp}isAfter(other){requireNonNull(other,"other");const thisEpochSec=this.toEpochSecond(),otherEpochSec=other.toEpochSecond();return thisEpochSec>otherEpochSec||thisEpochSec===otherEpochSec&&this.toLocalTime().nano()>other.toLocalTime().nano()}isBefore(other){requireNonNull(other,"other");const thisEpochSec=this.toEpochSecond(),otherEpochSec=other.toEpochSecond();return thisEpochSec<otherEpochSec||thisEpochSec===otherEpochSec&&this.toLocalTime().nano()<other.toLocalTime().nano()}isEqual(other){return requireNonNull(other,"other"),this.toEpochSecond()===other.toEpochSecond()&&this.toLocalTime().nano()===other.toLocalTime().nano()}equals(other){return this===other||other instanceof OffsetDateTime&&(this._dateTime.equals(other._dateTime)&&this._offset.equals(other._offset))}hashCode(){return this._dateTime.hashCode()^this._offset.hashCode()}toString(){return this._dateTime.toString()+this._offset.toString()}toJSON(){return this.toString()}format(formatter){return requireNonNull(formatter,"formatter"),formatter.format(this)}}export function _init(){OffsetDateTime.MIN=LocalDateTime.MIN.atOffset(ZoneOffset.MAX),OffsetDateTime.MAX=LocalDateTime.MAX.atOffset(ZoneOffset.MIN),OffsetDateTime.FROM=createTemporalQuery("OffsetDateTime.FROM",(temporal=>OffsetDateTime.from(temporal)))}