/**
 * @copyright (c) 2016-present, Philipp Thürwächter & Pattrick Hüper  & js-joda contributors
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{Temporal}from"./temporal/Temporal";import{Clock}from"./Clock";import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{Instant,LocalTime}from"./js-joda";import{MathUtil}from"./MathUtil";import{OffsetDateTime}from"./OffsetDateTime";import{TemporalQueries}from"./temporal/TemporalQueries";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";import{createTemporalQuery}from"./temporal/TemporalQuery";import{requireInstance,requireNonNull}from"./assert";export class OffsetTime extends Temporal{static from(temporal){if(requireNonNull(temporal,"temporal"),temporal instanceof OffsetTime)return temporal;if(temporal instanceof OffsetDateTime)return temporal.toOffsetTime();try{const time=LocalTime.from(temporal),offset=ZoneOffset.from(temporal);return new OffsetTime(time,offset)}catch(ex){throw new DateTimeException(`Unable to obtain OffsetTime TemporalAccessor: ${temporal}, type ${null!=temporal.constructor?temporal.constructor.name:""}`)}}static now(clockOrZone){return 0===arguments.length?OffsetTime._now(Clock.systemDefaultZone()):clockOrZone instanceof Clock?OffsetTime._now(clockOrZone):OffsetTime._now(Clock.system(clockOrZone))}static _now(clock){requireNonNull(clock,"clock");const now=clock.instant();return OffsetTime.ofInstant(now,clock.zone().rules().offset(now))}static of(){return arguments.length<=2?OffsetTime.ofTimeAndOffset.apply(this,arguments):OffsetTime.ofNumbers.apply(this,arguments)}static ofNumbers(hour,minute,second,nanoOfSecond,offset){const time=LocalTime.of(hour,minute,second,nanoOfSecond);return new OffsetTime(time,offset)}static ofTimeAndOffset(time,offset){return new OffsetTime(time,offset)}static ofInstant(instant,zone){requireNonNull(instant,"instant"),requireInstance(instant,Instant,"instant"),requireNonNull(zone,"zone"),requireInstance(zone,ZoneId,"zone");const offset=zone.rules().offset(instant);let secsOfDay=instant.epochSecond()%LocalTime.SECONDS_PER_DAY;secsOfDay=(secsOfDay+offset.totalSeconds())%LocalTime.SECONDS_PER_DAY,secsOfDay<0&&(secsOfDay+=LocalTime.SECONDS_PER_DAY);const time=LocalTime.ofSecondOfDay(secsOfDay,instant.nano());return new OffsetTime(time,offset)}static parse(text,formatter=DateTimeFormatter.ISO_OFFSET_TIME){return requireNonNull(formatter,"formatter"),formatter.parse(text,OffsetTime.FROM)}constructor(time,offset){super(),requireNonNull(time,"time"),requireInstance(time,LocalTime,"time"),requireNonNull(offset,"offset"),requireInstance(offset,ZoneOffset,"offset"),this._time=time,this._offset=offset}adjustInto(temporal){return temporal.with(ChronoField.NANO_OF_DAY,this._time.toNanoOfDay()).with(ChronoField.OFFSET_SECONDS,this.offset().totalSeconds())}atDate(date){return OffsetDateTime.of(date,this._time,this._offset)}format(formatter){return requireNonNull(formatter,"formatter"),formatter.format(this,OffsetTime.FROM)}get(field){return super.get(field)}getLong(field){return field instanceof ChronoField?field===ChronoField.OFFSET_SECONDS?this._offset.totalSeconds():this._time.getLong(field):field.getFrom(this)}hour(){return this._time.hour()}minute(){return this._time.minute()}second(){return this._time.second()}nano(){return this._time.nano()}offset(){return this._offset}isAfter(other){return requireNonNull(other,"other"),this._toEpochNano()>other._toEpochNano()}isBefore(other){return requireNonNull(other,"other"),this._toEpochNano()<other._toEpochNano()}isEqual(other){return requireNonNull(other,"other"),this._toEpochNano()===other._toEpochNano()}isSupported(fieldOrUnit){return fieldOrUnit instanceof ChronoField?fieldOrUnit.isTimeBased()||fieldOrUnit===ChronoField.OFFSET_SECONDS:fieldOrUnit instanceof ChronoUnit?fieldOrUnit.isTimeBased():null!=fieldOrUnit&&fieldOrUnit.isSupportedBy(this)}minusHours(hours){return this._withLocalTimeOffset(this._time.minusHours(hours),this._offset)}minusMinutes(minutes){return this._withLocalTimeOffset(this._time.minusMinutes(minutes),this._offset)}minusSeconds(seconds){return this._withLocalTimeOffset(this._time.minusSeconds(seconds),this._offset)}minusNanos(nanos){return this._withLocalTimeOffset(this._time.minusNanos(nanos),this._offset)}_minusAmount(amount){return requireNonNull(amount),amount.subtractFrom(this)}_minusUnit(amountToSubtract,unit){return this.plus(-1*amountToSubtract,unit)}_plusAmount(amount){return requireNonNull(amount),amount.addTo(this)}_plusUnit(amountToAdd,unit){return unit instanceof ChronoUnit?this._withLocalTimeOffset(this._time.plus(amountToAdd,unit),this._offset):unit.addTo(this,amountToAdd)}plusHours(hours){return this._withLocalTimeOffset(this._time.plusHours(hours),this._offset)}plusMinutes(minutes){return this._withLocalTimeOffset(this._time.plusMinutes(minutes),this._offset)}plusSeconds(seconds){return this._withLocalTimeOffset(this._time.plusSeconds(seconds),this._offset)}plusNanos(nanos){return this._withLocalTimeOffset(this._time.plusNanos(nanos),this._offset)}query(query){return requireNonNull(query,"query"),query===TemporalQueries.precision()?ChronoUnit.NANOS:query===TemporalQueries.offset()||query===TemporalQueries.zone()?this.offset():query===TemporalQueries.localTime()?this._time:query===TemporalQueries.chronology()||query===TemporalQueries.localDate()||query===TemporalQueries.zoneId()?null:super.query(query)}range(field){return field instanceof ChronoField?field===ChronoField.OFFSET_SECONDS?field.range():this._time.range(field):field.rangeRefinedBy(this)}toLocalTime(){return this._time}truncatedTo(unit){return this._withLocalTimeOffset(this._time.truncatedTo(unit),this._offset)}until(endExclusive,unit){requireNonNull(endExclusive,"endExclusive"),requireNonNull(unit,"unit");const end=OffsetTime.from(endExclusive);if(unit instanceof ChronoUnit){const nanosUntil=end._toEpochNano()-this._toEpochNano();switch(unit){case ChronoUnit.NANOS:return nanosUntil;case ChronoUnit.MICROS:return MathUtil.intDiv(nanosUntil,1e3);case ChronoUnit.MILLIS:return MathUtil.intDiv(nanosUntil,1e6);case ChronoUnit.SECONDS:return MathUtil.intDiv(nanosUntil,LocalTime.NANOS_PER_SECOND);case ChronoUnit.MINUTES:return MathUtil.intDiv(nanosUntil,LocalTime.NANOS_PER_MINUTE);case ChronoUnit.HOURS:return MathUtil.intDiv(nanosUntil,LocalTime.NANOS_PER_HOUR);case ChronoUnit.HALF_DAYS:return MathUtil.intDiv(nanosUntil,12*LocalTime.NANOS_PER_HOUR)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.between(this,end)}withHour(hour){return this._withLocalTimeOffset(this._time.withHour(hour),this._offset)}withMinute(minute){return this._withLocalTimeOffset(this._time.withMinute(minute),this._offset)}withSecond(second){return this._withLocalTimeOffset(this._time.withSecond(second),this._offset)}withNano(nano){return this._withLocalTimeOffset(this._time.withNano(nano),this._offset)}withOffsetSameInstant(offset){if(requireNonNull(offset,"offset"),offset.equals(this._offset))return this;const difference=offset.totalSeconds()-this._offset.totalSeconds(),adjusted=this._time.plusSeconds(difference);return new OffsetTime(adjusted,offset)}withOffsetSameLocal(offset){return null!=offset&&offset.equals(this._offset)?this:new OffsetTime(this._time,offset)}_toEpochNano(){return this._time.toNanoOfDay()-this._offset.totalSeconds()*LocalTime.NANOS_PER_SECOND}_withAdjuster(adjuster){return requireNonNull(adjuster,"adjuster"),adjuster instanceof LocalTime?this._withLocalTimeOffset(adjuster,this._offset):adjuster instanceof ZoneOffset?this._withLocalTimeOffset(this._time,adjuster):adjuster instanceof OffsetTime?adjuster:adjuster.adjustInto(this)}_withField(field,newValue){return requireNonNull(field,"field"),field instanceof ChronoField?field===ChronoField.OFFSET_SECONDS?this._withLocalTimeOffset(this._time,ZoneOffset.ofTotalSeconds(field.checkValidIntValue(newValue))):this._withLocalTimeOffset(this._time.with(field,newValue),this._offset):field.adjustInto(this,newValue)}_withLocalTimeOffset(time,offset){return this._time===time&&this._offset.equals(offset)?this:new OffsetTime(time,offset)}compareTo(other){if(requireNonNull(other,"other"),requireInstance(other,OffsetTime,"other"),this._offset.equals(other._offset))return this._time.compareTo(other._time);const compare=MathUtil.compareNumbers(this._toEpochNano(),other._toEpochNano());return 0===compare?this._time.compareTo(other._time):compare}equals(other){return this===other||other instanceof OffsetTime&&(this._time.equals(other._time)&&this._offset.equals(other._offset))}hashCode(){return this._time.hashCode()^this._offset.hashCode()}toString(){return this._time.toString()+this._offset.toString()}toJSON(){return this.toString()}}export function _init(){OffsetTime.MIN=OffsetTime.ofNumbers(0,0,0,0,ZoneOffset.MAX),OffsetTime.MAX=OffsetTime.ofNumbers(23,59,59,999999999,ZoneOffset.MIN),OffsetTime.FROM=createTemporalQuery("OffsetTime.FROM",(temporal=>OffsetTime.from(temporal)))}