/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{DateTimeException,UnsupportedTemporalTypeException}from"./errors";import{requireNonNull,requireInstance}from"./assert";import{MathUtil}from"./MathUtil";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{Clock}from"./Clock";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{DateTimeFormatterBuilder}from"./format/DateTimeFormatterBuilder";import{IsoChronology}from"./chrono/IsoChronology";import{LocalDate}from"./LocalDate";import{Month}from"./Month";import{MonthDay}from"./MonthDay";import{SignStyle}from"./format/SignStyle";import{Temporal}from"./temporal/Temporal";import{TemporalAccessor}from"./temporal/TemporalAccessor";import{TemporalField}from"./temporal/TemporalField";import{TemporalQueries}from"./temporal/TemporalQueries";import{TemporalQuery,createTemporalQuery}from"./temporal/TemporalQuery";import{TemporalUnit}from"./temporal/TemporalUnit";import{YearConstants}from"./YearConstants";import{YearMonth}from"./YearMonth";import{ZoneId}from"./ZoneId";export class Year extends Temporal{constructor(value){super(),this._year=MathUtil.safeToInt(value)}value(){return this._year}static now(zoneIdOrClock){return void 0===zoneIdOrClock?Year.now0():zoneIdOrClock instanceof ZoneId?Year.nowZoneId(zoneIdOrClock):Year.nowClock(zoneIdOrClock)}static now0(){return Year.nowClock(Clock.systemDefaultZone())}static nowZoneId(zone){return requireNonNull(zone,"zone"),requireInstance(zone,ZoneId,"zone"),Year.nowClock(Clock.system(zone))}static nowClock(clock){requireNonNull(clock,"clock"),requireInstance(clock,Clock,"clock");const now=LocalDate.now(clock);return Year.of(now.year())}static of(isoYear){return requireNonNull(isoYear,"isoYear"),ChronoField.YEAR.checkValidValue(isoYear),new Year(isoYear)}static from(temporal){if(requireNonNull(temporal,"temporal"),requireInstance(temporal,TemporalAccessor,"temporal"),temporal instanceof Year)return temporal;try{return Year.of(temporal.get(ChronoField.YEAR))}catch(ex){throw new DateTimeException(`Unable to obtain Year from TemporalAccessor: ${temporal}, type ${temporal&&null!=temporal.constructor?temporal.constructor.name:""}`)}}static parse(text,formatter){return arguments.length<=1?Year.parseText(text):Year.parseTextFormatter(text,formatter)}static parseText(text){return requireNonNull(text,"text"),Year.parse(text,PARSER)}static parseTextFormatter(text,formatter=PARSER){return requireNonNull(text,"text"),requireNonNull(formatter,"formatter"),requireInstance(formatter,DateTimeFormatter,"formatter"),formatter.parse(text,Year.FROM)}static isLeap(year){return 0===MathUtil.intMod(year,4)&&(0!==MathUtil.intMod(year,100)||0===MathUtil.intMod(year,400))}isSupported(fieldOrUnit){return 1===arguments.length&&fieldOrUnit instanceof TemporalField?this.isSupportedField(fieldOrUnit):this.isSupportedUnit(fieldOrUnit)}isSupportedField(field){return field instanceof ChronoField?field===ChronoField.YEAR||field===ChronoField.YEAR_OF_ERA||field===ChronoField.ERA:null!=field&&field.isSupportedBy(this)}isSupportedUnit(unit){return unit instanceof ChronoUnit?unit===ChronoUnit.YEARS||unit===ChronoUnit.DECADES||unit===ChronoUnit.CENTURIES||unit===ChronoUnit.MILLENNIA||unit===ChronoUnit.ERAS:null!=unit&&unit.isSupportedBy(this)}range(field){if(this.isSupported(field))return field.range();if(field instanceof ChronoField)throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`);return super.range(field)}get(field){return this.range(field).checkValidIntValue(this.getLong(field),field)}getLong(field){if(requireNonNull(field,"field"),field instanceof ChronoField){switch(field){case ChronoField.YEAR_OF_ERA:return this._year<1?1-this._year:this._year;case ChronoField.YEAR:return this._year;case ChronoField.ERA:return this._year<1?0:1}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}return field.getFrom(this)}isLeap(){return Year.isLeap(this._year)}_withField(field,newValue){if(requireNonNull(field,"field"),requireInstance(field,TemporalField,"field"),field instanceof ChronoField){switch(field.checkValidValue(newValue),field){case ChronoField.YEAR_OF_ERA:return Year.of(this._year<1?1-newValue:newValue);case ChronoField.YEAR:return Year.of(newValue);case ChronoField.ERA:return this.getLong(ChronoField.ERA)===newValue?this:Year.of(1-this._year)}throw new UnsupportedTemporalTypeException(`Unsupported field: ${field}`)}return field.adjustInto(this,newValue)}_plusUnit(amountToAdd,unit){if(requireNonNull(amountToAdd,"amountToAdd"),requireNonNull(unit,"unit"),requireInstance(unit,TemporalUnit,"unit"),unit instanceof ChronoUnit){switch(unit){case ChronoUnit.YEARS:return this.plusYears(amountToAdd);case ChronoUnit.DECADES:return this.plusYears(MathUtil.safeMultiply(amountToAdd,10));case ChronoUnit.CENTURIES:return this.plusYears(MathUtil.safeMultiply(amountToAdd,100));case ChronoUnit.MILLENNIA:return this.plusYears(MathUtil.safeMultiply(amountToAdd,1e3));case ChronoUnit.ERAS:return this.with(ChronoField.ERA,MathUtil.safeAdd(this.getLong(ChronoField.ERA),amountToAdd))}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.addTo(this,amountToAdd)}plusYears(yearsToAdd){return 0===yearsToAdd?this:Year.of(ChronoField.YEAR.checkValidIntValue(MathUtil.safeAdd(this._year,yearsToAdd)))}minusYears(yearsToSubtract){return yearsToSubtract===MathUtil.MIN_SAFE_INTEGER?this.plusYears(MathUtil.MAX_SAFE_INTEGER).plusYears(1):this.plusYears(-yearsToSubtract)}adjustInto(temporal){return requireNonNull(temporal,"temporal"),temporal.with(ChronoField.YEAR,this._year)}isValidMonthDay(monthDay){return null!=monthDay&&monthDay.isValidYear(this._year)}length(){return this.isLeap()?366:365}atDay(dayOfYear){return LocalDate.ofYearDay(this._year,dayOfYear)}atMonth(monthOrNumber){return 1===arguments.length&&monthOrNumber instanceof Month?this.atMonthMonth(monthOrNumber):this.atMonthNumber(monthOrNumber)}atMonthMonth(month){return requireNonNull(month,"month"),requireInstance(month,Month,"month"),YearMonth.of(this._year,month)}atMonthNumber(month){return requireNonNull(month,"month"),YearMonth.of(this._year,month)}atMonthDay(monthDay){return requireNonNull(monthDay,"monthDay"),requireInstance(monthDay,MonthDay,"monthDay"),monthDay.atYear(this._year)}query(query){return requireNonNull(query,"query()"),requireInstance(query,TemporalQuery,"query()"),query===TemporalQueries.chronology()?IsoChronology.INSTANCE:query===TemporalQueries.precision()?ChronoUnit.YEARS:query===TemporalQueries.localDate()||query===TemporalQueries.localTime()||query===TemporalQueries.zone()||query===TemporalQueries.zoneId()||query===TemporalQueries.offset()?null:super.query(query)}compareTo(other){return requireNonNull(other,"other"),requireInstance(other,Year,"other"),this._year-other._year}isAfter(other){return requireNonNull(other,"other"),requireInstance(other,Year,"other"),this._year>other._year}isBefore(other){return requireNonNull(other,"other"),requireInstance(other,Year,"other"),this._year<other._year}format(formatter){return requireNonNull(formatter,"formatter"),requireInstance(formatter,DateTimeFormatter,"formatter"),formatter.format(this)}equals(other){return this===other||other instanceof Year&&this.value()===other.value()}toString(){return`${this._year}`}toJSON(){return this.toString()}until(endExclusive,unit){const end=Year.from(endExclusive);if(unit instanceof ChronoUnit){const yearsUntil=end.value()-this.value();switch(unit){case ChronoUnit.YEARS:return yearsUntil;case ChronoUnit.DECADES:return MathUtil.intDiv(yearsUntil,10);case ChronoUnit.CENTURIES:return MathUtil.intDiv(yearsUntil,100);case ChronoUnit.MILLENNIA:return MathUtil.intDiv(yearsUntil,1e3);case ChronoUnit.ERAS:return end.getLong(ChronoField.ERA)-this.getLong(ChronoField.ERA)}throw new UnsupportedTemporalTypeException(`Unsupported unit: ${unit}`)}return unit.between(this,end)}}let PARSER;export function _init(){Year.MIN_VALUE=YearConstants.MIN_VALUE,Year.MAX_VALUE=YearConstants.MAX_VALUE,PARSER=(new DateTimeFormatterBuilder).appendValue(ChronoField.YEAR,4,10,SignStyle.EXCEEDS_PAD).toFormatter(),Year.FROM=createTemporalQuery("Year.FROM",(temporal=>Year.from(temporal)))}