/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull}from"./assert";import{DateTimeException,IllegalArgumentException}from"./errors";import{MathUtil}from"./MathUtil";import{Clock}from"./Clock";import{Instant}from"./Instant";import{LocalDate}from"./LocalDate";import{LocalDateTime}from"./LocalDateTime";import{LocalTime}from"./LocalTime";import{OffsetDateTime}from"./OffsetDateTime";import{ZoneId}from"./ZoneId";import{ZoneOffset}from"./ZoneOffset";import{ChronoZonedDateTime}from"./chrono/ChronoZonedDateTime";import{DateTimeFormatter}from"./format/DateTimeFormatter";import{ChronoField}from"./temporal/ChronoField";import{ChronoUnit}from"./temporal/ChronoUnit";import{createTemporalQuery}from"./temporal/TemporalQuery";import{TemporalQueries}from"./temporal/TemporalQueries";export class ZonedDateTime extends ChronoZonedDateTime{static now(clockOrZone){let clock;return clock=clockOrZone instanceof ZoneId?Clock.system(clockOrZone):null==clockOrZone?Clock.systemDefaultZone():clockOrZone,ZonedDateTime.ofInstant(clock.instant(),clock.zone())}static of(){return arguments.length<=2?ZonedDateTime.of2.apply(this,arguments):3===arguments.length&&arguments[0]instanceof LocalDate?ZonedDateTime.of3.apply(this,arguments):ZonedDateTime.of8.apply(this,arguments)}static of3(date,time,zone){return ZonedDateTime.of2(LocalDateTime.of(date,time),zone)}static of2(localDateTime,zone){return ZonedDateTime.ofLocal(localDateTime,zone,null)}static of8(year,month,dayOfMonth,hour,minute,second,nanoOfSecond,zone){const dt=LocalDateTime.of(year,month,dayOfMonth,hour,minute,second,nanoOfSecond);return ZonedDateTime.ofLocal(dt,zone,null)}static ofLocal(localDateTime,zone,preferredOffset){if(requireNonNull(localDateTime,"localDateTime"),requireNonNull(zone,"zone"),zone instanceof ZoneOffset)return new ZonedDateTime(localDateTime,zone,zone);let offset=null;const rules=zone.rules(),validOffsets=rules.validOffsets(localDateTime);if(1===validOffsets.length)offset=validOffsets[0];else if(0===validOffsets.length){const trans=rules.transition(localDateTime);localDateTime=localDateTime.plusSeconds(trans.duration().seconds()),offset=trans.offsetAfter()}else offset=null!=preferredOffset&&validOffsets.some((validOffset=>validOffset.equals(preferredOffset)))?preferredOffset:requireNonNull(validOffsets[0],"offset");return new ZonedDateTime(localDateTime,offset,zone)}static ofInstant(){return 2===arguments.length?ZonedDateTime.ofInstant2.apply(this,arguments):ZonedDateTime.ofInstant3.apply(this,arguments)}static ofInstant2(instant,zone){return requireNonNull(instant,"instant"),requireNonNull(zone,"zone"),ZonedDateTime._create(instant.epochSecond(),instant.nano(),zone)}static ofInstant3(localDateTime,offset,zone){return requireNonNull(localDateTime,"localDateTime"),requireNonNull(offset,"offset"),requireNonNull(zone,"zone"),ZonedDateTime._create(localDateTime.toEpochSecond(offset),localDateTime.nano(),zone)}static _create(epochSecond,nanoOfSecond,zone){const rules=zone.rules(),instant=Instant.ofEpochSecond(epochSecond,nanoOfSecond),offset=rules.offset(instant),ldt=LocalDateTime.ofEpochSecond(epochSecond,nanoOfSecond,offset);return new ZonedDateTime(ldt,offset,zone)}static ofStrict(localDateTime,offset,zone){requireNonNull(localDateTime,"localDateTime"),requireNonNull(offset,"offset"),requireNonNull(zone,"zone");const rules=zone.rules();if(!1===rules.isValidOffset(localDateTime,offset)){const trans=rules.transition(localDateTime);if(null!=trans&&trans.isGap())throw new DateTimeException(`LocalDateTime ${localDateTime} does not exist in zone ${zone} due to a gap in the local time-line, typically caused by daylight savings`);throw new DateTimeException(`ZoneOffset "${offset}" is not valid for LocalDateTime "${localDateTime}" in zone "${zone}"`)}return new ZonedDateTime(localDateTime,offset,zone)}static ofLenient(localDateTime,offset,zone){if(requireNonNull(localDateTime,"localDateTime"),requireNonNull(offset,"offset"),requireNonNull(zone,"zone"),zone instanceof ZoneOffset&&!1===offset.equals(zone))throw new IllegalArgumentException("ZoneId must match ZoneOffset");return new ZonedDateTime(localDateTime,offset,zone)}static from(temporal){if(requireNonNull(temporal,"temporal"),temporal instanceof ZonedDateTime)return temporal;const zone=ZoneId.from(temporal);if(temporal.isSupported(ChronoField.INSTANT_SECONDS)){const zdt=ZonedDateTime._from(temporal,zone);if(null!=zdt)return zdt}const ldt=LocalDateTime.from(temporal);return ZonedDateTime.of2(ldt,zone)}static _from(temporal,zone){try{return ZonedDateTime.__from(temporal,zone)}catch(ex){if(!(ex instanceof DateTimeException))throw ex}}static __from(temporal,zone){const epochSecond=temporal.getLong(ChronoField.INSTANT_SECONDS),nanoOfSecond=temporal.get(ChronoField.NANO_OF_SECOND);return ZonedDateTime._create(epochSecond,nanoOfSecond,zone)}static parse(text,formatter=DateTimeFormatter.ISO_ZONED_DATE_TIME){return requireNonNull(formatter,"formatter"),formatter.parse(text,ZonedDateTime.FROM)}constructor(dateTime,offset,zone){requireNonNull(dateTime,"dateTime"),requireNonNull(offset,"offset"),requireNonNull(zone,"zone"),super(),this._dateTime=dateTime,this._offset=offset,this._zone=zone}_resolveLocal(newDateTime){return requireNonNull(newDateTime,"newDateTime"),ZonedDateTime.ofLocal(newDateTime,this._zone,this._offset)}_resolveInstant(newDateTime){return ZonedDateTime.ofInstant3(newDateTime,this._offset,this._zone)}_resolveOffset(offset){return!1===offset.equals(this._offset)&&this._zone.rules().isValidOffset(this._dateTime,offset)?new ZonedDateTime(this._dateTime,offset,this._zone):this}isSupported(fieldOrUnit){return fieldOrUnit instanceof ChronoField||(fieldOrUnit instanceof ChronoUnit?fieldOrUnit.isDateBased()||fieldOrUnit.isTimeBased():null!=fieldOrUnit&&fieldOrUnit.isSupportedBy(this))}range(field){return field instanceof ChronoField?field===ChronoField.INSTANT_SECONDS||field===ChronoField.OFFSET_SECONDS?field.range():this._dateTime.range(field):field.rangeRefinedBy(this)}get(field){return this.getLong(field)}getLong(field){if(field instanceof ChronoField){switch(field){case ChronoField.INSTANT_SECONDS:return this.toEpochSecond();case ChronoField.OFFSET_SECONDS:return this._offset.totalSeconds()}return this._dateTime.getLong(field)}return requireNonNull(field,"field"),field.getFrom(this)}offset(){return this._offset}withEarlierOffsetAtOverlap(){const trans=this._zone.rules().transition(this._dateTime);if(null!=trans&&trans.isOverlap()){const earlierOffset=trans.offsetBefore();if(!1===earlierOffset.equals(this._offset))return new ZonedDateTime(this._dateTime,earlierOffset,this._zone)}return this}withLaterOffsetAtOverlap(){const trans=this._zone.rules().transition(this.toLocalDateTime());if(null!=trans){const laterOffset=trans.offsetAfter();if(!1===laterOffset.equals(this._offset))return new ZonedDateTime(this._dateTime,laterOffset,this._zone)}return this}zone(){return this._zone}withZoneSameLocal(zone){return requireNonNull(zone,"zone"),this._zone.equals(zone)?this:ZonedDateTime.ofLocal(this._dateTime,zone,this._offset)}withZoneSameInstant(zone){return requireNonNull(zone,"zone"),this._zone.equals(zone)?this:ZonedDateTime._create(this._dateTime.toEpochSecond(this._offset),this._dateTime.nano(),zone)}withFixedOffsetZone(){return this._zone.equals(this._offset)?this:new ZonedDateTime(this._dateTime,this._offset,this._offset)}year(){return this._dateTime.year()}monthValue(){return this._dateTime.monthValue()}month(){return this._dateTime.month()}dayOfMonth(){return this._dateTime.dayOfMonth()}dayOfYear(){return this._dateTime.dayOfYear()}dayOfWeek(){return this._dateTime.dayOfWeek()}hour(){return this._dateTime.hour()}minute(){return this._dateTime.minute()}second(){return this._dateTime.second()}nano(){return this._dateTime.nano()}_withAdjuster(adjuster){if(adjuster instanceof LocalDate)return this._resolveLocal(LocalDateTime.of(adjuster,this._dateTime.toLocalTime()));if(adjuster instanceof LocalTime)return this._resolveLocal(LocalDateTime.of(this._dateTime.toLocalDate(),adjuster));if(adjuster instanceof LocalDateTime)return this._resolveLocal(adjuster);if(adjuster instanceof Instant){const instant=adjuster;return ZonedDateTime._create(instant.epochSecond(),instant.nano(),this._zone)}return adjuster instanceof ZoneOffset?this._resolveOffset(adjuster):super._withAdjuster(adjuster)}_withField(field,newValue){if(field instanceof ChronoField){switch(field){case ChronoField.INSTANT_SECONDS:return ZonedDateTime._create(newValue,this.nano(),this._zone);case ChronoField.OFFSET_SECONDS:{const offset=ZoneOffset.ofTotalSeconds(field.checkValidIntValue(newValue));return this._resolveOffset(offset)}}return this._resolveLocal(this._dateTime.with(field,newValue))}return field.adjustInto(this,newValue)}withYear(year){return this._resolveLocal(this._dateTime.withYear(year))}withMonth(month){return this._resolveLocal(this._dateTime.withMonth(month))}withDayOfMonth(dayOfMonth){return this._resolveLocal(this._dateTime.withDayOfMonth(dayOfMonth))}withDayOfYear(dayOfYear){return this._resolveLocal(this._dateTime.withDayOfYear(dayOfYear))}withHour(hour){return this._resolveLocal(this._dateTime.withHour(hour))}withMinute(minute){return this._resolveLocal(this._dateTime.withMinute(minute))}withSecond(second){return this._resolveLocal(this._dateTime.withSecond(second))}withNano(nanoOfSecond){return this._resolveLocal(this._dateTime.withNano(nanoOfSecond))}truncatedTo(unit){return this._resolveLocal(this._dateTime.truncatedTo(unit))}_plusUnit(amountToAdd,unit){return unit instanceof ChronoUnit?unit.isDateBased()?this._resolveLocal(this._dateTime.plus(amountToAdd,unit)):this._resolveInstant(this._dateTime.plus(amountToAdd,unit)):(requireNonNull(unit,"unit"),unit.addTo(this,amountToAdd))}plusYears(years){return this._resolveLocal(this._dateTime.plusYears(years))}plusMonths(months){return this._resolveLocal(this._dateTime.plusMonths(months))}plusWeeks(weeks){return this._resolveLocal(this._dateTime.plusWeeks(weeks))}plusDays(days){return this._resolveLocal(this._dateTime.plusDays(days))}plusHours(hours){return this._resolveInstant(this._dateTime.plusHours(hours))}plusMinutes(minutes){return this._resolveInstant(this._dateTime.plusMinutes(minutes))}plusSeconds(seconds){return this._resolveInstant(this._dateTime.plusSeconds(seconds))}plusNanos(nanos){return this._resolveInstant(this._dateTime.plusNanos(nanos))}_minusUnit(amountToSubtract,unit){return this._plusUnit(-1*amountToSubtract,unit)}minusYears(years){return this.plusYears(-1*years)}minusMonths(months){return this.plusMonths(-1*months)}minusWeeks(weeks){return this.plusWeeks(-1*weeks)}minusDays(days){return this.plusDays(-1*days)}minusHours(hours){return this.plusHours(-1*hours)}minusMinutes(minutes){return this.plusMinutes(-1*minutes)}minusSeconds(seconds){return this.plusSeconds(-1*seconds)}minusNanos(nanos){return this.plusNanos(-1*nanos)}query(query){return query===TemporalQueries.localDate()?this.toLocalDate():(requireNonNull(query,"query"),super.query(query))}until(endExclusive,unit){let end=ZonedDateTime.from(endExclusive);if(unit instanceof ChronoUnit){if(end=end.withZoneSameInstant(this._zone),unit.isDateBased())return this._dateTime.until(end._dateTime,unit);{const difference=this._offset.totalSeconds()-end._offset.totalSeconds(),adjustedEnd=end._dateTime.plusSeconds(difference);return this._dateTime.until(adjustedEnd,unit)}}return unit.between(this,end)}toLocalDateTime(){return this._dateTime}toLocalDate(){return this._dateTime.toLocalDate()}toLocalTime(){return this._dateTime.toLocalTime()}toOffsetDateTime(){return OffsetDateTime.of(this._dateTime,this._offset)}equals(other){return this===other||other instanceof ZonedDateTime&&(this._dateTime.equals(other._dateTime)&&this._offset.equals(other._offset)&&this._zone.equals(other._zone))}hashCode(){return MathUtil.hashCode(this._dateTime.hashCode(),this._offset.hashCode(),this._zone.hashCode())}toString(){let str=this._dateTime.toString()+this._offset.toString();return this._offset!==this._zone&&(str+=`[${this._zone.toString()}]`),str}toJSON(){return this.toString()}format(formatter){return super.format(formatter)}}export function _init(){ZonedDateTime.FROM=createTemporalQuery("ZonedDateTime.FROM",(temporal=>ZonedDateTime.from(temporal)))}