/*
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{requireNonNull}from"../assert";import{DateTimeException}from"../errors";import{MathUtil}from"../MathUtil";import{EnumMap}from"./EnumMap";import{ResolverStyle}from"./ResolverStyle";import{IsoChronology}from"../chrono/IsoChronology";import{ChronoLocalDate}from"../chrono/ChronoLocalDate";import{ChronoField}from"../temporal/ChronoField";import{TemporalAccessor}from"../temporal/TemporalAccessor";import{TemporalQueries}from"../temporal/TemporalQueries";import{LocalTime}from"../LocalTime";import{LocalDate}from"../LocalDate";import{Period}from"../Period";import{ZoneOffset}from"../ZoneOffset";export class DateTimeBuilder extends TemporalAccessor{static create(field,value){const dtb=new DateTimeBuilder;return dtb._addFieldValue(field,value),dtb}constructor(){super(),this.fieldValues=new EnumMap,this.chrono=null,this.zone=null,this.date=null,this.time=null,this.leapSecond=!1,this.excessDays=null}getFieldValue0(field){return this.fieldValues.get(field)}_addFieldValue(field,value){requireNonNull(field,"field");const old=this.getFieldValue0(field);if(null!=old&&old!==value)throw new DateTimeException(`Conflict found: ${field} ${old} differs from ${field} ${value}: ${this}`);return this._putFieldValue0(field,value)}_putFieldValue0(field,value){return this.fieldValues.put(field,value),this}resolve(resolverStyle,resolverFields){return null!=resolverFields&&this.fieldValues.retainAll(resolverFields),this._mergeDate(resolverStyle),this._mergeTime(resolverStyle),this._resolveTimeInferZeroes(resolverStyle),null!=this.excessDays&&!1===this.excessDays.isZero()&&null!=this.date&&null!=this.time&&(this.date=this.date.plus(this.excessDays),this.excessDays=Period.ZERO),this._resolveInstant(),this}_mergeDate(resolverStyle){this._checkDate(IsoChronology.INSTANCE.resolveDate(this.fieldValues,resolverStyle))}_checkDate(date){if(null!=date){this._addObject(date);for(const fieldName in this.fieldValues.keySet()){const field=ChronoField.byName(fieldName);if(field&&void 0!==this.fieldValues.get(field)&&field.isDateBased()){let val1;try{val1=date.getLong(field)}catch(ex){if(ex instanceof DateTimeException)continue;throw ex}const val2=this.fieldValues.get(field);if(val1!==val2)throw new DateTimeException(`Conflict found: Field ${field} ${val1} differs from ${field} ${val2} derived from ${date}`)}}}}_mergeTime(resolverStyle){if(this.fieldValues.containsKey(ChronoField.CLOCK_HOUR_OF_DAY)){const ch=this.fieldValues.remove(ChronoField.CLOCK_HOUR_OF_DAY);resolverStyle!==ResolverStyle.LENIENT&&(resolverStyle===ResolverStyle.SMART&&0===ch||ChronoField.CLOCK_HOUR_OF_DAY.checkValidValue(ch)),this._addFieldValue(ChronoField.HOUR_OF_DAY,24===ch?0:ch)}if(this.fieldValues.containsKey(ChronoField.CLOCK_HOUR_OF_AMPM)){const ch=this.fieldValues.remove(ChronoField.CLOCK_HOUR_OF_AMPM);resolverStyle!==ResolverStyle.LENIENT&&(resolverStyle===ResolverStyle.SMART&&0===ch||ChronoField.CLOCK_HOUR_OF_AMPM.checkValidValue(ch)),this._addFieldValue(ChronoField.HOUR_OF_AMPM,12===ch?0:ch)}if(resolverStyle!==ResolverStyle.LENIENT&&(this.fieldValues.containsKey(ChronoField.AMPM_OF_DAY)&&ChronoField.AMPM_OF_DAY.checkValidValue(this.fieldValues.get(ChronoField.AMPM_OF_DAY)),this.fieldValues.containsKey(ChronoField.HOUR_OF_AMPM)&&ChronoField.HOUR_OF_AMPM.checkValidValue(this.fieldValues.get(ChronoField.HOUR_OF_AMPM))),this.fieldValues.containsKey(ChronoField.AMPM_OF_DAY)&&this.fieldValues.containsKey(ChronoField.HOUR_OF_AMPM)){const ap=this.fieldValues.remove(ChronoField.AMPM_OF_DAY),hap=this.fieldValues.remove(ChronoField.HOUR_OF_AMPM);this._addFieldValue(ChronoField.HOUR_OF_DAY,12*ap+hap)}if(this.fieldValues.containsKey(ChronoField.NANO_OF_DAY)){const nod=this.fieldValues.remove(ChronoField.NANO_OF_DAY);resolverStyle!==ResolverStyle.LENIENT&&ChronoField.NANO_OF_DAY.checkValidValue(nod),this._addFieldValue(ChronoField.SECOND_OF_DAY,MathUtil.intDiv(nod,1e9)),this._addFieldValue(ChronoField.NANO_OF_SECOND,MathUtil.intMod(nod,1e9))}if(this.fieldValues.containsKey(ChronoField.MICRO_OF_DAY)){const cod=this.fieldValues.remove(ChronoField.MICRO_OF_DAY);resolverStyle!==ResolverStyle.LENIENT&&ChronoField.MICRO_OF_DAY.checkValidValue(cod),this._addFieldValue(ChronoField.SECOND_OF_DAY,MathUtil.intDiv(cod,1e6)),this._addFieldValue(ChronoField.MICRO_OF_SECOND,MathUtil.intMod(cod,1e6))}if(this.fieldValues.containsKey(ChronoField.MILLI_OF_DAY)){const lod=this.fieldValues.remove(ChronoField.MILLI_OF_DAY);resolverStyle!==ResolverStyle.LENIENT&&ChronoField.MILLI_OF_DAY.checkValidValue(lod),this._addFieldValue(ChronoField.SECOND_OF_DAY,MathUtil.intDiv(lod,1e3)),this._addFieldValue(ChronoField.MILLI_OF_SECOND,MathUtil.intMod(lod,1e3))}if(this.fieldValues.containsKey(ChronoField.SECOND_OF_DAY)){const sod=this.fieldValues.remove(ChronoField.SECOND_OF_DAY);resolverStyle!==ResolverStyle.LENIENT&&ChronoField.SECOND_OF_DAY.checkValidValue(sod),this._addFieldValue(ChronoField.HOUR_OF_DAY,MathUtil.intDiv(sod,3600)),this._addFieldValue(ChronoField.MINUTE_OF_HOUR,MathUtil.intMod(MathUtil.intDiv(sod,60),60)),this._addFieldValue(ChronoField.SECOND_OF_MINUTE,MathUtil.intMod(sod,60))}if(this.fieldValues.containsKey(ChronoField.MINUTE_OF_DAY)){const mod=this.fieldValues.remove(ChronoField.MINUTE_OF_DAY);resolverStyle!==ResolverStyle.LENIENT&&ChronoField.MINUTE_OF_DAY.checkValidValue(mod),this._addFieldValue(ChronoField.HOUR_OF_DAY,MathUtil.intDiv(mod,60)),this._addFieldValue(ChronoField.MINUTE_OF_HOUR,MathUtil.intMod(mod,60))}if(resolverStyle!==ResolverStyle.LENIENT&&(this.fieldValues.containsKey(ChronoField.MILLI_OF_SECOND)&&ChronoField.MILLI_OF_SECOND.checkValidValue(this.fieldValues.get(ChronoField.MILLI_OF_SECOND)),this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)&&ChronoField.MICRO_OF_SECOND.checkValidValue(this.fieldValues.get(ChronoField.MICRO_OF_SECOND))),this.fieldValues.containsKey(ChronoField.MILLI_OF_SECOND)&&this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)){const los=this.fieldValues.remove(ChronoField.MILLI_OF_SECOND),cos=this.fieldValues.get(ChronoField.MICRO_OF_SECOND);this._putFieldValue0(ChronoField.MICRO_OF_SECOND,1e3*los+MathUtil.intMod(cos,1e3))}if(this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)&&this.fieldValues.containsKey(ChronoField.NANO_OF_SECOND)){const nos=this.fieldValues.get(ChronoField.NANO_OF_SECOND);this._putFieldValue0(ChronoField.MICRO_OF_SECOND,MathUtil.intDiv(nos,1e3)),this.fieldValues.remove(ChronoField.MICRO_OF_SECOND)}if(this.fieldValues.containsKey(ChronoField.MILLI_OF_SECOND)&&this.fieldValues.containsKey(ChronoField.NANO_OF_SECOND)){const nos=this.fieldValues.get(ChronoField.NANO_OF_SECOND);this._putFieldValue0(ChronoField.MILLI_OF_SECOND,MathUtil.intDiv(nos,1e6)),this.fieldValues.remove(ChronoField.MILLI_OF_SECOND)}if(this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)){const cos=this.fieldValues.remove(ChronoField.MICRO_OF_SECOND);this._putFieldValue0(ChronoField.NANO_OF_SECOND,1e3*cos)}else if(this.fieldValues.containsKey(ChronoField.MILLI_OF_SECOND)){const los=this.fieldValues.remove(ChronoField.MILLI_OF_SECOND);this._putFieldValue0(ChronoField.NANO_OF_SECOND,1e6*los)}}_resolveTimeInferZeroes(resolverStyle){let hod=this.fieldValues.get(ChronoField.HOUR_OF_DAY);const moh=this.fieldValues.get(ChronoField.MINUTE_OF_HOUR),som=this.fieldValues.get(ChronoField.SECOND_OF_MINUTE);let nos=this.fieldValues.get(ChronoField.NANO_OF_SECOND);if(null!=hod&&(null!=moh||null==som&&null==nos)&&(null==moh||null!=som||null==nos)){if(resolverStyle!==ResolverStyle.LENIENT){if(null!=hod){resolverStyle!==ResolverStyle.SMART||24!==hod||null!=moh&&0!==moh||null!=som&&0!==som||null!=nos&&0!==nos||(hod=0,this.excessDays=Period.ofDays(1));const hodVal=ChronoField.HOUR_OF_DAY.checkValidIntValue(hod);if(null!=moh){const mohVal=ChronoField.MINUTE_OF_HOUR.checkValidIntValue(moh);if(null!=som){const somVal=ChronoField.SECOND_OF_MINUTE.checkValidIntValue(som);if(null!=nos){const nosVal=ChronoField.NANO_OF_SECOND.checkValidIntValue(nos);this._addObject(LocalTime.of(hodVal,mohVal,somVal,nosVal))}else this._addObject(LocalTime.of(hodVal,mohVal,somVal))}else null==nos&&this._addObject(LocalTime.of(hodVal,mohVal))}else null==som&&null==nos&&this._addObject(LocalTime.of(hodVal,0))}}else if(null!=hod){let hodVal=hod;if(null!=moh)if(null!=som){null==nos&&(nos=0);let totalNanos=MathUtil.safeMultiply(hodVal,36e11);totalNanos=MathUtil.safeAdd(totalNanos,MathUtil.safeMultiply(moh,6e10)),totalNanos=MathUtil.safeAdd(totalNanos,MathUtil.safeMultiply(som,1e9)),totalNanos=MathUtil.safeAdd(totalNanos,nos);const excessDays=MathUtil.floorDiv(totalNanos,864e11),nod=MathUtil.floorMod(totalNanos,864e11);this._addObject(LocalTime.ofNanoOfDay(nod)),this.excessDays=Period.ofDays(excessDays)}else{let totalSecs=MathUtil.safeMultiply(hodVal,3600);totalSecs=MathUtil.safeAdd(totalSecs,MathUtil.safeMultiply(moh,60));const excessDays=MathUtil.floorDiv(totalSecs,86400),sod=MathUtil.floorMod(totalSecs,86400);this._addObject(LocalTime.ofSecondOfDay(sod)),this.excessDays=Period.ofDays(excessDays)}else{const excessDays=MathUtil.safeToInt(MathUtil.floorDiv(hodVal,24));hodVal=MathUtil.floorMod(hodVal,24),this._addObject(LocalTime.of(hodVal,0)),this.excessDays=Period.ofDays(excessDays)}}this.fieldValues.remove(ChronoField.HOUR_OF_DAY),this.fieldValues.remove(ChronoField.MINUTE_OF_HOUR),this.fieldValues.remove(ChronoField.SECOND_OF_MINUTE),this.fieldValues.remove(ChronoField.NANO_OF_SECOND)}}_addObject(dateOrTime){dateOrTime instanceof ChronoLocalDate?this.date=dateOrTime:dateOrTime instanceof LocalTime&&(this.time=dateOrTime)}_resolveInstant(){if(null!=this.date&&null!=this.time){const offsetSecs=this.fieldValues.get(ChronoField.OFFSET_SECONDS);if(null!=offsetSecs){const offset=ZoneOffset.ofTotalSeconds(offsetSecs),instant=this.date.atTime(this.time).atZone(offset).getLong(ChronoField.INSTANT_SECONDS);this.fieldValues.put(ChronoField.INSTANT_SECONDS,instant)}else if(null!=this.zone){const instant=this.date.atTime(this.time).atZone(this.zone).getLong(ChronoField.INSTANT_SECONDS);this.fieldValues.put(ChronoField.INSTANT_SECONDS,instant)}}}build(type){return type.queryFrom(this)}isSupported(field){return null!=field&&(this.fieldValues.containsKey(field)&&void 0!==this.fieldValues.get(field)||null!=this.date&&this.date.isSupported(field)||null!=this.time&&this.time.isSupported(field))}getLong(field){requireNonNull(field,"field");const value=this.getFieldValue0(field);if(null==value){if(null!=this.date&&this.date.isSupported(field))return this.date.getLong(field);if(null!=this.time&&this.time.isSupported(field))return this.time.getLong(field);throw new DateTimeException(`Field not found: ${field}`)}return value}query(query){return query===TemporalQueries.zoneId()?this.zone:query===TemporalQueries.chronology()?this.chrono:query===TemporalQueries.localDate()?null!=this.date?LocalDate.from(this.date):null:query===TemporalQueries.localTime()?this.time:query===TemporalQueries.zone()||query===TemporalQueries.offset()?query.queryFrom(this):query===TemporalQueries.precision()?null:query.queryFrom(this)}}