/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{assert,requireNonNull,requireInstance}from"../assert";import{IllegalArgumentException,IllegalStateException}from"../errors";import{MathUtil}from"../MathUtil";import{LocalDate}from"../LocalDate";import{LocalDateTime}from"../LocalDateTime";import{ZoneOffset}from"../ZoneOffset";import{ChronoLocalDate}from"../chrono/ChronoLocalDate";import{ChronoField}from"../temporal/ChronoField";import{IsoFields}from"../temporal/IsoFields";import{TemporalQueries}from"../temporal/TemporalQueries";import{DateTimeFormatter}from"./DateTimeFormatter";import{DecimalStyle}from"./DecimalStyle";import{SignStyle}from"./SignStyle";import{TextStyle}from"./TextStyle";import{ResolverStyle}from"./ResolverStyle";import{CharLiteralPrinterParser}from"./parser/CharLiteralPrinterParser";import{CompositePrinterParser}from"./parser/CompositePrinterParser";import{FractionPrinterParser}from"./parser/FractionPrinterParser";import{NumberPrinterParser,ReducedPrinterParser}from"./parser/NumberPrinterParser";import{OffsetIdPrinterParser}from"./parser/OffsetIdPrinterParser";import{PadPrinterParserDecorator}from"./parser/PadPrinterParserDecorator";import{SettingsParser}from"./parser/SettingsParser";import{StringLiteralPrinterParser}from"./parser/StringLiteralPrinterParser";import{ZoneIdPrinterParser}from"./parser/ZoneIdPrinterParser";const MAX_WIDTH=15;export class DateTimeFormatterBuilder{constructor(){this._active=this,this._parent=null,this._printerParsers=[],this._optional=!1,this._padNextWidth=0,this._padNextChar=null,this._valueParserIndex=-1}static _of(parent,optional){requireNonNull(parent,"parent"),requireNonNull(optional,"optional");const dtFormatterBuilder=new DateTimeFormatterBuilder;return dtFormatterBuilder._parent=parent,dtFormatterBuilder._optional=optional,dtFormatterBuilder}parseCaseSensitive(){return this._appendInternalPrinterParser(SettingsParser.SENSITIVE),this}parseCaseInsensitive(){return this._appendInternalPrinterParser(SettingsParser.INSENSITIVE),this}parseStrict(){return this._appendInternalPrinterParser(SettingsParser.STRICT),this}parseLenient(){return this._appendInternalPrinterParser(SettingsParser.LENIENT),this}appendValue(){return 1===arguments.length?this._appendValue1.apply(this,arguments):2===arguments.length?this._appendValue2.apply(this,arguments):this._appendValue4.apply(this,arguments)}_appendValue1(field){return requireNonNull(field),this._appendValuePrinterParser(new NumberPrinterParser(field,1,MAX_WIDTH,SignStyle.NORMAL)),this}_appendValue2(field,width){if(requireNonNull(field),width<1||width>MAX_WIDTH)throw new IllegalArgumentException(`The width must be from 1 to ${MAX_WIDTH} inclusive but was ${width}`);const pp=new NumberPrinterParser(field,width,width,SignStyle.NOT_NEGATIVE);return this._appendValuePrinterParser(pp),this}_appendValue4(field,minWidth,maxWidth,signStyle){if(requireNonNull(field),requireNonNull(signStyle),minWidth===maxWidth&&signStyle===SignStyle.NOT_NEGATIVE)return this._appendValue2(field,maxWidth);if(minWidth<1||minWidth>MAX_WIDTH)throw new IllegalArgumentException(`The minimum width must be from 1 to ${MAX_WIDTH} inclusive but was ${minWidth}`);if(maxWidth<1||maxWidth>MAX_WIDTH)throw new IllegalArgumentException(`The minimum width must be from 1 to ${MAX_WIDTH} inclusive but was ${maxWidth}`);if(maxWidth<minWidth)throw new IllegalArgumentException(`The maximum width must exceed or equal the minimum width but ${maxWidth} < ${minWidth}`);const pp=new NumberPrinterParser(field,minWidth,maxWidth,signStyle);return this._appendValuePrinterParser(pp),this}appendValueReduced(){return 4===arguments.length&&arguments[3]instanceof ChronoLocalDate?this._appendValueReducedFieldWidthMaxWidthBaseDate.apply(this,arguments):this._appendValueReducedFieldWidthMaxWidthBaseValue.apply(this,arguments)}_appendValueReducedFieldWidthMaxWidthBaseValue(field,width,maxWidth,baseValue){requireNonNull(field,"field");const pp=new ReducedPrinterParser(field,width,maxWidth,baseValue,null);return this._appendValuePrinterParser(pp),this}_appendValueReducedFieldWidthMaxWidthBaseDate(field,width,maxWidth,baseDate){requireNonNull(field,"field"),requireNonNull(baseDate,"baseDate"),requireInstance(baseDate,ChronoLocalDate,"baseDate");const pp=new ReducedPrinterParser(field,width,maxWidth,0,baseDate);return this._appendValuePrinterParser(pp),this}_appendValuePrinterParser(pp){if(assert(null!=pp),this._active._valueParserIndex>=0&&this._active._printerParsers[this._active._valueParserIndex]instanceof NumberPrinterParser){const activeValueParser=this._active._valueParserIndex;let basePP=this._active._printerParsers[activeValueParser];pp.minWidth()===pp.maxWidth()&&pp.signStyle()===SignStyle.NOT_NEGATIVE?(basePP=basePP.withSubsequentWidth(pp.maxWidth()),this._appendInternal(pp.withFixedWidth()),this._active._valueParserIndex=activeValueParser):(basePP=basePP.withFixedWidth(),this._active._valueParserIndex=this._appendInternal(pp)),this._active._printerParsers[activeValueParser]=basePP}else this._active._valueParserIndex=this._appendInternal(pp);return this}appendFraction(field,minWidth,maxWidth,decimalPoint){return this._appendInternal(new FractionPrinterParser(field,minWidth,maxWidth,decimalPoint)),this}appendInstant(fractionalDigits=-2){if(fractionalDigits<-2||fractionalDigits>9)throw new IllegalArgumentException(`Invalid fractional digits: ${fractionalDigits}`);return this._appendInternal(new InstantPrinterParser(fractionalDigits)),this}appendOffsetId(){return this._appendInternal(OffsetIdPrinterParser.INSTANCE_ID),this}appendOffset(pattern,noOffsetText){return this._appendInternalPrinterParser(new OffsetIdPrinterParser(noOffsetText,pattern)),this}appendZoneId(){return this._appendInternal(new ZoneIdPrinterParser(TemporalQueries.zoneId(),"ZoneId()")),this}appendPattern(pattern){return requireNonNull(pattern,"pattern"),this._parsePattern(pattern),this}appendZoneText(){throw new IllegalArgumentException("Pattern using (localized) text not implemented, use @js-joda/locale plugin!")}appendText(){throw new IllegalArgumentException("Pattern using (localized) text not implemented, use @js-joda/locale plugin!")}appendLocalizedOffset(){throw new IllegalArgumentException("Pattern using (localized) text not implemented, use @js-joda/locale plugin!")}appendWeekField(){throw new IllegalArgumentException("Pattern using (localized) text not implemented, use @js-joda/locale plugin!")}_parsePattern(pattern){const FIELD_MAP={G:ChronoField.ERA,y:ChronoField.YEAR_OF_ERA,u:ChronoField.YEAR,Q:IsoFields.QUARTER_OF_YEAR,q:IsoFields.QUARTER_OF_YEAR,M:ChronoField.MONTH_OF_YEAR,L:ChronoField.MONTH_OF_YEAR,D:ChronoField.DAY_OF_YEAR,d:ChronoField.DAY_OF_MONTH,F:ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH,E:ChronoField.DAY_OF_WEEK,c:ChronoField.DAY_OF_WEEK,e:ChronoField.DAY_OF_WEEK,a:ChronoField.AMPM_OF_DAY,H:ChronoField.HOUR_OF_DAY,k:ChronoField.CLOCK_HOUR_OF_DAY,K:ChronoField.HOUR_OF_AMPM,h:ChronoField.CLOCK_HOUR_OF_AMPM,m:ChronoField.MINUTE_OF_HOUR,s:ChronoField.SECOND_OF_MINUTE,S:ChronoField.NANO_OF_SECOND,A:ChronoField.MILLI_OF_DAY,n:ChronoField.NANO_OF_SECOND,N:ChronoField.NANO_OF_DAY};for(let pos=0;pos<pattern.length;pos++){let cur=pattern.charAt(pos);if(cur>="A"&&cur<="Z"||cur>="a"&&cur<="z"){let start=pos++;for(;pos<pattern.length&&pattern.charAt(pos)===cur;pos++);let count=pos-start;if("p"===cur){let pad=0;if(pos<pattern.length&&(cur=pattern.charAt(pos),cur>="A"&&cur<="Z"||cur>="a"&&cur<="z")){for(pad=count,start=pos++;pos<pattern.length&&pattern.charAt(pos)===cur;pos++);count=pos-start}if(0===pad)throw new IllegalArgumentException(`Pad letter 'p' must be followed by valid pad pattern: ${pattern}`);this.padNext(pad)}const field=FIELD_MAP[cur];if(null!=field)this._parseField(cur,count,field);else if("z"===cur){if(count>4)throw new IllegalArgumentException(`Too many pattern letters: ${cur}`);4===count?this.appendZoneText(TextStyle.FULL):this.appendZoneText(TextStyle.SHORT)}else if("V"===cur){if(2!==count)throw new IllegalArgumentException(`Pattern letter count must be 2: ${cur}`);this.appendZoneId()}else if("Z"===cur)if(count<4)this.appendOffset("+HHMM","+0000");else if(4===count)this.appendLocalizedOffset(TextStyle.FULL);else{if(5!==count)throw new IllegalArgumentException(`Too many pattern letters: ${cur}`);this.appendOffset("+HH:MM:ss","Z")}else if("O"===cur)if(1===count)this.appendLocalizedOffset(TextStyle.SHORT);else{if(4!==count)throw new IllegalArgumentException(`Pattern letter count must be 1 or 4: ${cur}`);this.appendLocalizedOffset(TextStyle.FULL)}else if("X"===cur){if(count>5)throw new IllegalArgumentException(`Too many pattern letters: ${cur}`);this.appendOffset(OffsetIdPrinterParser.PATTERNS[count+(1===count?0:1)],"Z")}else if("x"===cur){if(count>5)throw new IllegalArgumentException(`Too many pattern letters: ${cur}`);const zero=1===count?"+00":count%2==0?"+0000":"+00:00";this.appendOffset(OffsetIdPrinterParser.PATTERNS[count+(1===count?0:1)],zero)}else if("W"===cur){if(count>1)throw new IllegalArgumentException(`Too many pattern letters: ${cur}`);this.appendWeekField("W",count)}else if("w"===cur){if(count>2)throw new IllegalArgumentException(`Too many pattern letters: ${cur}`);this.appendWeekField("w",count)}else{if("Y"!==cur)throw new IllegalArgumentException(`Unknown pattern letter: ${cur}`);this.appendWeekField("Y",count)}pos--}else if("'"===cur){const start=pos++;for(;pos<pattern.length;pos++)if("'"===pattern.charAt(pos)){if(!(pos+1<pattern.length&&"'"===pattern.charAt(pos+1)))break;pos++}if(pos>=pattern.length)throw new IllegalArgumentException(`Pattern ends with an incomplete string literal: ${pattern}`);const str=pattern.substring(start+1,pos);0===str.length?this.appendLiteral("'"):this.appendLiteral(str.replace("''","'"))}else if("["===cur)this.optionalStart();else if("]"===cur){if(null===this._active._parent)throw new IllegalArgumentException("Pattern invalid as it contains ] without previous [");this.optionalEnd()}else{if("{"===cur||"}"===cur||"#"===cur)throw new IllegalArgumentException(`Pattern includes reserved character: '${cur}'`);this.appendLiteral(cur)}}}_parseField(cur,count,field){switch(cur){case"u":case"y":2===count?this.appendValueReduced(field,2,2,ReducedPrinterParser.BASE_DATE):count<4?this.appendValue(field,count,MAX_WIDTH,SignStyle.NORMAL):this.appendValue(field,count,MAX_WIDTH,SignStyle.EXCEEDS_PAD);break;case"M":case"Q":switch(count){case 1:this.appendValue(field);break;case 2:this.appendValue(field,2);break;case 3:this.appendText(field,TextStyle.SHORT);break;case 4:this.appendText(field,TextStyle.FULL);break;case 5:this.appendText(field,TextStyle.NARROW);break;default:throw new IllegalArgumentException(`Too many pattern letters: ${cur}`)}break;case"L":case"q":switch(count){case 1:this.appendValue(field);break;case 2:this.appendValue(field,2);break;case 3:this.appendText(field,TextStyle.SHORT_STANDALONE);break;case 4:this.appendText(field,TextStyle.FULL_STANDALONE);break;case 5:this.appendText(field,TextStyle.NARROW_STANDALONE);break;default:throw new IllegalArgumentException(`Too many pattern letters: ${cur}`)}break;case"e":switch(count){case 1:case 2:this.appendWeekField("e",count);break;case 3:this.appendText(field,TextStyle.SHORT);break;case 4:this.appendText(field,TextStyle.FULL);break;case 5:this.appendText(field,TextStyle.NARROW);break;default:throw new IllegalArgumentException(`Too many pattern letters: ${cur}`)}break;case"c":switch(count){case 1:this.appendWeekField("c",count);break;case 2:throw new IllegalArgumentException(`Invalid number of pattern letters: ${cur}`);case 3:this.appendText(field,TextStyle.SHORT_STANDALONE);break;case 4:this.appendText(field,TextStyle.FULL_STANDALONE);break;case 5:this.appendText(field,TextStyle.NARROW_STANDALONE);break;default:throw new IllegalArgumentException(`Too many pattern letters: ${cur}`)}break;case"a":if(1!==count)throw new IllegalArgumentException(`Too many pattern letters: ${cur}`);this.appendText(field,TextStyle.SHORT);break;case"E":case"G":switch(count){case 1:case 2:case 3:this.appendText(field,TextStyle.SHORT);break;case 4:this.appendText(field,TextStyle.FULL);break;case 5:this.appendText(field,TextStyle.NARROW);break;default:throw new IllegalArgumentException(`Too many pattern letters: ${cur}`)}break;case"S":this.appendFraction(ChronoField.NANO_OF_SECOND,count,count,!1);break;case"F":if(1!==count)throw new IllegalArgumentException(`Too many pattern letters: ${cur}`);this.appendValue(field);break;case"d":case"h":case"H":case"k":case"K":case"m":case"s":if(1===count)this.appendValue(field);else{if(2!==count)throw new IllegalArgumentException(`Too many pattern letters: ${cur}`);this.appendValue(field,count)}break;case"D":if(1===count)this.appendValue(field);else{if(!(count<=3))throw new IllegalArgumentException(`Too many pattern letters: ${cur}`);this.appendValue(field,count)}break;default:1===count?this.appendValue(field):this.appendValue(field,count)}}padNext(){return 1===arguments.length?this._padNext1.apply(this,arguments):this._padNext2.apply(this,arguments)}_padNext1(padWidth){return this._padNext2(padWidth," ")}_padNext2(padWidth,padChar){if(padWidth<1)throw new IllegalArgumentException(`The pad width must be at least one but was ${padWidth}`);return this._active._padNextWidth=padWidth,this._active._padNextChar=padChar,this._active._valueParserIndex=-1,this}optionalStart(){return this._active._valueParserIndex=-1,this._active=DateTimeFormatterBuilder._of(this._active,!0),this}optionalEnd(){if(null==this._active._parent)throw new IllegalStateException("Cannot call optionalEnd() as there was no previous call to optionalStart()");if(this._active._printerParsers.length>0){const cpp=new CompositePrinterParser(this._active._printerParsers,this._active._optional);this._active=this._active._parent,this._appendInternal(cpp)}else this._active=this._active._parent;return this}_appendInternal(pp){return assert(null!=pp),this._active._padNextWidth>0&&(null!=pp&&(pp=new PadPrinterParserDecorator(pp,this._active._padNextWidth,this._active._padNextChar)),this._active._padNextWidth=0,this._active._padNextChar=0),this._active._printerParsers.push(pp),this._active._valueParserIndex=-1,this._active._printerParsers.length-1}appendLiteral(literal){return assert(null!=literal),literal.length>0&&(1===literal.length?this._appendInternalPrinterParser(new CharLiteralPrinterParser(literal.charAt(0))):this._appendInternalPrinterParser(new StringLiteralPrinterParser(literal))),this}_appendInternalPrinterParser(pp){return assert(null!=pp),this._active._padNextWidth>0&&(null!=pp&&(pp=new PadPrinterParserDecorator(pp,this._active._padNextWidth,this._active._padNextChar)),this._active._padNextWidth=0,this._active._padNextChar=0),this._active._printerParsers.push(pp),this._active._valueParserIndex=-1,this._active._printerParsers.length-1}append(formatter){return requireNonNull(formatter,"formatter"),this._appendInternal(formatter._toPrinterParser(!1)),this}toFormatter(resolverStyle=ResolverStyle.SMART){for(;null!=this._active._parent;)this.optionalEnd();const pp=new CompositePrinterParser(this._printerParsers,!1);return new DateTimeFormatter(pp,null,DecimalStyle.STANDARD,resolverStyle,null,null,null)}}const SECONDS_PER_10000_YEARS=31556952e4,SECONDS_0000_TO_1970=62167219200;class InstantPrinterParser{constructor(fractionalDigits){this.fractionalDigits=fractionalDigits}print(context,buf){const inSecs=context.getValue(ChronoField.INSTANT_SECONDS);let inNanos=0;if(context.temporal().isSupported(ChronoField.NANO_OF_SECOND)&&(inNanos=context.temporal().getLong(ChronoField.NANO_OF_SECOND)),null==inSecs)return!1;const inSec=inSecs;let inNano=ChronoField.NANO_OF_SECOND.checkValidIntValue(inNanos);if(inSec>=-SECONDS_0000_TO_1970){const zeroSecs=inSec-SECONDS_PER_10000_YEARS+SECONDS_0000_TO_1970,hi=MathUtil.floorDiv(zeroSecs,SECONDS_PER_10000_YEARS)+1,lo=MathUtil.floorMod(zeroSecs,SECONDS_PER_10000_YEARS),ldt=LocalDateTime.ofEpochSecond(lo-SECONDS_0000_TO_1970,0,ZoneOffset.UTC);hi>0&&buf.append("+").append(hi),buf.append(ldt.toString()),0===ldt.second()&&buf.append(":00")}else{const zeroSecs=inSec+SECONDS_0000_TO_1970,hi=MathUtil.intDiv(zeroSecs,SECONDS_PER_10000_YEARS),lo=MathUtil.intMod(zeroSecs,SECONDS_PER_10000_YEARS),ldt=LocalDateTime.ofEpochSecond(lo-SECONDS_0000_TO_1970,0,ZoneOffset.UTC),pos=buf.length();buf.append(ldt.toString()),0===ldt.second()&&buf.append(":00"),hi<0&&(-1e4===ldt.year()?buf.replace(pos,pos+2,""+(hi-1)):0===lo?buf.insert(pos,hi):buf.insert(pos+1,Math.abs(hi)))}if(-2===this.fractionalDigits)0!==inNano&&(buf.append("."),0===MathUtil.intMod(inNano,1e6)?buf.append(`${MathUtil.intDiv(inNano,1e6)+1e3}`.substring(1)):0===MathUtil.intMod(inNano,1e3)?buf.append(`${MathUtil.intDiv(inNano,1e3)+1e6}`.substring(1)):buf.append(`${inNano+1e9}`.substring(1)));else if(this.fractionalDigits>0||-1===this.fractionalDigits&&inNano>0){buf.append(".");let div=1e8;for(let i=0;-1===this.fractionalDigits&&inNano>0||i<this.fractionalDigits;i++){const digit=MathUtil.intDiv(inNano,div);buf.append(digit),inNano-=digit*div,div=MathUtil.intDiv(div,10)}}return buf.append("Z"),!0}parse(context,text,position){const newContext=context.copy(),minDigits=this.fractionalDigits<0?0:this.fractionalDigits,maxDigits=this.fractionalDigits<0?9:this.fractionalDigits,pos=(new DateTimeFormatterBuilder).append(DateTimeFormatter.ISO_LOCAL_DATE).appendLiteral("T").appendValue(ChronoField.HOUR_OF_DAY,2).appendLiteral(":").appendValue(ChronoField.MINUTE_OF_HOUR,2).appendLiteral(":").appendValue(ChronoField.SECOND_OF_MINUTE,2).appendFraction(ChronoField.NANO_OF_SECOND,minDigits,maxDigits,!0).appendLiteral("Z").toFormatter()._toPrinterParser(!1).parse(newContext,text,position);if(pos<0)return pos;const yearParsed=newContext.getParsed(ChronoField.YEAR),month=newContext.getParsed(ChronoField.MONTH_OF_YEAR),day=newContext.getParsed(ChronoField.DAY_OF_MONTH);let hour=newContext.getParsed(ChronoField.HOUR_OF_DAY);const min=newContext.getParsed(ChronoField.MINUTE_OF_HOUR),secVal=newContext.getParsed(ChronoField.SECOND_OF_MINUTE),nanoVal=newContext.getParsed(ChronoField.NANO_OF_SECOND);let sec=null!=secVal?secVal:0;const nano=null!=nanoVal?nanoVal:0,year=MathUtil.intMod(yearParsed,1e4);let instantSecs,days=0;24===hour&&0===min&&0===sec&&0===nano?(hour=0,days=1):23===hour&&59===min&&60===sec&&(context.setParsedLeapSecond(),sec=59);try{instantSecs=LocalDateTime.of(year,month,day,hour,min,sec,0).plusDays(days).toEpochSecond(ZoneOffset.UTC),instantSecs+=MathUtil.safeMultiply(MathUtil.intDiv(yearParsed,1e4),SECONDS_PER_10000_YEARS)}catch(ex){return~position}let successPos=pos;return successPos=context.setParsedField(ChronoField.INSTANT_SECONDS,instantSecs,position,successPos),context.setParsedField(ChronoField.NANO_OF_SECOND,nano,position,successPos)}toString(){return"Instant()"}}export function _init(){ReducedPrinterParser.BASE_DATE=LocalDate.of(2e3,1,1),DateTimeFormatterBuilder.CompositePrinterParser=CompositePrinterParser,DateTimeFormatterBuilder.PadPrinterParserDecorator=PadPrinterParserDecorator,DateTimeFormatterBuilder.SettingsParser=SettingsParser,DateTimeFormatterBuilder.CharLiteralPrinterParser=StringLiteralPrinterParser,DateTimeFormatterBuilder.StringLiteralPrinterParser=StringLiteralPrinterParser,DateTimeFormatterBuilder.CharLiteralPrinterParser=CharLiteralPrinterParser,DateTimeFormatterBuilder.NumberPrinterParser=NumberPrinterParser,DateTimeFormatterBuilder.ReducedPrinterParser=ReducedPrinterParser,DateTimeFormatterBuilder.FractionPrinterParser=FractionPrinterParser,DateTimeFormatterBuilder.OffsetIdPrinterParser=OffsetIdPrinterParser,DateTimeFormatterBuilder.ZoneIdPrinterParser=ZoneIdPrinterParser}