/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{assert}from"../../assert";import{ArithmeticException,DateTimeException,IllegalArgumentException}from"../../errors";import{MathUtil}from"../../MathUtil";import{IsoChronology}from"../../chrono/IsoChronology";import{SignStyle}from"../SignStyle";const MAX_WIDTH=15,EXCEED_POINTS=[0,10,100,1e3,1e4,1e5,1e6,1e7,1e8,1e9];export class NumberPrinterParser{constructor(field,minWidth,maxWidth,signStyle,subsequentWidth=0){this._field=field,this._minWidth=minWidth,this._maxWidth=maxWidth,this._signStyle=signStyle,this._subsequentWidth=subsequentWidth}field(){return this._field}minWidth(){return this._minWidth}maxWidth(){return this._maxWidth}signStyle(){return this._signStyle}withFixedWidth(){return-1===this._subsequentWidth?this:new NumberPrinterParser(this._field,this._minWidth,this._maxWidth,this._signStyle,-1)}withSubsequentWidth(subsequentWidth){return new NumberPrinterParser(this._field,this._minWidth,this._maxWidth,this._signStyle,this._subsequentWidth+subsequentWidth)}_isFixedWidth(){return-1===this._subsequentWidth||this._subsequentWidth>0&&this._minWidth===this._maxWidth&&this._signStyle===SignStyle.NOT_NEGATIVE}print(context,buf){const contextValue=context.getValue(this._field);if(null==contextValue)return!1;const value=this._getValue(context,contextValue),symbols=context.symbols();let str=`${Math.abs(value)}`;if(str.length>this._maxWidth)throw new DateTimeException(`Field ${this._field} cannot be printed as the value ${value} exceeds the maximum print width of ${this._maxWidth}`);if(str=symbols.convertNumberToI18N(str),value>=0)switch(this._signStyle){case SignStyle.EXCEEDS_PAD:this._minWidth<MAX_WIDTH&&value>=EXCEED_POINTS[this._minWidth]&&buf.append(symbols.positiveSign());break;case SignStyle.ALWAYS:buf.append(symbols.positiveSign())}else switch(this._signStyle){case SignStyle.NORMAL:case SignStyle.EXCEEDS_PAD:case SignStyle.ALWAYS:buf.append(symbols.negativeSign());break;case SignStyle.NOT_NEGATIVE:throw new DateTimeException(`Field ${this._field} cannot be printed as the value ${value} cannot be negative according to the SignStyle`)}for(let i=0;i<this._minWidth-str.length;i++)buf.append(symbols.zeroDigit());return buf.append(str),!0}parse(context,text,position){const length=text.length;if(position===length)return~position;assert(position>=0&&position<length);const sign=text.charAt(position);let negative=!1,positive=!1;if(sign===context.symbols().positiveSign()){if(!1===this._signStyle.parse(!0,context.isStrict(),this._minWidth===this._maxWidth))return~position;positive=!0,position++}else if(sign===context.symbols().negativeSign()){if(!1===this._signStyle.parse(!1,context.isStrict(),this._minWidth===this._maxWidth))return~position;negative=!0,position++}else if(this._signStyle===SignStyle.ALWAYS&&context.isStrict())return~position;const effMinWidth=context.isStrict()||this._isFixedWidth()?this._minWidth:1,minEndPos=position+effMinWidth;if(minEndPos>length)return~position;let effMaxWidth=(context.isStrict()||this._isFixedWidth()?this._maxWidth:9)+Math.max(this._subsequentWidth,0),total=0,pos=position;for(let pass=0;pass<2;pass++){const maxEndPos=Math.min(pos+effMaxWidth,length);for(;pos<maxEndPos;){const ch=text.charAt(pos++),digit=context.symbols().convertToDigit(ch);if(digit<0){if(pos--,pos<minEndPos)return~position;break}if(pos-position>MAX_WIDTH)throw new ArithmeticException("number text exceeds length");total=10*total+digit}if(!(this._subsequentWidth>0&&0===pass))break;{const parseLen=pos-position;effMaxWidth=Math.max(effMinWidth,parseLen-this._subsequentWidth),pos=position,total=0}}if(negative){if(0===total&&context.isStrict())return~(position-1);0!==total&&(total=-total)}else if(this._signStyle===SignStyle.EXCEEDS_PAD&&context.isStrict()){const parseLen=pos-position;if(positive){if(parseLen<=this._minWidth)return~(position-1)}else if(parseLen>this._minWidth)return~position}return this._setValue(context,total,position,pos)}_getValue(context,value){return value}_setValue(context,value,errorPos,successPos){return context.setParsedField(this._field,value,errorPos,successPos)}toString(){return 1===this._minWidth&&this._maxWidth===MAX_WIDTH&&this._signStyle===SignStyle.NORMAL?`Value(${this._field})`:this._minWidth===this._maxWidth&&this._signStyle===SignStyle.NOT_NEGATIVE?`Value(${this._field},${this._minWidth})`:`Value(${this._field},${this._minWidth},${this._maxWidth},${this._signStyle})`}}export class ReducedPrinterParser extends NumberPrinterParser{constructor(field,width,maxWidth,baseValue,baseDate){if(super(field,width,maxWidth,SignStyle.NOT_NEGATIVE),width<1||width>10)throw new IllegalArgumentException(`The width must be from 1 to 10 inclusive but was ${width}`);if(maxWidth<1||maxWidth>10)throw new IllegalArgumentException(`The maxWidth must be from 1 to 10 inclusive but was ${maxWidth}`);if(maxWidth<width)throw new IllegalArgumentException("The maxWidth must be greater than the width");if(null===baseDate){if(!1===field.range().isValidValue(baseValue))throw new IllegalArgumentException("The base value must be within the range of the field");if(baseValue+EXCEED_POINTS[width]>MathUtil.MAX_SAFE_INTEGER)throw new DateTimeException("Unable to add printer-parser as the range exceeds the capacity of an int")}this._baseValue=baseValue,this._baseDate=baseDate}_getValue(context,value){const absValue=Math.abs(value);let baseValue=this._baseValue;if(null!==this._baseDate){context.temporal();baseValue=IsoChronology.INSTANCE.date(this._baseDate).get(this._field)}return value>=baseValue&&value<baseValue+EXCEED_POINTS[this._minWidth]?absValue%EXCEED_POINTS[this._minWidth]:absValue%EXCEED_POINTS[this._maxWidth]}_setValue(context,value,errorPos,successPos){let baseValue=this._baseValue;if(null!=this._baseDate){baseValue=context.getEffectiveChronology().date(this._baseDate).get(this._field)}if(successPos-errorPos===this._minWidth&&value>=0){const range=EXCEED_POINTS[this._minWidth],basePart=baseValue-baseValue%range;(value=baseValue>0?basePart+value:basePart-value)<baseValue&&(value+=range)}return context.setParsedField(this._field,value,errorPos,successPos)}withFixedWidth(){return-1===this._subsequentWidth?this:new ReducedPrinterParser(this._field,this._minWidth,this._maxWidth,this._baseValue,this._baseDate,-1)}withSubsequentWidth(subsequentWidth){return new ReducedPrinterParser(this._field,this._minWidth,this._maxWidth,this._baseValue,this._baseDate,this._subsequentWidth+subsequentWidth)}isFixedWidth(context){return!1!==context.isStrict()&&super.isFixedWidth(context)}toString(){return`ReducedValue(${this._field},${this._minWidth},${this._maxWidth},${null!=this._baseDate?this._baseDate:this._baseValue})`}}