/**
 * @copyright (c) 2016, Philipp Thürwächter & Pattrick Hüper
 * @copyright (c) 2007-present, Stephen Colebourne & Michael Nascimento Santos
 * @license BSD-3-Clause (see LICENSE in the root directory of this source tree)
 */
import{MAX_SAFE_INTEGER,MIN_SAFE_INTEGER}from"../MathUtil";import{ChronoUnit}from"./ChronoUnit";import{TemporalField}from"./TemporalField";import{ValueRange}from"./ValueRange";import{YearConstants}from"../YearConstants";export class ChronoField extends TemporalField{static byName(fieldName){for(const prop in ChronoField)if(ChronoField[prop]&&ChronoField[prop]instanceof ChronoField&&ChronoField[prop].name()===fieldName)return ChronoField[prop]}constructor(name,baseUnit,rangeUnit,range){super(),this._name=name,this._baseUnit=baseUnit,this._rangeUnit=rangeUnit,this._range=range}name(){return this._name}baseUnit(){return this._baseUnit}rangeUnit(){return this._rangeUnit}range(){return this._range}displayName(){return this.toString()}checkValidValue(value){return this.range().checkValidValue(value,this)}checkValidIntValue(value){return this.range().checkValidIntValue(value,this)}isDateBased(){return this===ChronoField.DAY_OF_WEEK||this===ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH||this===ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR||this===ChronoField.DAY_OF_MONTH||this===ChronoField.DAY_OF_YEAR||this===ChronoField.EPOCH_DAY||this===ChronoField.ALIGNED_WEEK_OF_MONTH||this===ChronoField.ALIGNED_WEEK_OF_YEAR||this===ChronoField.MONTH_OF_YEAR||this===ChronoField.PROLEPTIC_MONTH||this===ChronoField.YEAR_OF_ERA||this===ChronoField.YEAR||this===ChronoField.ERA}isTimeBased(){return this===ChronoField.NANO_OF_SECOND||this===ChronoField.NANO_OF_DAY||this===ChronoField.MICRO_OF_SECOND||this===ChronoField.MICRO_OF_DAY||this===ChronoField.MILLI_OF_SECOND||this===ChronoField.MILLI_OF_DAY||this===ChronoField.SECOND_OF_MINUTE||this===ChronoField.SECOND_OF_DAY||this===ChronoField.MINUTE_OF_HOUR||this===ChronoField.MINUTE_OF_DAY||this===ChronoField.HOUR_OF_AMPM||this===ChronoField.CLOCK_HOUR_OF_AMPM||this===ChronoField.HOUR_OF_DAY||this===ChronoField.CLOCK_HOUR_OF_DAY||this===ChronoField.AMPM_OF_DAY}rangeRefinedBy(temporal){return temporal.range(this)}getFrom(temporal){return temporal.getLong(this)}toString(){return this.name()}equals(other){return this===other}adjustInto(temporal,newValue){return temporal.with(this,newValue)}isSupportedBy(temporal){return temporal.isSupported(this)}}export function _init(){ChronoField.NANO_OF_SECOND=new ChronoField("NanoOfSecond",ChronoUnit.NANOS,ChronoUnit.SECONDS,ValueRange.of(0,999999999)),ChronoField.NANO_OF_DAY=new ChronoField("NanoOfDay",ChronoUnit.NANOS,ChronoUnit.DAYS,ValueRange.of(0,86399999999999)),ChronoField.MICRO_OF_SECOND=new ChronoField("MicroOfSecond",ChronoUnit.MICROS,ChronoUnit.SECONDS,ValueRange.of(0,999999)),ChronoField.MICRO_OF_DAY=new ChronoField("MicroOfDay",ChronoUnit.MICROS,ChronoUnit.DAYS,ValueRange.of(0,86399999999)),ChronoField.MILLI_OF_SECOND=new ChronoField("MilliOfSecond",ChronoUnit.MILLIS,ChronoUnit.SECONDS,ValueRange.of(0,999)),ChronoField.MILLI_OF_DAY=new ChronoField("MilliOfDay",ChronoUnit.MILLIS,ChronoUnit.DAYS,ValueRange.of(0,86399999)),ChronoField.SECOND_OF_MINUTE=new ChronoField("SecondOfMinute",ChronoUnit.SECONDS,ChronoUnit.MINUTES,ValueRange.of(0,59)),ChronoField.SECOND_OF_DAY=new ChronoField("SecondOfDay",ChronoUnit.SECONDS,ChronoUnit.DAYS,ValueRange.of(0,86399)),ChronoField.MINUTE_OF_HOUR=new ChronoField("MinuteOfHour",ChronoUnit.MINUTES,ChronoUnit.HOURS,ValueRange.of(0,59)),ChronoField.MINUTE_OF_DAY=new ChronoField("MinuteOfDay",ChronoUnit.MINUTES,ChronoUnit.DAYS,ValueRange.of(0,1439)),ChronoField.HOUR_OF_AMPM=new ChronoField("HourOfAmPm",ChronoUnit.HOURS,ChronoUnit.HALF_DAYS,ValueRange.of(0,11)),ChronoField.CLOCK_HOUR_OF_AMPM=new ChronoField("ClockHourOfAmPm",ChronoUnit.HOURS,ChronoUnit.HALF_DAYS,ValueRange.of(1,12)),ChronoField.HOUR_OF_DAY=new ChronoField("HourOfDay",ChronoUnit.HOURS,ChronoUnit.DAYS,ValueRange.of(0,23)),ChronoField.CLOCK_HOUR_OF_DAY=new ChronoField("ClockHourOfDay",ChronoUnit.HOURS,ChronoUnit.DAYS,ValueRange.of(1,24)),ChronoField.AMPM_OF_DAY=new ChronoField("AmPmOfDay",ChronoUnit.HALF_DAYS,ChronoUnit.DAYS,ValueRange.of(0,1)),ChronoField.DAY_OF_WEEK=new ChronoField("DayOfWeek",ChronoUnit.DAYS,ChronoUnit.WEEKS,ValueRange.of(1,7)),ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH=new ChronoField("AlignedDayOfWeekInMonth",ChronoUnit.DAYS,ChronoUnit.WEEKS,ValueRange.of(1,7)),ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR=new ChronoField("AlignedDayOfWeekInYear",ChronoUnit.DAYS,ChronoUnit.WEEKS,ValueRange.of(1,7)),ChronoField.DAY_OF_MONTH=new ChronoField("DayOfMonth",ChronoUnit.DAYS,ChronoUnit.MONTHS,ValueRange.of(1,28,31),"day"),ChronoField.DAY_OF_YEAR=new ChronoField("DayOfYear",ChronoUnit.DAYS,ChronoUnit.YEARS,ValueRange.of(1,365,366)),ChronoField.EPOCH_DAY=new ChronoField("EpochDay",ChronoUnit.DAYS,ChronoUnit.FOREVER,ValueRange.of(-365961662,364522971)),ChronoField.ALIGNED_WEEK_OF_MONTH=new ChronoField("AlignedWeekOfMonth",ChronoUnit.WEEKS,ChronoUnit.MONTHS,ValueRange.of(1,4,5)),ChronoField.ALIGNED_WEEK_OF_YEAR=new ChronoField("AlignedWeekOfYear",ChronoUnit.WEEKS,ChronoUnit.YEARS,ValueRange.of(1,53)),ChronoField.MONTH_OF_YEAR=new ChronoField("MonthOfYear",ChronoUnit.MONTHS,ChronoUnit.YEARS,ValueRange.of(1,12),"month"),ChronoField.PROLEPTIC_MONTH=new ChronoField("ProlepticMonth",ChronoUnit.MONTHS,ChronoUnit.FOREVER,ValueRange.of(12*YearConstants.MIN_VALUE,12*YearConstants.MAX_VALUE+11)),ChronoField.YEAR_OF_ERA=new ChronoField("YearOfEra",ChronoUnit.YEARS,ChronoUnit.FOREVER,ValueRange.of(1,YearConstants.MAX_VALUE,YearConstants.MAX_VALUE+1)),ChronoField.YEAR=new ChronoField("Year",ChronoUnit.YEARS,ChronoUnit.FOREVER,ValueRange.of(YearConstants.MIN_VALUE,YearConstants.MAX_VALUE),"year"),ChronoField.ERA=new ChronoField("Era",ChronoUnit.ERAS,ChronoUnit.FOREVER,ValueRange.of(0,1)),ChronoField.INSTANT_SECONDS=new ChronoField("InstantSeconds",ChronoUnit.SECONDS,ChronoUnit.FOREVER,ValueRange.of(MIN_SAFE_INTEGER,MAX_SAFE_INTEGER)),ChronoField.OFFSET_SECONDS=new ChronoField("OffsetSeconds",ChronoUnit.SECONDS,ChronoUnit.FOREVER,ValueRange.of(-64800,64800))}